@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2015-2020 Canonical Ltd
+ * Copyright (C) 2015-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -23,12 +23,17 @@ import (
 	"bufio"
 	"bytes"
 	"crypto"
+	"encoding/json"
 	"fmt"
 	"io"
+	"reflect"
 	"sort"
 	"strconv"
 	"strings"
 	"unicode/utf8"
+
+	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/snap/naming"
 )
 
 type typeFlags int
@@ -36,6 +41,7 @@ type typeFlags int
 const (
 	noAuthority typeFlags = 1 << iota
 	sequenceForming
+	jsonBody
 )
 
 // MetaHeaders is a list of headers in assertions which are about the assertion
@@ -56,17 +62,48 @@ type AssertionType struct {
 	// PrimaryKey holds the names of the headers that constitute the
 	// unique primary key for this assertion type.
 	PrimaryKey []string
+	// OptionalPrimaryKeyDefaults holds the default values for
+	// optional primary key headers.
+	// Optional primary key headers can be added to types defined
+	// in previous versions of snapd, as long as they are added at
+	// the end of the old primary key together with a default value set in
+	// this map. So they must form a contiguous suffix of PrimaryKey with
+	// each member having a default value set in this map.
+	// Optional primary key headers are not supported for sequence
+	// forming types.
+	OptionalPrimaryKeyDefaults map[string]string
 
 	assembler func(assert assertionBase) (Assertion, error)
 	flags     typeFlags
 }
 
+func (at *AssertionType) validate() {
+	if len(at.OptionalPrimaryKeyDefaults) != 0 && at.flags&sequenceForming != 0 {
+		panic(fmt.Sprintf("assertion type %q cannot be both sequence forming and have optional primary keys", at.Name))
+	}
+	noptional := 0
+	for _, k := range at.PrimaryKey {
+		defl := at.OptionalPrimaryKeyDefaults[k]
+		if noptional > 0 {
+			if defl == "" {
+				panic(fmt.Sprintf("assertion type %q primary key header %q has no default, optional primary keys must be a proper suffix of the primary key", at.Name, k))
+			}
+		}
+		if defl != "" {
+			noptional++
+		}
+	}
+	if len(at.OptionalPrimaryKeyDefaults) != noptional {
+		panic(fmt.Sprintf("assertion type %q has defaults values for unknown primary key headers", at.Name))
+	}
+}
+
 // MaxSupportedFormat returns the maximum supported format iteration for the type.
 func (at *AssertionType) MaxSupportedFormat() int {
 	return maxSupportedFormat[at.Name]
 }
 
-// SequencingForming returns true if the assertion type has a positive
+// SequenceForming returns true if the assertion type has a positive
 // integer >= 1 as the last component (preferably called "sequence")
 // of its primary key over which the assertions of the type form
 // sequences, usually without gaps, one sequence per sequence key (the
@@ -76,52 +113,78 @@ func (at *AssertionType) SequenceForming() bool {
 	return at.flags&sequenceForming != 0
 }
 
+// AcceptablePrimaryKey returns whether the given key could be an acceptable primary key for this type, allowing for the omission of optional primary key headers.
+func (at *AssertionType) AcceptablePrimaryKey(key []string) bool {
+	n := len(at.PrimaryKey)
+	nopt := len(at.OptionalPrimaryKeyDefaults)
+	ninp := len(key)
+	if ninp > n || ninp < (n-nopt) {
+		return false
+	}
+	return true
+}
+
+// JSONBody returns true if the body for this assertion type must be JSON.
+func (at *AssertionType) JSONBody() bool {
+	return at.flags&jsonBody != 0
+}
+
 // Understood assertion types.
 var (
-	AccountType         = &AssertionType{"account", []string{"account-id"}, assembleAccount, 0}
-	AccountKeyType      = &AssertionType{"account-key", []string{"public-key-sha3-384"}, assembleAccountKey, 0}
-	RepairType          = &AssertionType{"repair", []string{"brand-id", "repair-id"}, assembleRepair, sequenceForming}
-	ModelType           = &AssertionType{"model", []string{"series", "brand-id", "model"}, assembleModel, 0}
-	SerialType          = &AssertionType{"serial", []string{"brand-id", "model", "serial"}, assembleSerial, 0}
-	BaseDeclarationType = &AssertionType{"base-declaration", []string{"series"}, assembleBaseDeclaration, 0}
-	SnapDeclarationType = &AssertionType{"snap-declaration", []string{"series", "snap-id"}, assembleSnapDeclaration, 0}
-	SnapBuildType       = &AssertionType{"snap-build", []string{"snap-sha3-384"}, assembleSnapBuild, 0}
-	SnapRevisionType    = &AssertionType{"snap-revision", []string{"snap-sha3-384"}, assembleSnapRevision, 0}
-	SnapDeveloperType   = &AssertionType{"snap-developer", []string{"snap-id", "publisher-id"}, assembleSnapDeveloper, 0}
-	SystemUserType      = &AssertionType{"system-user", []string{"brand-id", "email"}, assembleSystemUser, 0}
-	ValidationType      = &AssertionType{"validation", []string{"series", "snap-id", "approved-snap-id", "approved-snap-revision"}, assembleValidation, 0}
-	ValidationSetType   = &AssertionType{"validation-set", []string{"series", "account-id", "name", "sequence"}, assembleValidationSet, sequenceForming}
-	StoreType           = &AssertionType{"store", []string{"store"}, assembleStore, 0}
+	AccountType              = &AssertionType{"account", []string{"account-id"}, nil, assembleAccount, 0}
+	AccountKeyType           = &AssertionType{"account-key", []string{"public-key-sha3-384"}, nil, assembleAccountKey, 0}
+	RepairType               = &AssertionType{"repair", []string{"brand-id", "repair-id"}, nil, assembleRepair, sequenceForming}
+	ModelType                = &AssertionType{"model", []string{"series", "brand-id", "model"}, nil, assembleModel, 0}
+	SerialType               = &AssertionType{"serial", []string{"brand-id", "model", "serial"}, nil, assembleSerial, 0}
+	BaseDeclarationType      = &AssertionType{"base-declaration", []string{"series"}, nil, assembleBaseDeclaration, 0}
+	SnapDeclarationType      = &AssertionType{"snap-declaration", []string{"series", "snap-id"}, nil, assembleSnapDeclaration, 0}
+	SnapBuildType            = &AssertionType{"snap-build", []string{"snap-sha3-384"}, nil, assembleSnapBuild, 0}
+	SnapRevisionType         = &AssertionType{"snap-revision", []string{"snap-sha3-384", "provenance"}, map[string]string{"provenance": naming.DefaultProvenance}, assembleSnapRevision, 0}
+	SnapDeveloperType        = &AssertionType{"snap-developer", []string{"snap-id", "publisher-id"}, nil, assembleSnapDeveloper, 0}
+	SystemUserType           = &AssertionType{"system-user", []string{"brand-id", "email"}, nil, assembleSystemUser, 0}
+	ValidationType           = &AssertionType{"validation", []string{"series", "snap-id", "approved-snap-id", "approved-snap-revision"}, nil, assembleValidation, 0}
+	ValidationSetType        = &AssertionType{"validation-set", []string{"series", "account-id", "name", "sequence"}, nil, assembleValidationSet, sequenceForming}
+	StoreType                = &AssertionType{"store", []string{"store"}, nil, assembleStore, 0}
+	PreseedType              = &AssertionType{"preseed", []string{"series", "brand-id", "model", "system-label"}, nil, assemblePreseed, 0}
+	SnapResourceRevisionType = &AssertionType{"snap-resource-revision", []string{"snap-id", "resource-name", "resource-sha3-384", "provenance"}, map[string]string{"provenance": naming.DefaultProvenance}, assembleSnapResourceRevision, 0}
+	SnapResourcePairType     = &AssertionType{"snap-resource-pair", []string{"snap-id", "resource-name", "resource-revision", "snap-revision", "provenance"}, map[string]string{"provenance": naming.DefaultProvenance}, assembleSnapResourcePair, 0}
+	ConfdbSchemaType         = &AssertionType{"confdb-schema", []string{"account-id", "name"}, nil, assembleConfdbSchema, jsonBody}
 
-// ...
+	// ...
 )
 
 // Assertion types without a definite authority set (on the wire and/or self-signed).
 var (
-	DeviceSessionRequestType = &AssertionType{"device-session-request", []string{"brand-id", "model", "serial"}, assembleDeviceSessionRequest, noAuthority}
-	SerialRequestType        = &AssertionType{"serial-request", nil, assembleSerialRequest, noAuthority}
-	AccountKeyRequestType    = &AssertionType{"account-key-request", []string{"public-key-sha3-384"}, assembleAccountKeyRequest, noAuthority}
+	DeviceSessionRequestType = &AssertionType{"device-session-request", []string{"brand-id", "model", "serial"}, nil, assembleDeviceSessionRequest, noAuthority}
+	SerialRequestType        = &AssertionType{"serial-request", nil, nil, assembleSerialRequest, noAuthority}
+	AccountKeyRequestType    = &AssertionType{"account-key-request", []string{"public-key-sha3-384"}, nil, assembleAccountKeyRequest, noAuthority}
+	ConfdbControlType        = &AssertionType{"confdb-control", []string{"brand-id", "model", "serial"}, nil, assembleConfdbControl, noAuthority}
 )
 
 var typeRegistry = map[string]*AssertionType{
-	AccountType.Name:         AccountType,
-	AccountKeyType.Name:      AccountKeyType,
-	ModelType.Name:           ModelType,
-	SerialType.Name:          SerialType,
-	BaseDeclarationType.Name: BaseDeclarationType,
-	SnapDeclarationType.Name: SnapDeclarationType,
-	SnapBuildType.Name:       SnapBuildType,
-	SnapRevisionType.Name:    SnapRevisionType,
-	SnapDeveloperType.Name:   SnapDeveloperType,
-	SystemUserType.Name:      SystemUserType,
-	ValidationType.Name:      ValidationType,
-	ValidationSetType.Name:   ValidationSetType,
-	RepairType.Name:          RepairType,
-	StoreType.Name:           StoreType,
+	AccountType.Name:              AccountType,
+	AccountKeyType.Name:           AccountKeyType,
+	ModelType.Name:                ModelType,
+	SerialType.Name:               SerialType,
+	BaseDeclarationType.Name:      BaseDeclarationType,
+	SnapDeclarationType.Name:      SnapDeclarationType,
+	SnapBuildType.Name:            SnapBuildType,
+	SnapRevisionType.Name:         SnapRevisionType,
+	SnapDeveloperType.Name:        SnapDeveloperType,
+	SystemUserType.Name:           SystemUserType,
+	ValidationType.Name:           ValidationType,
+	ValidationSetType.Name:        ValidationSetType,
+	RepairType.Name:               RepairType,
+	StoreType.Name:                StoreType,
+	PreseedType.Name:              PreseedType,
+	SnapResourceRevisionType.Name: SnapResourceRevisionType,
+	SnapResourcePairType.Name:     SnapResourcePairType,
+	ConfdbSchemaType.Name:         ConfdbSchemaType,
 	// no authority
 	DeviceSessionRequestType.Name: DeviceSessionRequestType,
 	SerialRequestType.Name:        SerialRequestType,
 	AccountKeyRequestType.Name:    AccountKeyRequestType,
+	ConfdbControlType.Name:        ConfdbControlType,
 }
 
 // Type returns the AssertionType with name or nil
@@ -151,10 +214,19 @@ func init() {
 	// 3: support for on-store/on-brand/on-model device scope constraints
 	// 4: support for plug-names/slot-names constraints
 	// 5: alt attr matcher usage (was unused before, has new behavior now)
-	maxSupportedFormat[SnapDeclarationType.Name] = 5
+	// 6: support for $PLUG_PUBLISHER_ID/$SLOT_PUBLISHER_ID in attr constraints
+	maxSupportedFormat[SnapDeclarationType.Name] = 6
 
 	// 1: support to limit to device serials
-	maxSupportedFormat[SystemUserType.Name] = 1
+	// 2: support for user-presence constraint
+	maxSupportedFormat[SystemUserType.Name] = 2
+
+	// 1: support for constraints
+	maxSupportedFormat[AccountKeyType.Name] = 1
+
+	for _, at := range typeRegistry {
+		at.validate()
+	}
 }
 
 func MockMaxSupportedFormat(assertType *AssertionType, maxFormat int) (restore func()) {
@@ -165,7 +237,25 @@ func MockMaxSupportedFormat(assertType *AssertionType, maxFormat int) (restore f
 	}
 }
 
-var formatAnalyzer = map[*AssertionType]func(headers map[string]interface{}, body []byte) (formatnum int, err error){
+func MockOptionalPrimaryKey(assertType *AssertionType, key, defaultValue string) (restore func()) {
+	osutil.MustBeTestBinary("mocking new assertion optional primary keys can be done only from tests")
+	oldPrimaryKey := assertType.PrimaryKey
+	oldOptionalPrimaryKeyDefaults := assertType.OptionalPrimaryKeyDefaults
+	newOptionalPrimaryKeyDefaults := make(map[string]string, len(oldOptionalPrimaryKeyDefaults)+1)
+	for k, defl := range oldOptionalPrimaryKeyDefaults {
+		newOptionalPrimaryKeyDefaults[k] = defl
+	}
+	assertType.PrimaryKey = append(assertType.PrimaryKey, key)
+	assertType.OptionalPrimaryKeyDefaults = newOptionalPrimaryKeyDefaults
+	newOptionalPrimaryKeyDefaults[key] = defaultValue
+	return func() {
+		assertType.PrimaryKey = oldPrimaryKey
+		assertType.OptionalPrimaryKeyDefaults = oldOptionalPrimaryKeyDefaults
+	}
+}
+
+var formatAnalyzer = map[*AssertionType]func(headers map[string]any, body []byte) (formatnum int, err error){
+	AccountKeyType:      accountKeyFormatAnalyze,
 	SnapDeclarationType: snapDeclarationFormatAnalyze,
 	SystemUserType:      systemUserFormatAnalyze,
 }
@@ -189,7 +279,7 @@ func MaxSupportedFormats(min int) (maxFormats map[string]int) {
 }
 
 // SuggestFormat returns a minimum format that supports the features that would be used by an assertion with the given components.
-func SuggestFormat(assertType *AssertionType, headers map[string]interface{}, body []byte) (formatnum int, err error) {
+func SuggestFormat(assertType *AssertionType, headers map[string]any, body []byte) (formatnum int, err error) {
 	analyzer := formatAnalyzer[assertType]
 	if analyzer == nil {
 		// no analyzer, format 0 is all there is
@@ -204,16 +294,23 @@ func SuggestFormat(assertType *AssertionType, headers map[string]interface{}, bo
 
 // HeadersFromPrimaryKey constructs a headers mapping from the
 // primaryKey values and the assertion type, it errors if primaryKey
-// has the wrong length.
+// does not cover all the non-optional primary key headers or provides
+// too many values.
 func HeadersFromPrimaryKey(assertType *AssertionType, primaryKey []string) (headers map[string]string, err error) {
-	if len(primaryKey) != len(assertType.PrimaryKey) {
+	if !assertType.AcceptablePrimaryKey(primaryKey) {
 		return nil, fmt.Errorf("primary key has wrong length for %q assertion", assertType.Name)
 	}
+	ninp := len(primaryKey)
 	headers = make(map[string]string, len(assertType.PrimaryKey))
 	for i, name := range assertType.PrimaryKey {
-		keyVal := primaryKey[i]
-		if keyVal == "" {
-			return nil, fmt.Errorf("primary key %q header cannot be empty", name)
+		var keyVal string
+		if i < ninp {
+			keyVal = primaryKey[i]
+			if keyVal == "" {
+				return nil, fmt.Errorf("primary key %q header cannot be empty", name)
+			}
+		} else {
+			keyVal = assertType.OptionalPrimaryKeyDefaults[name]
 		}
 		headers[name] = keyVal
 	}
@@ -244,23 +341,52 @@ func HeadersFromSequenceKey(assertType *AssertionType, sequenceKey []string) (he
 
 // PrimaryKeyFromHeaders extracts the tuple of values from headers
 // corresponding to a primary key under the assertion type, it errors
-// if there are missing primary key headers.
+// if there are missing primary key headers unless they are optional
+// in which case it fills in their default values.
 func PrimaryKeyFromHeaders(assertType *AssertionType, headers map[string]string) (primaryKey []string, err error) {
-	return keysFromHeaders(assertType.PrimaryKey, headers)
+	return keysFromHeaders(assertType.PrimaryKey, headers, assertType.OptionalPrimaryKeyDefaults)
 }
 
-func keysFromHeaders(keys []string, headers map[string]string) (keyValues []string, err error) {
+func keysFromHeaders(keys []string, headers map[string]string, defaults map[string]string) (keyValues []string, err error) {
 	keyValues = make([]string, len(keys))
 	for i, k := range keys {
 		keyVal := headers[k]
 		if keyVal == "" {
-			return nil, fmt.Errorf("must provide primary key: %v", k)
+			keyVal = defaults[k]
+			if keyVal == "" {
+				return nil, fmt.Errorf("must provide primary key: %v", k)
+			}
 		}
 		keyValues[i] = keyVal
 	}
 	return keyValues, nil
 }
 
+// ReducePrimaryKey produces a primary key prefix by omitting any
+// suffix of optional primary key headers default values.
+// Too short or long primary keys are returned as is.
+func ReducePrimaryKey(assertType *AssertionType, primaryKey []string) []string {
+	n := len(assertType.PrimaryKey)
+	nopt := len(assertType.OptionalPrimaryKeyDefaults)
+	ninp := len(primaryKey)
+	if ninp > n || ninp < (n-nopt) {
+		return primaryKey
+	}
+	reduced := make([]string, n-nopt, n)
+	copy(reduced, primaryKey[:n-nopt])
+	rest := ninp - (n - nopt)
+	for i := ninp - 1; i >= n-nopt; i-- {
+		defl := assertType.OptionalPrimaryKeyDefaults[assertType.PrimaryKey[i]]
+		if primaryKey[i] != defl {
+			break
+		}
+		// it matches the default value, leave it out
+		rest--
+	}
+	reduced = append(reduced, primaryKey[n-nopt:n-nopt+rest]...)
+	return reduced
+}
+
 // Ref expresses a reference to an assertion.
 type Ref struct {
 	Type       *AssertionType
@@ -270,15 +396,26 @@ type Ref struct {
 func (ref *Ref) String() string {
 	pkStr := "-"
 	n := len(ref.Type.PrimaryKey)
-	if n != len(ref.PrimaryKey) {
+	nopt := len(ref.Type.OptionalPrimaryKeyDefaults)
+	ninp := len(ref.PrimaryKey)
+	if ninp > n || ninp < (n-nopt) {
 		pkStr = "???"
 	} else if n > 0 {
-		pkStr = ref.PrimaryKey[n-1]
+		pkStr = ref.PrimaryKey[n-nopt-1]
 		if n > 1 {
 			sfx := []string{pkStr + ";"}
-			for i, k := range ref.Type.PrimaryKey[:n-1] {
+			for i, k := range ref.Type.PrimaryKey[:n-nopt-1] {
 				sfx = append(sfx, fmt.Sprintf("%s:%s", k, ref.PrimaryKey[i]))
 			}
+			// optional primary keys
+			for i := n - nopt; i < ninp; i++ {
+				v := ref.PrimaryKey[i]
+				k := ref.Type.PrimaryKey[i]
+				defl := ref.Type.OptionalPrimaryKeyDefaults[k]
+				if v != defl {
+					sfx = append(sfx, fmt.Sprintf("%s:%s", k, v))
+				}
+			}
 			pkStr = strings.Join(sfx, " ")
 		}
 	}
@@ -287,7 +424,7 @@ func (ref *Ref) String() string {
 
 // Unique returns a unique string representing the reference that can be used as a key in maps.
 func (ref *Ref) Unique() string {
-	return fmt.Sprintf("%s/%s", ref.Type.Name, strings.Join(ref.PrimaryKey, "/"))
+	return fmt.Sprintf("%s/%s", ref.Type.Name, strings.Join(ReducePrimaryKey(ref.Type, ref.PrimaryKey), "/"))
 }
 
 // Resolve resolves the reference using the given find function.
@@ -394,14 +531,15 @@ type Assertion interface {
 	SupportedFormat() bool
 	// Revision returns the revision of this assertion
 	Revision() int
-	// AuthorityID returns the authority that signed this assertion
+	// AuthorityID returns the authority responsible for this
+	// assertion
 	AuthorityID() string
 
 	// Header retrieves the header with name
-	Header(name string) interface{}
+	Header(name string) any
 
 	// Headers returns the complete headers
-	Headers() map[string]interface{}
+	Headers() map[string]any
 
 	// HeaderString retrieves the string value of header with name or ""
 	HeaderString(name string) string
@@ -436,7 +574,7 @@ type SequenceMember interface {
 // customSigner represents an assertion with special arrangements for its signing key (e.g. self-signed), rather than the usual case where an assertion is signed by its authority.
 type customSigner interface {
 	// signKey returns the public key material for the key that signed this assertion.  See also SignKeyID.
-	signKey() PublicKey
+	signKey(db RODatabase) (PublicKey, error)
 }
 
 // MediaType is the media type for encoded assertions on the wire.
@@ -444,7 +582,7 @@ const MediaType = "application/x.ubuntu.assertion"
 
 // assertionBase is the concrete base to hold representation data for actual assertions.
 type assertionBase struct {
-	headers map[string]interface{}
+	headers map[string]any
 	body    []byte
 	// parsed format iteration
 	format int
@@ -484,13 +622,13 @@ func (ab *assertionBase) Revision() int {
 	return ab.revision
 }
 
-// AuthorityID returns the authority-id a.k.a the signer id of the assertion.
+// AuthorityID returns the authority-id a.k.a the authority responsible for the assertion.
 func (ab *assertionBase) AuthorityID() string {
 	return ab.HeaderString("authority-id")
 }
 
 // Header returns the value of an header by name.
-func (ab *assertionBase) Header(name string) interface{} {
+func (ab *assertionBase) Header(name string) any {
 	v := ab.headers[name]
 	if v == nil {
 		return nil
@@ -499,7 +637,7 @@ func (ab *assertionBase) Header(name string) interface{} {
 }
 
 // Headers returns the complete headers.
-func (ab *assertionBase) Headers() map[string]interface{} {
+func (ab *assertionBase) Headers() map[string]any {
 	return copyHeaders(ab.headers)
 }
 
@@ -541,33 +679,33 @@ func (ab *assertionBase) At() *AtRevision {
 	return &AtRevision{Ref: *ab.Ref(), Revision: ab.Revision()}
 }
 
-// sanity check
+// expected interface is implemented
 var _ Assertion = (*assertionBase)(nil)
 
 // Decode parses a serialized assertion.
 //
 // The expected serialisation format looks like:
 //
-//   HEADER ("\n\n" BODY?)? "\n\n" SIGNATURE
+//	HEADER ("\n\n" BODY?)? "\n\n" SIGNATURE
 //
 // where:
 //
-//    HEADER is a set of header entries separated by "\n"
-//    BODY can be arbitrary text,
-//    SIGNATURE is the signature
+//	HEADER is a set of header entries separated by "\n"
+//	BODY can be arbitrary text,
+//	SIGNATURE is the signature
 //
 // Both BODY and HEADER must be UTF8.
 //
 // A header entry for a single line value (no '\n' in it) looks like:
 //
-//   NAME ": " SIMPLEVALUE
+//	NAME ": " SIMPLEVALUE
 //
 // The format supports multiline text values (with '\n's in them) and
 // lists or maps, possibly nested, with string scalars in them.
 //
 // For those a header entry looks like:
 //
-//   NAME ":\n" MULTI(baseindent)
+//	NAME ":\n" MULTI(baseindent)
 //
 // where MULTI can be
 //
@@ -575,11 +713,11 @@ var _ Assertion = (*assertionBase)(nil)
 //
 // * entries of a list each of the form:
 //
-//     " "*baseindent "  -"  ( " " SIMPLEVALUE | "\n" MULTI )
+//	" "*baseindent "  -"  ( " " SIMPLEVALUE | "\n" MULTI )
 //
 // * entries of map each of the form:
 //
-//     " "*baseindent "  " NAME ":"  ( " " SIMPLEVALUE | "\n" MULTI )
+//	" "*baseindent "  " NAME ":"  ( " " SIMPLEVALUE | "\n" MULTI )
 //
 // baseindent starts at 0 and then grows with nesting matching the
 // previous level introduction (e.g. the " "*baseindent " -" bit)
@@ -587,8 +725,8 @@ var _ Assertion = (*assertionBase)(nil)
 //
 // In general the following headers are mandatory:
 //
-//   type
-//   authority-id (except for on the wire/self-signed assertions like serial-request)
+//	type
+//	authority-id (except for on the wire/self-signed assertions like serial-request)
 //
 // Further for a given assertion type all the primary key headers
 // must be non empty and must not contain '/'.
@@ -596,12 +734,11 @@ var _ Assertion = (*assertionBase)(nil)
 // The following headers expect string representing integer values and
 // if omitted otherwise are assumed to be 0:
 //
-//   revision (a positive int)
-//   body-length (expected to be equal to the length of BODY)
-//   format (a positive int for the format iteration of the type used)
+//	revision (a positive int)
+//	body-length (expected to be equal to the length of BODY)
+//	format (a positive int for the format iteration of the type used)
 //
 // Times are expected to be in the RFC3339 format: "2006-01-02T15:04:05Z07:00".
-//
 func Decode(serializedAssertion []byte) (Assertion, error) {
 	// copy to get an independent backstorage that can't be mutated later
 	assertionSnapshot := make([]byte, len(serializedAssertion))
@@ -826,7 +963,7 @@ func (d *Decoder) Decode() (Assertion, error) {
 	return assemble(headers, finalBody, finalContent, finalSig)
 }
 
-func checkIteration(headers map[string]interface{}, name string) (int, error) {
+func checkIteration(headers map[string]any, name string) (int, error) {
 	iternum, err := checkIntWithDefault(headers, name, 0)
 	if err != nil {
 		return -1, err
@@ -837,16 +974,16 @@ func checkIteration(headers map[string]interface{}, name string) (int, error) {
 	return iternum, nil
 }
 
-func checkFormat(headers map[string]interface{}) (int, error) {
+func checkFormat(headers map[string]any) (int, error) {
 	return checkIteration(headers, "format")
 }
 
-func checkRevision(headers map[string]interface{}) (int, error) {
+func checkRevision(headers map[string]any) (int, error) {
 	return checkIteration(headers, "revision")
 }
 
 // Assemble assembles an assertion from its components.
-func Assemble(headers map[string]interface{}, body, content, signature []byte) (Assertion, error) {
+func Assemble(headers map[string]any, body, content, signature []byte) (Assertion, error) {
 	err := checkHeaders(headers)
 	if err != nil {
 		return nil, err
@@ -854,8 +991,52 @@ func Assemble(headers map[string]interface{}, body, content, signature []byte) (
 	return assemble(headers, body, content, signature)
 }
 
+func checkAuthority(_ *AssertionType, headers map[string]any) error {
+	if _, err := checkNotEmptyString(headers, "authority-id"); err != nil {
+		return err
+	}
+	return nil
+}
+
+func checkNoAuthority(assertType *AssertionType, headers map[string]any) error {
+	if _, ok := headers["authority-id"]; ok {
+		return fmt.Errorf("%q assertion cannot have authority-id set", assertType.Name)
+	}
+	return nil
+}
+
+func checkJSON(assertType *AssertionType, body []byte) (err error) {
+	defer func() {
+		if err != nil {
+			err = fmt.Errorf("assertion %s: %v", assertType.Name, err)
+		}
+	}()
+
+	if body == nil {
+		return fmt.Errorf(`body must contain JSON`)
+	}
+
+	var val any
+	if err := json.Unmarshal(body, &val); err != nil {
+		return fmt.Errorf("invalid JSON in body: %v", err)
+	}
+
+	formatted, err := json.MarshalIndent(val, "", "  ")
+	if err != nil {
+		return fmt.Errorf("invalid JSON in body: %v", err)
+	}
+
+	if !reflect.DeepEqual(body, formatted) {
+		// TODO: replace this with a manual comparison so we can give more context
+		// in the error message
+		return fmt.Errorf(`JSON in body must be indented with 2 spaces and sort object entries by key`)
+	}
+
+	return nil
+}
+
 // assemble is the internal variant of Assemble, assumes headers are already checked for supported types
-func assemble(headers map[string]interface{}, body, content, signature []byte) (Assertion, error) {
+func assemble(headers map[string]any, body, content, signature []byte) (Assertion, error) {
 	length, err := checkIntWithDefault(headers, "body-length", 0)
 	if err != nil {
 		return nil, fmt.Errorf("assertion: %v", err)
@@ -865,7 +1046,7 @@ func assemble(headers map[string]interface{}, body, content, signature []byte) (
 	}
 
 	if !utf8.Valid(body) {
-		return nil, fmt.Errorf("body is not utf8")
+		return nil, fmt.Errorf("assertion body is not utf8")
 	}
 
 	if _, err := checkDigest(headers, "sign-key-sha3-384", crypto.SHA3_384); err != nil {
@@ -881,14 +1062,19 @@ func assemble(headers map[string]interface{}, body, content, signature []byte) (
 		return nil, fmt.Errorf("unknown assertion type: %q", typ)
 	}
 
+	if assertType.flags&jsonBody != 0 {
+		if err := checkJSON(assertType, body); err != nil {
+			return nil, err
+		}
+	}
+
 	if assertType.flags&noAuthority == 0 {
-		if _, err := checkNotEmptyString(headers, "authority-id"); err != nil {
+		if err := checkAuthority(assertType, headers); err != nil {
 			return nil, fmt.Errorf("assertion: %v", err)
 		}
 	} else {
-		_, ok := headers["authority-id"]
-		if ok {
-			return nil, fmt.Errorf("%q assertion cannot have authority-id set", assertType.Name)
+		if err := checkNoAuthority(assertType, headers); err != nil {
+			return nil, err
 		}
 	}
 
@@ -898,6 +1084,11 @@ func assemble(headers map[string]interface{}, body, content, signature []byte) (
 	}
 
 	for _, primKey := range assertType.PrimaryKey {
+		if _, ok := headers[primKey]; !ok {
+			if defl := assertType.OptionalPrimaryKeyDefaults[primKey]; defl != "" {
+				headers[primKey] = defl
+			}
+		}
 		if _, err := checkPrimaryKey(headers, primKey); err != nil {
 			return nil, fmt.Errorf("assertion %s: %v", assertType.Name, err)
 		}
@@ -926,17 +1117,18 @@ func assemble(headers map[string]interface{}, body, content, signature []byte) (
 	return assert, nil
 }
 
-func writeHeader(buf *bytes.Buffer, headers map[string]interface{}, name string) {
+func writeHeader(buf *bytes.Buffer, headers map[string]any, name string) {
 	appendEntry(buf, fmt.Sprintf("%s:", name), headers[name], 0)
 }
 
-func assembleAndSign(assertType *AssertionType, headers map[string]interface{}, body []byte, privKey PrivateKey) (Assertion, error) {
+func assembleAndSign(assertType *AssertionType, headers map[string]any, body []byte, privKey PrivateKey) (Assertion, error) {
 	err := checkAssertType(assertType)
 	if err != nil {
 		return nil, err
 	}
 
 	withAuthority := assertType.flags&noAuthority == 0
+	withJSONBody := assertType.flags&jsonBody != 0
 
 	err = checkHeaders(headers)
 	if err != nil {
@@ -949,6 +1141,12 @@ func assembleAndSign(assertType *AssertionType, headers map[string]interface{},
 		return nil, fmt.Errorf("assertion body is not utf8")
 	}
 
+	if withJSONBody {
+		if err := checkJSON(assertType, body); err != nil {
+			return nil, err
+		}
+	}
+
 	finalHeaders := copyHeaders(headers)
 	bodyLength := len(body)
 	finalBody := make([]byte, bodyLength)
@@ -958,13 +1156,12 @@ func assembleAndSign(assertType *AssertionType, headers map[string]interface{},
 	finalHeaders["sign-key-sha3-384"] = privKey.PublicKey().ID()
 
 	if withAuthority {
-		if _, err := checkNotEmptyString(finalHeaders, "authority-id"); err != nil {
+		if err = checkAuthority(assertType, finalHeaders); err != nil {
 			return nil, err
 		}
 	} else {
-		_, ok := finalHeaders["authority-id"]
-		if ok {
-			return nil, fmt.Errorf("%q assertion cannot have authority-id set", assertType.Name)
+		if err := checkNoAuthority(assertType, finalHeaders); err != nil {
+			return nil, err
 		}
 	}
 
@@ -1018,10 +1215,21 @@ func assembleAndSign(assertType *AssertionType, headers map[string]interface{},
 		"sign-key-sha3-384": true,
 	}
 	for _, primKey := range assertType.PrimaryKey {
-		if _, err := checkPrimaryKey(finalHeaders, primKey); err != nil {
+		defl := assertType.OptionalPrimaryKeyDefaults[primKey]
+		_, ok := finalHeaders[primKey]
+		if !ok && defl != "" {
+			// optional but expected to be set in headers
+			// in the result assertion
+			finalHeaders[primKey] = defl
+			continue
+		}
+		value, err := checkPrimaryKey(finalHeaders, primKey)
+		if err != nil {
 			return nil, err
 		}
-		writeHeader(buf, finalHeaders, primKey)
+		if value != defl {
+			writeHeader(buf, finalHeaders, primKey)
+		}
 		written[primKey] = true
 	}
 
@@ -1056,7 +1264,7 @@ func assembleAndSign(assertType *AssertionType, headers map[string]interface{},
 	}
 	content := buf.Bytes()
 
-	signature, err := signContent(content, privKey)
+	signature, err := signAndEncode(content, privKey)
 	if err != nil {
 		return nil, fmt.Errorf("cannot sign assertion: %v", err)
 	}
@@ -1078,7 +1286,7 @@ func assembleAndSign(assertType *AssertionType, headers map[string]interface{},
 }
 
 // SignWithoutAuthority assembles an assertion without a set authority with the provided information and signs it with the given private key.
-func SignWithoutAuthority(assertType *AssertionType, headers map[string]interface{}, body []byte, privKey PrivateKey) (Assertion, error) {
+func SignWithoutAuthority(assertType *AssertionType, headers map[string]any, body []byte, privKey PrivateKey) (Assertion, error) {
 	if assertType.flags&noAuthority == 0 {
 		return nil, fmt.Errorf("cannot sign assertions needing a definite authority with SignWithoutAuthority")
 	}
