@@ -54,11 +54,11 @@ func MockSeedReadSystemEssential(f func(seedDir, label string, essentialTypes []
 // disk encryption implementations. The state must be locked when these
 // functions are called.
 var (
-	// HasFDESetupHook purpose is to detect if the target kernel has a
-	// fde-setup-hook. If kernelInfo is nil the current kernel is checked
-	// assuming it is representative` of the target one.
-	HasFDESetupHook = func(kernelInfo *snap.Info) (bool, error) {
-		return false, nil
+	// HookKeyProtectorFactory returns a secboot.KeyProtectorFactory
+	// implementation, which will create a secboot.KeyProtector based on which
+	// sealing methods are detected as supported.
+	HookKeyProtectorFactory = func(kernelInfo *snap.Info) (secboot.KeyProtectorFactory, error) {
+		return nil, nil
 	}
 )
 
@@ -85,8 +85,9 @@ func MockSealKeyToModeenv(f func(key, saveKey secboot.BootstrappedContainer, pri
 }
 
 type sealKeyToModeenvFlags struct {
-	// HasFDESetupHook is true if the kernel has a fde-setup hook to use
-	HasFDESetupHook bool
+	// HookKeyProtectorFactory will be used to create a [secboot.KeyProtector].
+	// If nil, it is assumed that TPM sealing should be used.
+	HookKeyProtectorFactory secboot.KeyProtectorFactory
 	// FactoryReset indicates that the sealing is happening during factory
 	// reset.
 	FactoryReset bool
@@ -133,13 +134,13 @@ func sealKeyToModeenvImpl(
 	}
 
 	method := device.SealingMethodTPM
-	if flags.HasFDESetupHook {
+	if flags.HookKeyProtectorFactory != nil {
 		method = device.SealingMethodFDESetupHook
-	} else {
-		if flags.StateUnlocker != nil {
-			relock := flags.StateUnlocker()
-			defer relock()
-		}
+	}
+
+	if flags.StateUnlocker != nil && method == device.SealingMethodTPM {
+		relock := flags.StateUnlocker()
+		defer relock()
 	}
 
 	return sealKeyToModeenvForMethod(method, key, saveKey, primaryKey, volumesAuth, model, modeenv, flags)
@@ -169,6 +170,9 @@ type SealKeyForBootChainsParams struct {
 	InstallHostWritableDir string
 	// PrimaryKey is the chosen primary key if it was chosen. It can be nil if not.
 	PrimaryKey []byte
+	// KeyProtectorFactory will be used to create the key protector for sealing.
+	// Will be nil if we are using the TPM for sealing.
+	KeyProtectorFactory secboot.KeyProtectorFactory
 }
 
 func sealKeyForBootChainsImpl(
@@ -197,6 +201,7 @@ func sealKeyToModeenvForMethod(
 		UseTokens:              flags.UseTokens,
 		InstallHostWritableDir: InstallHostWritableDir(model),
 		PrimaryKey:             primaryKey,
+		KeyProtectorFactory:    flags.HookKeyProtectorFactory,
 	}
 
 	var tbl bootloader.TrustedAssetsBootloader
@@ -337,16 +342,10 @@ func WithBootChains(f func(bc BootChains) error, method device.SealingMethod) er
 // device such that they can be used as an input for resealing of encryption
 // keys.
 func bootChains(modeenv *Modeenv, method device.SealingMethod) (BootChains, error) {
-	requiresBootLoaders := true
-	switch method {
-	case device.SealingMethodFDESetupHook:
-		requiresBootLoaders = false
-	}
-
 	var bc BootChains
-
 	var tbl bootloader.TrustedAssetsBootloader
 
+	requiresBootLoaders := method != device.SealingMethodFDESetupHook
 	if requiresBootLoaders {
 		// build the recovery mode boot chain
 		rbl, err := bootloader.Find(InitramfsUbuntuSeedDir, &bootloader.Options{
