@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2020 Canonical Ltd
+ * Copyright (C) 2014-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -62,6 +62,10 @@ var _ bootloader.ExtractedRunKernelImageBootloader = (*MockExtractedRunKernelIma
 var _ bootloader.ExtractedRecoveryKernelImageBootloader = (*MockExtractedRecoveryKernelImageBootloader)(nil)
 var _ bootloader.RecoveryAwareBootloader = (*MockRecoveryAwareTrustedAssetsBootloader)(nil)
 var _ bootloader.TrustedAssetsBootloader = (*MockRecoveryAwareTrustedAssetsBootloader)(nil)
+var _ bootloader.NotScriptableBootloader = (*MockNotScriptableBootloader)(nil)
+var _ bootloader.NotScriptableBootloader = (*MockExtractedRecoveryKernelNotScriptableBootloader)(nil)
+var _ bootloader.ExtractedRecoveryKernelImageBootloader = (*MockExtractedRecoveryKernelNotScriptableBootloader)(nil)
+var _ bootloader.RebootBootloader = (*MockRebootBootloader)(nil)
 
 func Mock(name, bootdir string) *MockBootloader {
 	return &MockBootloader{
@@ -409,7 +413,7 @@ func (b *MockExtractedRunKernelImageMixin) DisableTryKernel() error {
 // MockTrustedAssetsMixin implements the bootloader.TrustedAssetsBootloader
 // interface.
 type MockTrustedAssetsMixin struct {
-	TrustedAssetsList  []string
+	TrustedAssetsMap   map[string]string
 	TrustedAssetsErr   error
 	TrustedAssetsCalls int
 
@@ -422,6 +426,13 @@ type MockTrustedAssetsMixin struct {
 	BootChainRunBl         []bootloader.Bootloader
 	BootChainKernelPath    []string
 
+	RevocationTriggeringAssetsCalls  int
+	RevocationTriggeringAssetsReturn []string
+	RevocationTriggeringAssetsError  error
+
+	KernelBootFileBuilder         func(kernelPath string) bootloader.BootFile
+	RecoveryKernelBootFileBuilder func(kernelPath string) bootloader.BootFile
+
 	UpdateErr                  error
 	UpdateCalls                int
 	Updated                    bool
@@ -431,6 +442,14 @@ type MockTrustedAssetsMixin struct {
 	CommandLineErr             error
 }
 
+type MockEfiLoadOptionMixin struct {
+	EfiLoadOptionErr  error
+	EfiLoadOptionDesc string
+	EfiLoadOptionPath string
+	EfiLoadOptionData []byte
+	SeenUpdatedAssets [][]string
+}
+
 // MockTrustedAssetsBootloader mocks a bootloader implementing the
 // bootloader.TrustedAssetsBootloader interface.
 type MockTrustedAssetsBootloader struct {
@@ -439,12 +458,34 @@ type MockTrustedAssetsBootloader struct {
 	MockTrustedAssetsMixin
 }
 
+type MockTrustedAssetsBootloaderWithEfi struct {
+	*MockBootloader
+
+	MockTrustedAssetsMixin
+	MockEfiLoadOptionMixin
+}
+
 func (b *MockBootloader) WithTrustedAssets() *MockTrustedAssetsBootloader {
 	return &MockTrustedAssetsBootloader{
 		MockBootloader: b,
 	}
 }
 
+func (b *MockBootloader) WithTrustedAssetsAndEfi() *MockTrustedAssetsBootloaderWithEfi {
+	return &MockTrustedAssetsBootloaderWithEfi{
+		MockBootloader: b,
+	}
+}
+
+func (b *MockEfiLoadOptionMixin) ParametersForEfiLoadOption(updatedAssets []string) (string, string, []byte, error) {
+	b.SeenUpdatedAssets = append(b.SeenUpdatedAssets, updatedAssets)
+	if b.EfiLoadOptionErr != nil {
+		return "", "", nil, b.EfiLoadOptionErr
+	} else {
+		return b.EfiLoadOptionDesc, b.EfiLoadOptionPath, b.EfiLoadOptionData, nil
+	}
+}
+
 func (b *MockTrustedAssetsMixin) ManagedAssets() []string {
 	return b.ManagedAssetsList
 }
@@ -487,20 +528,48 @@ func (b *MockTrustedAssetsMixin) CandidateCommandLine(pieces bootloader.CommandL
 	return glueCommandLine(pieces, b.CandidateStaticCommandLine)
 }
 
-func (b *MockTrustedAssetsMixin) TrustedAssets() ([]string, error) {
+func (b *MockTrustedAssetsMixin) DefaultCommandLine(candidate bool) (string, error) {
+	if b.CommandLineErr != nil {
+		return "", b.CommandLineErr
+	}
+	if candidate {
+		return b.CandidateStaticCommandLine, nil
+	}
+	return b.StaticCommandLine, nil
+}
+
+func (b *MockTrustedAssetsMixin) TrustedAssets() (map[string]string, error) {
 	b.TrustedAssetsCalls++
-	return b.TrustedAssetsList, b.TrustedAssetsErr
+	return b.TrustedAssetsMap, b.TrustedAssetsErr
 }
 
-func (b *MockTrustedAssetsMixin) RecoveryBootChain(kernelPath string) ([]bootloader.BootFile, error) {
+func (b *MockTrustedAssetsMixin) RecoveryBootChains(kernelPath string) ([][]bootloader.BootFile, error) {
 	b.RecoveryBootChainCalls = append(b.RecoveryBootChainCalls, kernelPath)
-	return b.RecoveryBootChainList, b.RecoveryBootChainErr
+
+	bootchain := b.RecoveryBootChainList
+	if b.RecoveryKernelBootFileBuilder != nil {
+		bootchain = append(bootchain, b.RecoveryKernelBootFileBuilder(kernelPath))
+	}
+
+	return [][]bootloader.BootFile{bootchain}, b.RecoveryBootChainErr
 }
 
-func (b *MockTrustedAssetsMixin) BootChain(runBl bootloader.Bootloader, kernelPath string) ([]bootloader.BootFile, error) {
+func (b *MockTrustedAssetsMixin) BootChains(runBl bootloader.Bootloader, kernelPath string) ([][]bootloader.BootFile, error) {
 	b.BootChainRunBl = append(b.BootChainRunBl, runBl)
 	b.BootChainKernelPath = append(b.BootChainKernelPath, kernelPath)
-	return b.BootChainList, b.BootChainErr
+
+	bootchain := b.BootChainList
+	if b.KernelBootFileBuilder != nil {
+		bootchain = append(bootchain, b.KernelBootFileBuilder(kernelPath))
+	}
+
+	return [][]bootloader.BootFile{bootchain}, b.BootChainErr
+}
+
+func (b *MockTrustedAssetsMixin) RevocationTriggeringAssets() ([]string, error) {
+	b.RevocationTriggeringAssetsCalls++
+
+	return b.RevocationTriggeringAssetsReturn, b.RevocationTriggeringAssetsError
 }
 
 // MockRecoveryAwareTrustedAssetsBootloader implements the
@@ -518,3 +587,78 @@ func (b *MockBootloader) WithRecoveryAwareTrustedAssets() *MockRecoveryAwareTrus
 		MockBootloader: b,
 	}
 }
+
+// MockNotScriptableBootloader implements the
+// bootloader.NotScriptableBootloader interface.
+type MockNotScriptableBootloader struct {
+	*MockBootloader
+}
+
+func (b *MockBootloader) WithNotScriptable() *MockNotScriptableBootloader {
+	return &MockNotScriptableBootloader{
+		MockBootloader: b,
+	}
+}
+
+func (b *MockNotScriptableBootloader) SetBootVarsFromInitramfs(values map[string]string) error {
+	for k, v := range values {
+		b.BootVars[k] = v
+	}
+	return nil
+}
+
+// MockExtractedRecoveryKernelNotScriptableBootloader implements the
+// bootloader.ExtractedRecoveryKernelImageBootloader interface and
+// includes MockNotScriptableBootloader
+type MockExtractedRecoveryKernelNotScriptableBootloader struct {
+	*MockNotScriptableBootloader
+
+	ExtractRecoveryKernelAssetsCalls []ExtractedRecoveryKernelCall
+}
+
+func (b *MockNotScriptableBootloader) WithExtractedRecoveryKernel() *MockExtractedRecoveryKernelNotScriptableBootloader {
+	return &MockExtractedRecoveryKernelNotScriptableBootloader{
+		MockNotScriptableBootloader: b,
+	}
+}
+
+// ExtractRecoveryKernelAssets extracts the kernel assets for the provided
+// kernel snap into the specified recovery system dir; part of
+// RecoveryAwareBootloader.
+func (b *MockExtractedRecoveryKernelNotScriptableBootloader) ExtractRecoveryKernelAssets(recoverySystemDir string, s snap.PlaceInfo, snapf snap.Container) error {
+	if recoverySystemDir == "" {
+		panic("MockBootloader.ExtractRecoveryKernelAssets called without recoverySystemDir")
+	}
+
+	b.ExtractRecoveryKernelAssetsCalls = append(
+		b.ExtractRecoveryKernelAssetsCalls,
+		ExtractedRecoveryKernelCall{
+			S:                 s,
+			RecoverySystemDir: recoverySystemDir},
+	)
+	return nil
+}
+
+// MockRebootBootloaderMixin implements the bootloader.RebootBootloader
+// interface.
+type MockRebootBootloaderMixin struct {
+	RebootArgs string
+}
+
+// MockRebootBootloader mocks a bootloader implementing the
+// bootloader.RebootBootloader interface.
+type MockRebootBootloader struct {
+	*MockBootloader
+
+	MockRebootBootloaderMixin
+}
+
+func (b *MockRebootBootloaderMixin) GetRebootArguments() (string, error) {
+	return b.RebootArgs, nil
+}
+
+func (b *MockBootloader) WithRebootBootloader() *MockRebootBootloader {
+	return &MockRebootBootloader{
+		MockBootloader: b,
+	}
+}
