@@ -47,12 +47,15 @@ import (
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/disks"
 	"github.com/snapcore/snapd/osutil/kcmdline"
-	fdeBackend "github.com/snapcore/snapd/overlord/fdestate/backend"
 	"github.com/snapcore/snapd/snapdtool"
 	"github.com/snapcore/snapd/systemd"
 
 	// to set sysconfig.ApplyFilesystemOnlyDefaultsImpl
 	_ "github.com/snapcore/snapd/overlord/configstate/configcore"
+
+	// to set [boot.SealKeyForBootChains]
+	_ "github.com/snapcore/snapd/overlord/fdestate/backend"
+
 	"github.com/snapcore/snapd/overlord/install"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/secboot"
@@ -84,8 +87,7 @@ func init() {
 type cmdInitramfsMounts struct{}
 
 func (c *cmdInitramfsMounts) Execute([]string) error {
-	boot.HasFDESetupHook = hasFDESetupHook
-	fdeBackend.RunFDESetupHook = runFDESetupHook
+	boot.HookKeyProtectorFactory = hookKeyProtectorFactory
 
 	logger.Noticef("snap-bootstrap version %v starting", snapdtool.Version)
 
@@ -297,9 +299,16 @@ func runFDESetupHook(req *fde.SetupRequest) ([]byte, error) {
 	}
 	return output, nil
 }
-func hasFDESetupHook(kernelInfo *snap.Info) (bool, error) {
-	_, ok := kernelInfo.Hooks["fde-setup"]
-	return ok, nil
+func hookKeyProtectorFactory(kernelInfo *snap.Info) (secboot.KeyProtectorFactory, error) {
+	if _, ok := kernelInfo.Hooks["fde-setup"]; ok {
+		return secboot.FDESetupHookKeyProtectorFactory(runFDESetupHook), nil
+	}
+
+	if secboot.FDEOpteeTAPresent() {
+		return secboot.OPTEEKeyProtectorFactory(), nil
+	}
+
+	return nil, secboot.ErrNoKeyProtector
 }
 
 func readSnapInfoFromSeed(seedSnap *seed.Snap) (*snap.Info, error) {
@@ -355,7 +364,13 @@ func doInstall(mst *initramfsMountsState, model *asserts.Model, sysSnaps map[sna
 	if err != nil {
 		return err
 	}
-	encryptionSupport, err := install.CheckEncryptionSupport(model, secboot.TPMProvisionFull, kernelSnap, gadgetInfo, runFDESetupHook)
+
+	encryptionSupport, err := install.CheckEncryptionSupport(install.EncryptionConstraints{
+		Model:   model,
+		Kernel:  kernelSnap,
+		Gadget:  gadgetInfo,
+		TPMMode: secboot.TPMProvisionFull,
+	}, runFDESetupHook)
 	if err != nil {
 		return err
 	}
@@ -472,7 +487,7 @@ func doInstall(mst *initramfsMountsState, model *asserts.Model, sysSnaps map[sna
 	}
 
 	if useEncryption {
-		if err := install.PrepareEncryptedSystemData(model, installedSystem.BootstrappedContainerForRole, nil, trustedInstallObserver); err != nil {
+		if err := install.PrepareEncryptedSystemData(model, installedSystem.BootstrappedContainerForRole, nil, nil, trustedInstallObserver); err != nil {
 			return err
 		}
 	}
@@ -547,6 +562,12 @@ func doInstall(mst *initramfsMountsState, model *asserts.Model, sysSnaps map[sna
 	}
 
 	if hasDriversTree {
+		// FIXME: we should not remove or stop units while
+		// booting. That causes inconsistent jobs when
+		// something is depending on the removed unit,
+		// like: "[unit] has 'start' job queued, but 'stop' is
+		// included in transaction"
+
 		// Unmount the kernel snap mount, we keep it only for UC20/22
 		stdout, stderr, err := osutil.RunSplitOutput("systemd-mount", "--umount", kernelMountDir)
 		if err != nil {
