@@ -261,8 +261,8 @@ var helpCategories = []helpCategory{
 	}, {
 		Label:           i18n.G("Development"),
 		Description:     i18n.G("developer-oriented features"),
-		Commands:        []string{"download", "pack", "run", "try"},
-		AllOnlyCommands: []string{"prepare-image"},
+		Commands:        []string{"download", "pack", "run", "try", "sign"},
+		AllOnlyCommands: []string{"prepare-image", "export-key"},
 	}, {
 		Label:       i18n.G("Quota Groups"),
 		Description: i18n.G("Manage quota groups for snaps"),
@@ -311,13 +311,14 @@ func printShortHelp() {
 	maxLen := utf8.RuneCountInString("... Other")
 	var otherCommands []string
 	var develCateg *helpCategory
-	for _, categ := range helpCategories {
+	for i := range helpCategories {
+		categ := &helpCategories[i]
 		if categ.Other {
 			otherCommands = append(otherCommands, categ.Commands...)
 			continue
 		}
 		if categ.Label == "Development" {
-			develCateg = &categ
+			develCateg = categ
 		}
 		if l := utf8.RuneCountInString(categ.Label); l > maxLen {
 			maxLen = l
