@@ -20,13 +20,16 @@
 package main
 
 import (
+	"errors"
 	"fmt"
 	"io"
+	"log/syslog"
 	"net/http"
 	"os"
 	"path/filepath"
 	"runtime"
 	"strings"
+	"time"
 	"unicode"
 	"unicode/utf8"
 
@@ -44,6 +47,7 @@ import (
 	"github.com/snapcore/snapd/snap/squashfs"
 	"github.com/snapcore/snapd/snapdenv"
 	"github.com/snapcore/snapd/snapdtool"
+	"github.com/snapcore/snapd/systemd"
 )
 
 func init() {
@@ -75,8 +79,8 @@ type argDesc struct {
 
 var optionsData options
 
-// ErrExtraArgs is returned  if extra arguments to a command are found
-var ErrExtraArgs = fmt.Errorf(i18n.G("too many arguments for command"))
+// ErrExtraArgs is returned if extra arguments to a command are found
+var ErrExtraArgs = errors.New(i18n.G("too many arguments for command"))
 
 // cmdInfo holds information needed to call parser.AddCommand(...).
 type cmdInfo struct {
@@ -365,6 +369,7 @@ func Parser(cli *client.Client) *flags.Parser {
 }
 
 var isStdinTTY = terminal.IsTerminal(0)
+var isStdoutTTY = terminal.IsTerminal(1)
 
 // ClientConfig is the configuration of the Client used by all commands.
 var ClientConfig = client.Config{
@@ -400,10 +405,7 @@ This command has been left available for documentation purposes only.
 }
 
 func init() {
-	err := logger.SimpleSetup()
-	if err != nil {
-		fmt.Fprintf(Stderr, i18n.G("WARNING: failed to activate logging: %v\n"), err)
-	}
+	logger.SimpleSetup(nil)
 }
 
 func resolveApp(snapApp string) (string, error) {
@@ -440,8 +442,18 @@ func exitCodeFromError(err error) int {
 }
 
 func main() {
+	if err := loggerWithJournalMaybe(); err != nil {
+		fmt.Fprintf(Stderr, "cannot initialize logger: %v\n", err)
+		os.Exit(1)
+	}
+
 	snapdtool.ExecInSnapdOrCoreSnap()
 
+	if err := snapdtool.MaybeSetupFIPS(); err != nil {
+		fmt.Fprintf(os.Stderr, "cannot check or enable FIPS mode: %v\n", err)
+		os.Exit(1)
+	}
+
 	// check for magic symlink to /usr/bin/snap:
 	// 1. symlink from command-not-found to /usr/bin/snap: run c-n-f
 	if os.Args[0] == filepath.Join(dirs.GlobalRootDir, "/usr/lib/command-not-found") {
@@ -529,9 +541,64 @@ func (e unknownCommandError) Error() string {
 	return e.msg
 }
 
+func loggerWithJournalMaybe() error {
+	maybeJournal := func() error {
+		if !osutil.GetenvBool("SNAP_LOG_TO_JOURNAL") {
+			return fmt.Errorf("no need to log to journal")
+		}
+		journalWriter, err := systemd.NewJournalStreamFile(systemd.JournalStreamFileParams{
+			Identifier: "snap",
+			Priority:   syslog.LOG_DEBUG,
+		})
+		if err != nil {
+			return err
+		}
+		l := logger.New(journalWriter, logger.DefaultFlags, nil)
+		logger.SetLogger(l)
+		return nil
+	}
+
+	if err := maybeJournal(); err != nil {
+		// try simple setup
+		logger.SimpleSetup(nil)
+	}
+	// unset so that it doesn't trickle down to the snap application
+	os.Unsetenv("SNAP_LOG_TO_JOURNAL")
+	return nil
+}
+
+func composeSubCmd(cmd *flags.Command, subArgIndex int, cmdNames []string) string {
+	subcmds := cmd.Commands()
+	if len(subcmds) > 0 && len(os.Args) > subArgIndex {
+		for _, subcmd := range subcmds {
+			if subcmd.Name == os.Args[subArgIndex] {
+				return composeSubCmd(subcmd, subArgIndex+1, append(cmdNames, subcmd.Name))
+			}
+		}
+	}
+	return strings.Join(cmdNames, " ")
+}
+
+func makeCommandHandler(allCommands []*flags.Command) func(flags.Commander, []string) error {
+	return func(command flags.Commander, args []string) error {
+		for _, cmd := range allCommands {
+			if cmd.Name == os.Args[1] {
+				logger.Trace("command-execution", "cmd", composeSubCmd(cmd, 2, []string{cmd.Name}))
+				break
+			}
+		}
+		return command.Execute(args)
+	}
+}
+
+var timeAfter func(d time.Duration) <-chan time.Time = time.After
+
 func run() error {
 	cli := mkClient()
 	parser := Parser(cli)
+	if osutil.GetenvBool("SNAPD_TRACE") {
+		parser.CommandHandler = makeCommandHandler(parser.Command.Commands())
+	}
 	xtra, err := parser.Parse()
 	if err != nil {
 		if e, ok := err.(*flags.Error); ok {
@@ -588,7 +655,7 @@ fixed-width fonts, so it can be hard to tell.
 	return nil
 }
 
-func panicOnDebug(msg string, v ...interface{}) {
+func panicOnDebug(msg string, v ...any) {
 	if osutil.GetenvBool("SNAPD_DEBUG") || snapdenv.Testing() {
 		logger.Panicf(msg, v...)
 	}
