@@ -20,13 +20,19 @@
 package daemon_test
 
 import (
+	"bytes"
 	"context"
 	"crypto"
+	"encoding/json"
 	"fmt"
+	"io"
 	"net/http"
 	"os"
-	"os/user"
 	"path/filepath"
+	"strings"
+	"sync"
+	"sync/atomic"
+	"testing"
 	"time"
 
 	"github.com/gorilla/mux"
@@ -39,7 +45,9 @@ import (
 	"github.com/snapcore/snapd/asserts/sysdb"
 	"github.com/snapcore/snapd/daemon"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/interfaces/ifacetest"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/osutil/user"
 	"github.com/snapcore/snapd/overlord"
 	"github.com/snapcore/snapd/overlord/assertstate"
 	"github.com/snapcore/snapd/overlord/assertstate/assertstatetest"
@@ -51,11 +59,13 @@ import (
 	"github.com/snapcore/snapd/overlord/snapstate/sequence"
 	"github.com/snapcore/snapd/overlord/snapstate/snapstatetest"
 	"github.com/snapcore/snapd/overlord/state"
+	"github.com/snapcore/snapd/overlord/swfeats"
 	"github.com/snapcore/snapd/sandbox"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
 	"github.com/snapcore/snapd/store"
 	"github.com/snapcore/snapd/store/storetest"
+	"github.com/snapcore/snapd/strutil"
 	"github.com/snapcore/snapd/systemd"
 	"github.com/snapcore/snapd/testutil"
 )
@@ -97,6 +107,98 @@ type apiBaseSuite struct {
 
 	expectedReadAccess  daemon.AccessChecker
 	expectedWriteAccess daemon.AccessChecker
+
+	unwrapNewChange            func()
+	missingChangeRegistrations sync.Map
+}
+
+var (
+	actionsMap   *concurrentActionsMap
+	callCount    int64
+	disableMutex sync.RWMutex
+	disableMap   map[string][]string
+)
+
+func skipActionCoverage() bool {
+	if callCount <= 1 {
+		// If only one test suite ran, then it makes no sense to check action coverage
+		return true
+	}
+	for _, arg := range os.Args {
+		if strings.HasPrefix(arg, "-check.f") {
+			// If running a subset of tests, it doesn't make sense to check action coverage
+			return true
+		}
+	}
+	return false
+}
+
+func TestMain(m *testing.M) {
+	actionsMap = &concurrentActionsMap{data: map[*daemon.Command][]string{}}
+	disableMap = map[string][]string{}
+	code := m.Run()
+	if skipActionCoverage() {
+		os.Exit(code)
+	}
+	for _, cmd := range actionsMap.Keys() {
+		if cmd.Path == "/v2/debug" {
+			// API coverage for /v2/debug doesn't matter
+			continue
+		}
+		actions := actionsMap.Actions(cmd)
+		var path string
+		if cmd.Path != "" {
+			path = cmd.Path
+		} else {
+			path = cmd.PathPrefix
+		}
+		for _, action := range cmd.Actions {
+			if l, exists := disableMap[path]; exists && strutil.ListContains(l, action) {
+				continue
+			}
+			if !strutil.ListContains(actions, action) {
+				fmt.Printf("No test for action %s of command %s - if that action no longer exists, remove it from the Actions field slice in the relevant command. If it should not appear in unit tests, disable it calling apiBaseSuite.DisableActionsCheck(\"%s\", \"%s\")\n", action, path, path, action)
+				code = 1
+			}
+		}
+	}
+	os.Exit(code)
+}
+
+type concurrentActionsMap struct {
+	mu   sync.RWMutex
+	data map[*daemon.Command][]string
+}
+
+func (m *concurrentActionsMap) AddAction(cmd *daemon.Command, action string) {
+	m.mu.Lock()
+	defer m.mu.Unlock()
+
+	if _, exists := m.data[cmd]; !exists {
+		m.data[cmd] = []string{}
+	}
+	m.data[cmd] = append(m.data[cmd], action)
+}
+
+func (m *concurrentActionsMap) Keys() []*daemon.Command {
+	m.mu.RLock()
+	defer m.mu.RUnlock()
+	keys := make([]*daemon.Command, 0, len(m.data))
+	for cmd := range m.data {
+		keys = append(keys, cmd)
+	}
+	return keys
+}
+
+func (m *concurrentActionsMap) Actions(cmd *daemon.Command) []string {
+	m.mu.RLock()
+	defer m.mu.RUnlock()
+
+	actionsMap, exists := m.data[cmd]
+	if !exists {
+		return nil
+	}
+	return actionsMap
 }
 
 func (s *apiBaseSuite) pokeStateLock() {
@@ -169,18 +271,46 @@ func (s *apiBaseSuite) muxVars(*http.Request) map[string]string {
 	return s.vars
 }
 
+// DisableActionsCheck disables the final check for command action coverage
+func (s *apiBaseSuite) DisableActionsCheck(path, action string) {
+	disableMutex.Lock()
+	defer disableMutex.Unlock()
+	if _, exists := disableMap[path]; !exists {
+		disableMap[path] = []string{}
+	}
+	disableMap[path] = append(disableMap[path], action)
+}
+
+func mapToSlice(m *sync.Map) []string {
+	var slice []string
+	m.Range(func(key, _ any) bool {
+		slice = append(slice, key.(string))
+		return true
+	})
+	return slice
+}
+
 func (s *apiBaseSuite) SetUpSuite(c *check.C) {
+	atomic.AddInt64(&callCount, 1)
 	s.restoreMuxVars = daemon.MockMuxVars(s.muxVars)
 	s.restoreRelease = sandbox.MockForceDevMode(false)
 	s.systemctlRestorer = systemd.MockSystemctl(s.systemctl)
 	s.restoreSanitize = snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {})
+	s.unwrapNewChange = daemon.BeforeNewChange(func(_ *state.State, kind, _ string, _ []*state.TaskSet, _ []string) {
+		if !strutil.ListContains(swfeats.KnownChangeKinds(), kind) {
+			s.missingChangeRegistrations.Store(kind, nil)
+		}
+	})
 }
 
 func (s *apiBaseSuite) TearDownSuite(c *check.C) {
+	missingReg := mapToSlice(&s.missingChangeRegistrations)
+	c.Assert(missingReg, check.HasLen, 0, check.Commentf("Found missing change kind registrations %v Register new change kinds using swfeats.RegChangeKind", missingReg))
 	s.restoreMuxVars()
 	s.restoreRelease()
 	s.systemctlRestorer()
 	s.restoreSanitize()
+	s.unwrapNewChange()
 }
 
 func (s *apiBaseSuite) systemctl(args ...string) (buf []byte, err error) {
@@ -211,6 +341,7 @@ func (s *apiBaseSuite) SetUpTest(c *check.C) {
 	c.Assert(err, check.IsNil)
 	c.Assert(os.MkdirAll(dirs.SnapMountDir, 0755), check.IsNil)
 	c.Assert(os.MkdirAll(dirs.SnapBlobDir, 0755), check.IsNil)
+	c.Assert(os.MkdirAll(filepath.Dir(dirs.SnapSystemKeyFile), 0755), check.IsNil)
 
 	s.rsnaps = nil
 	s.suggestedCurrency = ""
@@ -256,12 +387,60 @@ func (s *apiBaseSuite) SetUpTest(c *check.C) {
 			HomeDir:  "",
 		}, nil
 	}))
+
+	s.AddCleanup(daemon.MockSnapstateStoreInstallGoal(newStoreInstallGoalRecorder))
+	s.AddCleanup(daemon.MockSnapstatePathUpdateGoal(newPathUpdateGoalRecorder))
+	s.AddCleanup(daemon.MockSnapstateStoreUpdateGoal(newStoreUpdateGoalRecorder))
+}
+
+type storeInstallGoalRecorder struct {
+	snapstate.InstallGoal
+	snaps []snapstate.StoreSnap
+}
+
+func newStoreInstallGoalRecorder(snaps ...snapstate.StoreSnap) snapstate.InstallGoal {
+	return &storeInstallGoalRecorder{
+		snaps:       snaps,
+		InstallGoal: snapstate.StoreInstallGoal(snaps...),
+	}
+}
+
+type pathUpdateGoalRecorder struct {
+	snapstate.UpdateGoal
+	snaps []snapstate.PathSnap
+}
+
+func newPathUpdateGoalRecorder(snaps ...snapstate.PathSnap) snapstate.UpdateGoal {
+	return &pathUpdateGoalRecorder{
+		snaps:      snaps,
+		UpdateGoal: snapstate.PathUpdateGoal(snaps...),
+	}
+}
+
+type storeUpdateGoalRecorder struct {
+	snapstate.UpdateGoal
+	snaps []snapstate.StoreUpdate
+}
+
+func (s *storeUpdateGoalRecorder) names() []string {
+	names := make([]string, 0, len(s.snaps))
+	for _, snap := range s.snaps {
+		names = append(names, snap.InstanceName)
+	}
+	return names
+}
+
+func newStoreUpdateGoalRecorder(snaps ...snapstate.StoreUpdate) snapstate.UpdateGoal {
+	return &storeUpdateGoalRecorder{
+		snaps:      snaps,
+		UpdateGoal: snapstate.StoreUpdateGoal(snaps...),
+	}
 }
 
 func (s *apiBaseSuite) mockModel(st *state.State, model *asserts.Model) {
 	// realistic model setup
 	if model == nil {
-		model = s.Brands.Model("can0nical", "pc", map[string]interface{}{
+		model = s.Brands.Model("can0nical", "pc", map[string]any{
 			"architecture": "amd64",
 			"gadget":       "gadget",
 			"kernel":       "kernel",
@@ -287,8 +466,10 @@ func (s *apiBaseSuite) daemonWithStore(c *check.C, sto snapstate.StoreService) *
 	c.Assert(err, check.IsNil)
 
 	st := d.Overlord().State()
-	// mark as already seeded
 	st.Lock()
+	// set a fake fde state
+	st.Set("fde", &struct{}{})
+	// mark as already seeded
 	st.Set("seeded", true)
 	// and registered
 	s.mockModel(st, nil)
@@ -387,6 +568,9 @@ func newFakeSnapManager(st *state.State, runner *state.TaskRunner) *fakeSnapMana
 	runner.AddHandler("fake-install-snap-error", func(t *state.Task, _ *tomb.Tomb) error {
 		return fmt.Errorf("fake-install-snap-error errored")
 	}, nil)
+	runner.AddHandler("fake-refresh-snap", func(t *state.Task, _ *tomb.Tomb) error {
+		return nil
+	}, nil)
 
 	return &fakeSnapManager{}
 }
@@ -428,7 +612,8 @@ func (s *apiBaseSuite) mockSnap(c *check.C, yamlText string) *snap.Info {
 		panic("call s.daemon(c) etc in your test first")
 	}
 
-	snapInfo := snaptest.MockSnap(c, yamlText, &snap.SideInfo{Revision: snap.R(1)})
+	appSet := ifacetest.MockSnapAndAppSet(c, yamlText, nil, &snap.SideInfo{Revision: snap.R(1)})
+	snapInfo := appSet.Info()
 
 	st := s.d.Overlord().State()
 
@@ -451,7 +636,7 @@ func (s *apiBaseSuite) mockSnap(c *check.C, yamlText string) *snap.Info {
 
 	// Put the snap into the interface repository
 	repo := s.d.Overlord().InterfaceManager().Repository()
-	err := repo.AddSnap(snapInfo)
+	err := repo.AddAppSet(appSet)
 	c.Assert(err, check.IsNil)
 	return snapInfo
 }
@@ -516,7 +701,7 @@ version: %s
 		return snapInfo
 	}
 
-	devAcct := assertstest.NewAccount(s.StoreSigning, developer, map[string]interface{}{
+	devAcct := assertstest.NewAccount(s.StoreSigning, developer, map[string]any{
 		"account-id": developer + "-id",
 	}, "")
 
@@ -527,7 +712,7 @@ version: %s
 		Validation:  devAcct.Validation(),
 	}
 
-	snapDecl, err := s.StoreSigning.Sign(asserts.SnapDeclarationType, map[string]interface{}{
+	snapDecl, err := s.StoreSigning.Sign(asserts.SnapDeclarationType, map[string]any{
 		"series":       "16",
 		"snap-id":      snapID,
 		"snap-name":    snapName,
@@ -541,7 +726,7 @@ version: %s
 	h := sha3.Sum384(content)
 	dgst, err := asserts.EncodeDigest(crypto.SHA3_384, h[:])
 	c.Assert(err, check.IsNil)
-	snapRev, err := s.StoreSigning.Sign(asserts.SnapRevisionType, map[string]interface{}{
+	snapRev, err := s.StoreSigning.Sign(asserts.SnapRevisionType, map[string]any{
 		"snap-sha3-384": string(dgst),
 		"snap-size":     "999",
 		"snap-id":       snapID,
@@ -601,7 +786,14 @@ func (s *apiBaseSuite) expectWriteAccess(a daemon.AccessChecker) {
 	s.expectedWriteAccess = a
 }
 
-func (s *apiBaseSuite) req(c *check.C, req *http.Request, u *auth.UserState) daemon.Response {
+type actionExpectedBool bool
+
+const (
+	actionIsUnexpected actionExpectedBool = false
+	actionIsExpected   actionExpectedBool = true
+)
+
+func (s *apiBaseSuite) req(c *check.C, req *http.Request, u *auth.UserState, actionExpected actionExpectedBool) daemon.Response {
 	if s.d == nil {
 		panic("call s.daemon(c) etc in your test first")
 	}
@@ -622,6 +814,22 @@ func (s *apiBaseSuite) req(c *check.C, req *http.Request, u *auth.UserState) dae
 		acc = cmd.WriteAccess
 		expAcc = s.expectedWriteAccess
 		whichAcc = "WriteAccess"
+		if actionExpected && req.Body != nil && (req.Header.Get("Content-Type") == "application/json" || req.Header.Get("Content-Type") == "") {
+			bodyBytes, err := io.ReadAll(req.Body)
+			c.Assert(err, check.IsNil)
+			var data struct {
+				Action string `json:"action"`
+			}
+			if err := json.Unmarshal(bodyBytes, &data); err == nil {
+				if data.Action != "" {
+					actionsMap.AddAction(cmd, data.Action)
+					if !strutil.ListContains(cmd.Actions, data.Action) {
+						c.Errorf("The action, %s, is not registered in the list of Actions of the corresponding command %s", data.Action, cmd.Path)
+					}
+				}
+			}
+			req.Body = io.NopCloser(bytes.NewBuffer(bodyBytes))
+		}
 	case "PUT":
 		f = cmd.PUT
 		acc = cmd.WriteAccess
@@ -637,26 +845,26 @@ func (s *apiBaseSuite) req(c *check.C, req *http.Request, u *auth.UserState) dae
 	return f(cmd, req, u)
 }
 
-func (s *apiBaseSuite) jsonReq(c *check.C, req *http.Request, u *auth.UserState) *daemon.RespJSON {
-	rsp, ok := s.req(c, req, u).(daemon.StructuredResponse)
+func (s *apiBaseSuite) jsonReq(c *check.C, req *http.Request, u *auth.UserState, actionExpected actionExpectedBool) *daemon.RespJSON {
+	rsp, ok := s.req(c, req, u, actionExpected).(daemon.StructuredResponse)
 	c.Assert(ok, check.Equals, true, check.Commentf("expected structured response"))
 	return rsp.JSON()
 }
 
-func (s *apiBaseSuite) syncReq(c *check.C, req *http.Request, u *auth.UserState) *daemon.RespJSON {
-	rsp := s.jsonReq(c, req, u)
+func (s *apiBaseSuite) syncReq(c *check.C, req *http.Request, u *auth.UserState, actionExpected actionExpectedBool) *daemon.RespJSON {
+	rsp := s.jsonReq(c, req, u, actionExpected)
 	c.Assert(rsp.Type, check.Equals, daemon.ResponseTypeSync, check.Commentf("expected sync resp: %#v, result: %+v", rsp, rsp.Result))
 	return rsp
 }
 
-func (s *apiBaseSuite) asyncReq(c *check.C, req *http.Request, u *auth.UserState) *daemon.RespJSON {
-	rsp := s.jsonReq(c, req, u)
+func (s *apiBaseSuite) asyncReq(c *check.C, req *http.Request, u *auth.UserState, actionExpected actionExpectedBool) *daemon.RespJSON {
+	rsp := s.jsonReq(c, req, u, actionExpected)
 	c.Assert(rsp.Type, check.Equals, daemon.ResponseTypeAsync, check.Commentf("expected async resp: %#v, result %v", rsp, rsp.Result))
 	return rsp
 }
 
-func (s *apiBaseSuite) errorReq(c *check.C, req *http.Request, u *auth.UserState) *daemon.APIError {
-	rsp := s.req(c, req, u)
+func (s *apiBaseSuite) errorReq(c *check.C, req *http.Request, u *auth.UserState, actionExpected actionExpectedBool) *daemon.APIError {
+	rsp := s.req(c, req, u, actionExpected)
 	rspe, ok := rsp.(*daemon.APIError)
 	c.Assert(ok, check.Equals, true, check.Commentf("expected apiError resp: %#v", rsp))
 	return rspe
@@ -669,6 +877,22 @@ func (s *apiBaseSuite) serveHTTP(c *check.C, w http.ResponseWriter, req *http.Re
 
 	cmd, vars := handlerCommand(c, s.d, req)
 	s.vars = vars
+	if req.Method == "POST" && req.Body != nil && (req.Header.Get("Content-Type") == "application/json" || req.Header.Get("Content-Type") == "") {
+		bodyBytes, err := io.ReadAll(req.Body)
+		c.Assert(err, check.IsNil)
+		var data struct {
+			Action string `json:"action"`
+		}
+		if err := json.Unmarshal(bodyBytes, &data); err == nil {
+			if data.Action != "" {
+				actionsMap.AddAction(cmd, data.Action)
+				if !strutil.ListContains(cmd.Actions, data.Action) {
+					c.Errorf("The action, %s, is not registered in the list of Actions of the corresponding command %s", data.Action, cmd.Path)
+				}
+			}
+		}
+		req.Body = io.NopCloser(bytes.NewBuffer(bodyBytes))
+	}
 
 	cmd.ServeHTTP(w, req)
 }
@@ -690,3 +914,12 @@ func (s *apiBaseSuite) simulateConflict(name string) {
 	chg := st.NewChange("manip", "...")
 	chg.AddTask(t)
 }
+
+func assertResponseBody(c *check.C, b io.Reader, expected map[string]any) {
+	var body map[string]any
+	dec := json.NewDecoder(b)
+	err := dec.Decode(&body)
+	c.Check(err, check.IsNil)
+	c.Check(body, check.DeepEquals, expected)
+	c.Check(dec.More(), check.Equals, false)
+}
