@@ -40,6 +40,7 @@ import (
 	"github.com/snapcore/snapd/overlord/install"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/secboot"
 	"github.com/snapcore/snapd/snap"
 )
 
@@ -66,7 +67,7 @@ var systemsActionCmd = &Command{
 	Actions: []string{
 		"do", "reboot", "install",
 		"create", "remove", "check-passphrase",
-		"check-pin",
+		"check-pin", "fix-encryption-support",
 	},
 	WriteAccess: rootAccess{},
 }
@@ -123,9 +124,22 @@ func getAllSystems(c *Command, r *http.Request, user *auth.UserState) Response {
 }
 
 // wrapped for unit tests
-var deviceManagerSystemAndGadgetAndEncryptionInfo = func(dm *devicestate.DeviceManager, systemLabel string) (*devicestate.System, *gadget.Info, *install.EncryptionSupportInfo, error) {
-	return dm.SystemAndGadgetAndEncryptionInfo(systemLabel)
-}
+var deviceManagerSystemAndGadgetAndEncryptionInfo func(
+	dm *devicestate.DeviceManager,
+	systemLabel string,
+	encInfoFromCache bool,
+) (
+	*devicestate.System, *gadget.Info, *install.EncryptionSupportInfo, error,
+) = (*devicestate.DeviceManager).SystemAndGadgetAndEncryptionInfo
+
+// wrapped for unit tests
+var deviceManagerApplyActionOnSystemAndGadgetAndEncryptionInfo func(
+	dm *devicestate.DeviceManager,
+	systemLabel string,
+	checkAction *secboot.PreinstallAction,
+) (
+	*devicestate.System, *gadget.Info, *install.EncryptionSupportInfo, error,
+) = (*devicestate.DeviceManager).ApplyActionOnSystemAndGadgetAndEncryptionInfo
 
 func storageEncryption(encInfo *install.EncryptionSupportInfo) *client.StorageEncryption {
 	if encInfo.Disabled {
@@ -173,12 +187,21 @@ func getSystemDetails(c *Command, r *http.Request, user *auth.UserState) Respons
 
 	deviceMgr := c.d.overlord.DeviceManager()
 
-	sys, gadgetInfo, encryptionInfo, err := deviceManagerSystemAndGadgetAndEncryptionInfo(deviceMgr, wantedSystemLabel)
+	// do not use cached encryption information; perform a fresh encryption
+	// availability check
+	const encInfoFromCache = false
+
+	sys, gadgetInfo, encryptionInfo, err := deviceManagerSystemAndGadgetAndEncryptionInfo(deviceMgr, wantedSystemLabel, encInfoFromCache)
 	if err != nil {
 		return InternalError(err.Error())
 	}
 
-	rsp := client.SystemDetails{
+	details := systemDetailsFrom(sys, gadgetInfo, encryptionInfo)
+	return SyncResponse(*details)
+}
+
+func systemDetailsFrom(sys *devicestate.System, gadgetInfo *gadget.Info, encryptionInfo *install.EncryptionSupportInfo) *client.SystemDetails {
+	details := &client.SystemDetails{
 		Current: sys.Current,
 		Label:   sys.Label,
 		Brand: snap.StoreAccount{
@@ -197,13 +220,12 @@ func getSystemDetails(c *Command, r *http.Request, user *auth.UserState) Respons
 		StorageEncryption: storageEncryption(encryptionInfo),
 	}
 	for _, sa := range sys.Actions {
-		rsp.Actions = append(rsp.Actions, client.SystemAction{
+		details.Actions = append(details.Actions, client.SystemAction{
 			Title: sa.Title,
 			Mode:  sa.Mode,
 		})
 	}
-
-	return SyncResponse(rsp)
+	return details
 }
 
 type systemActionRequest struct {
@@ -213,6 +235,7 @@ type systemActionRequest struct {
 	client.InstallSystemOptions
 	client.CreateSystemOptions
 	client.QualityCheckOptions
+	client.FixEncryptionSupportOptions
 }
 
 func postSystemsAction(c *Command, r *http.Request, user *auth.UserState) Response {
@@ -290,6 +313,8 @@ func postSystemsActionJSON(c *Command, r *http.Request) Response {
 		return postSystemActionCheckPassphrase(c, systemLabel, &req)
 	case "check-pin":
 		return postSystemActionCheckPIN(c, systemLabel, &req)
+	case "fix-encryption-support":
+		return postSystemActionFixEncryptionSupport(c, systemLabel, &req)
 	default:
 		return BadRequest("unsupported action %q", req.Action)
 	}
@@ -622,34 +647,6 @@ func postSystemActionRemove(c *Command, systemLabel string) Response {
 	return AsyncResponse(nil, chg.ID())
 }
 
-type encryptionSupportInfoKey struct{ systemLabel string }
-
-// cachedEncryptionSupportInfoByLabel returns encryption support info for specified system from cache.
-// If no cached value exist it is computed once and reused for future calls.
-//
-// Note that the cached value is never cleared as system seeds are assumed to be immutable.
-func cachedEncryptionSupportInfoByLabel(c *Command, systemLabel string) (*install.EncryptionSupportInfo, error) {
-	c.d.state.Lock()
-	cached := c.d.state.Cached(encryptionSupportInfoKey{systemLabel})
-	c.d.state.Unlock()
-	if cached != nil {
-		encryptionSupportInfo, ok := cached.(*install.EncryptionSupportInfo)
-		if ok {
-			return encryptionSupportInfo, nil
-		}
-	}
-	// no entry found in cache, let's compute encryption support info for target system
-	deviceMgr := c.d.overlord.DeviceManager()
-	_, _, encryptionInfo, err := deviceManagerSystemAndGadgetAndEncryptionInfo(deviceMgr, systemLabel)
-	if err != nil {
-		return nil, err
-	}
-	c.d.state.Lock()
-	c.d.state.Cache(encryptionSupportInfoKey{systemLabel}, encryptionInfo)
-	c.d.state.Unlock()
-	return encryptionInfo, nil
-}
-
 var deviceValidatePassphrase = device.ValidatePassphrase
 
 func postValidatePassphrase(mode device.AuthMode, passphrase string) Response {
@@ -693,7 +690,12 @@ func postSystemActionCheckPassphrase(c *Command, systemLabel string, req *system
 		return BadRequest("passphrase must be provided in request body for action %q", req.Action)
 	}
 
-	encryptionInfo, err := cachedEncryptionSupportInfoByLabel(c, systemLabel)
+	// use cached encryption information when available; skips the expensive
+	// availability check and still checks the passphrase
+	const encInfoFromCache = true
+
+	deviceMgr := c.d.overlord.DeviceManager()
+	_, _, encryptionInfo, err := deviceManagerSystemAndGadgetAndEncryptionInfo(deviceMgr, systemLabel, encInfoFromCache)
 	if err != nil {
 		return InternalError(err.Error())
 	}
@@ -716,7 +718,12 @@ func postSystemActionCheckPIN(c *Command, systemLabel string, req *systemActionR
 		return BadRequest("pin must be provided in request body for action %q", req.Action)
 	}
 
-	encryptionInfo, err := cachedEncryptionSupportInfoByLabel(c, systemLabel)
+	// use cached encryption information when available; skips the expensive
+	// availability check and still checks the PIN
+	const encInfoFromCache = true
+
+	deviceMgr := c.d.overlord.DeviceManager()
+	_, _, encryptionInfo, err := deviceManagerSystemAndGadgetAndEncryptionInfo(deviceMgr, systemLabel, encInfoFromCache)
 	if err != nil {
 		return InternalError(err.Error())
 	}
@@ -730,3 +737,40 @@ func postSystemActionCheckPIN(c *Command, systemLabel string, req *systemActionR
 
 	return postValidatePassphrase(device.AuthModePIN, req.PIN)
 }
+
+func postSystemActionFixEncryptionSupport(c *Command, systemLabel string, req *systemActionRequest) Response {
+	if systemLabel == "" {
+		return BadRequest("system action requires the system label to be provided")
+	}
+
+	// FixAction set to "" is valid and maps to secboot constant ActionNone.
+	// Omission of FixAction is not allowed.
+	if req.FixAction == nil {
+		return BadRequest("fix action must be provided in request body for action %q", req.Action)
+	}
+
+	// Args is optional, but when specified it must contain at least one
+	// argument entry.
+	if req.Args != nil && len(req.Args) == 0 {
+		return BadRequest("optional fix action args, when provided, must contain one or more arguments %q", req.Action)
+	}
+
+	checkAction := &secboot.PreinstallAction{
+		Action: *req.FixAction,
+		Args:   req.Args,
+	}
+
+	// TODO:FDEM: In the future, snapd should be able to identify actions it is responsible for handling,
+	// and avoid forwarding those actions to secboot. Similarly, actions intended for the installer
+	// should result in an error. A mechanism is needed to determine ownership of each action.
+
+	deviceMgr := c.d.overlord.DeviceManager()
+
+	sys, gadgetInfo, encryptionInfo, err := deviceManagerApplyActionOnSystemAndGadgetAndEncryptionInfo(deviceMgr, systemLabel, checkAction)
+	if err != nil {
+		return InternalError(err.Error())
+	}
+
+	details := systemDetailsFrom(sys, gadgetInfo, encryptionInfo)
+	return SyncResponse(*details)
+}
