@@ -19,7 +19,7 @@
 
 package notification
 
-import "github.com/godbus/dbus"
+import "github.com/godbus/dbus/v5"
 
 var (
 	NewFdoBackend = newFdoBackend
