@@ -27,7 +27,9 @@ import (
 	"fmt"
 	"math"
 	"os"
+	"path"
 	"path/filepath"
+	"reflect"
 	"regexp"
 	"sort"
 	"strings"
@@ -35,6 +37,8 @@ import (
 	"gopkg.in/yaml.v2"
 
 	"github.com/snapcore/snapd/asserts"
+	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/gadget/device"
 	"github.com/snapcore/snapd/gadget/edition"
 	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/logger"
@@ -42,7 +46,6 @@ import (
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/disks"
 	"github.com/snapcore/snapd/osutil/kcmdline"
-	"github.com/snapcore/snapd/secboot"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/naming"
 	"github.com/snapcore/snapd/snap/snapfile"
@@ -55,6 +58,8 @@ const (
 	schemaMBR = "mbr"
 	// schemaGPT identifies a GUID Partition Table partitioning schema
 	schemaGPT = "gpt"
+	// schemaEMMC identifies a schema for eMMC
+	schemaEMMC = "emmc"
 
 	SystemBoot     = "system-boot"
 	SystemData     = "system-data"
@@ -117,10 +122,15 @@ type KernelCmdline struct {
 }
 
 type Info struct {
+	// Volumes is the list of volumes defined by the gadget
 	Volumes map[string]*Volume `yaml:"volumes,omitempty"`
+	// VolumeAssingments carries a list of possible assignments of a volume to an
+	// actual device visible in the system. Can carry alternative assignments of the same
+	// volume to a device. Best match is selected at runtime.
+	VolumeAssignments []*VolumeAssignment `yaml:"volume-assignments,omitempty"`
 
 	// Default configuration for snaps (snap-id => key => value).
-	Defaults map[string]map[string]interface{} `yaml:"defaults,omitempty"`
+	Defaults map[string]map[string]any `yaml:"defaults,omitempty"`
 
 	Connections []Connection `yaml:"connections"`
 
@@ -130,14 +140,7 @@ type Info struct {
 // HasRole returns true if any of the volume structures in this Info has the
 // given role.
 func (i *Info) HasRole(role string) bool {
-	for _, v := range i.Volumes {
-		for _, s := range v.Structure {
-			if s.Role == role {
-				return true
-			}
-		}
-	}
-	return false
+	return VolumesHaveRole(i.Volumes, role)
 }
 
 // PartialProperty is a gadget property that can be partially defined.
@@ -173,6 +176,23 @@ type Volume struct {
 	Structure []VolumeStructure `yaml:"structure" json:"structure"`
 	// Name is the name of the volume from the gadget.yaml
 	Name string `json:"-"`
+	// AssignedDevice is set during runtime to assign a specific gadget
+	// volume to a device, eg. /dev/disk/by-path/pci-42:0. This is only
+	// set optionally if matched against the volume-assignments.
+	AssignedDevice string `json:"-"`
+}
+
+// VolumesHaveRole checks if any of the volumes has a structure with the given
+// role.
+func VolumesHaveRole(volumes map[string]*Volume, role string) bool {
+	for _, v := range volumes {
+		for _, s := range v.Structure {
+			if s.Role == role {
+				return true
+			}
+		}
+	}
+	return false
 }
 
 // HasPartial checks if the volume has a partially defined part.
@@ -185,21 +205,35 @@ func (v *Volume) HasPartial(pp PartialProperty) bool {
 	return false
 }
 
-// MinSize returns the minimum size required by a volume, as implicitly
-// defined by the size structures. It assumes sorted structures.
-func (v *Volume) MinSize() quantity.Size {
+// size returns the size of the volume using the structureSizer function to calculate
+// structures size. It assumes sorted structures.
+func (v *Volume) size(structureSizer func(VolumeStructure) quantity.Size) quantity.Size {
 	endVol := quantity.Offset(0)
 	for _, s := range v.Structure {
 		if s.Offset != nil {
-			endVol = *s.Offset + quantity.Offset(s.MinSize)
+			endVol = *s.Offset + quantity.Offset(structureSizer(s))
 		} else {
-			endVol += quantity.Offset(s.MinSize)
+			endVol += quantity.Offset(structureSizer(s))
 		}
 	}
 
 	return quantity.Size(endVol)
 }
 
+// MinSize returns the minimum size required by a volume.
+func (v *Volume) MinSize() quantity.Size {
+	return v.size(func(s VolumeStructure) quantity.Size {
+		return s.MinSize
+	})
+}
+
+// Size returns the current size required by a volume.
+func (v *Volume) Size() quantity.Size {
+	return v.size(func(s VolumeStructure) quantity.Size {
+		return s.Size
+	})
+}
+
 // StructFromYamlIndex returns the structure defined at a given yaml index from
 // the original yaml file.
 func (v *Volume) StructFromYamlIndex(yamlIdx int) *VolumeStructure {
@@ -531,6 +565,117 @@ type VolumeUpdate struct {
 	Preserve []string       `yaml:"preserve" json:"preserve"`
 }
 
+// VolumeAssignment is an optional set of volume-to-disk assignments
+// that can be specified. This is a nice way of reusing the same gadget
+// for multiple devices, or a way to map multiple volumes to the same disk
+// for cases like eMMC. Each assignment in this structure refers to a volume
+// and a device path (/dev/disk/** for now).
+type VolumeAssignment struct {
+	// Name is the assignment variant name
+	Name string `yaml:"assignment-name"`
+	// Assignments maps a volume name to an actual device assignment
+	Assignments map[string]*DeviceAssignment `yaml:"assignment"`
+}
+
+func (va *VolumeAssignment) validate(volumes map[string]*Volume) error {
+	if len(va.Assignments) == 0 {
+		return fmt.Errorf("no assignments specified")
+	}
+
+	for name, asv := range va.Assignments {
+		if v := volumes[name]; v == nil {
+			return fmt.Errorf("volume %q is mentioned in assignment but has not been defined", name)
+		}
+		if err := asv.validate(); err != nil {
+			return fmt.Errorf("%q: %v", name, err)
+		}
+	}
+	return nil
+}
+
+// DeviceAssignment is the device data for each volume assignment. Currently
+// just keeps the device the volume should be assigned to.
+type DeviceAssignment struct {
+	// Device is the actual block device available in the system,
+	// eg. /dev/disk/by-path/pci-42:0
+	Device string `yaml:"device"`
+}
+
+func (da *DeviceAssignment) validate() error {
+	// Device disk paths must be either under by-path or by-id
+	// as those are the only sensible ones that can be predefined
+	if !(strings.HasPrefix(da.Device, "/dev/disk/by-path") || strings.HasPrefix(da.Device, "/dev/disk/by-id")) {
+		return fmt.Errorf("unsupported device path %q, for now only paths under /dev/disk/{by-path,by-id} are valid", da.Device)
+	}
+	return nil
+}
+
+func areAssignmentsMatchingCurrentDevice(name string, assignments map[string]*DeviceAssignment) bool {
+	for _, va := range assignments {
+		if _, err := os.Stat(path.Join(dirs.GlobalRootDir, va.Device)); err != nil {
+			// XXX: expect this to mean no path, consider actually
+			// ensuring the error is not-exists
+			logger.Debugf("cannot stat device %s for volume assignment %q: %v",
+				va.Device, name, err)
+			return false
+		}
+	}
+	return true
+}
+
+// Indirection here for mocking
+var VolumesForCurrentDeviceAssignment = volumesForCurrentDeviceAssignmentImpl
+
+// volumesForCurrentDeviceAssignmentImpl finds a matching assignment based on
+// the available /dev/disk environment. If none, or multiple matching are found,
+// an error is returned.
+func volumesForCurrentDeviceAssignmentImpl(gi *Info) (map[string]*Volume, error) {
+	var matched []*VolumeAssignment
+	for _, vas := range gi.VolumeAssignments {
+		if !areAssignmentsMatchingCurrentDevice(vas.Name, vas.Assignments) {
+			continue
+		}
+		matched = append(matched, vas)
+	}
+
+	// If we find nothing, throw an error about no assignments being matched
+	// for the current device. If we find more than one, error out on that too.
+	switch {
+	case len(matched) == 0:
+		return nil, fmt.Errorf("no matching volume-assignment for current device")
+	case len(matched) == 1:
+		// update volume assignments, we can assume only one match
+		// in the list.
+		vas := matched[0]
+		logger.Noticef("found valid device-assignment: %s", vas.Name)
+		volumes := make(map[string]*Volume)
+		for vol := range vas.Assignments {
+			gi.Volumes[vol].AssignedDevice = vas.Assignments[vol].Device
+			volumes[vol] = gi.Volumes[vol]
+		}
+		return volumes, nil
+	default:
+		return nil, fmt.Errorf("multiple matching volume-assignment for current device")
+	}
+}
+
+// VolumesForCurrentDevice returns the correct list of volumes for the currently
+// running device. If there is an active assignment, then those volumes are returned
+// together with `assigned` being true.
+func VolumesForCurrentDevice(gi *Info) (vols map[string]*Volume, assigned bool, err error) {
+	if len(gi.VolumeAssignments) != 0 {
+		vols, err = VolumesForCurrentDeviceAssignment(gi)
+		if err != nil {
+			return nil, false, err
+		}
+		assigned = true
+	} else {
+		vols = gi.Volumes
+		assigned = false
+	}
+	return vols, assigned, nil
+}
+
 // DiskVolumeDeviceTraits is a set of traits about a disk that were measured at
 // a previous point in time on the same device, and is used primarily to try and
 // map a volume in the gadget.yaml to a physical device on the system after the
@@ -696,6 +841,68 @@ func LoadDiskVolumesDeviceTraits(dir string) (map[string]DiskVolumeDeviceTraits,
 	return mapping, nil
 }
 
+// MaybeDeviceForStructure does a best-effort of resolving a device
+// node for the provided volume structure.
+func MaybeDeviceForStructure(vs *VolumeStructure) (string, error) {
+	structureDevice, err := FindDeviceForStructure(vs)
+	if err != nil && err != ErrDeviceNotFound {
+		return "", err
+	}
+
+	if structureDevice != "" {
+		// we found a device for this structure, get the parent disk
+		// and save that as the device for this volume
+		disk, err := disks.DiskFromPartitionDeviceNode(structureDevice)
+		if err != nil {
+			return "", err
+		}
+
+		return disk.KernelDeviceNode(), nil
+	}
+	return "", nil
+}
+
+// MaybeDeviceForVolume does a best-effort of finding a matching device
+// node for the provided volume. Optionally a device path can be specified
+// that should be resolved instead of finding a matching device for one
+// of the volume structures.
+func MaybeDeviceForVolume(volume *Volume) (string, error) {
+	if volume.AssignedDevice != "" {
+		disk, err := disks.DiskFromDeviceName(volume.AssignedDevice)
+		if err != nil {
+			return "", err
+		}
+
+		return disk.KernelDeviceNode(), nil
+	}
+
+	for _, vs := range volume.Structure {
+		// TODO: This code works for volumes that have at least one
+		// partition (i.e. not type: bare structure), but does not work for
+		// volumes which contain only type: bare structures with no other
+		// structures on them. It is entirely unclear how to identify such
+		// a volume, since there is no information on the disk about where
+		// such raw structures begin and end and thus no way to validate
+		// that a given disk "has" such raw structures at particular
+		// locations, aside from potentially reading and comparing the bytes
+		// at the expected locations, but that is probably fragile and very
+		// non-performant.
+		if !vs.IsPartition() {
+			// skip trying to find non-partitions on disk, it won't work
+			continue
+		}
+
+		devNode, err := MaybeDeviceForStructure(&vs)
+		if err != nil {
+			return "", err
+		}
+		if devNode != "" {
+			return devNode, nil
+		}
+	}
+	return "", nil
+}
+
 // AllDiskVolumeDeviceTraits takes a mapping of volume name to Volume
 // and produces a map of volume name to DiskVolumeDeviceTraits. Since
 // doing so uses DiskVolumeDeviceTraitsForDevice, it will also
@@ -703,51 +910,18 @@ func LoadDiskVolumesDeviceTraits(dir string) (map[string]DiskVolumeDeviceTraits,
 // and matching before returning.
 func AllDiskVolumeDeviceTraits(allVols map[string]*Volume, optsPerVolume map[string]*DiskVolumeValidationOptions) (map[string]DiskVolumeDeviceTraits, error) {
 	// build up the mapping of volumes to disk device traits
-
-	allTraits := map[string]DiskVolumeDeviceTraits{}
-
 	// find all devices which map to volumes to save the current state of the
 	// system
+	allTraits := map[string]DiskVolumeDeviceTraits{}
 	for name, vol := range allVols {
 		// try to find a device for a structure inside the volume, we have a
 		// loop to attempt to use all structures in the volume in case there are
 		// partitions we can't map to a device directly at first using the
 		// device symlinks that FindDeviceForStructure uses
-		dev := ""
-		for _, vs := range vol.Structure {
-			// TODO: This code works for volumes that have at least one
-			// partition (i.e. not type: bare structure), but does not work for
-			// volumes which contain only type: bare structures with no other
-			// structures on them. It is entirely unclear how to identify such
-			// a volume, since there is no information on the disk about where
-			// such raw structures begin and end and thus no way to validate
-			// that a given disk "has" such raw structures at particular
-			// locations, aside from potentially reading and comparing the bytes
-			// at the expected locations, but that is probably fragile and very
-			// non-performant.
-
-			if !vs.IsPartition() {
-				// skip trying to find non-partitions on disk, it won't work
-				continue
-			}
-
-			structureDevice, err := FindDeviceForStructure(&vs)
-			if err != nil && err != ErrDeviceNotFound {
-				return nil, err
-			}
-			if structureDevice != "" {
-				// we found a device for this structure, get the parent disk
-				// and save that as the device for this volume
-				disk, err := disks.DiskFromPartitionDeviceNode(structureDevice)
-				if err != nil {
-					return nil, err
-				}
-
-				dev = disk.KernelDeviceNode()
-				break
-			}
+		dev, err := MaybeDeviceForVolume(vol)
+		if err != nil {
+			return nil, fmt.Errorf("cannot find disk for volume %s from gadget: %v", name, err)
 		}
-
 		if dev == "" {
 			return nil, fmt.Errorf("cannot find disk for volume %s from gadget", name)
 		}
@@ -793,7 +967,7 @@ func (gcplug *ConnectionPlug) Empty() bool {
 	return gcplug.SnapID == "" && gcplug.Plug == ""
 }
 
-func (gcplug *ConnectionPlug) UnmarshalYAML(unmarshal func(interface{}) error) error {
+func (gcplug *ConnectionPlug) UnmarshalYAML(unmarshal func(any) error) error {
 	var s string
 	if err := unmarshal(&s); err != nil {
 		return err
@@ -816,7 +990,7 @@ func (gcslot *ConnectionSlot) Empty() bool {
 	return gcslot.SnapID == "" && gcslot.Slot == ""
 }
 
-func (gcslot *ConnectionSlot) UnmarshalYAML(unmarshal func(interface{}) error) error {
+func (gcslot *ConnectionSlot) UnmarshalYAML(unmarshal func(any) error) error {
 	var s string
 	if err := unmarshal(&s); err != nil {
 		return err
@@ -952,6 +1126,45 @@ func validatePartial(v *Volume) error {
 	return nil
 }
 
+func validateVolumeAssignments(volassigns []*VolumeAssignment, volumes map[string]*Volume) error {
+	// do basic validation for volume-assignments
+	for _, va := range volassigns {
+		if err := va.validate(volumes); err != nil {
+			return fmt.Errorf("volume-assignment variant %q: %v", va.Name, err)
+		}
+	}
+
+	// ensure no two variants are identical in terms of contents, as we
+	// can only do matching based on the first correct we find.
+	checker := make(map[string][]*VolumeAssignment, len(volassigns))
+	for _, vavariant := range volassigns {
+		var keys []string
+
+		// include the device path to ensure that the assignments
+		// at least differ, as the same volumes may be reused for multiple
+		// devices
+		for name, da := range vavariant.Assignments {
+			keys = append(keys, name+da.Device)
+		}
+		sort.Strings(keys)
+
+		key := strings.Join(keys, " ")
+		if matches, ok := checker[key]; ok {
+			// In case the user has done something weird where the keys
+			// end up matching (possible edge-case), we double check the
+			// maps here - this is a highly unlikely situation, so the
+			// performance implications should be non-existent
+			for _, match := range matches {
+				if reflect.DeepEqual(vavariant.Assignments, match.Assignments) {
+					return fmt.Errorf("volume-assignment variant %q: identical to %q", vavariant.Name, match.Name)
+				}
+			}
+		}
+		checker[key] = append(checker[key], vavariant)
+	}
+	return nil
+}
+
 // InfoFromGadgetYaml parses the provided gadget metadata.
 // If model is nil only self-consistency checks are performed.
 // If model is not nil implied values for filesystem labels will be set
@@ -972,7 +1185,7 @@ func InfoFromGadgetYaml(gadgetYaml []byte, model Model) (*Info, error) {
 		if err != nil {
 			return nil, fmt.Errorf("default value %q of %q: %v", v, k, err)
 		}
-		gi.Defaults[k] = dflt.(map[string]interface{})
+		gi.Defaults[k] = dflt.(map[string]any)
 	}
 
 	for i, gconn := range gi.Connections {
@@ -992,6 +1205,9 @@ func InfoFromGadgetYaml(gadgetYaml []byte, model Model) (*Info, error) {
 	}
 
 	// basic validation
+	// XXX: With the addition of volume-assignments we could even do per
+	// "device" validation here and check that atleast one bootloader
+	// assignment has been set for each of the assignments
 	var bootloadersFound int
 	for name := range gi.Volumes {
 		v := gi.Volumes[name]
@@ -1039,6 +1255,9 @@ func InfoFromGadgetYaml(gadgetYaml []byte, model Model) (*Info, error) {
 		return nil, fmt.Errorf("too many (%d) bootloaders declared", bootloadersFound)
 	}
 
+	if err := validateVolumeAssignments(gi.VolumeAssignments, gi.Volumes); err != nil {
+		return nil, err
+	}
 	return &gi, nil
 }
 
@@ -1088,6 +1307,24 @@ func asOffsetPtr(offs quantity.Offset) *quantity.Offset {
 	return &offs
 }
 
+func setVolumeStructureOffset(vs *VolumeStructure, startPtr *quantity.Offset) (next *quantity.Offset) {
+	if vs.Offset == nil && startPtr != nil {
+		var start quantity.Offset
+		if vs.Role != schemaMBR && *startPtr < NonMBRStartOffset {
+			start = NonMBRStartOffset
+		} else {
+			start = *startPtr
+		}
+		vs.Offset = &start
+	}
+	// We know the end of the structure only if we could define an offset
+	// and the size is fixed.
+	if vs.Offset != nil && vs.isFixedSize() {
+		return asOffsetPtr(*vs.Offset + quantity.Offset(vs.Size))
+	}
+	return nil
+}
+
 func setImplicitForVolume(vol *Volume, model Model) error {
 	rs := whichVolRuleset(model)
 	if vol.HasPartial(PartialSchema) {
@@ -1112,41 +1349,34 @@ func setImplicitForVolume(vol *Volume, model Model) error {
 
 	previousEnd := asOffsetPtr(0)
 	for i := range vol.Structure {
+		vs := &vol.Structure[i]
+
 		// set the VolumeName for the structure from the volume itself
-		vol.Structure[i].VolumeName = vol.Name
+		vs.VolumeName = vol.Name
 		// Store index as we will reorder later
-		vol.Structure[i].YamlIndex = i
+		vs.YamlIndex = i
 		// MinSize is Size if not explicitly set
-		if vol.Structure[i].MinSize == 0 {
-			vol.Structure[i].MinSize = vol.Structure[i].Size
+		if vs.MinSize == 0 {
+			vs.MinSize = vs.Size
 		}
 		// Set the pointer to the volume
-		vol.Structure[i].EnclosingVolume = vol
+		vs.EnclosingVolume = vol
 
 		// set other implicit data for the structure
-		if err := setImplicitForVolumeStructure(&vol.Structure[i], rs, knownFsLabels, knownVfatFsLabels); err != nil {
+		if err := setImplicitForVolumeStructure(vs, rs, knownFsLabels, knownVfatFsLabels); err != nil {
 			return err
 		}
 
 		// Set offset if it was not set (must be after setImplicitForVolumeStructure
 		// so roles are good). This is possible only if the previous structure had
 		// a well-defined end.
-		if vol.Structure[i].Offset == nil && previousEnd != nil {
-			var start quantity.Offset
-			if vol.Structure[i].Role != schemaMBR && *previousEnd < NonMBRStartOffset {
-				start = NonMBRStartOffset
-			} else {
-				start = *previousEnd
-			}
-			vol.Structure[i].Offset = &start
-		}
-		// We know the end of the structure only if we could define an offset
-		// and the size is fixed.
-		if vol.Structure[i].Offset != nil && vol.Structure[i].isFixedSize() {
-			previousEnd = asOffsetPtr(*vol.Structure[i].Offset +
-				quantity.Offset(vol.Structure[i].Size))
-		} else {
-			previousEnd = nil
+		switch vol.Schema {
+		case schemaEMMC:
+			// For eMMC, we do not support partition offsets. The partitions
+			// are hardware partitions that act more like traditional disks.
+			vs.Offset = asOffsetPtr(0)
+		default:
+			previousEnd = setVolumeStructureOffset(vs, previousEnd)
 		}
 	}
 
@@ -1267,11 +1497,17 @@ func fmtIndexAndName(idx int, name string) string {
 	return fmt.Sprintf("#%v", idx)
 }
 
+var validSchemaNames = []string{schemaMBR, schemaGPT, schemaEMMC}
+
+func isValidSchema(schema string) bool {
+	return strutil.ListContains(validSchemaNames, schema)
+}
+
 func validateVolume(vol *Volume) error {
 	if !validVolumeName.MatchString(vol.Name) {
 		return errors.New("invalid name")
 	}
-	if !vol.HasPartial(PartialSchema) && vol.Schema != schemaGPT && vol.Schema != schemaMBR {
+	if !vol.HasPartial(PartialSchema) && !isValidSchema(vol.Schema) {
 		return fmt.Errorf("invalid schema %q", vol.Schema)
 	}
 
@@ -1338,6 +1574,12 @@ func isMBR(vs *VolumeStructure) bool {
 }
 
 func validateCrossVolumeStructure(vol *Volume) error {
+	// eMMC have no traditional volumes, instead eMMC has the concept
+	// of hardware partitions that act like separate disks.
+	if vol.Schema == schemaEMMC {
+		return nil
+	}
+
 	previousEnd := quantity.Offset(0)
 	// cross structure validation:
 	// - relative offsets that reference other structures by name
@@ -1393,7 +1635,66 @@ func validateOffsetWrite(s, firstStruct *VolumeStructure, volSize quantity.Size)
 	return nil
 }
 
+func contentCheckerCreate(vs *VolumeStructure, vol *Volume) func(*VolumeContent) error {
+	switch {
+	case vol.Schema == schemaEMMC:
+		return validateEMMCContent
+	case vs.HasFilesystem():
+		return validateFilesystemContent
+	default:
+		// default to bare content checker if no filesystem
+		// is present
+		return validateBareContent
+	}
+}
+
+// For emmc volume structures, we do not allow anything but
+// content and name
+func validateEMMCVolumeStructure(vs *VolumeStructure) error {
+	fielderr := func(field string) error {
+		return fmt.Errorf("%q not allowed for emmc volume structures", field)
+	}
+
+	if vs.Size != 0 {
+		return fielderr("size")
+	}
+	if vs.MinSize != 0 {
+		return fielderr("min-size")
+	}
+	if vs.Label != "" {
+		return fielderr("filesystem-label")
+	}
+	if vs.Offset != nil && *vs.Offset != 0 {
+		return fielderr("offset")
+	}
+	if vs.OffsetWrite != nil {
+		return fielderr("offset-write")
+	}
+	if vs.Filesystem != "" {
+		return fielderr("filesystem")
+	}
+	if vs.Type != "" {
+		return fielderr("type")
+	}
+	if vs.ID != "" {
+		return fielderr("id")
+	}
+	if vs.Role != "" {
+		return fielderr("role")
+	}
+
+	for i, c := range vs.Content {
+		if err := validateEMMCContent(&c); err != nil {
+			return fmt.Errorf("invalid content #%v: %v", i, err)
+		}
+	}
+	return validateStructureUpdate(vs)
+}
+
 func validateVolumeStructure(vs *VolumeStructure, vol *Volume) error {
+	if isVolumeEMMC(vol) {
+		return validateEMMCVolumeStructure(vs)
+	}
 	if !vs.hasPartialSize() {
 		if vs.Size == 0 {
 			return errors.New("missing size")
@@ -1419,20 +1720,14 @@ func validateVolumeStructure(vs *VolumeStructure, vol *Volume) error {
 		return fmt.Errorf("invalid filesystem %q", vs.Filesystem)
 	}
 
-	var contentChecker func(*VolumeContent) error
-
-	if vs.HasFilesystem() {
-		contentChecker = validateFilesystemContent
-	} else {
-		contentChecker = validateBareContent
-	}
+	contentChecker := contentCheckerCreate(vs, vol)
 	for i, c := range vs.Content {
 		if err := contentChecker(&c); err != nil {
 			return fmt.Errorf("invalid content #%v: %v", i, err)
 		}
 	}
 
-	if err := validateStructureUpdate(vs, vol); err != nil {
+	if err := validateStructureUpdate(vs); err != nil {
 		return err
 	}
 
@@ -1442,6 +1737,14 @@ func validateVolumeStructure(vs *VolumeStructure, vol *Volume) error {
 	return nil
 }
 
+func validateStructureTypeEMMC(s string) error {
+	// for eMMC we don't support the type being set
+	if s != "" {
+		return errors.New(`type is not supported for "emmc" schema`)
+	}
+	return nil
+}
+
 func validateStructureType(s string, vol *Volume) error {
 	// Type can be one of:
 	// - "mbr" (backwards compatible)
@@ -1453,6 +1756,11 @@ func validateStructureType(s string, vol *Volume) error {
 	// Hybrid ID is 2 hex digits of MBR type, followed by 36 GUUID
 	// example: EF,C12A7328-F81F-11D2-BA4B-00A0C93EC93B
 
+	// eMMC volumes we treat differently
+	if vol.Schema == schemaEMMC {
+		return validateStructureTypeEMMC(s)
+	}
+
 	if s == "" {
 		return errors.New(`type is not specified`)
 	}
@@ -1562,6 +1870,20 @@ func validateBareContent(vc *VolumeContent) error {
 	return nil
 }
 
+func validateEMMCContent(vc *VolumeContent) error {
+	if vc.Offset != nil || vc.Size != 0 {
+		return fmt.Errorf("cannot specify size or offset for content")
+	}
+
+	if vc.UnresolvedSource != "" || vc.Target != "" {
+		return fmt.Errorf("cannot use non-image content for hardware partitions")
+	}
+	if vc.Image == "" {
+		return fmt.Errorf("missing image file name")
+	}
+	return nil
+}
+
 func validateFilesystemContent(vc *VolumeContent) error {
 	if vc.Image != "" || vc.Offset != nil || vc.Size != 0 {
 		return fmt.Errorf("cannot use image content for non-bare file system")
@@ -1575,7 +1897,7 @@ func validateFilesystemContent(vc *VolumeContent) error {
 	return nil
 }
 
-func validateStructureUpdate(vs *VolumeStructure, v *Volume) error {
+func validateStructureUpdate(vs *VolumeStructure) error {
 	if !vs.HasFilesystem() && len(vs.Update.Preserve) > 0 {
 		return errors.New("preserving files during update is not supported for non-filesystem structures")
 	}
@@ -1648,7 +1970,7 @@ func parseRelativeOffset(grs string) (*RelativeOffset, error) {
 	}, nil
 }
 
-func (s *RelativeOffset) UnmarshalYAML(unmarshal func(interface{}) error) error {
+func (s *RelativeOffset) UnmarshalYAML(unmarshal func(any) error) error {
 	var grs string
 	if err := unmarshal(&grs); err != nil {
 		return errors.New(`cannot unmarshal gadget relative offset`)
@@ -1705,7 +2027,12 @@ func checkCompatibleSchema(old, new *Volume) error {
 // LaidOutVolumesFromGadget takes gadget volumes, gadget and kernel rootdirs
 // and lays out the partitions on all volumes as specified. It returns the
 // volumes mentioned in the gadget.yaml and their laid out representations.
-func LaidOutVolumesFromGadget(vols map[string]*Volume, gadgetRoot, kernelRoot string, encType secboot.EncryptionType, volToGadgetToDiskStruct map[string]map[int]*OnDiskStructure) (all map[string]*LaidOutVolume, err error) {
+func LaidOutVolumesFromGadget(
+	vols map[string]*Volume,
+	gadgetRoot, kernelRoot string,
+	encType device.EncryptionType,
+	volToGadgetToDiskStruct map[string]map[int]*OnDiskStructure,
+) (all map[string]*LaidOutVolume, err error) {
 
 	all = make(map[string]*LaidOutVolume)
 	// layout all volumes saving them
@@ -1742,8 +2069,8 @@ func FindBootVolume(vols map[string]*Volume) (*Volume, error) {
 	return nil, fmt.Errorf("no volume has system-boot role")
 }
 
-func flatten(path string, cfg interface{}, out map[string]interface{}) {
-	if cfgMap, ok := cfg.(map[string]interface{}); ok {
+func flatten(path string, cfg any, out map[string]any) {
+	if cfgMap, ok := cfg.(map[string]any); ok {
 		for k, v := range cfgMap {
 			p := k
 			if path != "" {
@@ -1757,10 +2084,10 @@ func flatten(path string, cfg interface{}, out map[string]interface{}) {
 }
 
 // SystemDefaults returns default system configuration from gadget defaults.
-func SystemDefaults(gadgetDefaults map[string]map[string]interface{}) map[string]interface{} {
+func SystemDefaults(gadgetDefaults map[string]map[string]any) map[string]any {
 	for _, systemSnap := range []string{"system", naming.WellKnownSnapID("core")} {
 		if defaults, ok := gadgetDefaults[systemSnap]; ok {
-			coreDefaults := map[string]interface{}{}
+			coreDefaults := map[string]any{}
 			flatten("", defaults, coreDefaults)
 			return coreDefaults
 		}
