@@ -491,8 +491,9 @@ var templateCommon = `
   /run/lock/snap.@{SNAP_INSTANCE_NAME}/ rw,
   /run/lock/snap.@{SNAP_INSTANCE_NAME}/** mrwklix,
 
-  # Allow reading systemd-provided credentials.
+  # Allow listing and reading systemd-provided credentials.
   /run/credentials/ r,
+  /run/credentials/snap.@{SNAP_INSTANCE_NAME}.*.service/ r,
   /run/credentials/snap.@{SNAP_INSTANCE_NAME}.*.service/** r,
   
   ###DEVMODE_SNAP_CONFINE###
@@ -631,6 +632,7 @@ var defaultCoreRuntimeTemplateRules = `
   /{,usr/}bin/stty ixr,
   /{,usr/}bin/sync ixr,
   /{,usr/}bin/systemd-cat ixr,
+  /{,usr/}bin/systemd-creds ixr,
   /{,usr/}bin/tac ixr,
   /{,usr/}bin/tail ixr,
   /{,usr/}bin/tar ixr,
@@ -1010,6 +1012,9 @@ profile snap-update-ns.###SNAP_INSTANCE_NAME### (attach_disconnected) {
   # Allow reading own cgroups
   owner @{PROC}/@{pid}/cgroup r,
 
+  # Allow reading own mountinfo (Go runtime 1.25+)
+  owner @{PROC}/@{pid}/mountinfo r,
+
   # Allow reading somaxconn, required in newer distro releases
   @{PROC}/sys/net/core/somaxconn r,
   # but silence noisy denial of inet/inet6
@@ -1044,6 +1049,10 @@ profile snap-update-ns.###SNAP_INSTANCE_NAME### (attach_disconnected) {
   # mount profile at a given moment.
   /run/snapd/ns/snap.###SNAP_INSTANCE_NAME###.fstab{,.*} rw,
 
+  # Allow writing to a log file for both per-snap and per-snap-and-user log files.
+  /run/snapd/ns/snap.###SNAP_INSTANCE_NAME###.log w,
+  /run/snapd/ns/snap.###SNAP_INSTANCE_NAME###.user.*.log w,
+
   # NOTE: at this stage the /snap directory is stable as we have called
   # pivot_root already.
 
