@@ -91,8 +91,12 @@ type commonInterface struct {
 	controlsDeviceCgroup bool
 
 	serviceSnippets []interfaces.PlugServicesSnippet
+
+	conflictingConnectedInterfaces []string
 }
 
+var _ = interfaces.ConflictingConnectedInterfacesDefiner(&commonInterface{})
+
 // Name returns the interface name.
 func (iface *commonInterface) Name() string {
 	return iface.name
@@ -219,3 +223,7 @@ func (iface *commonInterface) UDevConnectedPlug(spec *udev.Specification, plug *
 
 	return nil
 }
+
+func (iface *commonInterface) ConflictsWithOtherConnectedInterfaces() []string {
+	return iface.conflictingConnectedInterfaces
+}
