@@ -148,10 +148,13 @@ func (s *baseDeclSuite) TestAutoConnection(c *C) {
 		"cuda-driver-libs":       true,
 		"desktop":                true,
 		"egl-driver-libs":        true,
+		"gbm-driver-libs":        true,
 		"home":                   true,
 		"lxd-support":            true,
 		"microstack-support":     true,
 		"multipass-support":      true,
+		"opengl-driver-libs":     true,
+		"opengles-driver-libs":   true,
 		"packagekit-control":     true,
 		"pkcs11":                 true,
 		"remoteproc":             true,
@@ -874,19 +877,22 @@ var (
 		"wayland":                   {"app", "core"},
 		"x11":                       {"app", "core"},
 		// snowflakes
-		"classic-support":   nil,
-		"cuda-driver-libs":  nil,
-		"custom-device":     nil,
-		"docker":            nil,
-		"egl-driver-libs":   nil,
-		"lxd":               nil,
-		"microceph":         nil,
-		"microceph-support": nil,
-		"microovn":          nil,
-		"pkcs11":            nil,
-		"posix-mq":          nil,
-		"shared-memory":     nil,
-		"gpio-chardev":      nil,
+		"classic-support":      nil,
+		"cuda-driver-libs":     nil,
+		"custom-device":        nil,
+		"docker":               nil,
+		"egl-driver-libs":      nil,
+		"gbm-driver-libs":      nil,
+		"lxd":                  nil,
+		"microceph":            nil,
+		"microceph-support":    nil,
+		"microovn":             nil,
+		"opengl-driver-libs":   nil,
+		"opengles-driver-libs": nil,
+		"pkcs11":               nil,
+		"posix-mq":             nil,
+		"shared-memory":        nil,
+		"gpio-chardev":         nil,
 	}
 
 	restrictedPlugInstallation = map[string][]string{
@@ -1046,6 +1052,7 @@ func (s *baseDeclSuite) TestPlugInstallation(c *C) {
 		"dm-crypt":                         true,
 		"docker-support":                   true,
 		"egl-driver-libs":                  true,
+		"gbm-driver-libs":                  true,
 		"greengrass-support":               true,
 		"gpio-control":                     true,
 		"ion-memory-control":               true,
@@ -1060,6 +1067,8 @@ func (s *baseDeclSuite) TestPlugInstallation(c *C) {
 		"multipass-support":                true,
 		"nomad-support":                    true,
 		"nvidia-drivers-support":           true,
+		"opengl-driver-libs":               true,
+		"opengles-driver-libs":             true,
 		"packagekit-control":               true,
 		"personal-files":                   true,
 		"polkit":                           true,
@@ -1355,6 +1364,7 @@ func (s *baseDeclSuite) TestValidity(c *C) {
 		"dm-crypt":                         true,
 		"docker-support":                   true,
 		"egl-driver-libs":                  true,
+		"gbm-driver-libs":                  true,
 		"greengrass-support":               true,
 		"gpio-control":                     true,
 		"ion-memory-control":               true,
@@ -1369,6 +1379,8 @@ func (s *baseDeclSuite) TestValidity(c *C) {
 		"multipass-support":                true,
 		"nomad-support":                    true,
 		"nvidia-drivers-support":           true,
+		"opengl-driver-libs":               true,
+		"opengles-driver-libs":             true,
 		"packagekit-control":               true,
 		"personal-files":                   true,
 		"pkcs11":                           true,
