@@ -55,6 +55,15 @@ set_tls
 usr26
 usr32
 
+# Requires input fd and so should not pose more security 
+# issues than access to the file in the first place
+# Flags are currently unused and should be 0
+cachestat - - - 0
+
+# Flags are currently unused and should be 0
+mseal - - 0
+map_shadow_stack
+
 capget
 # AppArmor mediates capabilities, so allow capset (useful for apps that for
 # example want to drop capabilities)
@@ -68,6 +77,7 @@ fchdir
 chmod
 fchmod
 fchmodat
+fchmodat2
 
 # Daemons typically run as 'root' so allow chown to 'root'. DAC will prevent
 # non-root from chowning to root.
@@ -125,6 +135,7 @@ epoll_create1
 epoll_ctl
 epoll_ctl_old
 epoll_pwait
+epoll_pwait2
 epoll_wait
 epoll_wait_old
 eventfd
@@ -146,8 +157,11 @@ flock
 fork
 ftime
 futex
+futex_requeue
 futex_time64
+futex_wait
 futex_waitv
+futex_wake
 get_mempolicy
 get_robust_list
 get_thread_area
@@ -188,6 +202,7 @@ getuid32
 getxattr
 fgetxattr
 lgetxattr
+getxattrat
 
 inotify_add_watch
 inotify_init
@@ -205,6 +220,9 @@ inotify_rm_watch
 # TODO: this should be scaled back even more
 ~ioctl - TIOCSTI
 ~ioctl - TIOCLINUX
+# see CVE-2019-7303
+~ioctl - 4294967295|TIOCSTI
+~ioctl - 4294967295|TIOCLINUX
 ioctl
 
 io_cancel
@@ -231,6 +249,7 @@ linkat
 listxattr
 llistxattr
 flistxattr
+listxattrat
 
 lseek
 llseek
@@ -301,6 +320,7 @@ openat
 pause
 personality
 pipe
+~pipe2 - |O_NOTIFICATION_PIPE
 pipe2
 poll
 ppoll
@@ -333,6 +353,7 @@ remap_file_pages
 removexattr
 fremovexattr
 lremovexattr
+removexattrat
 
 rename
 renameat
@@ -381,6 +402,13 @@ sched_yield
 # permissions)
 seccomp
 
+# Allow restricting access with Landlock. This is OK because the kernel
+# enforces that each new restriction only drops accesses for the calling
+# process (i.e., no widening permissions).
+landlock_create_ruleset
+landlock_add_rule
+landlock_restrict_self
+
 select
 _newselect
 pselect
@@ -425,6 +453,7 @@ set_tid_address
 setxattr
 fsetxattr
 lsetxattr
+setxattrat
 
 shmat
 shmctl
@@ -592,6 +621,7 @@ writev
 pwrite
 pwrite64
 pwritev
+pwritev2
 `)
 
 // Go's net package attempts to bind early to check whether IPv6 is available or not.
