@@ -48,11 +48,11 @@ if command -v git >/dev/null && [ -d "$(dirname "$0")/.git" ] ; then
     # dirty and append that last, including it here will make dirty trees 
     # directly on top of tags show up with version_from_git as 2.46-dirty which
     # will not match 2.46 from the changelog and then result in a final version
-    # like 2.46+git2.46.2.46 which is silly and unhelpful
+    # like 2.46+g2.46.2.46 which is silly and unhelpful
     # tracking the dirty independently like this will produce instead 2.46-dirty
-    # for a dirty tree on top of a tag, and 2.46+git83.g1671726-dirty for a 
+    # for a dirty tree on top of a tag, and 2.46+g83.1671726-dirty for a
     # commit not directly on top of a tag
-    version_from_git="$(git describe --always | sed -e 's/-/+git/;y/-/./' )"
+    version_from_git="$(git describe --always | sed -e  's/-/+/;y/-/./;s/\.g/g./;s/\([0-9]\+\)g/g\1/' )"
 
     # check if we are using a dirty tree
     if git describe --always --dirty | grep -q dirty; then
@@ -63,8 +63,9 @@ fi
 # at this point we maybe in _build/src/github etc where we have no
 # debian/changelog (dh-golang only exports the sources here)
 # switch to the real source dir for the changelog parsing
-if command -v dpkg-parsechangelog >/dev/null; then
-    version_from_changelog="$(cd "$PKG_BUILDDIR"; dpkg-parsechangelog --show-field Version)";
+: "${DPKG_PARSECHANGELOG:=$(command -v dpkg-parsechangelog)}"
+if [ -n "$DPKG_PARSECHANGELOG" ]; then
+    version_from_changelog="$(cd "$PKG_BUILDDIR"; "$DPKG_PARSECHANGELOG" --show-field Version)";
 fi
 
 # select version based on priority
@@ -90,7 +91,7 @@ fi
 # and append the git revno and commit hash. A simpler approach would be
 # to git tag all pre/rc releases.
 if [ -z "$version_from_user" ] && [ "$version_from_git" != "" ] && \
-       [ -n "$version_from_changelog" ] && [ "$version_from_git" != "$version_from_changelog" ]; then
+       [ -n "$version_from_changelog" ] && [ "$version_from_git" != "${version_from_changelog%+fips}" ]; then
     # if the changelog version has "git" in it and we also have a git version
     # directly, that is a bad changelog version, so fail, otherwise the below
     # code will produce a duplicated git info
@@ -99,8 +100,8 @@ if [ -z "$version_from_user" ] && [ "$version_from_git" != "" ] && \
         exit 1
     else
         revno=$(git describe --always --abbrev=7|cut -d- -f2)
-        commit=$(git describe --always --abbrev=7|cut -d- -f3)
-        v="${version_from_changelog}+git${revno}.${commit}"
+        commit=$(git describe --always --abbrev=7|cut -d- -f3|sed -e 's/^g//')
+        v="${version_from_changelog}+g${revno}.${commit}"
         o="changelog+git"
     fi
 fi
