@@ -20,16 +20,16 @@
 package disks_test
 
 import (
+	"errors"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/gadget/gadgettest"
 	"github.com/snapcore/snapd/gadget/quantity"
-	"github.com/snapcore/snapd/kernel/fde"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/disks"
 	"github.com/snapcore/snapd/testutil"
@@ -121,7 +121,7 @@ func createVirtioDevicesInSysfs(c *C, path string, devsToPartition map[string]bo
 		err := os.MkdirAll(filepath.Join(diskDir, dev), 0755)
 		c.Assert(err, IsNil)
 		if isPartition {
-			err = ioutil.WriteFile(filepath.Join(diskDir, dev, "partition"), []byte("1"), 0644)
+			err = os.WriteFile(filepath.Join(diskDir, dev, "partition"), []byte("1"), 0644)
 			c.Assert(err, IsNil)
 		}
 	}
@@ -179,6 +179,10 @@ func (s *diskSuite) TestDiskFromDeviceNameHappy(c *C) {
 }
 
 func (s *diskSuite) TestDiskFromDevicePathHappy(c *C) {
+	// for udevadm trigger and udevadm settle which are called on the partitions
+	mockUdevadm := testutil.MockCommand(c, "udevadm", ``)
+	defer mockUdevadm.Restore()
+
 	const vdaSysfsPath = "/devices/pci0000:00/0000:00:04.0/virtio2/block/vdb"
 	fullSysPath := filepath.Join("/sys", vdaSysfsPath)
 	n := 0
@@ -287,7 +291,7 @@ func (s *diskSuite) TestDiskFromDevicePathHappy(c *C) {
 			KernelDeviceNode: "/dev/vdb2",
 			SizeInBytes:      124473665 * 512,
 			StartInBytes:     uint64(257 * quantity.SizeMiB),
-			StructureIndex:   2,
+			DiskIndex:        2,
 		},
 		{
 			Major:            1,
@@ -298,11 +302,18 @@ func (s *diskSuite) TestDiskFromDevicePathHappy(c *C) {
 			KernelDeviceNode: "/dev/vdb1",
 			SizeInBytes:      uint64(256 * quantity.SizeMiB),
 			StartInBytes:     uint64(quantity.SizeMiB),
-			StructureIndex:   1,
+			DiskIndex:        1,
 		},
 	})
 
 	c.Assert(n, Equals, 4)
+
+	c.Assert(mockUdevadm.Calls(), DeepEquals, [][]string{
+		{"udevadm", "trigger", "--name-match=vdb1"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vdb2"},
+		{"udevadm", "settle", "--timeout=180"},
+	})
 }
 
 func (s *diskSuite) TestDiskFromPartitionDeviceNodeHappy(c *C) {
@@ -491,7 +502,7 @@ func (s *diskSuite) TestDiskFromMountPointUnhappyIsDecryptedDeviceNotDiskDevice(
 
 	opts := &disks.Options{IsDecryptedDevice: true}
 	_, err := disks.DiskFromMountPoint("/run/mnt/point", opts)
-	c.Assert(err, ErrorMatches, `cannot find disk from mountpoint source /dev/vda4 of /run/mnt/point: not a decrypted device: devtype is not disk \(is partition\)`)
+	c.Assert(err, ErrorMatches, `cannot process properties of /dev/vda4 parent device: not a decrypted device: devtype is not disk \(is partition\)`)
 }
 
 func (s *diskSuite) TestDiskFromMountPointUnhappyIsDecryptedDeviceNoSysfs(c *C) {
@@ -519,10 +530,14 @@ func (s *diskSuite) TestDiskFromMountPointUnhappyIsDecryptedDeviceNoSysfs(c *C)
 
 	opts := &disks.Options{IsDecryptedDevice: true}
 	_, err := disks.DiskFromMountPoint("/run/mnt/point", opts)
-	c.Assert(err, ErrorMatches, fmt.Sprintf(`cannot find disk from mountpoint source /dev/mapper/something of /run/mnt/point: not a decrypted device: could not read device mapper metadata: open %s/dev/block/252:0/dm/uuid: no such file or directory`, dirs.SysfsDir))
+	c.Assert(err, ErrorMatches, fmt.Sprintf(`cannot process properties of /dev/mapper/something parent device: not a decrypted device: could not read device mapper metadata: open %s/dev/block/252:0/dm/uuid: no such file or directory`, dirs.SysfsDir))
 }
 
 func (s *diskSuite) TestDiskFromMountPointHappySinglePartitionIgnoresNonPartitionsInSysfs(c *C) {
+	// for udevadm trigger and udevadm settle which are called on the partitions
+	mockUdevadm := testutil.MockCommand(c, "udevadm", ``)
+	defer mockUdevadm.Restore()
+
 	restore := osutil.MockMountInfo(`130 30 47:1 / /run/mnt/point rw,relatime shared:54 - ext4 /dev/vda4 rw
 `)
 	defer restore()
@@ -609,7 +624,7 @@ func (s *diskSuite) TestDiskFromMountPointHappySinglePartitionIgnoresNonPartitio
 			Minor:            4,
 			PartitionType:    "SOME-GPT-UUID-TYPE",
 			SizeInBytes:      3000 * 512,
-			StructureIndex:   4,
+			DiskIndex:        4,
 			StartInBytes:     2500 * 512,
 		},
 	})
@@ -625,6 +640,11 @@ func (s *diskSuite) TestDiskFromMountPointHappySinglePartitionIgnoresNonPartitio
 		SearchType:  "filesystem-label",
 		SearchQuery: "ubuntu-boot",
 	})
+
+	c.Assert(mockUdevadm.Calls(), DeepEquals, [][]string{
+		{"udevadm", "trigger", "--name-match=vda4"},
+		{"udevadm", "settle", "--timeout=180"},
+	})
 }
 
 func (s *diskSuite) TestDiskFromMountPointHappyRealUdevadm(c *C) {
@@ -723,11 +743,11 @@ func (s *diskSuite) TestDiskFromMountPointIsDecryptedLUKSDeviceVolumeHappy(c *C)
 	c.Assert(err, IsNil)
 
 	b := []byte("something")
-	err = ioutil.WriteFile(filepath.Join(dmDir, "name"), b, 0644)
+	err = os.WriteFile(filepath.Join(dmDir, "name"), b, 0644)
 	c.Assert(err, IsNil)
 
 	b = []byte("CRYPT-LUKS2-5a522809c87e4dfa81a88dc5667d1304-something")
-	err = ioutil.WriteFile(filepath.Join(dmDir, "uuid"), b, 0644)
+	err = os.WriteFile(filepath.Join(dmDir, "uuid"), b, 0644)
 	c.Assert(err, IsNil)
 
 	opts := &disks.Options{IsDecryptedDevice: true}
@@ -740,7 +760,7 @@ func (s *diskSuite) TestDiskFromMountPointIsDecryptedLUKSDeviceVolumeHappy(c *C)
 	}()
 
 	_, err = disks.DiskFromMountPoint("/run/mnt/point", opts)
-	c.Assert(err, ErrorMatches, `cannot find disk from mountpoint source /dev/mapper/something of /run/mnt/point: internal error: no back resolver supports decrypted device mapper with UUID "CRYPT-LUKS2-5a522809c87e4dfa81a88dc5667d1304-something" and name "something"`)
+	c.Assert(err, ErrorMatches, `cannot process properties of /dev/mapper/something parent device: internal error: no back resolver supports decrypted device mapper with UUID "CRYPT-LUKS2-5a522809c87e4dfa81a88dc5667d1304-something" and name "something"`)
 
 	// but when it is available it works
 	disks.RegisterDeviceMapperBackResolver("crypt-luks2", disks.CryptLuks2DeviceMapperBackResolver)
@@ -752,114 +772,6 @@ func (s *diskSuite) TestDiskFromMountPointIsDecryptedLUKSDeviceVolumeHappy(c *C)
 	c.Assert(d.Schema(), Equals, "dos")
 }
 
-func (s *diskSuite) TestDiskFromMountPointIsDecryptedUnlockedDeviceVolumeHappy(c *C) {
-	restore := osutil.MockMountInfo(`130 30 242:1 / /run/mnt/point rw,relatime shared:54 - ext4 /dev/mapper/something-device-locked rw
-`)
-	defer restore()
-
-	// un-register the fde handler so we can make sure that the default
-	// handler(s) don't match this device mapper - this name needs to be kept in
-	// sync with what's in kernel/fde/fde.go
-	disks.UnregisterDeviceMapperBackResolver("device-unlock-kernel-fde")
-	defer func() {
-		// this is registered by default in this file since we import the fde
-		// package so make sure it is re-registered at the end
-		disks.RegisterDeviceMapperBackResolver("device-unlock-kernel-fde", fde.DeviceUnlockKernelHookDeviceMapperBackResolver)
-	}()
-
-	restore = disks.MockUdevPropertiesForDevice(func(typeOpt, dev string) (map[string]string, error) {
-		c.Assert(typeOpt, Equals, "--name")
-		switch dev {
-		case "/dev/mapper/something-device-locked":
-			return map[string]string{
-				"DEVTYPE": "disk",
-				"MAJOR":   "242",
-				"MINOR":   "1",
-			}, nil
-		case "/dev/disk/by-partuuid/5a522809-c87e-4dfa-81a8-8dc5667d1304":
-			return map[string]string{
-				"DEVTYPE":            "disk",
-				"ID_PART_ENTRY_DISK": "41:3",
-			}, nil
-		case "/dev/block/41:3":
-			return map[string]string{
-				"DEVTYPE":            "disk",
-				"DEVNAME":            "/dev/sda",
-				"DEVPATH":            "/block/foo",
-				"ID_PART_TABLE_UUID": "foo-foo-foo-foo",
-				"ID_PART_TABLE_TYPE": "gpt",
-			}, nil
-		case "sda1":
-			return map[string]string{
-				"DEVPATH":              "/block/foo/sda1",
-				"DEVNAME":              "/dev/sda1",
-				"ID_PART_ENTRY_UUID":   "foo-foo-foo-foo-1",
-				"ID_PART_ENTRY_TYPE":   "gooooooo",
-				"ID_PART_ENTRY_SIZE":   "500000",
-				"ID_PART_ENTRY_NUMBER": "1",
-				"ID_PART_ENTRY_OFFSET": "2048",
-				"MAJOR":                "41",
-				"MINOR":                "4",
-			}, nil
-		default:
-			c.Errorf("unexpected udev device properties requested: %s", dev)
-			return nil, fmt.Errorf("unexpected udev device: %s", dev)
-		}
-	})
-	defer restore()
-
-	// mock the sysfs dm uuid and name files
-	dmDir := filepath.Join(filepath.Join(dirs.SysfsDir, "dev", "block"), "242:1", "dm")
-	err := os.MkdirAll(dmDir, 0755)
-	c.Assert(err, IsNil)
-
-	// name expected by fde
-	b := []byte("something-device-locked")
-	err = ioutil.WriteFile(filepath.Join(dmDir, "name"), b, 0644)
-	c.Assert(err, IsNil)
-
-	b = []byte("5a522809-c87e-4dfa-81a8-8dc5667d1304")
-	err = ioutil.WriteFile(filepath.Join(dmDir, "uuid"), b, 0644)
-	c.Assert(err, IsNil)
-
-	opts := &disks.Options{IsDecryptedDevice: true}
-
-	// first try without the handler fails
-	_, err = disks.DiskFromMountPoint("/run/mnt/point", opts)
-	c.Assert(err, ErrorMatches, `cannot find disk from mountpoint source /dev/mapper/something-device-locked of /run/mnt/point: internal error: no back resolver supports decrypted device mapper with UUID "5a522809-c87e-4dfa-81a8-8dc5667d1304" and name "something-device-locked"`)
-
-	// next try with the FDE package handler works
-	disks.RegisterDeviceMapperBackResolver("device-unlock-kernel-fde", fde.DeviceUnlockKernelHookDeviceMapperBackResolver)
-
-	d, err := disks.DiskFromMountPoint("/run/mnt/point", opts)
-	c.Assert(err, IsNil)
-	c.Assert(d.Dev(), Equals, "41:3")
-	c.Assert(d.HasPartitions(), Equals, true)
-
-	// mock a partition
-	partFile := filepath.Join(dirs.SysfsDir, "/block/foo/sda1/partition")
-	err = os.MkdirAll(filepath.Dir(partFile), 0755)
-	c.Assert(err, IsNil)
-	err = ioutil.WriteFile(partFile, nil, 0644)
-	c.Assert(err, IsNil)
-
-	parts, err := d.Partitions()
-	c.Assert(err, IsNil)
-	c.Assert(parts, DeepEquals, []disks.Partition{
-		{
-			PartitionType:    "GOOOOOOO",
-			StructureIndex:   1,
-			StartInBytes:     512 * 2048,
-			SizeInBytes:      500000 * 512,
-			PartitionUUID:    "foo-foo-foo-foo-1",
-			Major:            41,
-			Minor:            4,
-			KernelDevicePath: filepath.Join(dirs.SysfsDir, "/block/foo/sda1"),
-			KernelDeviceNode: "/dev/sda1",
-		},
-	})
-}
-
 func (s *diskSuite) TestDiskFromMountPointNotDiskUnsupported(c *C) {
 	restore := osutil.MockMountInfo(`130 30 42:1 / /run/mnt/point rw,relatime shared:54 - ext4 /dev/not-a-disk rw
 `)
@@ -919,6 +831,10 @@ fi
 }
 
 func (s *diskSuite) TestDiskFromMountPointPartitionsHappy(c *C) {
+	// for udevadm trigger and udevadm settle which are called on the partitions
+	mockUdevadm := testutil.MockCommand(c, "udevadm", ``)
+	defer mockUdevadm.Restore()
+
 	restore := osutil.MockMountInfo(`130 30 42:4 / /run/mnt/data rw,relatime shared:54 - ext4 /dev/vda4 rw
  130 30 42:4 / /run/mnt/ubuntu-boot rw,relatime shared:54 - ext4 /dev/vda3 rw
 `)
@@ -1085,9 +1001,32 @@ func (s *diskSuite) TestDiskFromMountPointPartitionsHappy(c *C) {
 		SearchType:  "partition-label",
 		SearchQuery: "NOT BIOS Boot",
 	})
+
+	c.Assert(mockUdevadm.Calls(), DeepEquals, [][]string{
+		{"udevadm", "trigger", "--name-match=vda1"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda2"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda3"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda4"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda1"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda2"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda3"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda4"},
+		{"udevadm", "settle", "--timeout=180"},
+	})
 }
 
 func (s *diskSuite) TestDiskFromMountPointDecryptedDevicePartitionsHappy(c *C) {
+	// for udevadm trigger and udevadm settle which are called on the partitions
+	mockUdevadm := testutil.MockCommand(c, "udevadm", ``)
+	defer mockUdevadm.Restore()
+
 	restore := osutil.MockMountInfo(`130 30 252:0 / /run/mnt/data rw,relatime shared:54 - ext4 /dev/mapper/ubuntu-data-3776bab4-8bcc-46b7-9da2-6a84ce7f93b4 rw
  130 30 42:4 / /run/mnt/ubuntu-boot rw,relatime shared:54 - ext4 /dev/vda3 rw
 `)
@@ -1107,7 +1046,7 @@ func (s *diskSuite) TestDiskFromMountPointDecryptedDevicePartitionsHappy(c *C) {
 			KernelDeviceNode: "/dev/vda4",
 			PartitionType:    "0FC63DAF-8483-4772-8E79-3D69D8477DE4",
 			SizeInBytes:      8552415 * 512,
-			StructureIndex:   4,
+			DiskIndex:        4,
 			StartInBytes:     3997696 * 512,
 		},
 		"ubuntu-boot": {
@@ -1120,7 +1059,7 @@ func (s *diskSuite) TestDiskFromMountPointDecryptedDevicePartitionsHappy(c *C) {
 			KernelDeviceNode: "/dev/vda3",
 			PartitionType:    "0FC63DAF-8483-4772-8E79-3D69D8477DE4",
 			SizeInBytes:      1536000 * 512,
-			StructureIndex:   3,
+			DiskIndex:        3,
 			StartInBytes:     2461696 * 512,
 		},
 		"ubuntu-seed": {
@@ -1133,7 +1072,7 @@ func (s *diskSuite) TestDiskFromMountPointDecryptedDevicePartitionsHappy(c *C) {
 			KernelDeviceNode: "/dev/vda2",
 			PartitionType:    "C12A7328-F81F-11D2-BA4B-00A0C93EC93B",
 			SizeInBytes:      2457600 * 512,
-			StructureIndex:   2,
+			DiskIndex:        2,
 			StartInBytes:     4096 * 512,
 		},
 		"bios-boot": {
@@ -1145,7 +1084,7 @@ func (s *diskSuite) TestDiskFromMountPointDecryptedDevicePartitionsHappy(c *C) {
 			KernelDeviceNode: "/dev/vda1",
 			PartitionType:    "21686148-6449-6E6F-744E-656564454649",
 			SizeInBytes:      2048 * 512,
-			StructureIndex:   1,
+			DiskIndex:        1,
 			StartInBytes:     2048 * 512,
 		},
 	}
@@ -1261,11 +1200,11 @@ func (s *diskSuite) TestDiskFromMountPointDecryptedDevicePartitionsHappy(c *C) {
 	c.Assert(err, IsNil)
 
 	b := []byte("ubuntu-data-3776bab4-8bcc-46b7-9da2-6a84ce7f93b4")
-	err = ioutil.WriteFile(filepath.Join(dmDir, "name"), b, 0644)
+	err = os.WriteFile(filepath.Join(dmDir, "name"), b, 0644)
 	c.Assert(err, IsNil)
 
 	b = []byte("CRYPT-LUKS2-5a522809c87e4dfa81a88dc5667d1304-ubuntu-data-3776bab4-8bcc-46b7-9da2-6a84ce7f93b4")
-	err = ioutil.WriteFile(filepath.Join(dmDir, "uuid"), b, 0644)
+	err = os.WriteFile(filepath.Join(dmDir, "uuid"), b, 0644)
 	c.Assert(err, IsNil)
 
 	// mock the dev nodes in sysfs for the partitions
@@ -1326,6 +1265,25 @@ func (s *diskSuite) TestDiskFromMountPointDecryptedDevicePartitionsHappy(c *C) {
 	matches, err = ubuntuBootDisk.MountPointIsFromDisk("/run/mnt/data", opts)
 	c.Assert(err, IsNil)
 	c.Assert(matches, Equals, true)
+
+	c.Assert(mockUdevadm.Calls(), DeepEquals, [][]string{
+		{"udevadm", "trigger", "--name-match=vda1"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda2"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda3"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda4"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda1"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda2"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda3"},
+		{"udevadm", "settle", "--timeout=180"},
+		{"udevadm", "trigger", "--name-match=vda4"},
+		{"udevadm", "settle", "--timeout=180"},
+	})
 }
 
 func (s *diskSuite) TestMountPointsForPartitionRoot(c *C) {
@@ -1457,7 +1415,97 @@ func (s *diskSuite) TestDiskSizeRelatedMethodsGPT(c *C) {
 	})
 	defer restore()
 
+	restoreCalculateLBA := disks.MockCalculateLastUsableLBA(42, nil)
+	defer restoreCalculateLBA()
+
 	sfdiskCmd := testutil.MockCommand(c, "sfdisk", `
+if [ "$1" = --version ]; then
+	echo 'sfdisk from util-linux 2.37.2'
+	exit 0
+fi
+echo '{
+	"partitiontable": {
+		"unit": "sectors",
+		"lastlba": 42
+	}
+}'
+`)
+	defer sfdiskCmd.Restore()
+
+	d, err := disks.DiskFromDeviceName("sda")
+	c.Assert(err, IsNil)
+	c.Assert(d.Schema(), Equals, "gpt")
+	c.Assert(d.KernelDeviceNode(), Equals, "/dev/sda")
+
+	blockDevCmd := testutil.MockCommand(c, "blockdev", `
+if [ "$1" = "--getsize64" ]; then
+	echo 5120000
+elif [ "$1" = "--getss" ]; then
+	echo 512
+else
+	echo "fail, test broken"
+	exit 1
+fi
+`)
+	defer blockDevCmd.Restore()
+
+	endSectors, err := d.UsableSectorsEnd()
+	c.Assert(err, IsNil)
+	c.Assert(endSectors, Equals, uint64(43))
+	c.Assert(sfdiskCmd.Calls(), DeepEquals, [][]string{
+		{"sfdisk", "--version"},
+	})
+	c.Assert(blockDevCmd.Calls(), DeepEquals, [][]string{
+		{"blockdev", "--getsize64", "/dev/sda"},
+		{"blockdev", "--getss", "/dev/sda"},
+	})
+	blockDevCmd.ForgetCalls()
+	sfdiskCmd.ForgetCalls()
+
+	sz, err := d.SizeInBytes()
+	c.Assert(err, IsNil)
+	c.Assert(sz, Equals, uint64(10000*512))
+	c.Assert(blockDevCmd.Calls(), DeepEquals, [][]string{
+		{"blockdev", "--getsize64", "/dev/sda"},
+	})
+
+	blockDevCmd.ForgetCalls()
+
+	sectorSz, err := d.SectorSize()
+	c.Assert(err, IsNil)
+	c.Assert(sectorSz, Equals, uint64(512))
+	c.Assert(blockDevCmd.Calls(), DeepEquals, [][]string{
+		{"blockdev", "--getss", "/dev/sda"},
+	})
+
+	// we didn't use sfdisk again at all
+	c.Assert(sfdiskCmd.Calls(), HasLen, 0)
+}
+
+func (s *diskSuite) TestDiskSizeRelatedMethodsGPTFallback(c *C) {
+	restore := disks.MockUdevPropertiesForDevice(func(typeOpt, dev string) (map[string]string, error) {
+		c.Assert(typeOpt, Equals, "--name")
+		c.Assert(dev, Equals, "sda")
+		return map[string]string{
+			"MAJOR":              "1",
+			"MINOR":              "2",
+			"DEVTYPE":            "disk",
+			"DEVNAME":            "/dev/sda",
+			"ID_PART_TABLE_UUID": "foo",
+			"ID_PART_TABLE_TYPE": "gpt",
+			"DEVPATH":            "/devices/foo/sda",
+		}, nil
+	})
+	defer restore()
+
+	restoreCalculateLBA := disks.MockCalculateLastUsableLBA(0, errors.New("Some error"))
+	defer restoreCalculateLBA()
+
+	sfdiskCmd := testutil.MockCommand(c, "sfdisk", `
+if [ "$1" = --version ]; then
+	echo 'sfdisk from util-linux 2.34.1'
+	exit 0
+fi
 echo '{
 	"partitiontable": {
 		"unit": "sectors",
@@ -1476,40 +1524,9 @@ echo '{
 	c.Assert(err, IsNil)
 	c.Assert(endSectors, Equals, uint64(43))
 	c.Assert(sfdiskCmd.Calls(), DeepEquals, [][]string{
+		{"sfdisk", "--version"},
 		{"sfdisk", "--json", "/dev/sda"},
 	})
-	sfdiskCmd.ForgetCalls()
-
-	blockDevCmd := testutil.MockCommand(c, "blockdev", `
-if [ "$1" = "--getsz" ]; then
-	echo 10000
-elif [ "$1" = "--getss" ]; then
-	echo 512
-else
-	echo "fail, test broken"
-	exit 1
-fi
-`)
-	defer blockDevCmd.Restore()
-
-	sz, err := d.SizeInBytes()
-	c.Assert(err, IsNil)
-	c.Assert(sz, Equals, uint64(10000*512))
-	c.Assert(blockDevCmd.Calls(), DeepEquals, [][]string{
-		{"blockdev", "--getsz", "/dev/sda"},
-	})
-
-	blockDevCmd.ForgetCalls()
-
-	sectorSz, err := d.SectorSize()
-	c.Assert(err, IsNil)
-	c.Assert(sectorSz, Equals, uint64(512))
-	c.Assert(blockDevCmd.Calls(), DeepEquals, [][]string{
-		{"blockdev", "--getss", "/dev/sda"},
-	})
-
-	// we didn't use sfdisk again at all
-	c.Assert(sfdiskCmd.Calls(), HasLen, 0)
 }
 
 func (s *diskSuite) TestDiskUsableSectorsEndGPTUnexpectedSfdiskUnit(c *C) {
@@ -1529,6 +1546,10 @@ func (s *diskSuite) TestDiskUsableSectorsEndGPTUnexpectedSfdiskUnit(c *C) {
 	defer restore()
 
 	cmd := testutil.MockCommand(c, "sfdisk", `
+if [ "$1" = --version ]; then
+	echo 'sfdisk from util-linux 2.34.1'
+	exit 0
+fi
 echo '{
 	"partitiontable": {
 		"unit": "not-sectors",
@@ -1547,6 +1568,7 @@ echo '{
 	c.Assert(err, ErrorMatches, "cannot get size in sectors, sfdisk reported unknown unit not-sectors")
 
 	c.Assert(cmd.Calls(), DeepEquals, [][]string{
+		{"sfdisk", "--version"},
 		{"sfdisk", "--json", "/dev/sda"},
 	})
 }
@@ -1568,6 +1590,10 @@ func (s *diskSuite) TestDiskSizeRelatedMethodsGPTSectorSize4K(c *C) {
 	defer restore()
 
 	sfdiskCmd := testutil.MockCommand(c, "sfdisk", `
+if [ "$1" = --version ]; then
+	echo 'sfdisk from util-linux 2.34.1'
+	exit 0
+fi
 echo '{
 	"partitiontable": {
 		"unit": "sectors",
@@ -1586,14 +1612,15 @@ echo '{
 	c.Assert(err, IsNil)
 	c.Assert(endSectors, Equals, uint64(43))
 	c.Assert(sfdiskCmd.Calls(), DeepEquals, [][]string{
+		{"sfdisk", "--version"},
 		{"sfdisk", "--json", "/dev/sda"},
 	})
 
 	sfdiskCmd.ForgetCalls()
 
 	blockDevCmd := testutil.MockCommand(c, "blockdev", `
-if [ "$1" = "--getsz" ]; then
-	echo 10000
+if [ "$1" = "--getsize64" ]; then
+	echo 5120000
 elif [ "$1" = "--getss" ]; then
 	echo 4096
 else
@@ -1605,12 +1632,9 @@ fi
 
 	sz, err := d.SizeInBytes()
 	c.Assert(err, IsNil)
-	// the size is still the result of --getsz * 512, we don't use the native
-	// sector size at all here since blockdev doesn't use the native sector size
-	// at all
-	c.Assert(sz, Equals, uint64(10000*512))
+	c.Assert(sz, Equals, uint64(5120000))
 	c.Assert(blockDevCmd.Calls(), DeepEquals, [][]string{
-		{"blockdev", "--getsz", "/dev/sda"},
+		{"blockdev", "--getsize64", "/dev/sda"},
 	})
 
 	blockDevCmd.ForgetCalls()
@@ -1626,60 +1650,6 @@ fi
 	c.Assert(sfdiskCmd.Calls(), HasLen, 0)
 }
 
-func (s *diskSuite) TestDiskSizeRelatedMethodsGPTNon512MultipleSectorSizeError(c *C) {
-	restore := disks.MockUdevPropertiesForDevice(func(typeOpt, dev string) (map[string]string, error) {
-		c.Assert(typeOpt, Equals, "--name")
-		c.Assert(dev, Equals, "sda")
-		return map[string]string{
-			"MAJOR":              "1",
-			"MINOR":              "2",
-			"DEVTYPE":            "disk",
-			"DEVNAME":            "/dev/sda",
-			"ID_PART_TABLE_UUID": "foo",
-			"ID_PART_TABLE_TYPE": "gpt",
-			"DEVPATH":            "/devices/foo/sda",
-		}, nil
-	})
-	defer restore()
-
-	sfdiskCmd := testutil.MockCommand(c, "sfdisk", `
-echo '{
-	"partitiontable": {
-		"unit": "sectors",
-		"lastlba": 42
-	}
-}'
-`)
-	defer sfdiskCmd.Restore()
-
-	d, err := disks.DiskFromDeviceName("sda")
-	c.Assert(err, IsNil)
-	c.Assert(d.Schema(), Equals, "gpt")
-	c.Assert(d.KernelDeviceNode(), Equals, "/dev/sda")
-
-	// the end of sectors end does not query the size of the sectors
-	endSectors, err := d.UsableSectorsEnd()
-	c.Assert(err, IsNil)
-	c.Assert(endSectors, Equals, uint64(43))
-
-	sfdiskCmd.ForgetCalls()
-
-	blockDevCmd := testutil.MockCommand(c, "blockdev", `
-echo 513
-`)
-	defer blockDevCmd.Restore()
-
-	// but getting the sector size itself fails
-	_, err = d.SectorSize()
-	c.Assert(err, ErrorMatches, `sector size \(513\) is not a multiple of 512`)
-
-	c.Assert(blockDevCmd.Calls(), DeepEquals, [][]string{
-		{"blockdev", "--getss", "/dev/sda"},
-	})
-
-	c.Assert(sfdiskCmd.Calls(), HasLen, 0)
-}
-
 func (s *diskSuite) TestDiskSizeRelatedMethodsDOS(c *C) {
 	restore := disks.MockUdevPropertiesForDevice(func(typeOpt, dev string) (map[string]string, error) {
 		c.Assert(typeOpt, Equals, "--name")
@@ -1700,8 +1670,8 @@ func (s *diskSuite) TestDiskSizeRelatedMethodsDOS(c *C) {
 	defer sfdiskCmd.Restore()
 
 	blockDevCmd := testutil.MockCommand(c, "blockdev", `
-if [ "$1" = "--getsz" ]; then
-	echo 10000
+if [ "$1" = "--getsize64" ]; then
+	echo 5120000
 elif [ "$1" = "--getss" ]; then
 	echo 512
 else
@@ -1723,7 +1693,7 @@ fi
 	c.Assert(endSectors, Equals, uint64(10000))
 
 	c.Assert(blockDevCmd.Calls(), DeepEquals, [][]string{
-		{"blockdev", "--getsz", "/dev/sda"},
+		{"blockdev", "--getsize64", "/dev/sda"},
 		{"blockdev", "--getss", "/dev/sda"},
 	})
 
@@ -1735,7 +1705,7 @@ fi
 	c.Assert(sz, Equals, uint64(10000*512))
 
 	c.Assert(blockDevCmd.Calls(), DeepEquals, [][]string{
-		{"blockdev", "--getsz", "/dev/sda"},
+		{"blockdev", "--getsize64", "/dev/sda"},
 	})
 
 	// we never used sfdisk
@@ -1762,8 +1732,8 @@ func (s *diskSuite) TestDiskSizeRelatedMethodsDOS4096SectorSize(c *C) {
 	defer sfdiskCmd.Restore()
 
 	blockDevCmd := testutil.MockCommand(c, "blockdev", `
-if [ "$1" = "--getsz" ]; then
-	echo 10000
+if [ "$1" = "--getsize64" ]; then
+	echo 5120000
 	elif [ "$1" = "--getss" ]; then
 	echo 4096
 else
@@ -1783,7 +1753,7 @@ fi
 	c.Assert(endSectors, Equals, uint64(10000*512/4096))
 
 	c.Assert(blockDevCmd.Calls(), DeepEquals, [][]string{
-		{"blockdev", "--getsz", "/dev/sda"},
+		{"blockdev", "--getsize64", "/dev/sda"},
 		{"blockdev", "--getss", "/dev/sda"},
 	})
 
@@ -1795,7 +1765,7 @@ fi
 	c.Assert(sz, Equals, uint64(10000*512))
 
 	c.Assert(blockDevCmd.Calls(), DeepEquals, [][]string{
-		{"blockdev", "--getsz", "/dev/sda"},
+		{"blockdev", "--getsize64", "/dev/sda"},
 	})
 
 	// we never used sfdisk
@@ -1810,7 +1780,7 @@ func (s *diskSuite) TestAllPhysicalDisks(c *C) {
 	c.Assert(err, IsNil)
 	devsToCreate := []string{"sda", "loop1", "loop2", "sdb", "nvme0n1", "mmcblk0"}
 	for _, dev := range devsToCreate {
-		err := ioutil.WriteFile(filepath.Join(blockDir, dev), nil, 0644)
+		err := os.WriteFile(filepath.Join(blockDir, dev), nil, 0644)
 		c.Assert(err, IsNil)
 	}
 
@@ -1878,3 +1848,252 @@ func (s *diskSuite) TestAllPhysicalDisks(c *C) {
 	c.Assert(d[2].KernelDeviceNode(), Equals, "/dev/sda")
 	c.Assert(d[3].KernelDeviceNode(), Equals, "/dev/sdb")
 }
+
+func (s *diskSuite) TestDMCryptUUIDFromMountPointErrs(c *C) {
+	restore := osutil.MockMountInfo(``)
+	defer restore()
+
+	_, err := disks.DMCryptUUIDFromMountPoint("/run/mnt/blah")
+	c.Assert(err, ErrorMatches, `cannot find mountpoint "/run/mnt/blah"`)
+	c.Assert(errors.Is(err, disks.ErrMountPointNotFound), Equals, true)
+
+	restore = osutil.MockMountInfo(`130 30 42:1 / /run/mnt/point rw,relatime shared:54 - ext4 /dev/vda4 rw
+`)
+	defer restore()
+
+	restore = disks.MockUdevPropertiesForDevice(func(typeOpt, dev string) (map[string]string, error) {
+		c.Assert(typeOpt, Equals, "--name")
+		c.Assert(dev, Equals, "/dev/vda4")
+		return map[string]string{
+			"DEVNAME": "vda4",
+			"prop":    "hello",
+		}, nil
+	})
+	defer restore()
+
+	_, err = disks.DMCryptUUIDFromMountPoint("/run/mnt/point")
+	c.Assert(err, ErrorMatches, "device has no DM_UUID")
+	c.Assert(err, Equals, disks.ErrNoDmUUID)
+
+	restore = disks.MockUdevPropertiesForDevice(func(typeOpt, dev string) (map[string]string, error) {
+		c.Assert(typeOpt, Equals, "--name")
+		c.Assert(dev, Equals, "/dev/vda4")
+		return map[string]string{
+			"DEVNAME": "vda4",
+			"prop":    "hello",
+			"DM_UUID": "garbage",
+		}, nil
+	})
+	defer restore()
+
+	_, err = disks.DMCryptUUIDFromMountPoint("/run/mnt/point")
+	c.Assert(err, ErrorMatches, "value of DM_UUID is not recognized")
+}
+
+func (s *diskSuite) TestDMCryptUUIDFromMountPoint(c *C) {
+	restore := osutil.MockMountInfo(`130 30 42:1 / /run/mnt/point rw,relatime shared:54 - ext4 /dev/mapper/something rw
+`)
+	defer restore()
+	restore = disks.MockUdevPropertiesForDevice(func(typeOpt, dev string) (map[string]string, error) {
+		c.Assert(typeOpt, Equals, "--name")
+		switch dev {
+		case "/dev/mapper/something":
+			return map[string]string{
+				"DM_UUID": "CRYPT-LUKS2-5a522809c87e4dfa81a88dc5667d1304-something",
+			}, nil
+		default:
+			c.Errorf("unexpected udev device properties requested: %s", dev)
+			return nil, fmt.Errorf("unexpected udev device: %s", dev)
+		}
+	})
+	defer restore()
+
+	uuid, err := disks.DMCryptUUIDFromMountPoint("/run/mnt/point")
+	c.Assert(err, IsNil)
+	c.Assert(uuid, Equals, "5a522809-c87e-4dfa-81a8-8dc5667d1304")
+}
+
+func (s *diskSuite) TestDMCryptUUIDFromMountPointFallback(c *C) {
+	restore := osutil.MockMountInfo(`130 30 42:1 / /run/mnt/point rw,relatime shared:54 - ext4 /dev/mapper/something rw
+`)
+	defer restore()
+	restore = disks.MockUdevPropertiesForDevice(func(typeOpt, dev string) (map[string]string, error) {
+		c.Assert(typeOpt, Equals, "--name")
+		switch dev {
+		case "/dev/mapper/something":
+			return map[string]string{
+				"DEVPATH": "/devices/virtual/mydevice",
+			}, nil
+		default:
+			c.Errorf("unexpected udev device properties requested: %s", dev)
+			return nil, fmt.Errorf("unexpected udev device: %s", dev)
+		}
+	})
+	defer restore()
+
+	err := os.MkdirAll(filepath.Join(dirs.SysfsDir, "devices/virtual/mydevice", "dm"), 0755)
+	c.Assert(err, IsNil)
+	b := []byte("CRYPT-LUKS2-5a522809c87e4dfa81a88dc5667d1304-something\n")
+	err = os.WriteFile(filepath.Join(dirs.SysfsDir, "devices/virtual/mydevice", "dm/uuid"), b, 0644)
+	c.Assert(err, IsNil)
+
+	uuid, err := disks.DMCryptUUIDFromMountPoint("/run/mnt/point")
+	c.Assert(err, IsNil)
+	c.Assert(uuid, Equals, "5a522809-c87e-4dfa-81a8-8dc5667d1304")
+}
+
+func (s *diskSuite) TestFilesystemTypeForPartition(c *C) {
+	restore := disks.MockUdevPropertiesForDevice(func(typeOpt, dev string) (map[string]string, error) {
+		c.Assert(typeOpt, Equals, "--name")
+		switch dev {
+		case "/dev/vda4":
+			return map[string]string{
+				"ID_FS_TYPE": "vfat",
+			}, nil
+		case "/dev/no-fs":
+			return map[string]string{}, nil
+		case "/dev/mock-failure":
+			return nil, fmt.Errorf("mock failure")
+		default:
+			c.Errorf("unexpected udev device properties requested: %s", dev)
+			return nil, fmt.Errorf("unexpected udev device: %s", dev)
+		}
+	})
+	defer restore()
+
+	fs, err := disks.FilesystemTypeForPartition("/dev/vda4")
+	c.Assert(err, IsNil)
+	c.Check(fs, Equals, "vfat")
+
+	fs, err = disks.FilesystemTypeForPartition("/dev/no-fs")
+	c.Assert(err, IsNil)
+	c.Check(fs, Equals, "")
+
+	fs, err = disks.FilesystemTypeForPartition("/dev/mock-failure")
+	c.Assert(err.Error(), Equals, "mock failure")
+	c.Check(fs, Equals, "")
+}
+
+func (s *diskSuite) TestFindMatchingPartitionWithFsLabel(c *C) {
+	// mock disk
+	restore := disks.MockDeviceNameToDiskMapping(map[string]*disks.MockDiskMapping{
+		"/dev/vda": gadgettest.VMSystemVolumeDiskMappingSeedFsLabelCaps,
+	})
+	defer restore()
+
+	d, err := disks.DiskFromDeviceName("/dev/vda")
+	c.Assert(err, IsNil)
+
+	// seed partition is vfat, capitals are ignored when searching
+	for _, searchLabel := range []string{"ubuntu-seed", "UBUNTU-SEED", "ubuntu-SEED"} {
+		p, err := d.FindMatchingPartitionWithFsLabel(searchLabel)
+		c.Assert(err, IsNil)
+		c.Check(p.KernelDeviceNode, Equals, "/dev/vda1")
+		c.Check(p.FilesystemLabel, Equals, "UBUNTU-SEED")
+	}
+
+	// boot partition is not vfat, case-sensitive search
+	for _, searchLabel := range []string{"ubuntu-boot", "UBUNTU-BOOT", "ubuntu-BOOT"} {
+		p, err := d.FindMatchingPartitionWithFsLabel(searchLabel)
+		if searchLabel == "ubuntu-boot" {
+			c.Assert(err, IsNil)
+			c.Check(p.KernelDeviceNode, Equals, "/dev/vda2")
+			c.Check(p.FilesystemLabel, Equals, "ubuntu-boot")
+		} else {
+			c.Assert(err.Error(), Equals, fmt.Sprintf("filesystem label %q not found", searchLabel))
+			c.Check(p, Equals, disks.Partition{})
+		}
+	}
+}
+
+func (s *diskSuite) TestMockDisksChecking(c *C) {
+	f := func() {
+		disks.MockDeviceNameToDiskMapping(map[string]*disks.MockDiskMapping{
+			"/dev/vda": {
+				Structure: []disks.Partition{
+					{KernelDeviceNode: "/dev/vda1"},
+					{KernelDeviceNode: "/dev/vda1"},
+				},
+			},
+		})
+	}
+	c.Check(f, Panics, "mock error: duplicated kernel device nodes for partitions in disk mapping")
+}
+
+func (s *diskSuite) TestFilesystemUUID(c *C) {
+	restore := disks.MockUdevPropertiesForDevice(func(typeOpt, dev string) (map[string]string, error) {
+		c.Assert(typeOpt, Equals, "--name")
+		switch dev {
+		case "/dev/vda4":
+			return map[string]string{
+				"ID_FS_UUID": "the-expected-uuid",
+			}, nil
+		case "/dev/vda5":
+			return map[string]string{}, nil
+		case "/dev/vda6":
+			return nil, fmt.Errorf("some error")
+		default:
+			c.Errorf("unexpected udev device properties requested: %s", dev)
+			return nil, fmt.Errorf("unexpected udev device: %s", dev)
+		}
+	})
+	defer restore()
+
+	uuid, err := disks.FilesystemUUID("/dev/vda4")
+	c.Assert(err, IsNil)
+	c.Check(uuid, Equals, "the-expected-uuid")
+
+	_, err = disks.FilesystemUUID("/dev/vda5")
+	c.Check(err, ErrorMatches, `cannot get required udev ID_FS_UUID property`)
+
+	_, err = disks.FilesystemUUID("/dev/vda6")
+	c.Check(err, ErrorMatches, `cannot process udev properties: some error`)
+}
+
+func (s *diskSuite) TestDevlinks(c *C) {
+	restore := disks.MockUdevPropertiesForDevice(func(typeOpt, dev string) (map[string]string, error) {
+		c.Assert(typeOpt, Equals, "--name")
+		switch dev {
+		case "/dev/some/main/link":
+			return map[string]string{
+				"DEVLINKS": "/dev/some/other/link /dev/yet/another/link",
+			}, nil
+		case "/dev/no/links":
+			return map[string]string{}, nil
+		case "/dev/some/error":
+			return nil, fmt.Errorf("some error")
+		default:
+			c.Errorf("unexpected udev device properties requested: %s", dev)
+			return nil, fmt.Errorf("unexpected udev device: %s", dev)
+		}
+	})
+	defer restore()
+
+	links, err := disks.Devlinks("/dev/some/main/link")
+	c.Assert(err, IsNil)
+	c.Check(links, DeepEquals, []string{"/dev/some/other/link", "/dev/yet/another/link"})
+
+	_, err = disks.Devlinks("/dev/no/links")
+	c.Check(err, ErrorMatches, `cannot get required udev DEVLINKS property`)
+
+	_, err = disks.Devlinks("/dev/some/error")
+	c.Check(err, ErrorMatches, `cannot process udev properties: some error`)
+}
+
+func (s *diskSuite) TestFindMatchingPartitionWithPartUUID(c *C) {
+	restore := disks.MockDeviceNameToDiskMapping(map[string]*disks.MockDiskMapping{
+		"/dev/vda": gadgettest.VMSystemVolumeDiskMappingSeedFsLabelCaps,
+	})
+	defer restore()
+
+	d, err := disks.DiskFromDeviceName("/dev/vda")
+	c.Assert(err, IsNil)
+
+	p, err := d.FindMatchingPartitionWithPartUUID("ade3ba65-7831-fd40-bbe2-e01c9774ed5b")
+	c.Assert(err, IsNil)
+	c.Check(p.KernelDeviceNode, Equals, "/dev/vda2")
+	c.Check(p.PartitionUUID, Equals, "ade3ba65-7831-fd40-bbe2-e01c9774ed5b")
+
+	_, err = d.FindMatchingPartitionWithPartUUID("fe1ec853-15b1-4c72-a207-6a9b185dcbbb")
+	c.Assert(err, ErrorMatches, "partition uuid \"fe1ec853-15b1-4c72-a207-6a9b185dcbbb\" not found")
+}
