@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2017-2022 Canonical Ltd
+ * Copyright (C) 2017-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -20,8 +20,18 @@
 package configcore
 
 import (
+	"time"
+
+	"github.com/snapcore/snapd/boot"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/sys"
+	"github.com/snapcore/snapd/osutil/user"
+	"github.com/snapcore/snapd/overlord/hookstate"
+	"github.com/snapcore/snapd/overlord/restart"
+	"github.com/snapcore/snapd/overlord/servicestate"
+	"github.com/snapcore/snapd/overlord/state"
+	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/sysconfig"
 	"github.com/snapcore/snapd/testutil"
 )
@@ -34,6 +44,8 @@ var (
 	AddFSOnlyHandler     = addFSOnlyHandler
 	FilesystemOnlyApply  = filesystemOnlyApply
 	UpdateHomedirsConfig = updateHomedirsConfig
+
+	DoExperimentalApparmorPromptingDaemonRestart = doExperimentalApparmorPromptingDaemonRestart
 )
 
 type PlainCoreConfig = plainCoreConfig
@@ -47,47 +59,51 @@ func FilesystemOnlyRun(dev sysconfig.Device, cfg ConfGetter) error {
 }
 
 func MockFindGid(f func(string) (uint64, error)) func() {
-	old := osutilFindGid
-	osutilFindGid = f
-	return func() {
-		osutilFindGid = old
-	}
+	return testutil.Mock(&osutilFindGid, f)
 }
 
 func MockChownPath(f func(string, sys.UserID, sys.GroupID) error) func() {
-	old := sysChownPath
-	sysChownPath = f
-	return func() {
-		sysChownPath = old
-	}
+	return testutil.Mock(&sysChownPath, f)
 }
 
 func MockEnsureFileState(f func(string, osutil.FileState) error) func() {
-	r := testutil.Backup(&osutilEnsureFileState)
-	osutilEnsureFileState = f
-	return r
+	return testutil.Mock(&osutilEnsureFileState, f)
 }
 
 func MockDirExists(f func(string) (bool, bool, error)) func() {
-	r := testutil.Backup(&osutilDirExists)
-	osutilDirExists = f
-	return r
+	return testutil.Mock(&osutilDirExists, f)
 }
 
 func MockApparmorUpdateHomedirsTunable(f func([]string) error) func() {
-	r := testutil.Backup(&apparmorUpdateHomedirsTunable)
-	apparmorUpdateHomedirsTunable = f
-	return r
+	return testutil.Mock(&apparmorUpdateHomedirsTunable, f)
 }
 
 func MockApparmorSetupSnapConfineSnippets(f func() (bool, error)) func() {
-	r := testutil.Backup(&apparmorSetupSnapConfineSnippets)
-	apparmorSetupSnapConfineSnippets = f
-	return r
+	return testutil.Mock(&apparmorSetupSnapConfineSnippets, f)
 }
 
 func MockApparmorReloadAllSnapProfiles(f func() error) func() {
-	r := testutil.Backup(&apparmorReloadAllSnapProfiles)
-	apparmorReloadAllSnapProfiles = f
-	return r
+	return testutil.Mock(&apparmorReloadAllSnapProfiles, f)
+}
+
+func MockLoggerSimpleSetup(f func(opts *logger.LoggerOptions)) func() {
+	return testutil.Mock(&loggerSimpleSetup, f)
+}
+
+func MockRestartRequest(f func(st *state.State, t restart.RestartType, rebootInfo *boot.RebootInfo)) func() {
+	return testutil.Mock(&restartRequest, f)
+}
+
+func MockServicestateControl(f func(st *state.State, appInfos []*snap.AppInfo, inst *servicestate.Instruction, cu *user.User, flags *servicestate.Flags, context *hookstate.Context) ([]*state.TaskSet, error)) func() {
+	return testutil.Mock(&servicestateControl, f)
+}
+
+func MockServicestateChangeTimeout(v time.Duration) func() {
+	return testutil.Mock(&serviceStartChangeTimeout, v)
+}
+
+func MockEnvPath(newEnvPath string) func() {
+	oldEnvPath := envFilePath
+	envFilePath = newEnvPath
+	return func() { envFilePath = oldEnvPath }
 }
