@@ -27,7 +27,7 @@ import (
 	"path/filepath"
 	"time"
 
-	"github.com/godbus/dbus"
+	"github.com/godbus/dbus/v5"
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/dbusutil"
@@ -126,7 +126,7 @@ func (s *netplanSuite) TestNetplanGetFromDBusNoSuchService(c *C) {
 	// Note that we do not export any netplan dbus api here
 
 	tr := config.NewTransaction(s.state)
-	netplanCfg := make(map[string]interface{})
+	netplanCfg := make(map[string]any)
 	err := tr.Get("core", "system.network.netplan", &netplanCfg)
 	c.Assert(err, ErrorMatches, `snap "core" has no "system.network.netplan" configuration option`)
 }
@@ -157,7 +157,7 @@ func (s *netplanSuite) TestNetplanGetNoSupportOnClassic(c *C) {
 	s.backend.ExportApiV2()
 
 	tr := config.NewTransaction(s.state)
-	netplanCfg := make(map[string]interface{})
+	netplanCfg := make(map[string]any)
 	err := tr.Get("core", "system.network.netplan", &netplanCfg)
 	c.Assert(err, ErrorMatches, `snap "core" has no "system.network.netplan" configuration option`)
 }
@@ -172,28 +172,32 @@ func (s *netplanSuite) TestNetplanGetFromDBusHappy(c *C) {
 	tr := config.NewTransaction(s.state)
 
 	// full doc
-	netplanCfg := make(map[string]interface{})
+	netplanCfg := make(map[string]any)
 	err := tr.Get("core", "system.network.netplan", &netplanCfg)
 	c.Assert(err, IsNil)
-	c.Check(netplanCfg, DeepEquals, map[string]interface{}{
-		"network": map[string]interface{}{
+	c.Check(netplanCfg, DeepEquals, map[string]any{
+		"network": map[string]any{
 			"renderer": "NetworkManager",
 			"version":  json.Number("2"),
 		},
 	})
-	// the config snapshot is discarded after it was read
-	c.Check(s.backend.ConfigApiCancelCalls, Equals, 1)
+	s.backend.WithLocked(func() {
+		// the config snapshot is discarded after it was read
+		c.Check(s.backend.ConfigApiCancelCalls, Equals, 1)
+	})
 
 	// only the "network" subset
-	netplanCfg = make(map[string]interface{})
+	netplanCfg = make(map[string]any)
 	err = tr.Get("core", "system.network.netplan.network", &netplanCfg)
 	c.Assert(err, IsNil)
-	c.Check(netplanCfg, DeepEquals, map[string]interface{}{
+	c.Check(netplanCfg, DeepEquals, map[string]any{
 		"renderer": "NetworkManager",
 		"version":  json.Number("2"),
 	})
-	// the config snapshot is discarded after it was read
-	c.Check(s.backend.ConfigApiCancelCalls, Equals, 2)
+	s.backend.WithLocked(func() {
+		// the config snapshot is discarded after it was read
+		c.Check(s.backend.ConfigApiCancelCalls, Equals, 2)
+	})
 
 	// only the "network.version" subset
 	var ver json.Number
@@ -201,11 +205,13 @@ func (s *netplanSuite) TestNetplanGetFromDBusHappy(c *C) {
 	c.Assert(err, IsNil)
 	c.Check(ver, Equals, json.Number("2"))
 
-	// the config snapshot is discarded after it was read
-	c.Check(s.backend.ConfigApiCancelCalls, Equals, 3)
-	// but nothing was applied
-	c.Check(s.backend.ConfigApiTryCalls, Equals, 0)
-	c.Check(s.backend.ConfigApiApplyCalls, Equals, 0)
+	s.backend.WithLocked(func() {
+		// the config snapshot is discarded after it was read
+		c.Check(s.backend.ConfigApiCancelCalls, Equals, 3)
+		// but nothing was applied
+		c.Check(s.backend.ConfigApiTryCalls, Equals, 0)
+		c.Check(s.backend.ConfigApiApplyCalls, Equals, 0)
+	})
 }
 
 func (s *netplanSuite) TestNetplanGetFromDBusCancelFails(c *C) {
@@ -215,16 +221,16 @@ func (s *netplanSuite) TestNetplanGetFromDBusCancelFails(c *C) {
 	logbuf, restore := logger.MockLogger()
 	defer restore()
 
+	s.backend.ConfigApiCancelRet = false
 	// export the V2 api, things work with that
 	s.backend.ExportApiV2()
-	s.backend.ConfigApiCancelRet = false
 
 	tr := config.NewTransaction(s.state)
-	netplanCfg := make(map[string]interface{})
+	netplanCfg := make(map[string]any)
 	err := tr.Get("core", "system.network.netplan", &netplanCfg)
 	c.Assert(err, IsNil)
-	c.Check(netplanCfg, DeepEquals, map[string]interface{}{
-		"network": map[string]interface{}{
+	c.Check(netplanCfg, DeepEquals, map[string]any{
+		"network": map[string]any{
 			"renderer": "NetworkManager",
 			"version":  json.Number("2"),
 		},
@@ -239,16 +245,16 @@ func (s *netplanSuite) TestNetplanGetFromDBusCancelErr(c *C) {
 	logbuf, restore := logger.MockLogger()
 	defer restore()
 
+	s.backend.ConfigApiCancelErr = dbus.MakeFailedError(fmt.Errorf("netplan failed with some error"))
 	// export the V2 api, things work with that
 	s.backend.ExportApiV2()
-	s.backend.ConfigApiCancelErr = dbus.MakeFailedError(fmt.Errorf("netplan failed with some error"))
 
 	tr := config.NewTransaction(s.state)
-	netplanCfg := make(map[string]interface{})
+	netplanCfg := make(map[string]any)
 	err := tr.Get("core", "system.network.netplan", &netplanCfg)
 	c.Assert(err, IsNil)
-	c.Check(netplanCfg, DeepEquals, map[string]interface{}{
-		"network": map[string]interface{}{
+	c.Check(netplanCfg, DeepEquals, map[string]any{
+		"network": map[string]any{
 			"renderer": "NetworkManager",
 			"version":  json.Number("2"),
 		},
@@ -300,10 +306,11 @@ func (s *netplanSuite) TestNetplanConnectivityCheck(c *C) {
 }
 
 func (s *netplanSuite) TestNetplanWriteConfigSetReturnsFalse(c *C) {
-	s.backend.ExportApiV2()
 	s.backend.ConfigApiSetRet = false
 	s.backend.ConfigApiCancelRet = true
 
+	s.backend.ExportApiV2()
+
 	s.state.Lock()
 	rt := configcore.NewRunTransaction(config.NewTransaction(s.state), nil)
 
@@ -316,9 +323,9 @@ func (s *netplanSuite) TestNetplanWriteConfigSetReturnsFalse(c *C) {
 
 func (s *netplanSuite) TestNetplanWriteConfigSetFailsDBusErr(c *C) {
 	s.backend.ConfigApiCancelRet = true
+	s.backend.ConfigApiSetErr = dbus.MakeFailedError(fmt.Errorf("netplan failed with some error"))
 
 	s.backend.ExportApiV2()
-	s.backend.ConfigApiSetErr = dbus.MakeFailedError(fmt.Errorf("netplan failed with some error"))
 
 	s.state.Lock()
 	rt := configcore.NewRunTransaction(config.NewTransaction(s.state), nil)
@@ -330,11 +337,12 @@ func (s *netplanSuite) TestNetplanWriteConfigSetFailsDBusErr(c *C) {
 }
 
 func (s *netplanSuite) TestNetplanWriteConfigTryReturnsFalse(c *C) {
-	s.backend.ExportApiV2()
 	s.backend.ConfigApiSetRet = true
 	s.backend.ConfigApiTryRet = false
 	s.backend.ConfigApiCancelRet = true
 
+	s.backend.ExportApiV2()
+
 	s.state.Lock()
 	rt := configcore.NewRunTransaction(config.NewTransaction(s.state), nil)
 	s.state.Unlock()
@@ -345,11 +353,12 @@ func (s *netplanSuite) TestNetplanWriteConfigTryReturnsFalse(c *C) {
 }
 
 func (s *netplanSuite) TestNetplanWriteConfigTryFailsDBusErr(c *C) {
-	s.backend.ExportApiV2()
 	s.backend.ConfigApiSetRet = true
 	s.backend.ConfigApiTryErr = dbus.MakeFailedError(fmt.Errorf("netplan failed with some error"))
 	s.backend.ConfigApiCancelRet = true
 
+	s.backend.ExportApiV2()
+
 	s.state.Lock()
 	rt := configcore.NewRunTransaction(config.NewTransaction(s.state), nil)
 	s.state.Unlock()
@@ -372,15 +381,15 @@ func (s *netplanSuite) testNetplanWriteConfigHappy(c *C, seeded bool, expectedOr
 	s.state.Set("seeded", seeded)
 	s.state.Unlock()
 
-	// export the V2 api, things work with that
-	s.backend.ExportApiV2()
-
 	// and everything is fine
 	s.fakestore.status = map[string]bool{"host1": true}
 	s.backend.ConfigApiSetRet = true
 	s.backend.ConfigApiTryRet = true
 	s.backend.ConfigApiApplyRet = true
 
+	// export the V2 api, things work with that
+	s.backend.ExportApiV2()
+
 	s.state.Lock()
 	rt := configcore.NewRunTransaction(config.NewTransaction(s.state), nil)
 	s.state.Unlock()
@@ -390,25 +399,26 @@ func (s *netplanSuite) testNetplanWriteConfigHappy(c *C, seeded bool, expectedOr
 	err := configcore.Run(coreDev, rt)
 	c.Assert(err, IsNil)
 
-	c.Check(s.backend.ConfigApiSetCalls, DeepEquals, []string{
-		fmt.Sprintf(`network=null/%s`, expectedOriginHint),
-		fmt.Sprintf(`network={"ethernets":{"eth0":{"dhcp4":true}},"renderer":"NetworkManager","version":2,"wifi":{"wlan0":{"dhcp4":true}}}/%s`, expectedOriginHint),
+	s.backend.WithLocked(func() {
+		c.Check(s.backend.ConfigApiSetCalls, DeepEquals, []string{
+			fmt.Sprintf(`network=null/%s`, expectedOriginHint),
+			fmt.Sprintf(`network={"ethernets":{"eth0":{"dhcp4":true}},"renderer":"NetworkManager","version":2,"wifi":{"wlan0":{"dhcp4":true}}}/%s`, expectedOriginHint),
+		})
+		c.Check(s.backend.ConfigApiTryCalls, Equals, 1)
+		c.Check(s.backend.ConfigApiApplyCalls, Equals, 1)
 	})
-	c.Check(s.backend.ConfigApiTryCalls, Equals, 1)
-	c.Check(s.backend.ConfigApiApplyCalls, Equals, 1)
 }
 
 func (s *netplanSuite) TestNetplanApplyConfigFails(c *C) {
-	s.backend.ConfigApiCancelRet = true
-
-	// export the V2 api, things work with that
-	s.backend.ExportApiV2()
-
 	s.fakestore.status = map[string]bool{"host1": true}
 	s.backend.ConfigApiSetRet = true
 	s.backend.ConfigApiTryRet = true
+	s.backend.ConfigApiCancelRet = true
 	s.backend.ConfigApiApplyRet = false
 
+	// export the V2 api, things work with that
+	s.backend.ExportApiV2()
+
 	s.state.Lock()
 	rt := configcore.NewRunTransaction(config.NewTransaction(s.state), nil)
 	s.state.Unlock()
@@ -420,16 +430,15 @@ func (s *netplanSuite) TestNetplanApplyConfigFails(c *C) {
 }
 
 func (s *netplanSuite) TestNetplanApplyConfigErr(c *C) {
-	s.backend.ConfigApiCancelRet = true
-
-	// export the V2 api, things work with that
-	s.backend.ExportApiV2()
-
 	s.fakestore.status = map[string]bool{"host1": true}
 	s.backend.ConfigApiSetRet = true
 	s.backend.ConfigApiTryRet = true
+	s.backend.ConfigApiCancelRet = true
 	s.backend.ConfigApiApplyErr = dbus.MakeFailedError(fmt.Errorf("netplan failed with some error"))
 
+	// export the V2 api, things work with that
+	s.backend.ExportApiV2()
+
 	s.state.Lock()
 	rt := configcore.NewRunTransaction(config.NewTransaction(s.state), nil)
 	s.state.Unlock()
@@ -441,9 +450,6 @@ func (s *netplanSuite) TestNetplanApplyConfigErr(c *C) {
 }
 
 func (s *netplanSuite) TestNetplanWriteConfigNoNetworkAfterTry(c *C) {
-	// export the V2 api, things work with that
-	s.backend.ExportApiV2()
-
 	// we have connectivity but it stops
 	s.fakestore.statusSeq = []map[string]bool{
 		{"host1": true},
@@ -459,6 +465,9 @@ func (s *netplanSuite) TestNetplanWriteConfigNoNetworkAfterTry(c *C) {
 	s.backend.ConfigApiApplyRet = true
 	s.backend.ConfigApiCancelRet = true
 
+	// export the V2 api, things work with that
+	s.backend.ExportApiV2()
+
 	s.state.Lock()
 	rt := configcore.NewRunTransaction(config.NewTransaction(s.state), nil)
 	s.state.Unlock()
@@ -467,16 +476,17 @@ func (s *netplanSuite) TestNetplanWriteConfigNoNetworkAfterTry(c *C) {
 	err := configcore.Run(coreDev, rt)
 	c.Assert(err, ErrorMatches, `cannot set netplan config: store no longer reachable`)
 
-	c.Check(s.backend.ConfigApiTryCalls, Equals, 1)
-	// one cancel for the initial "Get()" and one after the Try failed
-	c.Check(s.backend.ConfigApiCancelCalls, Equals, 2)
-	c.Check(s.backend.ConfigApiApplyCalls, Equals, 0)
-	// 1 initial call and 10 retries
-	c.Check(s.fakestore.seq, Equals, 6)
+	s.backend.WithLocked(func() {
+		c.Check(s.backend.ConfigApiTryCalls, Equals, 1)
+		// one cancel for the initial "Get()" and one after the Try failed
+		c.Check(s.backend.ConfigApiCancelCalls, Equals, 2)
+		c.Check(s.backend.ConfigApiApplyCalls, Equals, 0)
+		// 1 initial call and 10 retries
+		c.Check(s.fakestore.seq, Equals, 6)
+	})
 }
 
 func (s *netplanSuite) TestNetplanWriteConfigCancelFails(c *C) {
-	s.backend.ExportApiV2()
 	s.fakestore.statusSeq = []map[string]bool{
 		{"host1": true},
 		// and is retried 5 times
@@ -490,6 +500,8 @@ func (s *netplanSuite) TestNetplanWriteConfigCancelFails(c *C) {
 	s.backend.ConfigApiTryRet = true
 	s.backend.ConfigApiCancelRet = false
 
+	s.backend.ExportApiV2()
+
 	s.state.Lock()
 	rt := configcore.NewRunTransaction(config.NewTransaction(s.state), nil)
 	s.state.Unlock()
@@ -500,9 +512,6 @@ func (s *netplanSuite) TestNetplanWriteConfigCancelFails(c *C) {
 }
 
 func (s *netplanSuite) TestNetplanWriteConfigCancelFailsWithDbusErr(c *C) {
-	s.backend.ConfigApiCancelRet = true
-
-	s.backend.ExportApiV2()
 	s.fakestore.statusSeq = []map[string]bool{
 		{"host1": true},
 		// and is retried 5 times
@@ -512,10 +521,13 @@ func (s *netplanSuite) TestNetplanWriteConfigCancelFailsWithDbusErr(c *C) {
 		{"host1": false},
 		{"host1": false},
 	}
+	s.backend.ConfigApiCancelRet = true
 	s.backend.ConfigApiSetRet = true
 	s.backend.ConfigApiTryRet = true
 	s.backend.ConfigApiCancelErr = dbus.MakeFailedError(fmt.Errorf("netplan failed with some error"))
 
+	s.backend.ExportApiV2()
+
 	s.state.Lock()
 	rt := configcore.NewRunTransaction(config.NewTransaction(s.state), nil)
 	s.state.Unlock()
@@ -535,7 +547,6 @@ network:
     br54:
       dhcp4: true
       dhcp6: true`
-	s.backend.ExportApiV2()
 
 	// we have connectivity
 	s.fakestore.status = map[string]bool{"host1": true}
@@ -543,6 +554,8 @@ network:
 	s.backend.ConfigApiTryRet = true
 	s.backend.ConfigApiApplyRet = true
 
+	s.backend.ExportApiV2()
+
 	// we cannot use mockConf because we need the external config
 	// integration from the config.Transaction
 	s.state.Lock()
@@ -553,9 +566,11 @@ network:
 	err := configcore.Run(coreDev, rt)
 	c.Assert(err, IsNil)
 
-	c.Check(s.backend.ConfigApiSetCalls, DeepEquals, []string{
-		`network=null/90-snapd-config`,
-		`network={"bridges":{"br54":{"dhcp6":true}},"renderer":"networkd","version":2}/90-snapd-config`,
+	s.backend.WithLocked(func() {
+		c.Check(s.backend.ConfigApiSetCalls, DeepEquals, []string{
+			`network=null/90-snapd-config`,
+			`network={"bridges":{"br54":{"dhcp6":true}},"renderer":"networkd","version":2}/90-snapd-config`,
+		})
 	})
 }
 
@@ -565,7 +580,7 @@ func (s *netplanSuite) TestNetplanNoApplyOnClassic(c *C) {
 
 	err := configcore.Run(coreDev, &mockConf{
 		state: s.state,
-		changes: map[string]interface{}{
+		changes: map[string]any{
 			"system.network.netplan.network.renderer": "networkd",
 		},
 	})
@@ -573,9 +588,9 @@ func (s *netplanSuite) TestNetplanNoApplyOnClassic(c *C) {
 
 	err = configcore.Run(coreDev, &mockConf{
 		state: s.state,
-		changes: map[string]interface{}{
-			"system.network.netplan": map[string]interface{}{
-				"network": map[string]interface{}{
+		changes: map[string]any{
+			"system.network.netplan": map[string]any{
+				"network": map[string]any{
 					"version": 2,
 				},
 			},
