@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2023 Canonical Ltd
+ * Copyright (C) 2016-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -26,6 +26,7 @@ import (
 	"os"
 	"path/filepath"
 	"regexp"
+	"strconv"
 	"strings"
 	"time"
 
@@ -50,6 +51,7 @@ import (
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/overlord/storecontext"
+	"github.com/snapcore/snapd/overlord/swfeats"
 	"github.com/snapcore/snapd/progress"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/secboot"
@@ -72,10 +74,49 @@ var (
 	secbootMarkSuccessful = secboot.MarkSuccessful
 )
 
+var (
+	becomeOperationalChangeKind = swfeats.RegisterChangeKind("become-operational")
+	seedChangeKind              = swfeats.RegisterChangeKind("seed")
+	installSystemChangeKind     = swfeats.RegisterChangeKind("install-system")
+	factoryResetChangeKind      = swfeats.RegisterChangeKind("factory-reset")
+)
+
+func init() {
+	swfeats.RegisterEnsure("DeviceManager", "ensureOperational")
+	swfeats.RegisterEnsure("DeviceManager", "ensureSeeded")
+	swfeats.RegisterEnsure("DeviceManager", "ensureAutoImportAssertions")
+	swfeats.RegisterEnsure("DeviceManager", "ensureSerialBoundSystemUserAssertionsProcessed")
+	swfeats.RegisterEnsure("DeviceManager", "ensureBootOk")
+	swfeats.RegisterEnsure("DeviceManager", "ensureCloudInitRestricted")
+	swfeats.RegisterEnsure("DeviceManager", "ensureInstalled")
+	swfeats.RegisterEnsure("DeviceManager", "ensureFactoryReset")
+	swfeats.RegisterEnsure("DeviceManager", "ensureSeedInConfig")
+	swfeats.RegisterEnsure("DeviceManager", "ensureSeedInConfig")
+	swfeats.RegisterEnsure("DeviceManager", "ensureTriedRecoverySystem")
+	swfeats.RegisterEnsure("DeviceManager", "ensurePostFactoryReset")
+	swfeats.RegisterEnsure("DeviceManager", "ensureExpiredUsersRemoved")
+}
+
 // EarlyConfig is a hook set by configstate that can process early configuration
 // during managers' startup.
 var EarlyConfig func(st *state.State, preloadGadget func() (sysconfig.Device, *gadget.Info, error)) error
 
+// ErrNoDeviceIdentityYet is returned when the device doesn't have a serial assertion.
+// It's a special case of ErrNoState.
+var ErrNoDeviceIdentityYet = &noDeviceIdentityYetError{}
+
+// noDeviceIdentityYetError is returned when the device doesn't have a serial assertion.
+type noDeviceIdentityYetError struct{}
+
+func (e *noDeviceIdentityYetError) Error() string {
+	return "device has no identity yet"
+}
+
+func (e *noDeviceIdentityYetError) Is(err error) bool {
+	_, ok := err.(*noDeviceIdentityYetError)
+	return ok || errors.Is(err, state.ErrNoState)
+}
+
 // DeviceManager is responsible for managing the device identity and device
 // policies.
 type DeviceManager struct {
@@ -100,7 +141,10 @@ type DeviceManager struct {
 
 	seedTimings *timings.Timings
 	// this is used during early phases until seeding is under way
-	earlyDeviceSeed     seed.Seed
+	earlyDeviceSeed seed.Seed
+	// these are details about the chosen seed we will be seeding from,
+	// set and valid only before seeding has happened. Should not be
+	// used by tasks that are not explicitly happening prior to system being marked seeded.
 	seedLabel, seedMode string
 	seedChosen          bool
 
@@ -144,13 +188,14 @@ func Manager(s *state.State, hookManager *hookstate.HookManager, runner *state.T
 	m.populateStateFromSeed = m.populateStateFromSeedImpl
 
 	if !m.preseed {
-		modeenv, err := maybeReadModeenv()
+		mode, explicit, err := boot.SystemMode("")
 		if err != nil {
 			return nil, err
 		}
-		if modeenv != nil {
-			logger.Debugf("modeenv for model %q found", modeenv.Model)
-			m.sysMode = modeenv.Mode
+
+		if explicit {
+			logger.Debugf("explicitly set system mode")
+			m.sysMode = mode
 		}
 	} else {
 		// cache system label for preseeding of core20; note, this will fail on
@@ -179,6 +224,8 @@ func Manager(s *state.State, hookManager *hookstate.HookManager, runner *state.T
 
 	runner.AddHandler("generate-device-key", m.doGenerateDeviceKey, nil)
 	runner.AddHandler("request-serial", m.doRequestSerial, nil)
+	// Mark-preseeded touches and records the system-key, ensure that it does
+	// not run in parallel with other tasks touching the system-key
 	runner.AddHandler("mark-preseeded", m.doMarkPreseeded, nil)
 	runner.AddHandler("mark-seeded", m.doMarkSeeded, nil)
 	runner.AddHandler("setup-ubuntu-save", m.doSetupUbuntuSave, nil)
@@ -217,8 +264,7 @@ func Manager(s *state.State, hookManager *hookstate.HookManager, runner *state.T
 	runner.AddBlocked(gadgetUpdateBlocked)
 
 	// wire FDE kernel hook support into boot
-	boot.HasFDESetupHook = m.hasFDESetupHook
-	boot.RunFDESetupHook = m.runFDESetupHook
+	boot.HookKeyProtectorFactory = m.hookKeyProtectorFactory
 	hookManager.Register(regexp.MustCompile("^fde-setup$"), newFdeSetupHandler)
 
 	return m, nil
@@ -248,24 +294,19 @@ func newBasicHookStateHandler(context *hookstate.Context) hookstate.Handler {
 	return genericHook{}
 }
 
-func maybeReadModeenv() (*boot.Modeenv, error) {
-	modeenv, err := boot.ReadModeenv("")
-	if err != nil && !os.IsNotExist(err) {
-		return nil, fmt.Errorf("cannot read modeenv: %v", err)
-	}
-	return modeenv, nil
-}
-
 // ReloadModeenv is only useful for integration testing
 func (m *DeviceManager) ReloadModeenv() error {
 	osutil.MustBeTestBinary("ReloadModeenv can only be called from tests")
-	modeenv, err := maybeReadModeenv()
+
+	mode, explicit, err := boot.SystemMode("")
 	if err != nil {
 		return err
 	}
-	if modeenv != nil {
-		m.sysMode = modeenv.Mode
+
+	if explicit {
+		m.sysMode = mode
 	}
+
 	return nil
 }
 
@@ -559,6 +600,8 @@ func (m *DeviceManager) ensureOperational() error {
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureOperational")
+
 	perfTimings := timings.New(map[string]string{"ensure": "become-operational"})
 
 	// conditions to trigger device registration
@@ -717,7 +760,7 @@ func (m *DeviceManager) ensureOperational() error {
 		tasks = append(tasks, requestSerial)
 	}
 
-	chg := m.state.NewChange("become-operational", i18n.G("Initialize device"))
+	chg := m.state.NewChange(becomeOperationalChangeKind, i18n.G("Initialize device"))
 	chg.AddAll(state.NewTaskSet(tasks...))
 
 	state.TagTimingsWithChange(perfTimings, chg)
@@ -846,7 +889,7 @@ func (m *DeviceManager) seedLabelAndMode() (seedLabel, seedMode string, err erro
 			seedLabel = m.systemForPreseeding()
 		}
 	} else {
-		modeenv, err := maybeReadModeenv()
+		modeenv, err := boot.MaybeReadModeenv()
 		if err != nil {
 			return "", "", err
 		}
@@ -997,7 +1040,9 @@ func (m *DeviceManager) ensureSeeded() error {
 		return nil
 	}
 
-	chg := m.state.NewChange("seed", "Initialize system state")
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureSeeded")
+
+	chg := m.state.NewChange(seedChangeKind, "Initialize system state")
 	for _, ts := range tsAll {
 		chg.AddAll(ts)
 	}
@@ -1050,6 +1095,8 @@ func (m *DeviceManager) ensureAutoImportAssertions() error {
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureAutoImportAssertions")
+
 	commitTo := func(batch *asserts.Batch) error {
 		return assertstate.AddBatch(m.state, batch, nil)
 	}
@@ -1066,6 +1113,66 @@ func (m *DeviceManager) ensureAutoImportAssertions() error {
 	return nil
 }
 
+func (m *DeviceManager) ensureSerialBoundSystemUserAssertionsProcessed() error {
+	// in situations where a device serial can be anticipated, it is
+	// possible to create a serial-bound system-user assertion beforehand,
+	// this Ensure logic takes care of creating the corresponding user even
+	// if system-user gets presented to the device before the actual serial
+	// assertion is acquired, see the corresponding code setting the
+	// system-user-waiting-on-serial flag in createAllKnownSystemUsers
+	// (users.go).
+	if release.OnClassic {
+		return nil
+	}
+
+	m.state.Lock()
+	defer m.state.Unlock()
+
+	var waitingOnSerial bool
+	err := m.state.Get("system-user-waiting-on-serial", &waitingOnSerial)
+	if err != nil && !errors.Is(err, state.ErrNoState) {
+		return err
+	}
+	if !waitingOnSerial {
+		return nil
+	}
+
+	var seeded bool
+	if err := m.state.Get("seeded", &seeded); err != nil && !errors.Is(err, state.ErrNoState) {
+		return err
+	}
+	if !seeded {
+		return nil
+	}
+
+	// we should always have a model if we are seeded and not on classic
+	model, err := m.Model()
+	if err != nil {
+		return err
+	}
+
+	serial, err := m.Serial()
+	if err != nil {
+		if errors.Is(err, state.ErrNoState) {
+			return nil
+		}
+		return err
+	}
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureSerialBoundSystemUserAssertionsProcessed")
+
+	db := assertstate.DB(m.state)
+
+	const sudoer = true
+	_, err = createAllKnownSystemUsers(m.state, db, model, serial, sudoer)
+	if err != nil {
+		return err
+	}
+
+	m.state.Set("system-user-waiting-on-serial", false)
+
+	return nil
+}
+
 func (m *DeviceManager) ensureBootOk() error {
 	m.state.Lock()
 	defer m.state.Unlock()
@@ -1091,6 +1198,7 @@ func (m *DeviceManager) ensureBootOk() error {
 
 		m.bootOkRan = true
 	}
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureBootOk")
 
 	if !m.bootRevisionsUpdated {
 		if err := snapstate.UpdateBootRevisions(m.state); err != nil {
@@ -1145,6 +1253,7 @@ func (m *DeviceManager) ensureCloudInitRestricted() error {
 	if err != nil {
 		return err
 	}
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureCloudInitRestricted")
 	statusMsg := ""
 
 	switch cloudInitStatus {
@@ -1335,6 +1444,8 @@ func (m *DeviceManager) ensureInstalled() error {
 		return fmt.Errorf("internal error: %v", err)
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureInstalled")
+
 	m.ensureInstalledRan = true
 
 	// Create both setup-run-system and restart-system-to-run-mode tasks as they
@@ -1367,7 +1478,7 @@ func (m *DeviceManager) ensureInstalled() error {
 
 	addTask(restartSystem)
 
-	chg := m.state.NewChange("install-system", i18n.G("Install the system"))
+	chg := m.state.NewChange(installSystemChangeKind, i18n.G("Install the system"))
 	chg.AddAll(state.NewTaskSet(tasks...))
 
 	state.TagTimingsWithChange(perfTimings, chg)
@@ -1400,6 +1511,7 @@ func (m *DeviceManager) ensureFactoryReset() error {
 	if !seeded {
 		return nil
 	}
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureFactoryReset")
 
 	perfTimings := timings.New(map[string]string{"ensure": "factory-reset"})
 
@@ -1445,7 +1557,7 @@ func (m *DeviceManager) ensureFactoryReset() error {
 
 	addTask(restartSystem)
 
-	chg := m.state.NewChange("factory-reset", i18n.G("Perform factory reset"))
+	chg := m.state.NewChange(factoryResetChangeKind, i18n.G("Perform factory reset"))
 	chg.AddAll(state.NewTaskSet(tasks...))
 
 	state.TagTimingsWithChange(perfTimings, chg)
@@ -1518,6 +1630,7 @@ func (m *DeviceManager) ensureSeedInConfig() error {
 			// doMarkSeeded will run "EnsureBefore(0)"
 			return nil
 		}
+		logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureSeedInConfig")
 
 		// Sync seeding with the configuration state. We need to
 		// do this here to ensure that old systems which did not
@@ -1527,6 +1640,8 @@ func (m *DeviceManager) ensureSeedInConfig() error {
 			return err
 		}
 		m.ensureSeedInConfigRan = true
+	} else {
+		logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureSeedInConfig")
 	}
 
 	return nil
@@ -1561,6 +1676,8 @@ func (m *DeviceManager) ensureTriedRecoverySystem() error {
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureTriedRecoverySystem")
+
 	m.state.Lock()
 	defer m.state.Unlock()
 
@@ -1600,8 +1717,6 @@ func (m *DeviceManager) ensureTriedRecoverySystem() error {
 	return nil
 }
 
-var bootMarkFactoryResetComplete = boot.MarkFactoryResetComplete
-
 func (m *DeviceManager) ensurePostFactoryReset() error {
 	m.state.Lock()
 	defer m.state.Unlock()
@@ -1628,6 +1743,8 @@ func (m *DeviceManager) ensurePostFactoryReset() error {
 		return nil
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensurePostFactoryReset")
+
 	m.ensurePostFactoryResetRan = true
 
 	factoryResetMarker := filepath.Join(dirs.SnapDeviceDir, "factory-reset")
@@ -1647,14 +1764,9 @@ func (m *DeviceManager) ensurePostFactoryReset() error {
 		return fmt.Errorf("cannot verify factory reset marker: %v", err)
 	}
 
-	// if encrypted, rotates the fallback keys on disk
-	if err := bootMarkFactoryResetComplete(encrypted); err != nil {
-		return fmt.Errorf("cannot complete factory reset: %v", err)
-	}
-
 	if encrypted {
-		if err := rotateEncryptionKeys(); err != nil {
-			return fmt.Errorf("cannot transition encryption keys: %v", err)
+		if err := rotateSaveKeyAndDeleteOldKeys(boot.InitramfsUbuntuSaveDir); err != nil {
+			return fmt.Errorf("cannot remove old encryption keys: %v", err)
 		}
 	}
 
@@ -1689,6 +1801,8 @@ func (m *DeviceManager) ensureExpiredUsersRemoved() error {
 		return err
 	}
 
+	logger.Trace("ensure", "manager", "DeviceManager", "func", "ensureExpiredUsersRemoved")
+
 	for _, user := range users {
 		if !user.HasExpired() {
 			continue
@@ -1773,6 +1887,10 @@ func (m *DeviceManager) Ensure() error {
 			errs = append(errs, err)
 		}
 
+		if err := m.ensureSerialBoundSystemUserAssertionsProcessed(); err != nil {
+			errs = append(errs, err)
+		}
+
 		if err := m.ensureExpiredUsersRemoved(); err != nil {
 			errs = append(errs, err)
 		}
@@ -1900,6 +2018,32 @@ func (m *DeviceManager) keyPair() (asserts.PrivateKey, error) {
 	return privKey, nil
 }
 
+// SignConfdbControl signs a confdb-control assertion using the device's key as it needs to be attested by the device.
+func (m *DeviceManager) SignConfdbControl(groups []any, revision int) (*asserts.ConfdbControl, error) {
+	serial, err := m.Serial()
+	if err != nil {
+		return nil, fmt.Errorf("cannot sign confdb-control without a serial")
+	}
+
+	privKey, err := m.keyPair()
+	if err != nil {
+		return nil, fmt.Errorf("cannot sign confdb-control without device key")
+	}
+
+	a, err := asserts.SignWithoutAuthority(asserts.ConfdbControlType, map[string]any{
+		"brand-id": serial.BrandID(),
+		"model":    serial.Model(),
+		"serial":   serial.Serial(),
+		"revision": strconv.Itoa(revision),
+		"groups":   groups,
+	}, nil, privKey)
+	if err != nil {
+		return nil, err
+	}
+
+	return a.(*asserts.ConfdbControl), nil
+}
+
 // Registered returns a channel that is closed when the device is known to have been registered.
 func (m *DeviceManager) Registered() <-chan struct{} {
 	return m.reg
@@ -1975,6 +2119,35 @@ func (m *DeviceManager) Serial() (*asserts.Serial, error) {
 	return findSerial(m.state, nil)
 }
 
+// ConfdbControl returns the device's confdb-control assertion.
+func (m *DeviceManager) ConfdbControl() (*asserts.ConfdbControl, error) {
+	serial, err := m.Serial()
+	if err != nil {
+		return nil, ErrNoDeviceIdentityYet
+	}
+
+	db := assertstate.DB(m.state)
+	a, err := db.Find(asserts.ConfdbControlType, map[string]string{
+		"brand-id": serial.BrandID(),
+		"model":    serial.Model(),
+		"serial":   serial.Serial(),
+	})
+	if errors.Is(err, &asserts.NotFoundError{}) {
+		return nil, state.ErrNoState
+	}
+	if err != nil {
+		return nil, err
+	}
+
+	cc := a.(*asserts.ConfdbControl)
+	key := serial.DeviceKey()
+	if key.ID() != cc.SignKeyID() {
+		return nil, errors.New("confdb-control's signing key doesn't match the device key")
+	}
+
+	return cc, nil
+}
+
 type SystemModeInfo struct {
 	Mode              string
 	HasModeenv        bool
@@ -2041,6 +2214,10 @@ type System struct {
 	// DefaultRecoverySystem is true when the system is the default recovery
 	// system.
 	DefaultRecoverySystem bool
+	// OptionalContainers is a set of snaps and components that are optional in
+	// the system's model, but are available to be installed when installing this
+	// system.
+	OptionalContainers OptionalContainers
 }
 
 var defaultSystemActions = []SystemAction{
@@ -2110,15 +2287,79 @@ func (m *DeviceManager) systems() ([]*System, error) {
 	return systems, nil
 }
 
-// SystemAndGadgetAndEncryptionInfo return the system details
-// including the model assertion, gadget details and encryption info
-// for the given system label.
-func (m *DeviceManager) SystemAndGadgetAndEncryptionInfo(wantedSystemLabel string) (*System, *gadget.Info, *install.EncryptionSupportInfo, error) {
+// SystemAndGadgetAndEncryptionInfo resolves the target system by
+// label and returns the system details (including its model assertion), the
+// gadget details and encryption support information.
+//
+// This is a wrapper around systemAndGadgetAndEncryptionInfoWithAction that
+// performs an encryption availability check without any action.
+//
+// Cache semantics:
+//   - With encInfoFromCache set to true: returns cached EncryptionSupportInfo when
+//     available, skipping the expensive availability check.
+//   - Otherwise: computes fresh EncryptionSupportInfo and refreshes the cache on
+//     success.
+//
+// Errors are returned if the system cannot be resolved, gadget info cannot be
+// read/validated, or encryption support evaluation fails.
+func (m *DeviceManager) SystemAndGadgetAndEncryptionInfo(
+	wantedSystemLabel string,
+	encInfoFromCache bool,
+) (*System, *gadget.Info, *install.EncryptionSupportInfo, error) {
+	var checkAction *secboot.PreinstallAction = nil
+	return m.systemAndGadgetAndEncryptionInfoWithAction(wantedSystemLabel, checkAction, encInfoFromCache)
+}
+
+// ApplyActionOnSystemAndGadgetAndEncryptionInfo resolves the target system by
+// label and evaluates encryption support after applying the provided action. It
+// returns the system details (including its model assertion), the gadget
+// details and encryption support information.
+//
+// Action and cache semantics:
+//   - Always computes fresh EncryptionSupportInfo (encInfoFromCache is false).
+//   - Requires EncryptionSupportInfo containing a CheckContext to exist in
+//     cache for the provided label.
+//
+// Errors are returned if the system cannot be resolved, gadget info cannot be
+// read/validated, or encryption support evaluation fails.
+func (m *DeviceManager) ApplyActionOnSystemAndGadgetAndEncryptionInfo(
+	wantedSystemLabel string,
+	checkAction *secboot.PreinstallAction,
+) (*System, *gadget.Info, *install.EncryptionSupportInfo, error) {
+	if checkAction == nil {
+		return nil, nil, nil, errors.New("cannot apply empty action")
+	}
+	const encInfoFromCache = false
+	return m.systemAndGadgetAndEncryptionInfoWithAction(wantedSystemLabel, checkAction, encInfoFromCache)
+}
+
+// systemAndGadgetAndEncryptionInfoWithAction resolves the target system by
+// label and returns the system details (including its model assertion), the
+// gadget details and encryption support information.
+//
+// Action and cache semantics:
+//   - With a checkAction: Computes fresh EncryptionSupportInfo. Requires
+//     EncryptionSupportInfo containing a CheckContext to exist in cache for the
+//     provided label. It is incompatible with encInfoFromCache set to true.
+//   - Without a checkAction and encInfoFromCache set to true: return the cached
+//     EncryptionSupportInfo when available, skipping the expensive availability
+//     check.
+//   - Otherwise: computes fresh EncryptionSupportInfo and refreshes the cache on
+//     success.
+//
+// Errors are returned if the system cannot be resolved, gadget info cannot be
+// read/validated, or encryption support evaluation fails. See
+// encryptionSupportInfoUnlocked for the exact cache/CheckContext behavior.
+func (m *DeviceManager) systemAndGadgetAndEncryptionInfoWithAction(
+	wantedSystemLabel string,
+	checkAction *secboot.PreinstallAction,
+	encInfoFromCache bool,
+) (*System, *gadget.Info, *install.EncryptionSupportInfo, error) {
 	// TODO check that the system is not a classic boot one when the
 	// installer is not anymore.
 
 	// System information
-	systemAndSnaps, err := m.loadSystemAndEssentialSnaps(wantedSystemLabel, []snap.Type{snap.TypeKernel, snap.TypeGadget})
+	systemAndSnaps, err := m.loadSystemAndEssentialSnaps(wantedSystemLabel, []snap.Type{snap.TypeSnapd, snap.TypeKernel, snap.TypeGadget}, seed.AllModes)
 	if err != nil {
 		return nil, nil, nil, err
 	}
@@ -2134,7 +2375,14 @@ func (m *DeviceManager) SystemAndGadgetAndEncryptionInfo(wantedSystemLabel strin
 	}
 
 	// Encryption details
-	encInfo, err := m.encryptionSupportInfo(systemAndSnaps.Model, secboot.TPMProvisionFull, systemAndSnaps.InfosByType[snap.TypeKernel], gadgetInfo)
+	encInfo, err := m.encryptionSupportInfoUnlocked(wantedSystemLabel, install.EncryptionConstraints{
+		Model:         systemAndSnaps.Model,
+		Kernel:        systemAndSnaps.InfosByType[snap.TypeKernel],
+		Gadget:        gadgetInfo,
+		TPMMode:       secboot.TPMProvisionFull,
+		SnapdVersions: systemAndSnaps.SystemSnapdVersions,
+		CheckAction:   checkAction,
+	}, encInfoFromCache)
 	if err != nil {
 		return nil, nil, nil, err
 	}
@@ -2147,14 +2395,16 @@ func (m *DeviceManager) SystemAndGadgetAndEncryptionInfo(wantedSystemLabel strin
 		return nil, nil, nil, fmt.Errorf("cannot validate gadget.yaml: %v", err)
 	}
 
-	return systemAndSnaps.System, gadgetInfo, &encInfo, err
+	return systemAndSnaps.System, gadgetInfo, encInfo, err
 }
 
 type systemAndEssentialSnaps struct {
 	*System
-	Seed            seed.Seed
-	InfosByType     map[snap.Type]*snap.Info
-	SeedSnapsByType map[snap.Type]*seed.Snap
+	Seed                seed.Seed
+	SystemSnapdVersions install.SystemSnapdVersions
+	InfosByType         map[snap.Type]*snap.Info
+	CompsByType         map[snap.Type][]install.ComponentSeedInfo
+	SeedSnapsByType     map[snap.Type]*seed.Snap
 }
 
 // DefaultRecoverySystem returns the default recovery system, if there is one.
@@ -2175,16 +2425,20 @@ func (m *DeviceManager) defaultRecoverySystem() (*DefaultRecoverySystem, error)
 }
 
 // loadSystemAndEssentialSnaps loads information for the given label, which
-// includes system, gadget information, gadget and kernel snaps info,
-// and gadget and kernel seed snap info.
+// includes system, gadget information, gadget and kernel snaps info, and
+// gadget and kernel seed snap info. In some cases we only want the components
+// of the essential snaps for a given mode.
 // TODO: make this method optionally return the system seed, since it might not
 // always be needed, and it is quite large.
-func (m *DeviceManager) loadSystemAndEssentialSnaps(wantedSystemLabel string, types []snap.Type) (*systemAndEssentialSnaps, error) {
+func (m *DeviceManager) loadSystemAndEssentialSnaps(wantedSystemLabel string, types []snap.Type, modeForComps string) (*systemAndEssentialSnaps, error) {
 	// get current system as input for loadSeedAndSystem()
 	systemMode := m.SystemMode(SysAny)
-	m.state.Lock()
-	currentSys, _ := currentSystemForMode(m.state, systemMode)
-	m.state.Unlock()
+	var currentSys *currentSystem
+	func() {
+		m.state.Lock()
+		defer m.state.Unlock()
+		currentSys, _ = currentSystemForMode(m.state, systemMode)
+	}()
 
 	defaultRecoverySystem, err := m.DefaultRecoverySystem()
 	if err != nil && !errors.Is(err, state.ErrNoState) {
@@ -2206,7 +2460,9 @@ func (m *DeviceManager) loadSystemAndEssentialSnaps(wantedSystemLabel string, ty
 	// like "snapd" will be skipped and not part of the EssentialSnaps list
 	//
 	snapInfos := make(map[snap.Type]*snap.Info)
+	compInfos := make(map[snap.Type][]install.ComponentSeedInfo)
 	seedSnaps := make(map[snap.Type]*seed.Snap)
+	systemSnapdVersions := install.SystemSnapdVersions{}
 	for _, seedSnap := range s.EssentialSnaps() {
 		typ := seedSnap.EssentialType
 		if seedSnap.Path == "" {
@@ -2223,18 +2479,59 @@ func (m *DeviceManager) loadSystemAndEssentialSnaps(wantedSystemLabel string, ty
 		if snapInfo.SnapType != typ {
 			return nil, fmt.Errorf("cannot use snap info, expected %s but got %s", typ, snapInfo.SnapType)
 		}
-		seedSnaps[typ] = seedSnap
+		// Read components in the seed too, for the mode we are interested in
+		snapForMode, err := s.ModeSnap(seedSnap.SnapName(), modeForComps)
+		if err != nil {
+			return nil, fmt.Errorf("internal error while retrieving %s for %s mode: %v",
+				seedSnap.SnapName(), modeForComps, err)
+		}
+		var compInfosForType []install.ComponentSeedInfo
+		if len(snapForMode.Components) > 0 {
+			compInfosForType = make([]install.ComponentSeedInfo, 0, len(snapForMode.Components))
+			for _, sc := range snapForMode.Components {
+				seedComp := sc
+				compf, err := snapfile.Open(seedComp.Path)
+				if err != nil {
+					return nil, fmt.Errorf("cannot open snap from %q: %v", snapForMode.Path, err)
+				}
+				compInfo, err := snap.ReadComponentInfoFromContainer(
+					compf, snapInfo, &seedComp.CompSideInfo)
+				if err != nil {
+					return nil, err
+				}
+				compInfosForType = append(compInfosForType, install.ComponentSeedInfo{
+					Info: compInfo,
+					Seed: &seedComp,
+				})
+			}
+		}
+		if typ == snap.TypeSnapd || typ == snap.TypeKernel {
+			snapdVersion, _, err := snap.SnapdInfoFromSnapFile(snapf, typ)
+			if err != nil {
+				return nil, err
+			}
+			switch typ {
+			case snap.TypeSnapd:
+				systemSnapdVersions.SnapdVersion = snapdVersion
+			case snap.TypeKernel:
+				systemSnapdVersions.SnapdInitramfsVersion = snapdVersion
+			}
+		}
+		seedSnaps[typ] = snapForMode
 		snapInfos[typ] = snapInfo
+		compInfos[typ] = compInfosForType
 	}
 	if len(snapInfos) != len(types) {
 		return nil, fmt.Errorf("internal error: retrieved snap infos (%d) does not match number of types (%d)", len(snapInfos), len(types))
 	}
 
 	return &systemAndEssentialSnaps{
-		System:          sys,
-		Seed:            s,
-		InfosByType:     snapInfos,
-		SeedSnapsByType: seedSnaps,
+		System:              sys,
+		Seed:                s,
+		SystemSnapdVersions: systemSnapdVersions,
+		InfosByType:         snapInfos,
+		CompsByType:         compInfos,
+		SeedSnapsByType:     seedSnaps,
 	}, nil
 }
 
@@ -2471,7 +2768,7 @@ func (scb storeContextBackend) SignDeviceSessionRequest(serial *asserts.Serial,
 		return nil, err
 	}
 
-	a, err := asserts.SignWithoutAuthority(asserts.DeviceSessionRequestType, map[string]interface{}{
+	a, err := asserts.SignWithoutAuthority(asserts.DeviceSessionRequestType, map[string]any{
 		"brand-id":  serial.BrandID(),
 		"model":     serial.Model(),
 		"serial":    serial.Serial(),
@@ -2515,24 +2812,32 @@ func (m *DeviceManager) ntpSyncedOrWaitedLongerThan(maxWait time.Duration) bool
 	return m.ntpSyncedOrTimedOut
 }
 
-func (m *DeviceManager) hasFDESetupHook(kernelInfo *snap.Info) (bool, error) {
+func (m *DeviceManager) hookKeyProtectorFactory(kernelInfo *snap.Info) (secboot.KeyProtectorFactory, error) {
 	// state must be locked
 	st := m.state
 
 	deviceCtx, err := DeviceCtx(st, nil, nil)
 	if err != nil {
-		return false, fmt.Errorf("cannot get device context: %v", err)
+		return nil, fmt.Errorf("cannot get device context: %v", err)
 	}
 
 	if kernelInfo == nil {
 		var err error
 		kernelInfo, err = snapstate.KernelInfo(st, deviceCtx)
 		if err != nil {
-			return false, fmt.Errorf("cannot get kernel info: %v", err)
+			return nil, fmt.Errorf("cannot get kernel info: %v", err)
 		}
 	}
-	_, ok := kernelInfo.Hooks["fde-setup"]
-	return ok, nil
+
+	if _, ok := kernelInfo.Hooks["fde-setup"]; ok {
+		return secboot.FDESetupHookKeyProtectorFactory(m.runFDESetupHook), nil
+	}
+
+	if secboot.FDEOpteeTAPresent() {
+		return secboot.OPTEEKeyProtectorFactory(), nil
+	}
+
+	return nil, secboot.ErrNoKeyProtector
 }
 
 func (m *DeviceManager) runFDESetupHook(req *fde.SetupRequest) ([]byte, error) {
@@ -2558,7 +2863,7 @@ func (m *DeviceManager) runFDESetupHook(req *fde.SetupRequest) ([]byte, error) {
 		// XXX: should this be configurable somehow?
 		Timeout: 5 * time.Minute,
 	}
-	contextData := map[string]interface{}{
+	contextData := map[string]any{
 		"fde-setup-request": req,
 	}
 	st.Unlock()
@@ -2716,10 +3021,10 @@ func (m *DeviceManager) RemoveRecoveryKeys() error {
 	return secbootRemoveRecoveryKeys(recoveryKeyDevices)
 }
 
-// checkEncryption verifies whether encryption should be used based on the
-// model grade and the availability of a TPM device or a fde-setup hook
-// in the kernel.
-func (m *DeviceManager) checkEncryption(st *state.State, deviceCtx snapstate.DeviceContext, tpmMode secboot.TPMProvisionMode) (secboot.EncryptionType, error) {
+// checkEncryption verifies whether encryption should be used based on the model
+// grade and the availability of a TPM device, fde-setup hook in the kernel, or
+// the OPTEE trusted application.
+func (m *DeviceManager) checkEncryption(st *state.State, deviceCtx snapstate.DeviceContext, tpmMode secboot.TPMProvisionMode) (device.EncryptionType, error) {
 	model := deviceCtx.Model()
 
 	kernelInfo, err := snapstate.KernelInfo(st, deviceCtx)
@@ -2735,9 +3040,129 @@ func (m *DeviceManager) checkEncryption(st *state.State, deviceCtx snapstate.Dev
 		return "", err
 	}
 
-	return install.CheckEncryptionSupport(model, tpmMode, kernelInfo, gadgetInfo, m.runFDESetupHook)
+	return install.CheckEncryptionSupport(install.EncryptionConstraints{
+		Model:   model,
+		TPMMode: tpmMode,
+		Kernel:  kernelInfo,
+		Gadget:  gadgetInfo,
+	}, m.runFDESetupHook)
 }
 
-func (m *DeviceManager) encryptionSupportInfo(model *asserts.Model, tpmMode secboot.TPMProvisionMode, kernelInfo *snap.Info, gadgetInfo *gadget.Info) (install.EncryptionSupportInfo, error) {
-	return install.GetEncryptionSupportInfo(model, tpmMode, kernelInfo, gadgetInfo, m.runFDESetupHook)
+// encryptionSupportInfoUnlocked is the encryptionSupportInfo variant to use when the state is not locked.
+func (m *DeviceManager) encryptionSupportInfoUnlocked(systemLabel string, constraints install.EncryptionConstraints, encInfoFromCache bool) (*install.EncryptionSupportInfo, error) {
+	return m.encryptionSupportInfo(
+		systemLabel,
+		constraints,
+		encInfoFromCache,
+		m.readCacheEncryptionSupportInfoUnlocked,
+		m.refreshCacheEncryptionSupportInfoUnlocked,
+	)
+}
+
+// encryptionSupportInfoLocked is the encryptionSupportInfo variant to use when the state is locked.
+func (m *DeviceManager) encryptionSupportInfoLocked(systemLabel string, constraints install.EncryptionConstraints, encInfoFromCache bool) (*install.EncryptionSupportInfo, error) {
+	return m.encryptionSupportInfo(
+		systemLabel,
+		constraints, encInfoFromCache,
+		m.readCacheEncryptionSupportInfoLocked,
+		m.refreshCacheEncryptionSupportInfoLocked,
+	)
+}
+
+// encryptionSupportInfo returns encryption support information, optionally
+// consulting and refreshing a cache.
+//
+// Cache behavior is implicit and depends on the combination of arguments:
+//   - With a CheckAction: the cache must provide a valid CheckContext. In this
+//     mode, only the CheckContext is used, and the rest of the cached
+//     EncryptionSupportInfo is ignored. This implies that any call with
+//     CheckAction must be preceded by a call without that populates the cache.
+//   - Without a CheckAction and encInfoFromCache set to true: the function
+//     returns the cached EncryptionSupportInfo directly, if available. This
+//     provides a way to skip the expensive encryption availability check in
+//     cases where it is not relevant.
+//   - Otherwise: the function computes fresh information via
+//     GetEncryptionSupportInfo and refreshes the cache on success.
+//
+// Errors are returned if inconsistent or impossible cache usage is requested
+//   - CheckAction requires a cache but none is available
+//   - CheckAction is combined with encInfoFromCache set to true
+func (m *DeviceManager) encryptionSupportInfo(
+	systemLabel string,
+	constraints install.EncryptionConstraints,
+	encInfoFromCache bool,
+	readCache func(systemLabel string) *install.EncryptionSupportInfo,
+	refreshCache func(systemLabel string, info *install.EncryptionSupportInfo),
+) (*install.EncryptionSupportInfo, error) {
+
+	if constraints.CheckAction != nil && encInfoFromCache {
+		return nil, errors.New("internal error: cannot apply check action and use cached encryption information")
+	}
+
+	cachedEncryptionSupportInfo := readCache(systemLabel)
+
+	if constraints.CheckAction != nil {
+		// a check action requires only the check context from the cache
+		// and need to run the check
+		if cachedEncryptionSupportInfo == nil {
+			return nil, errors.New("cannot use check action without cached encryption information")
+		}
+		checkContext := cachedEncryptionSupportInfo.CheckContext()
+		if checkContext == nil {
+			return nil, errors.New("cannot use check action without cached check context")
+		}
+		constraints.CheckContext = checkContext
+	} else if encInfoFromCache && cachedEncryptionSupportInfo != nil {
+		// in case of no check action use encryption support info from the
+		// cache when requested and available
+		return cachedEncryptionSupportInfo, nil
+	}
+
+	// GetEncryptionSupportInfo expects and uses constraints.CheckContext when
+	// constraints.CheckAction != nil, otherwise it is ignored. See
+	// install.encryptionAvailabilityCheck.
+	encInfo, err := install.GetEncryptionSupportInfo(constraints, m.runFDESetupHook)
+	if err == nil {
+		refreshCache(systemLabel, &encInfo)
+	}
+	return &encInfo, err
+}
+
+type encryptionSupportInfoKey struct{ systemLabel string }
+
+// SetEncryptionSupportInfoInCacheUnlocked is a test only helper for populating EncryptionSupportInfo in cache.
+func (m *DeviceManager) SetEncryptionSupportInfoInCacheUnlocked(systemLabel string, encryptionInfo *install.EncryptionSupportInfo) {
+	osutil.MustBeTestBinary("SetEncryptionSupportInfoInCacheUnlocked can only be used tests")
+	m.refreshCacheEncryptionSupportInfoUnlocked(systemLabel, encryptionInfo)
+}
+
+// refreshCacheEncryptionSupportUnlocked is the refreshCacheEncryptionSupport variant to use when the state is not locked.
+func (m *DeviceManager) refreshCacheEncryptionSupportInfoUnlocked(systemLabel string, encryptionInfo *install.EncryptionSupportInfo) {
+	m.state.Lock()
+	defer m.state.Unlock()
+	m.refreshCacheEncryptionSupportInfoLocked(systemLabel, encryptionInfo)
+}
+
+// refreshCacheEncryptionSupportLocked is the refreshCacheEncryptionSupport variant to use when the state is locked.
+func (m *DeviceManager) refreshCacheEncryptionSupportInfoLocked(systemLabel string, encryptionInfo *install.EncryptionSupportInfo) {
+	m.state.Cache(encryptionSupportInfoKey{systemLabel}, encryptionInfo)
+}
+
+// readCacheEncryptionSupportInfoStateUnlocked is the readCacheEncryptionSupportInfoState variant that does not lock state.
+func (m *DeviceManager) readCacheEncryptionSupportInfoUnlocked(systemLabel string) *install.EncryptionSupportInfo {
+	m.state.Lock()
+	defer m.state.Unlock()
+	return m.readCacheEncryptionSupportInfoLocked(systemLabel)
+}
+
+// readCacheEncryptionSupportInfoStateLocked is the readCacheEncryptionSupportInfoState variant that locks state.
+func (m *DeviceManager) readCacheEncryptionSupportInfoLocked(systemLabel string) *install.EncryptionSupportInfo {
+	cached := m.state.Cached(encryptionSupportInfoKey{systemLabel})
+	if cached != nil {
+		encryptionSupportInfo, ok := cached.(*install.EncryptionSupportInfo)
+		if ok {
+			return encryptionSupportInfo
+		}
+	}
+	return nil
 }
