@@ -35,22 +35,21 @@ import (
 )
 
 var (
-	secbootResealKeys                 = secboot.ResealKeys
-	secbootBuildPCRProtectionProfile  = secboot.BuildPCRProtectionProfile
-	secbootResealKeysWithFDESetupHook = secboot.ResealKeysWithFDESetupHook
-	secbootGetPrimaryKey              = secboot.GetPrimaryKey
-	secbootRevokeOldKeys              = (*secboot.UpdatedKeys).RevokeOldKeys
-	bootIsResealNeeded                = boot.IsResealNeeded
+	secbootResealKey                 = secboot.ResealKey
+	secbootBuildPCRProtectionProfile = secboot.BuildPCRProtectionProfile
+	secbootGetPrimaryKey             = secboot.GetPrimaryKey
+	secbootRevokeOldKeys             = (*secboot.UpdatedKeys).RevokeOldKeys
+	bootIsResealNeeded               = boot.IsResealNeeded
 )
 
-// MockSecbootResealKeys is only useful in testing. Note that this is a very low
+// MockSecbootResealKey is only useful in testing. Note that this is a very low
 // level call and may need significant environment setup.
-func MockSecbootResealKeys(f func(params *secboot.ResealKeysParams, newPCRPolicyVersion bool) (secboot.UpdatedKeys, error)) (restore func()) {
-	osutil.MustBeTestBinary("secbootResealKeys only can be mocked in tests")
-	old := secbootResealKeys
-	secbootResealKeys = f
+func MockSecbootResealKey(f func(key secboot.KeyDataLocation, params *secboot.ResealKeyParams) (secboot.UpdatedKeys, error)) (restore func()) {
+	osutil.MustBeTestBinary("secbootResealKey only can be mocked in tests")
+	old := secbootResealKey
+	secbootResealKey = f
 	return func() {
-		secbootResealKeys = old
+		secbootResealKey = old
 	}
 }
 
@@ -63,7 +62,7 @@ func MockSecbootBuildPCRProtectionProfile(f func(modelParams []*secboot.SealKeyM
 	}
 }
 
-func MockSecbootGetPrimaryKey(f func(devices []string, fallbackKeyFile string) ([]byte, error)) (restore func()) {
+func MockSecbootGetPrimaryKey(f func(devices []string, fallbackKeyFiles []string) ([]byte, error)) (restore func()) {
 	osutil.MustBeTestBinary("secbootGetPrimaryKey only can be mocked in tests")
 	old := secbootGetPrimaryKey
 	secbootGetPrimaryKey = f
@@ -149,7 +148,14 @@ type resealParamsAndLocation struct {
 	location secboot.KeyDataLocation
 }
 
-func doReseal(manager FDEStateManager, method device.SealingMethod, rootdir string, revokeOldKeys bool) error {
+type doResealOptions struct {
+	revokeOldKeys     bool
+	hintExpectFDEHook bool
+}
+
+func doReseal(manager FDEStateManager, rootdir string, opts doResealOptions) error {
+	revokeOldKeys := opts.revokeOldKeys
+
 	containers, err := manager.GetEncryptedContainers()
 	if err != nil {
 		return err
@@ -236,56 +242,50 @@ func doReseal(manager FDEStateManager, method device.SealingMethod, rootdir stri
 		}
 	}
 
-	switch method {
-	case device.SealingMethodFDESetupHook:
-		// Note this is the fallback key path for old installations. The file might not exist.
-		primaryKeyFile := filepath.Join(boot.InstallHostFDESaveDir, "aux-key")
-		// All devices should have the same primary key
-		primaryKeyGetter := func() ([]byte, error) {
-			return secbootGetPrimaryKey(devices, primaryKeyFile)
+	saveFDEDir := dirs.SnapFDEDirUnderSave(dirs.SnapSaveDirUnder(rootdir))
+	fallbackPrimaryKeyFiles := []string{
+		filepath.Join(saveFDEDir, "aux-key"),
+		filepath.Join(saveFDEDir, "tpm-policy-auth-key"),
+	}
+
+	var allResealedKeys secboot.UpdatedKeys
+	for _, key := range keys {
+		params := &secboot.ResealKeyParams{
+			// Because the save disk might be opened with
+			// an old plainkey, there might not be any
+			// primary key available in keyring for that
+			// save device. So we need to look at all
+			// devices.
+			PrimaryKeyDevices:       devices,
+			FallbackPrimaryKeyFiles: fallbackPrimaryKeyFiles,
+			BootModes:               key.params.BootModes,
+			Models:                  key.params.Models,
+			TpmPCRProfile:           key.params.TpmPCRProfile,
+			NewPCRPolicyVersion:     revokeOldKeys,
+			HintExpectFDEHook:       opts.hintExpectFDEHook,
 		}
-		for _, key := range keys {
-			if err := secbootResealKeysWithFDESetupHook([]secboot.KeyDataLocation{key.location}, primaryKeyGetter, key.params.Models, key.params.BootModes); err != nil {
-				return err
-			}
-		}
-		return nil
-	case device.SealingMethodTPM, device.SealingMethodLegacyTPM:
-		saveFDEDir := dirs.SnapFDEDirUnderSave(dirs.SnapSaveDirUnder(rootdir))
-		authKeyFile := filepath.Join(saveFDEDir, "tpm-policy-auth-key")
-		// All devices should have the same primary key
-		primaryKey, err := secbootGetPrimaryKey(devices, authKeyFile)
+		resealedKeys, err := secbootResealKey(key.location, params)
 		if err != nil {
 			return err
 		}
-		var allResealedKeys secboot.UpdatedKeys
-		for _, key := range keys {
-			keyParams := &secboot.ResealKeysParams{
-				PCRProfile: key.params.TpmPCRProfile,
-				Keys:       []secboot.KeyDataLocation{key.location},
-				PrimaryKey: primaryKey,
-			}
-
-			resealedKeys, err := secbootResealKeys(keyParams, revokeOldKeys)
-			if err != nil {
-				return fmt.Errorf("cannot reseal the encryption key: %v", err)
-			}
-			if revokeOldKeys {
-				allResealedKeys = append(allResealedKeys, resealedKeys...)
-			}
-		}
 		if revokeOldKeys {
-			if err := secbootRevokeOldKeys(&allResealedKeys, primaryKey); err != nil {
-				return fmt.Errorf("cannot revoke older keys: %v", err)
-			}
+			allResealedKeys = append(allResealedKeys, resealedKeys...)
 		}
-		return nil
-	default:
-		return fmt.Errorf("unknown key sealing method: %q", method)
 	}
+	if revokeOldKeys {
+		primaryKey, err := secbootGetPrimaryKey(devices, fallbackPrimaryKeyFiles)
+		if err != nil {
+			return err
+		}
+		if err := secbootRevokeOldKeys(&allResealedKeys, primaryKey); err != nil {
+			return fmt.Errorf("cannot revoke older keys: %v", err)
+		}
+	}
+
+	return nil
 }
 
-func recalculateParamatersFDEHook(manager FDEStateManager, method device.SealingMethod, rootdir string, inputs resealInputs, opts resealOptions) error {
+func recalculateParamatersFDEHook(manager FDEStateManager, inputs resealInputs) error {
 	params := inputs.bootChains
 	runModels := getUniqueModels(append(params.RunModeBootChains, params.RecoveryBootChainsForRunKey...))
 	recoveryModels := getUniqueModels(params.RecoveryBootChains)
@@ -317,7 +317,7 @@ func recalculateParamatersFDEHook(manager FDEStateManager, method device.Sealing
 	return nil
 }
 
-func recalculateParamatersTPM(manager FDEStateManager, method device.SealingMethod, rootdir string, inputs resealInputs, opts resealOptions) error {
+func recalculateParamatersTPM(manager FDEStateManager, rootdir string, inputs resealInputs, opts resealOptions) error {
 	params := inputs.bootChains
 	// reseal the run object
 	pbc := boot.ToPredictableBootChains(append(params.RunModeBootChains, params.RecoveryBootChainsForRunKey...))
@@ -614,7 +614,7 @@ func resealKeys(
 			return nil
 		}
 
-		if err := recalculateParamatersFDEHook(manager, method, rootdir, inputs, opts); err != nil {
+		if err := recalculateParamatersFDEHook(manager, inputs); err != nil {
 			return err
 		}
 	case device.SealingMethodTPM, device.SealingMethodLegacyTPM:
@@ -625,7 +625,7 @@ func resealKeys(
 			}
 		}
 
-		if err := recalculateParamatersTPM(manager, method, rootdir, inputs, opts); err != nil {
+		if err := recalculateParamatersTPM(manager, rootdir, inputs, opts); err != nil {
 			// TODO:FDEM:FIX: remove the save boot chains.
 			return err
 		}
@@ -633,7 +633,7 @@ func resealKeys(
 		return fmt.Errorf("unknown key sealing method: %q", method)
 	}
 
-	return doReseal(manager, method, rootdir, opts.Revoke)
+	return doReseal(manager, rootdir, doResealOptions{revokeOldKeys: opts.Revoke, hintExpectFDEHook: method == device.SealingMethodFDESetupHook})
 }
 
 func attachSignatureDbxUpdate(params []*secboot.SealKeyModelParams, update []byte) {
