@@ -399,7 +399,7 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 	defer restore()
 
 	resealCalls := 0
-	restore = backend.MockSecbootResealKeys(func(params *secboot.ResealKeysParams, newPCRPolicyVersion bool) (secboot.UpdatedKeys, error) {
+	restore = backend.MockSecbootResealKey(func(key secboot.KeyDataLocation, params *secboot.ResealKeyParams) (secboot.UpdatedKeys, error) {
 		resealCalls++
 
 		skippedCalls := 0
@@ -407,37 +407,36 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 			skippedCalls += 1
 		}
 
-		c.Check(params.PrimaryKey, DeepEquals, []byte{1, 2, 3, 4})
-		c.Check(newPCRPolicyVersion, Equals, revokeOldKeys && !missingRunParams && !missingRecoverParams)
+		c.Check(params.PrimaryKeyDevices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
+		c.Check(params.FallbackPrimaryKeyFiles, DeepEquals, []string{
+			filepath.Join(dirs.SnapSaveDir, "device/fde", "aux-key"),
+			filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"),
+		})
 
-		c.Check(params.PCRProfile, DeepEquals, secboot.SerializedPCRProfile(`"serialized-pcr-profile"`))
+		c.Logf("key: %v", key)
+
+		c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
 		switch resealCalls + skippedCalls {
 		case 1:
 			// Resealing the run+recover key for data partition
-			c.Check(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/123",
-					SlotName:   "default",
-					KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-boot/device/fde/ubuntu-data.sealed-key"),
-				},
+			c.Check(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/123",
+				SlotName:   "default",
+				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-boot/device/fde/ubuntu-data.sealed-key"),
 			})
 		case 2:
 			// Resealing the recovery key for both data partition
-			c.Check(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/123",
-					SlotName:   "default-fallback",
-					KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-data.recovery.sealed-key"),
-				},
+			c.Check(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/123",
+				SlotName:   "default-fallback",
+				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-data.recovery.sealed-key"),
 			})
 		case 3:
 			// Resealing the recovery key for both save partition
-			c.Check(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/456",
-					SlotName:   "default-fallback",
-					KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-save.recovery.sealed-key"),
-				},
+			c.Check(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/456",
+				SlotName:   "default-fallback",
+				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-save.recovery.sealed-key"),
 			})
 		default:
 			c.Errorf("unexpected additional call to secboot.ResealKey (call # %d)", resealCalls)
@@ -465,12 +464,6 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 		},
 	}
 
-	defer backend.MockSecbootGetPrimaryKey(func(devices []string, fallbackKeyFile string) ([]byte, error) {
-		c.Check(fallbackKeyFile, Equals, filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"))
-		c.Check(devices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
-		return []byte{1, 2, 3, 4}, nil
-	})()
-
 	defer backend.MockSecbootRevokeOldKeys(func(uk *secboot.UpdatedKeys, primaryKey []byte) error {
 		if !revokeOldKeys {
 			c.Errorf("unexpected call")
@@ -486,6 +479,15 @@ func (s *resealTestSuite) testTPMResealHappy(c *C, revokeOldKeys bool, missingRu
 		return nil
 	})()
 
+	defer backend.MockSecbootGetPrimaryKey(func(devices []string, fallbackKeyFiles []string) ([]byte, error) {
+		c.Check(fallbackKeyFiles, DeepEquals, []string{
+			filepath.Join(dirs.SnapSaveDir, "device/fde", "aux-key"),
+			filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"),
+		})
+		c.Check(devices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
+		return []byte{1, 2, 3, 4}, nil
+	})()
+
 	opts := boot.ResealKeyToModeenvOptions{ExpectReseal: true, RevokeOldKeys: revokeOldKeys}
 	err := backend.ResealKeyForBootChains(myState, device.SealingMethodTPM, s.rootdir, &boot.ResealKeyForBootChainsParams{BootChains: bootChains, Options: opts})
 	c.Assert(err, IsNil)
@@ -590,7 +592,7 @@ func (s *resealTestSuite) TestResealKeyForBootchainsWithSystemFallback(c *C) {
 		{noShim2: true, resealErr: nil, err: ""},
 		{noShim2: true, noGrub2: true, resealErr: nil, err: ""},
 		{resealErr: nil, err: ""},
-		{resealErr: errors.New("reseal error"), err: "cannot reseal the encryption key: reseal error"},
+		{resealErr: errors.New("reseal error"), err: "reseal error"},
 		{reuseRunPbc: true, reuseRecoveryPbc: true, resealErr: nil, err: ""},
 		// recovery boot chain is unchanged
 		{reuseRunPbc: false, reuseRecoveryPbc: true, resealErr: nil, err: ""},
@@ -834,40 +836,39 @@ func (s *resealTestSuite) TestResealKeyForBootchainsWithSystemFallback(c *C) {
 
 		// set mock key resealing
 		resealKeysCalls := 0
-		restore = backend.MockSecbootResealKeys(func(params *secboot.ResealKeysParams, newPCRPolicyVersion bool) (secboot.UpdatedKeys, error) {
-			c.Check(params.PrimaryKey, DeepEquals, []byte{1, 2, 3, 4})
-			c.Check(newPCRPolicyVersion, Equals, false)
+		restore = backend.MockSecbootResealKey(func(key secboot.KeyDataLocation, params *secboot.ResealKeyParams) (secboot.UpdatedKeys, error) {
+			c.Check(params.PrimaryKeyDevices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
+			c.Check(params.FallbackPrimaryKeyFiles, DeepEquals, []string{
+				filepath.Join(dirs.SnapSaveDir, "device/fde", "aux-key"),
+				filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"),
+			})
+			c.Check(params.NewPCRPolicyVersion, Equals, false)
+			c.Check(params.HintExpectFDEHook, Equals, false)
 
 			resealKeysCalls++
-			c.Check(params.PCRProfile, DeepEquals, secboot.SerializedPCRProfile(`"serialized-pcr-profile"`))
+			c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
 
 			checkRunParams := func() {
-				c.Check(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-					{
-						DevicePath: "/dev/disk/by-uuid/123",
-						SlotName:   "default",
-						KeyFile:    filepath.Join(boot.InitramfsBootEncryptionKeyDir, "ubuntu-data.sealed-key"),
-					},
+				c.Check(key, DeepEquals, secboot.KeyDataLocation{
+					DevicePath: "/dev/disk/by-uuid/123",
+					SlotName:   "default",
+					KeyFile:    filepath.Join(boot.InitramfsBootEncryptionKeyDir, "ubuntu-data.sealed-key"),
 				})
 			}
 
 			checkRecoveryParamsData := func() {
-				c.Check(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-					{
-						DevicePath: "/dev/disk/by-uuid/123",
-						SlotName:   "default-fallback",
-						KeyFile:    filepath.Join(boot.InitramfsSeedEncryptionKeyDir, "ubuntu-data.recovery.sealed-key"),
-					},
+				c.Check(key, DeepEquals, secboot.KeyDataLocation{
+					DevicePath: "/dev/disk/by-uuid/123",
+					SlotName:   "default-fallback",
+					KeyFile:    filepath.Join(boot.InitramfsSeedEncryptionKeyDir, "ubuntu-data.recovery.sealed-key"),
 				})
 			}
 
 			checkRecoveryParamsSave := func() {
-				c.Check(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-					{
-						DevicePath: "/dev/disk/by-uuid/456",
-						SlotName:   "default-fallback",
-						KeyFile:    filepath.Join(boot.InitramfsSeedEncryptionKeyDir, "ubuntu-save.recovery.sealed-key"),
-					},
+				c.Check(key, DeepEquals, secboot.KeyDataLocation{
+					DevicePath: "/dev/disk/by-uuid/456",
+					SlotName:   "default-fallback",
+					KeyFile:    filepath.Join(boot.InitramfsSeedEncryptionKeyDir, "ubuntu-save.recovery.sealed-key"),
 				})
 			}
 
@@ -1151,12 +1152,6 @@ func (s *resealTestSuite) TestResealKeyForBootchainsWithSystemFallback(c *C) {
 			},
 		}
 
-		defer backend.MockSecbootGetPrimaryKey(func(devices []string, fallbackKeyFile string) ([]byte, error) {
-			c.Check(fallbackKeyFile, Equals, filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"))
-			c.Check(devices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
-			return []byte{1, 2, 3, 4}, nil
-		})()
-
 		err := backend.ResealKeyForBootChains(myState, device.SealingMethodTPM, rootdir, &boot.ResealKeyForBootChainsParams{BootChains: bootChains})
 		if tc.err == "" {
 			c.Assert(err, IsNil)
@@ -1290,37 +1285,36 @@ func (s *resealTestSuite) TestResealKeyForBootchainsRecoveryKeysForGoodSystemsOn
 
 	// set mock key resealing
 	resealKeysCalls := 0
-	restore = backend.MockSecbootResealKeys(func(params *secboot.ResealKeysParams, newPCRPolicyVersion bool) (secboot.UpdatedKeys, error) {
-		c.Check(params.PrimaryKey, DeepEquals, []byte{1, 2, 3, 4})
-		c.Check(newPCRPolicyVersion, Equals, false)
+	restore = backend.MockSecbootResealKey(func(key secboot.KeyDataLocation, params *secboot.ResealKeyParams) (secboot.UpdatedKeys, error) {
+		c.Check(params.PrimaryKeyDevices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
+		c.Check(params.FallbackPrimaryKeyFiles, DeepEquals, []string{
+			filepath.Join(dirs.SnapSaveDir, "device/fde", "aux-key"),
+			filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"),
+		})
+		c.Check(params.NewPCRPolicyVersion, Equals, false)
+		c.Check(params.HintExpectFDEHook, Equals, false)
 
 		resealKeysCalls++
-		c.Check(params.PCRProfile, DeepEquals, secboot.SerializedPCRProfile(`"serialized-pcr-profile"`))
+		c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
 
 		switch resealKeysCalls {
 		case 1: // run key
-			c.Assert(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/123",
-					SlotName:   "default",
-					KeyFile:    filepath.Join(boot.InitramfsBootEncryptionKeyDir, "ubuntu-data.sealed-key"),
-				},
+			c.Assert(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/123",
+				SlotName:   "default",
+				KeyFile:    filepath.Join(boot.InitramfsBootEncryptionKeyDir, "ubuntu-data.sealed-key"),
 			})
 		case 2: // recovery keys
-			c.Check(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/123",
-					SlotName:   "default-fallback",
-					KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-data.recovery.sealed-key"),
-				},
+			c.Check(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/123",
+				SlotName:   "default-fallback",
+				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-data.recovery.sealed-key"),
 			})
 		case 3:
-			c.Check(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/456",
-					SlotName:   "default-fallback",
-					KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-save.recovery.sealed-key"),
-				},
+			c.Check(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/456",
+				SlotName:   "default-fallback",
+				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-save.recovery.sealed-key"),
 			})
 		default:
 			c.Errorf("unexpected additional call to secboot.ResealKeys (call # %d)", resealKeysCalls)
@@ -1476,12 +1470,6 @@ func (s *resealTestSuite) TestResealKeyForBootchainsRecoveryKeysForGoodSystemsOn
 		},
 	}
 
-	defer backend.MockSecbootGetPrimaryKey(func(devices []string, fallbackKeyFile string) ([]byte, error) {
-		c.Check(fallbackKeyFile, Equals, filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"))
-		c.Check(devices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
-		return []byte{1, 2, 3, 4}, nil
-	})()
-
 	err := backend.ResealKeyForBootChains(myState, device.SealingMethodTPM, s.rootdir, &boot.ResealKeyForBootChainsParams{BootChains: bootChains})
 	c.Assert(err, IsNil)
 	c.Assert(resealKeysCalls, Equals, 3)
@@ -1639,37 +1627,36 @@ func (s *resealTestSuite) testResealKeyForBootchainsWithTryModel(c *C, shimId, g
 
 	// set mock key resealing
 	resealKeysCalls := 0
-	restore = backend.MockSecbootResealKeys(func(params *secboot.ResealKeysParams, newPCRPolicyVersion bool) (secboot.UpdatedKeys, error) {
-		c.Check(params.PrimaryKey, DeepEquals, []byte{1, 2, 3, 4})
-		c.Check(newPCRPolicyVersion, Equals, false)
+	restore = backend.MockSecbootResealKey(func(key secboot.KeyDataLocation, params *secboot.ResealKeyParams) (secboot.UpdatedKeys, error) {
+		c.Check(params.PrimaryKeyDevices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
+		c.Check(params.FallbackPrimaryKeyFiles, DeepEquals, []string{
+			filepath.Join(dirs.SnapSaveDir, "device/fde", "aux-key"),
+			filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"),
+		})
+		c.Check(params.NewPCRPolicyVersion, Equals, false)
+		c.Check(params.HintExpectFDEHook, Equals, false)
 
 		resealKeysCalls++
-		c.Check(params.PCRProfile, DeepEquals, secboot.SerializedPCRProfile(`"serialized-pcr-profile"`))
+		c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
 
 		switch resealKeysCalls {
 		case 1: // run key
-			c.Assert(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/123",
-					SlotName:   "default",
-					KeyFile:    filepath.Join(boot.InitramfsBootEncryptionKeyDir, "ubuntu-data.sealed-key"),
-				},
+			c.Assert(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/123",
+				SlotName:   "default",
+				KeyFile:    filepath.Join(boot.InitramfsBootEncryptionKeyDir, "ubuntu-data.sealed-key"),
 			})
 		case 2: // recovery keys
-			c.Assert(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/123",
-					SlotName:   "default-fallback",
-					KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-data.recovery.sealed-key"),
-				},
+			c.Assert(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/123",
+				SlotName:   "default-fallback",
+				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-data.recovery.sealed-key"),
 			})
 		case 3:
-			c.Assert(params.Keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/456",
-					SlotName:   "default-fallback",
-					KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-save.recovery.sealed-key"),
-				},
+			c.Assert(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/456",
+				SlotName:   "default-fallback",
+				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-save.recovery.sealed-key"),
 			})
 		default:
 			c.Errorf("unexpected additional call to secboot.ResealKeys (call # %d)", resealKeysCalls)
@@ -1815,12 +1802,6 @@ func (s *resealTestSuite) testResealKeyForBootchainsWithTryModel(c *C, shimId, g
 		},
 	}
 
-	defer backend.MockSecbootGetPrimaryKey(func(devices []string, fallbackKeyFile string) ([]byte, error) {
-		c.Check(fallbackKeyFile, Equals, filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"))
-		c.Check(devices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
-		return []byte{1, 2, 3, 4}, nil
-	})()
-
 	err := backend.ResealKeyForBootChains(myState, device.SealingMethodTPM, s.rootdir, &boot.ResealKeyForBootChainsParams{BootChains: bootChains})
 	c.Assert(err, IsNil)
 	c.Assert(resealKeysCalls, Equals, 3)
@@ -1905,11 +1886,13 @@ func (s *resealTestSuite) TestResealKeyForBootchainsFallbackCmdline(c *C) {
 
 	// set mock key resealing
 	resealKeysCalls := 0
-	restore = backend.MockSecbootResealKeys(func(params *secboot.ResealKeysParams, newPCRPolicyVersion bool) (secboot.UpdatedKeys, error) {
+	restore = backend.MockSecbootResealKey(func(key secboot.KeyDataLocation, params *secboot.ResealKeyParams) (secboot.UpdatedKeys, error) {
 		resealKeysCalls++
 
-		c.Check(newPCRPolicyVersion, Equals, false)
-		c.Check(params.PCRProfile, DeepEquals, secboot.SerializedPCRProfile(`"serialized-pcr-profile"`))
+		c.Check(params.NewPCRPolicyVersion, Equals, false)
+		c.Check(params.HintExpectFDEHook, Equals, false)
+		c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile"`))
+
 		c.Logf("reseal: %+v", params)
 		switch resealKeysCalls {
 		case 1:
@@ -2017,12 +2000,6 @@ func (s *resealTestSuite) TestResealKeyForBootchainsFallbackCmdline(c *C) {
 		},
 	}
 
-	defer backend.MockSecbootGetPrimaryKey(func(devices []string, fallbackKeyFile string) ([]byte, error) {
-		c.Check(fallbackKeyFile, Equals, filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"))
-		c.Check(devices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
-		return []byte{1, 2, 3, 4}, nil
-	})()
-
 	err = backend.ResealKeyForBootChains(myState, device.SealingMethodTPM, s.rootdir, &boot.ResealKeyForBootChainsParams{BootChains: bootChains})
 	c.Assert(err, IsNil)
 	c.Assert(resealKeysCalls, Equals, 3)
@@ -2078,55 +2055,52 @@ func (s *resealTestSuite) TestHooksResealHappy(c *C) {
 	}
 
 	resealCalls := 0
-	restore := backend.MockSecbootResealKeysWithFDESetupHook(func(keys []secboot.KeyDataLocation, primaryKeyGetter func() ([]byte, error), models []secboot.ModelForSealing, bootModes []string) error {
+	restore := backend.MockSecbootResealKey(func(key secboot.KeyDataLocation, params *secboot.ResealKeyParams) (secboot.UpdatedKeys, error) {
+
 		resealCalls++
 
-		primaryKey, err := primaryKeyGetter()
-		c.Assert(err, IsNil)
+		c.Check(params.HintExpectFDEHook, Equals, true)
+
+		c.Check(params.PrimaryKeyDevices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
+		c.Check(params.FallbackPrimaryKeyFiles, DeepEquals, []string{
+			filepath.Join(dirs.SnapSaveDir, "device/fde", "aux-key"),
+			filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"),
+		})
 		switch resealCalls {
 		case 1:
 			// Resealing the run+recover key for data partition
-			c.Check(keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/123",
-					SlotName:   "default",
-					KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-boot/device/fde/ubuntu-data.sealed-key"),
-				},
+			c.Check(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/123",
+				SlotName:   "default",
+				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-boot/device/fde/ubuntu-data.sealed-key"),
 			})
-			c.Check(primaryKey, DeepEquals, []byte{1, 2, 3, 4})
-			c.Assert(models, HasLen, 1)
-			c.Check(models[0].Model(), Equals, model.Model())
-			c.Check(bootModes, DeepEquals, []string{"run", "recover"})
+			c.Assert(params.Models, HasLen, 1)
+			c.Check(params.Models[0].Model(), Equals, model.Model())
+			c.Check(params.BootModes, DeepEquals, []string{"run", "recover"})
 		case 2:
 			// Resealing the recovery key for both data partition
-			c.Check(keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/123",
-					SlotName:   "default-fallback",
-					KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-data.recovery.sealed-key"),
-				},
+			c.Check(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/123",
+				SlotName:   "default-fallback",
+				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-data.recovery.sealed-key"),
 			})
-			c.Check(primaryKey, DeepEquals, []byte{1, 2, 3, 4})
-			c.Assert(models, HasLen, 1)
-			c.Check(models[0].Model(), Equals, model.Model())
-			c.Check(bootModes, DeepEquals, []string{"recover"})
+			c.Assert(params.Models, HasLen, 1)
+			c.Check(params.Models[0].Model(), Equals, model.Model())
+			c.Check(params.BootModes, DeepEquals, []string{"recover"})
 		case 3:
 			// Resealing the recovery key for both save partition
-			c.Check(keys, DeepEquals, []secboot.KeyDataLocation{
-				{
-					DevicePath: "/dev/disk/by-uuid/456",
-					SlotName:   "default-fallback",
-					KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-save.recovery.sealed-key"),
-				},
+			c.Check(key, DeepEquals, secboot.KeyDataLocation{
+				DevicePath: "/dev/disk/by-uuid/456",
+				SlotName:   "default-fallback",
+				KeyFile:    filepath.Join(s.rootdir, "run/mnt/ubuntu-seed/device/fde/ubuntu-save.recovery.sealed-key"),
 			})
-			c.Check(primaryKey, DeepEquals, []byte{1, 2, 3, 4})
-			c.Assert(models, HasLen, 1)
-			c.Check(models[0].Model(), Equals, model.Model())
-			c.Check(bootModes, DeepEquals, []string{"recover", "factory-reset"})
+			c.Assert(params.Models, HasLen, 1)
+			c.Check(params.Models[0].Model(), Equals, model.Model())
+			c.Check(params.BootModes, DeepEquals, []string{"recover", "factory-reset"})
 		default:
 			c.Errorf("unexpected additional call to secboot.ResealKey (call # %d)", resealCalls)
 		}
-		return nil
+		return nil, nil
 	})
 
 	defer restore()
@@ -2150,12 +2124,6 @@ func (s *resealTestSuite) TestHooksResealHappy(c *C) {
 		},
 	}
 
-	defer backend.MockSecbootGetPrimaryKey(func(devices []string, fallbackKeyFile string) ([]byte, error) {
-		c.Check(fallbackKeyFile, Equals, filepath.Join(s.rootdir, "run/mnt/ubuntu-save/device/fde/aux-key"))
-		c.Check(devices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
-		return []byte{1, 2, 3, 4}, nil
-	})()
-
 	err := backend.ResealKeyForBootChains(myState, device.SealingMethodFDESetupHook, s.rootdir, &boot.ResealKeyForBootChainsParams{BootChains: bootChains})
 	c.Assert(err, IsNil)
 
@@ -2205,9 +2173,9 @@ func (s *resealTestSuite) TestHooksResealIgnoreFDEHooks(c *C) {
 		},
 	}
 
-	defer backend.MockSecbootResealKeysWithFDESetupHook(func(keys []secboot.KeyDataLocation, primaryKeyGetter func() ([]byte, error), models []secboot.ModelForSealing, bootModes []string) error {
+	defer backend.MockSecbootResealKey(func(key secboot.KeyDataLocation, params *secboot.ResealKeyParams) (secboot.UpdatedKeys, error) {
 		c.Errorf("unexpected call")
-		return fmt.Errorf("unexpected call")
+		return nil, fmt.Errorf("unexpected call")
 	})()
 
 	myState := &fakeState{}
@@ -2229,7 +2197,7 @@ func (s *resealTestSuite) TestHooksResealIgnoreFDEHooks(c *C) {
 		},
 	}
 
-	defer backend.MockSecbootGetPrimaryKey(func(devices []string, fallbackKeyFile string) ([]byte, error) {
+	defer backend.MockSecbootGetPrimaryKey(func(devices []string, fallbackKeyFiles []string) ([]byte, error) {
 		c.Errorf("unexpected call")
 		return nil, fmt.Errorf("unexpected call")
 	})()
@@ -2363,11 +2331,13 @@ func (s *resealTestSuite) TestResealKeyForSignatureDBUpdate(c *C) {
 
 	// set mock key resealing
 	resealKeysCalls := 0
-	restore = backend.MockSecbootResealKeys(func(params *secboot.ResealKeysParams, newPCRPolicyVersion bool) (secboot.UpdatedKeys, error) {
+	restore = backend.MockSecbootResealKey(func(key secboot.KeyDataLocation, params *secboot.ResealKeyParams) (secboot.UpdatedKeys, error) {
 		resealKeysCalls++
 
-		c.Check(newPCRPolicyVersion, Equals, false)
-		c.Check(params.PCRProfile, DeepEquals, secboot.SerializedPCRProfile(`"serialized-pcr-profile-with-dbx"`))
+		c.Check(params.NewPCRPolicyVersion, Equals, false)
+		c.Check(params.HintExpectFDEHook, Equals, false)
+
+		c.Check(params.TpmPCRProfile, DeepEquals, []byte(`"serialized-pcr-profile-with-dbx"`))
 		c.Logf("reseal: %+v", params)
 
 		return nil, nil
@@ -2393,12 +2363,6 @@ func (s *resealTestSuite) TestResealKeyForSignatureDBUpdate(c *C) {
 		},
 	}
 
-	defer backend.MockSecbootGetPrimaryKey(func(devices []string, fallbackKeyFile string) ([]byte, error) {
-		c.Check(fallbackKeyFile, Equals, filepath.Join(dirs.SnapSaveDir, "device/fde", "tpm-policy-auth-key"))
-		c.Check(devices, DeepEquals, []string{"/dev/disk/by-uuid/123", "/dev/disk/by-uuid/456"})
-		return []byte{1, 2, 3, 4}, nil
-	})()
-
 	err = backend.ResealKeysForSignaturesDBUpdate(myState, device.SealingMethodTPM, dirs.GlobalRootDir,
 		&boot.ResealKeyForBootChainsParams{BootChains: bootChains}, []byte("dbx-payload"))
 	c.Assert(err, IsNil)
@@ -2538,7 +2502,7 @@ func (s *resealTestSuite) TestTPMResealEnsureProvisioned(c *C) {
 	})()
 
 	resealCalls := 0
-	defer backend.MockSecbootResealKeys(func(params *secboot.ResealKeysParams, newPCRPolicyVersion bool) (secboot.UpdatedKeys, error) {
+	defer backend.MockSecbootResealKey(func(key secboot.KeyDataLocation, params *secboot.ResealKeyParams) (secboot.UpdatedKeys, error) {
 		resealCalls++
 		return nil, nil
 	})()
@@ -2562,10 +2526,6 @@ func (s *resealTestSuite) TestTPMResealEnsureProvisioned(c *C) {
 		},
 	}
 
-	defer backend.MockSecbootGetPrimaryKey(func(devices []string, fallbackKeyFile string) ([]byte, error) {
-		return []byte{1, 2, 3, 4}, nil
-	})()
-
 	provisioned := 0
 	defer backend.MockSecbootProvisionTPM(func(mode secboot.TPMProvisionMode, lockoutAuthFile string) error {
 		provisioned++
