@@ -30,7 +30,7 @@ var (
 	shortStartHelp = i18n.G("Start services")
 	longStartHelp  = i18n.G(`
 The start command starts the given services of the snap. If executed from the
-"configure" hook, the services will be started after the hook finishes.`)
+"configure" hook or "default-configure" hook, the services will be started after the hook finishes.`)
 )
 
 func init() {
