@@ -28,6 +28,7 @@ import (
 	"os/exec"
 	"path/filepath"
 	"runtime"
+	"strings"
 	"syscall"
 	"testing"
 	"time"
@@ -37,6 +38,7 @@ import (
 
 	"github.com/snapcore/snapd/boot"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/dirs/dirstest"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord"
 	"github.com/snapcore/snapd/overlord/devicestate/devicestatetest"
@@ -90,6 +92,7 @@ func (ovs *overlordSuite) SetUpTest(c *C) {
 	}
 
 	tmpdir := c.MkDir()
+	dirstest.MustMockCanonicalSnapMountDir(tmpdir)
 	dirs.SetRootDir(tmpdir)
 	ovs.AddCleanup(func() { dirs.SetRootDir("") })
 	ovs.AddCleanup(osutil.MockMountInfo(""))
@@ -124,6 +127,7 @@ func (ovs *overlordSuite) TestNew(c *C) {
 	c.Check(o.DeviceManager(), NotNil)
 	c.Check(o.CommandManager(), NotNil)
 	c.Check(o.SnapshotManager(), NotNil)
+	c.Check(o.FDEManager(), NotNil)
 	c.Check(configstateInitCalled, Equals, true)
 
 	o.InterfaceManager().DisableUDevMonitor()
@@ -193,13 +197,25 @@ func (ovs *overlordSuite) TestNewWithGoodState(c *C) {
 	d, err := state.MarshalJSON()
 	c.Assert(err, IsNil)
 
-	var got, expected map[string]interface{}
+	var got, expected map[string]any
 	err = json.Unmarshal(d, &got)
 	c.Assert(err, IsNil)
 	err = json.Unmarshal(fakeState, &expected)
 	c.Assert(err, IsNil)
 
-	data, _ := got["data"].(map[string]interface{})
+	if runtime.Version() < "go1.24" {
+		// Go versions prior to 1.24 do not recognize "omitzero", and since a
+		// zero time.Time is not technically empty, it is not omitted by
+		// "omitempty". We expect a zero-valued "last-notice-timestamp", so it
+		// should be "0001-01-01T00:00:00Z" on go versions prior to 1.24.
+		lastTS, ok := got["last-notice-timestamp"]
+		c.Check(ok, Equals, true)
+		c.Check(lastTS, Equals, "0001-01-01T00:00:00Z")
+		// Delete the field so the DeepEquals check can succeed
+		delete(got, "last-notice-timestamp")
+	}
+
+	data, _ := got["data"].(map[string]any)
 	c.Assert(data, NotNil)
 
 	c.Check(got, DeepEquals, expected)
@@ -1384,3 +1400,24 @@ func (ovs *overlordSuite) TestLockWithTimeoutFailed(c *C) {
 		"EXTEND_TIMEOUT_USEC=5000",
 	})
 }
+
+func (ovs *overlordSuite) TestAllStateManagersHaveEnsureLoggingTest(c *C) {
+	entries, err := os.ReadDir(".")
+	c.Assert(err, IsNil)
+
+	for _, entry := range entries {
+		if !entry.IsDir() || !strings.HasSuffix(entry.Name(), "state") {
+			continue
+		}
+		testPath := filepath.Join(entry.Name(), entry.Name()+"_test.go")
+		prefix := strings.TrimSuffix(entry.Name(), "state")
+		mgrPath := filepath.Join(entry.Name(), prefix+"mgr.go")
+		if !osutil.FileExists(testPath) || !osutil.FileExists(mgrPath) {
+			continue
+		}
+		content, err := os.ReadFile(testPath)
+		c.Assert(err, IsNil)
+		containsEnsureChecks := strings.Contains(string(content), fmt.Sprintf(`testutil.CheckEnsureLoopLogging("%s`, prefix+"mgr.go"))
+		c.Assert(containsEnsureChecks, Equals, true, Commentf("File %s does not contain a unit test that calls testutil.CheckEnsureLoopLogging on the file containing its Ensure() method", testPath))
+	}
+}
