@@ -20,8 +20,9 @@
 package servicestate_test
 
 import (
-	"io/ioutil"
+	"fmt"
 	"os"
+	"path"
 	"path/filepath"
 	"sort"
 	"strings"
@@ -32,15 +33,19 @@ import (
 
 	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/osutil/user"
 	"github.com/snapcore/snapd/overlord"
 	"github.com/snapcore/snapd/overlord/servicestate"
 	"github.com/snapcore/snapd/overlord/snapstate"
+	"github.com/snapcore/snapd/overlord/snapstate/snapstatetest"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
 	"github.com/snapcore/snapd/systemd"
 	"github.com/snapcore/snapd/systemd/systemdtest"
 	"github.com/snapcore/snapd/testutil"
+	"github.com/snapcore/snapd/wrappers"
 )
 
 func TestServiceControl(t *testing.T) { TestingT(t) }
@@ -66,6 +71,9 @@ apps:
     after: [bar]
   foo:
     daemon: simple
+  baz:
+    daemon: simple
+    daemon-scope: user
   bar:
     daemon: simple
     after: [foo]
@@ -113,7 +121,7 @@ func (s *serviceControlSuite) mockTestSnap(c *C) *snap.Info {
 	info := snaptest.MockSnap(c, servicesSnapYaml1, &si)
 	snapstate.Set(s.state, "test-snap", &snapstate.SnapState{
 		Active:   true,
-		Sequence: []*snap.SideInfo{&si},
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{&si}),
 		Current:  snap.R(7),
 		SnapType: "app",
 	})
@@ -121,7 +129,7 @@ func (s *serviceControlSuite) mockTestSnap(c *C) *snap.Info {
 	// mock systemd service units, this is required when testing "stop"
 	err := os.MkdirAll(filepath.Join(dirs.GlobalRootDir, "etc/systemd/system/"), 0775)
 	c.Assert(err, IsNil)
-	err = ioutil.WriteFile(filepath.Join(dirs.GlobalRootDir, "etc/systemd/system/snap.test-snap.foo.service"), nil, 0644)
+	err = os.WriteFile(filepath.Join(dirs.GlobalRootDir, "etc/systemd/system/snap.test-snap.foo.service"), nil, 0644)
 	c.Assert(err, IsNil)
 
 	return info
@@ -129,7 +137,7 @@ func (s *serviceControlSuite) mockTestSnap(c *C) *snap.Info {
 
 func verifyControlTasks(c *C, tasks []*state.Task, expectedAction, actionModifier string,
 	expectedServices []string, expectedExplicitServices []string) {
-	// sanity, ensures test checks below are hit
+	// precondition, ensures test checks below are hit
 	c.Assert(len(tasks) > 0, Equals, true)
 
 	splitServiceName := func(name string) (snapName, serviceName string) {
@@ -259,7 +267,7 @@ func makeControlChange(c *C, st *state.State, inst *servicestate.Instruction, in
 	}
 
 	flags := &servicestate.Flags{CreateExecCommandTasks: true}
-	tss, err := servicestate.Control(st, apps, inst, flags, nil)
+	tss, err := servicestate.Control(st, apps, inst, nil, flags, nil)
 	c.Assert(err, IsNil)
 
 	chg := st.NewChange("service-control", "...")
@@ -281,7 +289,7 @@ func (s *serviceControlSuite) TestControlDoesntCreateExecCommandTasksIfNoFlags(c
 	}
 
 	flags := &servicestate.Flags{}
-	tss, err := servicestate.Control(st, []*snap.AppInfo{info.Apps["foo"]}, inst, flags, nil)
+	tss, err := servicestate.Control(st, []*snap.AppInfo{info.Apps["foo"]}, inst, nil, flags, nil)
 	c.Assert(err, IsNil)
 	// service-control is the only task
 	c.Assert(tss, HasLen, 1)
@@ -304,7 +312,7 @@ func (s *serviceControlSuite) TestControlConflict(c *C) {
 	chg.AddTask(t)
 
 	inst := &servicestate.Instruction{Action: "start", Names: []string{"foo"}}
-	_, err := servicestate.Control(st, []*snap.AppInfo{inf.Apps["foo"]}, inst, nil, nil)
+	_, err := servicestate.Control(st, []*snap.AppInfo{inf.Apps["foo"]}, inst, nil, nil, nil)
 	c.Check(err, ErrorMatches, `snap "test-snap" has "manip" change in progress`)
 }
 
@@ -419,7 +427,7 @@ func (s *serviceControlSuite) TestControlUnknownInstruction(c *C) {
 		RestartOptions: client.RestartOptions{Reload: true},
 	}
 
-	_, err := servicestate.Control(st, []*snap.AppInfo{info.Apps["foo"]}, inst, nil, nil)
+	_, err := servicestate.Control(st, []*snap.AppInfo{info.Apps["foo"]}, inst, nil, nil, nil)
 	c.Assert(err, ErrorMatches, `unknown action "boo"`)
 }
 
@@ -505,7 +513,7 @@ func (s *serviceControlSuite) TestNoopWhenNoServices(c *C) {
 	snaptest.MockSnap(c, `name: test-snap`, &si)
 	snapstate.Set(st, "test-snap", &snapstate.SnapState{
 		Active:   true,
-		Sequence: []*snap.SideInfo{&si},
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{&si}),
 		Current:  snap.R(7),
 		SnapType: "app",
 	})
@@ -560,7 +568,7 @@ func (s *serviceControlSuite) TestUnknownService(c *C) {
 	cmd := &servicestate.ServiceAction{
 		SnapName: "test-snap",
 		Action:   "start",
-		Services: []string{"baz"},
+		Services: []string{"boo"},
 	}
 	t.Set("service-action", cmd)
 	chg.AddTask(t)
@@ -572,7 +580,7 @@ func (s *serviceControlSuite) TestUnknownService(c *C) {
 	c.Assert(err, IsNil)
 
 	c.Assert(t.Status(), Equals, state.ErrorStatus)
-	c.Check(chg.Err(), ErrorMatches, `cannot perform the following tasks:\n.*no such service: baz.*`)
+	c.Check(chg.Err(), ErrorMatches, `cannot perform the following tasks:\n.*no such service: boo.*`)
 }
 
 func (s *serviceControlSuite) TestNotAService(c *C) {
@@ -688,11 +696,46 @@ func (s *serviceControlSuite) TestStartEnableServices(c *C) {
 
 	c.Assert(t.Status(), Equals, state.DoneStatus)
 	c.Check(s.sysctlArgs, DeepEquals, [][]string{
-		{"enable", "snap.test-snap.foo.service"},
+		{"--no-reload", "enable", "snap.test-snap.foo.service"},
+		{"daemon-reload"},
 		{"start", "snap.test-snap.foo.service"},
 	})
 }
 
+func (s *serviceControlSuite) TestStartServicesWithScope(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	s.mockTestSnap(c)
+
+	chg := st.NewChange("service-control", "...")
+	t := st.NewTask("service-control", "...")
+	cmd := &servicestate.ServiceAction{
+		SnapName: "test-snap",
+		Action:   "start",
+		ScopeOptions: wrappers.ScopeOptions{
+			Scope: wrappers.ServiceScopeSystem,
+		},
+	}
+	t.Set("service-action", cmd)
+	chg.AddTask(t)
+
+	st.Unlock()
+	defer s.se.Stop()
+	err := s.o.Settle(5 * time.Second)
+	st.Lock()
+	c.Assert(err, IsNil)
+
+	// Baz should not appear in this list as we explicitly requested system services
+	c.Assert(t.Status(), Equals, state.DoneStatus)
+	c.Check(s.sysctlArgs, DeepEquals, [][]string{
+		{"start", "snap.test-snap.foo.service"},
+		{"start", "snap.test-snap.bar.service"},
+		{"start", "snap.test-snap.abc.service"},
+	})
+}
+
 func (s *serviceControlSuite) TestStartEnableMultipleServices(c *C) {
 	st := s.state
 	st.Lock()
@@ -718,9 +761,9 @@ func (s *serviceControlSuite) TestStartEnableMultipleServices(c *C) {
 
 	c.Assert(t.Status(), Equals, state.DoneStatus)
 	c.Check(s.sysctlArgs, DeepEquals, [][]string{
-		{"enable", "snap.test-snap.foo.service"},
-		{"enable", "snap.test-snap.bar.service"},
-		{"enable", "snap.test-snap.abc.service"},
+		{"--no-reload", "enable", "snap.test-snap.foo.service", "snap.test-snap.bar.service", "snap.test-snap.abc.service"},
+		{"daemon-reload"},
+		{"--user", "--global", "--no-reload", "enable", "snap.test-snap.baz.service"},
 		{"start", "snap.test-snap.foo.service"},
 		{"start", "snap.test-snap.bar.service"},
 		{"start", "snap.test-snap.abc.service"},
@@ -757,6 +800,39 @@ func (s *serviceControlSuite) TestStopServices(c *C) {
 	})
 }
 
+func (s *serviceControlSuite) TestStopServicesWithScope(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	s.mockTestSnap(c)
+
+	chg := st.NewChange("service-control", "...")
+	t := st.NewTask("service-control", "...")
+	cmd := &servicestate.ServiceAction{
+		SnapName: "test-snap",
+		Action:   "stop",
+		ScopeOptions: wrappers.ScopeOptions{
+			Scope: wrappers.ServiceScopeSystem,
+		},
+	}
+	t.Set("service-action", cmd)
+	chg.AddTask(t)
+
+	st.Unlock()
+	defer s.se.Stop()
+	err := s.o.Settle(5 * time.Second)
+	st.Lock()
+	c.Assert(err, IsNil)
+
+	// we only mock service unit for foo, expect only foo to appear
+	c.Assert(t.Status(), Equals, state.DoneStatus)
+	c.Check(s.sysctlArgs, DeepEquals, [][]string{
+		{"stop", "snap.test-snap.foo.service"},
+		{"show", "--property=ActiveState", "snap.test-snap.foo.service"},
+	})
+}
+
 func (s *serviceControlSuite) TestStopDisableServices(c *C) {
 	st := s.state
 	st.Lock()
@@ -785,7 +861,8 @@ func (s *serviceControlSuite) TestStopDisableServices(c *C) {
 	c.Check(s.sysctlArgs, DeepEquals, [][]string{
 		{"stop", "snap.test-snap.foo.service"},
 		{"show", "--property=ActiveState", "snap.test-snap.foo.service"},
-		{"disable", "snap.test-snap.foo.service"},
+		{"--no-reload", "disable", "snap.test-snap.foo.service"},
+		{"daemon-reload"},
 	})
 }
 
@@ -820,6 +897,158 @@ func (s *serviceControlSuite) TestRestartServices(c *C) {
 	})
 }
 
+func (s *serviceControlSuite) TestRestartServicesWithScope(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	s.mockTestSnap(c)
+
+	chg := st.NewChange("service-control", "...")
+	t := st.NewTask("service-control", "...")
+	cmd := &servicestate.ServiceAction{
+		SnapName: "test-snap",
+		Action:   "restart",
+		ScopeOptions: wrappers.ScopeOptions{
+			Scope: wrappers.ServiceScopeSystem,
+		},
+	}
+	t.Set("service-action", cmd)
+	chg.AddTask(t)
+
+	st.Unlock()
+	defer s.se.Stop()
+	err := s.o.Settle(5 * time.Second)
+	st.Lock()
+	c.Assert(err, IsNil)
+
+	c.Assert(t.Status(), Equals, state.DoneStatus)
+	c.Check(s.sysctlArgs, DeepEquals, [][]string{
+		{"stop", "snap.test-snap.foo.service"},
+		{"show", "--property=ActiveState", "snap.test-snap.foo.service"},
+		{"start", "snap.test-snap.foo.service"},
+		{"stop", "snap.test-snap.bar.service"},
+		{"show", "--property=ActiveState", "snap.test-snap.bar.service"},
+		{"start", "snap.test-snap.bar.service"},
+		{"stop", "snap.test-snap.abc.service"},
+		{"show", "--property=ActiveState", "snap.test-snap.abc.service"},
+		{"start", "snap.test-snap.abc.service"},
+	})
+}
+
+func (s *serviceControlSuite) TestRestartOnlyActiveServices(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	s.mockTestSnap(c)
+
+	srvFoo := "snap.test-snap.foo.service"
+	srvBar := "snap.test-snap.bar.service"
+
+	systemctlRestorer := systemd.MockSystemctl(func(cmd ...string) (buf []byte, err error) {
+		states := map[string]systemdtest.ServiceState{
+			srvFoo: {ActiveState: "inactive", UnitFileState: "enabled"},
+			srvBar: {ActiveState: "active", UnitFileState: "enabled"},
+		}
+		if out := systemdtest.HandleMockAllUnitsActiveOutput(cmd, states); out != nil {
+			return out, nil
+		}
+
+		s.sysctlArgs = append(s.sysctlArgs, cmd)
+		if cmd[0] == "show" {
+			return []byte("ActiveState=inactive\n"), nil
+		}
+		return nil, nil
+	})
+	s.AddCleanup(systemctlRestorer)
+
+	chg := st.NewChange("service-control", "...")
+	t := st.NewTask("service-control", "...")
+	cmd := &servicestate.ServiceAction{
+		SnapName:                "test-snap",
+		Action:                  "restart",
+		Services:                []string{"foo", "bar"},
+		RestartEnabledNonActive: false,
+	}
+	t.Set("service-action", cmd)
+	chg.AddTask(t)
+
+	st.Unlock()
+	defer s.se.Stop()
+	err := s.o.Settle(5 * time.Second)
+	st.Lock()
+	c.Assert(err, IsNil)
+
+	c.Assert(t.Status(), Equals, state.DoneStatus)
+
+	expectedInvocations := [][]string{
+		{"stop", srvBar},
+		{"show", "--property=ActiveState", srvBar},
+		{"start", srvBar},
+	}
+	// We expect to get only bar restarted, as foo is inactive
+	c.Check(s.sysctlArgs, DeepEquals, expectedInvocations)
+}
+
+func (s *serviceControlSuite) TestRestartAllEnabledServices(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	s.mockTestSnap(c)
+
+	srvFoo := "snap.test-snap.foo.service"
+	srvBar := "snap.test-snap.bar.service"
+
+	systemctlRestorer := systemd.MockSystemctl(func(cmd ...string) (buf []byte, err error) {
+		states := map[string]systemdtest.ServiceState{
+			srvFoo: {ActiveState: "inactive", UnitFileState: "enabled"},
+			srvBar: {ActiveState: "active", UnitFileState: "enabled"},
+		}
+		if out := systemdtest.HandleMockAllUnitsActiveOutput(cmd, states); out != nil {
+			return out, nil
+		}
+
+		s.sysctlArgs = append(s.sysctlArgs, cmd)
+		if cmd[0] == "show" {
+			return []byte("ActiveState=inactive\n"), nil
+		}
+		return nil, nil
+	})
+	s.AddCleanup(systemctlRestorer)
+
+	chg := st.NewChange("service-control", "...")
+	t := st.NewTask("service-control", "...")
+	cmd := &servicestate.ServiceAction{
+		SnapName:                "test-snap",
+		Action:                  "restart",
+		Services:                []string{"foo", "bar"},
+		RestartEnabledNonActive: true,
+	}
+	t.Set("service-action", cmd)
+	chg.AddTask(t)
+
+	st.Unlock()
+	defer s.se.Stop()
+	err := s.o.Settle(5 * time.Second)
+	st.Lock()
+	c.Assert(err, IsNil)
+
+	c.Assert(t.Status(), Equals, state.DoneStatus)
+
+	expectedInvocations := [][]string{
+		{"stop", srvFoo},
+		{"show", "--property=ActiveState", srvFoo},
+		{"start", srvFoo},
+		{"stop", srvBar},
+		{"show", "--property=ActiveState", srvBar},
+		{"start", srvBar},
+	}
+	// We expect both foo and bar restarted
+	c.Check(s.sysctlArgs, DeepEquals, expectedInvocations)
+}
+
 func (s *serviceControlSuite) testRestartWithExplicitServicesCommon(c *C,
 	explicitServices []string, expectedInvocations [][]string) {
 	st := s.state
@@ -1028,7 +1257,7 @@ func (s *serviceControlSuite) TestConflict(c *C) {
 	c.Assert(err, ErrorMatches, `snap "test-snap" has "service-control" change in progress`)
 }
 
-func (s *serviceControlSuite) TestUpdateSnapstateServices(c *C) {
+func (s *serviceControlSuite) TestUpdateSnapstateSystemServices(c *C) {
 	var tests = []struct {
 		enable                    []string
 		disable                   []string
@@ -1081,12 +1310,20 @@ func (s *serviceControlSuite) TestUpdateSnapstateServices(c *C) {
 	for _, tst := range tests {
 		var enable, disable []*snap.AppInfo
 		for _, srv := range tst.enable {
-			enable = append(enable, &snap.AppInfo{Name: srv})
+			enable = append(enable, &snap.AppInfo{
+				Name:        srv,
+				Daemon:      "simple",
+				DaemonScope: snap.SystemDaemon,
+			})
 		}
 		for _, srv := range tst.disable {
-			disable = append(disable, &snap.AppInfo{Name: srv})
+			disable = append(disable, &snap.AppInfo{
+				Name:        srv,
+				Daemon:      "simple",
+				DaemonScope: snap.SystemDaemon,
+			})
 		}
-		result, err := servicestate.UpdateSnapstateServices(&snapst, enable, disable)
+		result, err := servicestate.UpdateSnapstateServices(&snapst, enable, disable, wrappers.ScopeOptions{})
 		c.Assert(err, IsNil)
 		c.Check(result, Equals, tst.changed)
 		c.Check(snapst.ServicesEnabledByHooks, DeepEquals, tst.expectedSnapstateEnabled)
@@ -1094,6 +1331,264 @@ func (s *serviceControlSuite) TestUpdateSnapstateServices(c *C) {
 	}
 
 	services := []*snap.AppInfo{{Name: "foo"}}
-	_, err := servicestate.UpdateSnapstateServices(nil, services, services)
+	_, err := servicestate.UpdateSnapstateServices(nil, services, services, wrappers.ScopeOptions{})
 	c.Assert(err, ErrorMatches, `internal error: cannot handle enabled and disabled services at the same time`)
 }
+
+func (s *serviceControlSuite) TestUpdateSnapstateServicesIgnoresNonServices(c *C) {
+	snapst := snapstate.SnapState{}
+
+	enable := []*snap.AppInfo{
+		{
+			Name:        "foo",
+			Daemon:      "simple",
+			DaemonScope: snap.SystemDaemon,
+		},
+		{
+			Name: "baz",
+		},
+	}
+	disable := []*snap.AppInfo{
+		{
+			Name:        "bar",
+			Daemon:      "simple",
+			DaemonScope: snap.SystemDaemon,
+		},
+		{
+			Name: "jazz",
+		},
+	}
+
+	result, err := servicestate.UpdateSnapstateServices(&snapst, enable, nil, wrappers.ScopeOptions{})
+	c.Assert(err, IsNil)
+	c.Check(result, Equals, true)
+	c.Check(snapst.ServicesEnabledByHooks, DeepEquals, []string{"foo"})
+	c.Check(snapst.ServicesDisabledByHooks, HasLen, 0)
+
+	result, err = servicestate.UpdateSnapstateServices(&snapst, nil, disable, wrappers.ScopeOptions{})
+	c.Assert(err, IsNil)
+	c.Check(result, Equals, true)
+	c.Check(snapst.ServicesEnabledByHooks, DeepEquals, []string{"foo"})
+	c.Check(snapst.ServicesDisabledByHooks, DeepEquals, []string{"bar"})
+}
+
+func (s *serviceControlSuite) TestUpdateSnapstateUserServices(c *C) {
+	// fake two sockets, one for 0 and one for 1000
+	err := os.MkdirAll(path.Join(dirs.XdgRuntimeDirBase, "0", "snapd-session-agent.socket"), 0700)
+	c.Assert(err, IsNil)
+	err = os.MkdirAll(path.Join(dirs.XdgRuntimeDirBase, "1000", "snapd-session-agent.socket"), 0700)
+	c.Assert(err, IsNil)
+
+	osutil.MockUserLookup(func(s string) (*user.User, error) {
+		switch s {
+		case "root":
+			return &user.User{
+				Uid: "0",
+			}, nil
+		case "test":
+			return &user.User{
+				Uid: "1000",
+			}, nil
+		default:
+			return nil, fmt.Errorf("unknown user %s", s)
+		}
+	})
+
+	var tests = []struct {
+		enable                    []string
+		disable                   []string
+		users                     []string
+		expectedSnapstateEnabled  map[int][]string
+		expectedSnapstateDisabled map[int][]string
+		changed                   bool
+	}{
+		// These test scenarios share a single SnapState instance and accumulate
+		// changes to ServicesEnabledByHooks and ServicesDisabledByHooks.
+		{
+			changed: false,
+		},
+		// first one affects specific users
+		{
+			enable: []string{"a"},
+			users:  []string{"root", "test"},
+			expectedSnapstateEnabled: map[int][]string{
+				0:    {"a"},
+				1000: {"a"},
+			},
+			changed: true,
+		},
+		// second one enables for all, should cause no change
+		{
+			enable: []string{"a"},
+			users:  []string{},
+			expectedSnapstateEnabled: map[int][]string{
+				0:    {"a"},
+				1000: {"a"},
+			},
+			changed: false,
+		},
+		// third enable for a specific user again does nothing
+		{
+			enable: []string{"a"},
+			users:  []string{"test"},
+			expectedSnapstateEnabled: map[int][]string{
+				0:    {"a"},
+				1000: {"a"},
+			},
+			changed: false,
+		},
+		// next up we disable one for a specific user
+		{
+			disable: []string{"a"},
+			users:   []string{"root"},
+			expectedSnapstateEnabled: map[int][]string{
+				1000: {"a"},
+			},
+			expectedSnapstateDisabled: map[int][]string{
+				0: {"a"},
+			},
+			changed: true,
+		},
+		// now we re-enable multiple for specific user
+		{
+			enable: []string{"a", "c"},
+			users:  []string{"root"},
+			expectedSnapstateEnabled: map[int][]string{
+				0:    {"a", "c"},
+				1000: {"a"},
+			},
+			expectedSnapstateDisabled: map[int][]string{},
+			changed:                   true,
+		},
+		// We now mix it up with disabling for everyone
+		{
+			disable: []string{"b"},
+			users:   []string{},
+			expectedSnapstateEnabled: map[int][]string{
+				0:    {"a", "c"},
+				1000: {"a"},
+			},
+			expectedSnapstateDisabled: map[int][]string{
+				0:    {"b"},
+				1000: {"b"},
+			},
+			changed: true,
+		},
+		// We now mix it up with disabling an existing one that only is
+		// enabled for one user
+		{
+			disable: []string{"c"},
+			users:   []string{},
+			expectedSnapstateEnabled: map[int][]string{
+				0:    {"a"},
+				1000: {"a"},
+			},
+			expectedSnapstateDisabled: map[int][]string{
+				0:    {"b", "c"},
+				1000: {"b", "c"},
+			},
+			changed: true,
+		},
+		{
+			disable: []string{"b", "c"},
+			users:   []string{},
+			expectedSnapstateEnabled: map[int][]string{
+				0:    {"a"},
+				1000: {"a"},
+			},
+			expectedSnapstateDisabled: map[int][]string{
+				0:    {"b", "c"},
+				1000: {"b", "c"},
+			},
+			changed: false,
+		},
+	}
+
+	snapst := snapstate.SnapState{}
+
+	for _, tst := range tests {
+		var enable, disable []*snap.AppInfo
+		for _, srv := range tst.enable {
+			enable = append(enable, &snap.AppInfo{
+				Name:        srv,
+				Daemon:      "simple",
+				DaemonScope: snap.UserDaemon,
+			})
+		}
+		for _, srv := range tst.disable {
+			disable = append(disable, &snap.AppInfo{
+				Name:        srv,
+				Daemon:      "simple",
+				DaemonScope: snap.UserDaemon,
+			})
+		}
+		result, err := servicestate.UpdateSnapstateServices(&snapst, enable, disable, wrappers.ScopeOptions{Users: tst.users})
+		c.Assert(err, IsNil)
+		c.Assert(result, Equals, tst.changed)
+		c.Assert(snapst.UserServicesEnabledByHooks, DeepEquals, tst.expectedSnapstateEnabled)
+		c.Assert(snapst.UserServicesDisabledByHooks, DeepEquals, tst.expectedSnapstateDisabled)
+	}
+}
+
+func (s *serviceControlSuite) TestUpdateSnapstateUserServicesFailsOnUserError(c *C) {
+	// fake two sockets, one for 0 and one for 1000
+	err := os.MkdirAll(path.Join(dirs.XdgRuntimeDirBase, "0", "snapd-session-agent.socket"), 0700)
+	c.Assert(err, IsNil)
+	err = os.MkdirAll(path.Join(dirs.XdgRuntimeDirBase, "1000", "snapd-session-agent.socket"), 0700)
+	c.Assert(err, IsNil)
+
+	osutil.MockUserLookup(func(s string) (*user.User, error) {
+		return nil, fmt.Errorf("unknown user %s", s)
+	})
+
+	snapst := snapstate.SnapState{}
+	enable := []*snap.AppInfo{
+		{
+			Name:        "foo",
+			Daemon:      "simple",
+			DaemonScope: snap.SystemDaemon,
+		},
+	}
+
+	result, err := servicestate.UpdateSnapstateServices(&snapst, enable, nil, wrappers.ScopeOptions{
+		Scope: wrappers.ServiceScopeUser,
+		Users: []string{"test"},
+	})
+	c.Assert(err, ErrorMatches, `unknown user test`)
+	c.Assert(result, Equals, false)
+	c.Assert(snapst.UserServicesEnabledByHooks, HasLen, 0)
+	c.Assert(snapst.UserServicesDisabledByHooks, HasLen, 0)
+}
+
+func (s *serviceControlSuite) TestUpdateSnapstateUserServicesFailsOnInvalidUID(c *C) {
+	// fake two sockets, one for 0 and one for 1000
+	err := os.MkdirAll(path.Join(dirs.XdgRuntimeDirBase, "0", "snapd-session-agent.socket"), 0700)
+	c.Assert(err, IsNil)
+	err = os.MkdirAll(path.Join(dirs.XdgRuntimeDirBase, "1000", "snapd-session-agent.socket"), 0700)
+	c.Assert(err, IsNil)
+
+	osutil.MockUserLookup(func(s string) (*user.User, error) {
+		return &user.User{
+			Uid:      "wups",
+			Username: s,
+		}, nil
+	})
+
+	snapst := snapstate.SnapState{}
+	enable := []*snap.AppInfo{
+		{
+			Name:        "foo",
+			Daemon:      "simple",
+			DaemonScope: snap.SystemDaemon,
+		},
+	}
+
+	result, err := servicestate.UpdateSnapstateServices(&snapst, enable, nil, wrappers.ScopeOptions{
+		Scope: wrappers.ServiceScopeUser,
+		Users: []string{"test"},
+	})
+	c.Assert(err, ErrorMatches, `strconv.Atoi: parsing "wups": invalid syntax`)
+	c.Assert(result, Equals, false)
+	c.Assert(snapst.UserServicesEnabledByHooks, HasLen, 0)
+	c.Assert(snapst.UserServicesDisabledByHooks, HasLen, 0)
+}
