@@ -29,7 +29,6 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
 	"path"
 	"path/filepath"
@@ -63,10 +62,11 @@ var (
 	// Stop is used to ask Iter to stop iteration, without it being an error.
 	Stop = errors.New("stop iteration")
 
-	osOpen      = os.Open
-	dirNames    = (*os.File).Readdirnames
-	backendOpen = Open
-	timeNow     = time.Now
+	osOpen               = os.Open
+	dirNames             = (*os.File).Readdirnames
+	backendOpen          = Open
+	timeNow              = time.Now
+	snapReadSnapshotYaml = snap.ReadSnapshotYaml
 
 	usersForUsernames = usersForUsernamesImpl
 )
@@ -258,7 +258,7 @@ func isSnapshotFilename(filePath string) (ok bool, setID uint64) {
 }
 
 // EstimateSnapshotSize calculates estimated size of the snapshot.
-func EstimateSnapshotSize(si *snap.Info, usernames []string, opts *dirs.SnapDirOptions) (uint64, error) {
+func EstimateSnapshotSize(si *snap.Info, usernames []string, dirOpts *dirs.SnapDirOptions) (uint64, error) {
 	var total uint64
 	calculateSize := func(path string, finfo os.FileInfo, err error) error {
 		if finfo.Mode().IsRegular() {
@@ -284,15 +284,15 @@ func EstimateSnapshotSize(si *snap.Info, usernames []string, opts *dirs.SnapDirO
 		}
 	}
 
-	users, err := usersForUsernames(usernames, opts)
+	users, err := usersForUsernames(usernames, dirOpts)
 	if err != nil {
 		return 0, err
 	}
 	for _, usr := range users {
-		if err := visitDir(si.UserDataDir(usr.HomeDir, opts)); err != nil {
+		if err := visitDir(si.UserDataDir(usr.HomeDir, dirOpts)); err != nil {
 			return 0, err
 		}
-		if err := visitDir(si.UserCommonDataDir(usr.HomeDir, opts)); err != nil {
+		if err := visitDir(si.UserCommonDataDir(usr.HomeDir, dirOpts)); err != nil {
 			return 0, err
 		}
 	}
@@ -302,7 +302,7 @@ func EstimateSnapshotSize(si *snap.Info, usernames []string, opts *dirs.SnapDirO
 }
 
 // Save a snapshot
-func Save(ctx context.Context, id uint64, si *snap.Info, cfg map[string]interface{}, usernames []string, opts *dirs.SnapDirOptions) (*client.Snapshot, error) {
+func Save(ctx context.Context, id uint64, si *snap.Info, cfg map[string]any, usernames []string, dynSnapshotOpts *snap.SnapshotOptions, dirOpts *dirs.SnapDirOptions) (*client.Snapshot, error) {
 	if err := os.MkdirAll(dirs.SnapshotsDir, 0700); err != nil {
 		return nil, err
 	}
@@ -315,17 +315,26 @@ func Save(ctx context.Context, id uint64, si *snap.Info, cfg map[string]interfac
 		Version:  si.Version,
 		Epoch:    si.Epoch,
 		Time:     timeNow(),
+		// Pass only dynamic snapshot options here. Static options are tied to the snap version
+		// and should not be repeated in snapshot metadata on every save.
+		Options:  dynSnapshotOpts,
 		SHA3_384: make(map[string]string),
 		Size:     0,
 		Conf:     cfg,
 		// Note: Auto is no longer set in the Snapshot.
 	}
 
-	snapshotOptions, err := snap.ReadSnapshotYaml(si)
+	snapshotOptions, err := snapReadSnapshotYaml(si)
 	if err != nil {
 		return nil, err
 	}
 
+	if dynSnapshotOpts != nil {
+		if err := snapshotOptions.MergeDynamicExcludes(dynSnapshotOpts.Exclude); err != nil {
+			return nil, err
+		}
+	}
+
 	aw, err := osutil.NewAtomicFile(Filename(snapshot), 0600, 0, osutil.NoChown, osutil.NoChown)
 	if err != nil {
 		return nil, err
@@ -337,19 +346,19 @@ func Save(ctx context.Context, id uint64, si *snap.Info, cfg map[string]interfac
 	defer w.Close() // note this does not close the file descriptor (that's done by hand on the atomic writer, above)
 	savingUserData := false
 	baseDataDir := snap.BaseDataDir(si.InstanceName())
-	if err := addSnapDirToZip(ctx, snapshot, w, "root", archiveName, baseDataDir, savingUserData, snapshotOptions.ExcludePaths); err != nil {
+	if err := addSnapDirToZip(ctx, snapshot, w, "root", archiveName, baseDataDir, savingUserData, snapshotOptions.Exclude); err != nil {
 		return nil, err
 	}
 
-	users, err := usersForUsernames(usernames, opts)
+	users, err := usersForUsernames(usernames, dirOpts)
 	if err != nil {
 		return nil, err
 	}
 
 	savingUserData = true
 	for _, usr := range users {
-		snapDataDir := filepath.Dir(si.UserDataDir(usr.HomeDir, opts))
-		if err := addSnapDirToZip(ctx, snapshot, w, usr.Username, userArchiveName(usr), snapDataDir, savingUserData, snapshotOptions.ExcludePaths); err != nil {
+		snapDataDir := filepath.Dir(si.UserDataDir(usr.HomeDir, dirOpts))
+		if err := addSnapDirToZip(ctx, snapshot, w, usr.Username, userArchiveName(usr), snapDataDir, savingUserData, snapshotOptions.Exclude); err != nil {
 			return nil, err
 		}
 	}
@@ -557,7 +566,7 @@ func (me *multiError) Error() string {
 	return me.nestedError(0)
 }
 
-// helper to ensure formating of nested multiErrors works.
+// helper to ensure formatting of nested multiErrors works.
 func (me *multiError) nestedError(level int) string {
 	indent := strings.Repeat(" ", level)
 	buf := bytes.NewBufferString(fmt.Sprintf("%s:\n", me.header))
@@ -676,7 +685,7 @@ func (t *importTransaction) Commit() error {
 }
 
 func (t *importTransaction) lock() error {
-	return ioutil.WriteFile(t.lockPath, nil, 0644)
+	return os.WriteFile(t.lockPath, nil, 0644)
 }
 
 func (t *importTransaction) unlock() error {
@@ -685,14 +694,14 @@ func (t *importTransaction) unlock() error {
 
 var filepathGlob = filepath.Glob
 
-// CleanupAbandondedImports will clean any import that is in progress.
+// CleanupAbandonedImports will clean any import that is in progress.
 // This is meant to be called at startup of snapd before any real imports
 // happen. It is not safe to run this concurrently with any other snapshot
 // operation.
 //
 // The amount of snapshots cleaned is returned and an error if one or
 // more cleanups did not succeed.
-func CleanupAbandondedImports() (cleaned int, err error) {
+func CleanupAbandonedImports() (cleaned int, err error) {
 	inProgressSnapshots, err := filepathGlob(filepath.Join(dirs.SnapshotsDir, importingFnGlob))
 	if err != nil {
 		return 0, err
@@ -999,7 +1008,7 @@ func (se *SnapshotExport) Init() error {
 	// but a known issue with this approach here.
 	var sz osutil.Sizer
 	if err := se.StreamTo(&sz); err != nil {
-		return fmt.Errorf("cannot calculcate the size for %v: %s", se.setID, err)
+		return fmt.Errorf("cannot calculate the size for %v: %s", se.setID, err)
 	}
 	se.size = sz.Size()
 	return nil
@@ -1072,6 +1081,16 @@ func (se *SnapshotExport) StreamTo(w io.Writer) error {
 		files = append(files, path.Base(snapshotFile.Name()))
 	}
 
+	// SnapshotExporter has se.Close() set as a finalizer, thus when the object
+	// is no longer referenced, se.Close() (which closes all files) will be
+	// called automatically after/during a GC pass. We don't know if the caller
+	// retains a reference to the object (eg. for any outstanding calls to some
+	// of its functions), and the last explicit reference in the code above was
+	// kept for the purpose of accessing the snapshot files list, which is done
+	// before the final file is read, so we need to mark object as alive until
+	// after every file has been read.
+	runtime.KeepAlive(se)
+
 	// write the metadata last, then the client can use that to
 	// validate the archive is complete
 	meta := exportMetadata{
