@@ -128,6 +128,37 @@ func (n *Notice) Reoccur(now time.Time, data map[string]string, repeatAfter time
 	return repeated
 }
 
+// DeepCopy returns a deep copy of the receiver.
+func (n *Notice) DeepCopy() *Notice {
+	// Create deep copies of non-primitive fields (strings are fine)
+	var userID *uint32
+	if n.userID != nil {
+		userIDVal := *n.userID
+		userID = &userIDVal
+	}
+	var lastData map[string]string
+	if len(n.lastData) > 0 {
+		lastData = make(map[string]string, len(n.lastData))
+		for k, v := range n.lastData {
+			lastData[k] = v
+		}
+	}
+
+	return &Notice{
+		id:            n.id,
+		userID:        userID,
+		noticeType:    n.noticeType,
+		key:           n.key,
+		firstOccurred: n.firstOccurred,
+		lastOccurred:  n.lastOccurred,
+		lastRepeated:  n.lastRepeated,
+		occurrences:   n.occurrences,
+		lastData:      lastData,
+		repeatAfter:   n.repeatAfter,
+		expireAfter:   n.expireAfter,
+	}
+}
+
 func (n *Notice) String() string {
 	userIDStr := "public"
 	if n.userID != nil {
@@ -136,6 +167,11 @@ func (n *Notice) String() string {
 	return fmt.Sprintf("Notice %s (%s:%s:%s)", n.id, userIDStr, n.noticeType, n.key)
 }
 
+// ID returns the notice's ID.
+func (n *Notice) ID() string {
+	return n.id
+}
+
 // UserID returns the value of the notice's user ID and whether it is set.
 // If it is nil, then the returned userID is 0, and isSet is false.
 func (n *Notice) UserID() (userID uint32, isSet bool) {
@@ -150,6 +186,21 @@ func (n *Notice) Type() NoticeType {
 	return n.noticeType
 }
 
+// Key returns the notice's key.
+func (n *Notice) Key() string {
+	return n.key
+}
+
+// LastRepeated returns the last repeated timestamp for this notice.
+func (n *Notice) LastRepeated() time.Time {
+	return n.lastRepeated
+}
+
+// LastData returns the last data associated with this notice.
+func (n *Notice) LastData() map[string]string {
+	return n.lastData
+}
+
 func flattenUserID(userID *uint32) (uid uint32, isSet bool) {
 	if userID == nil {
 		return 0, false
@@ -157,8 +208,8 @@ func flattenUserID(userID *uint32) (uid uint32, isSet bool) {
 	return *userID, true
 }
 
-// expired reports whether this notice has expired (relative to the given "now").
-func (n *Notice) expired(now time.Time) bool {
+// Expired reports whether this notice has expired (relative to the given "now").
+func (n *Notice) Expired(now time.Time) bool {
 	return n.lastOccurred.Add(n.expireAfter).Before(now)
 }
 
@@ -337,6 +388,8 @@ func (s *State) AddNotice(userID *uint32, noticeType NoticeType, key string, opt
 	}
 
 	s.writing()
+	s.noticesMu.Lock()
+	defer s.noticesMu.Unlock()
 
 	now := options.Time
 	if now.IsZero() {
@@ -474,12 +527,14 @@ func (f *NoticeFilter) futureNoticesPossible(now time.Time) bool {
 // from state to another notice backend.
 func (s *State) DrainNotices(filter *NoticeFilter) []*Notice {
 	s.writing()
+	s.noticesMu.Lock()
+	defer s.noticesMu.Unlock()
 
 	now := time.Now()
 	var toRemove []noticeKey
 	var notices []*Notice
 	for k, n := range s.notices {
-		if n.expired(now) || !filter.matches(n) {
+		if n.Expired(now) || !filter.matches(n) {
 			continue
 		}
 		toRemove = append(toRemove, k)
@@ -488,27 +543,39 @@ func (s *State) DrainNotices(filter *NoticeFilter) []*Notice {
 	for _, k := range toRemove {
 		delete(s.notices, k)
 	}
+	SortNotices(notices)
+	return notices
+}
+
+// SortNotices sorts the given slice of notices according to the lastRepeated
+// timestamp.
+func SortNotices(notices []*Notice) {
 	sort.Slice(notices, func(i, j int) bool {
 		return notices[i].lastRepeated.Before(notices[j].lastRepeated)
 	})
-	return notices
 }
 
 // Notices returns the list of notices that match the filter (if any),
 // ordered by the last-repeated time.
 func (s *State) Notices(filter *NoticeFilter) []*Notice {
-	s.reading()
+	s.noticesMu.RLock()
+	defer s.noticesMu.RUnlock()
+	return s.doNotices(filter)
+}
 
-	notices := s.flattenNotices(filter)
-	sort.Slice(notices, func(i, j int) bool {
-		return notices[i].lastRepeated.Before(notices[j].lastRepeated)
-	})
+// doNotices returns the list of notices that match the filter (if any),
+// ordered by the last-repeated time. The caller must hold the noticesMu for
+// reading.
+func (s *State) doNotices(filter *NoticeFilter) []*Notice {
+	notices := s.filterNotices(filter)
+	SortNotices(notices)
 	return notices
 }
 
 // Notice returns a single notice by ID, or nil if not found.
 func (s *State) Notice(id string) *Notice {
-	s.reading()
+	s.noticesMu.RLock()
+	defer s.noticesMu.RUnlock()
 
 	// Could use another map for lookup, but the number of notices will likely
 	// be small, and this function is probably only used rarely, so performance
@@ -521,11 +588,24 @@ func (s *State) Notice(id string) *Notice {
 	return nil
 }
 
-func (s *State) flattenNotices(filter *NoticeFilter) []*Notice {
+// flattenNotices loops over the notices map and returns all non-expired notices
+// so that they can be marshalled to disk. The notices are not sorted.
+//
+// State lock does not need to be held, and this method acquires noticesMu for
+// reading, so noticesMu must not be held for writing by the caller.
+func (s *State) flattenNotices() []*Notice {
+	s.noticesMu.RLock()
+	defer s.noticesMu.RUnlock()
+	return s.filterNotices(nil)
+}
+
+// filterNotices returns the list of notices that match the filter (if any),
+// without sorting them. The caller must hold the noticesMu for reading.
+func (s *State) filterNotices(filter *NoticeFilter) []*Notice {
 	now := time.Now()
 	var notices []*Notice
 	for _, n := range s.notices {
-		if n.expired(now) || !filter.matches(n) {
+		if n.Expired(now) || !filter.matches(n) {
 			continue
 		}
 		notices = append(notices, n)
@@ -533,11 +613,17 @@ func (s *State) flattenNotices(filter *NoticeFilter) []*Notice {
 	return notices
 }
 
+// unflattenNotices takes a flat list of notices and replaces the notices map
+// with them, ignoring expired notices in the process.
+//
+// Call with the state lock held. Acquires the notices lock for writing.
 func (s *State) unflattenNotices(flat []*Notice) {
+	s.noticesMu.Lock()
+	defer s.noticesMu.Unlock()
 	now := time.Now()
 	s.notices = make(map[noticeKey]*Notice)
 	for _, n := range flat {
-		if n.expired(now) {
+		if n.Expired(now) {
 			continue
 		}
 		userID, hasUserID := n.UserID()
@@ -555,14 +641,28 @@ func (s *State) unflattenNotices(flat []*Notice) {
 // cancelled, or the timestamp of the BeforeOrAt filter has passed (if it is
 // nonzero). If there are existing notices that match the filter, WaitNotices
 // will return them immediately.
+//
+// The caller should not hold state lock, since this function will not release
+// state lock while waiting for a new notice to be added. Adding a new notice
+// requires both state lock and an internal notices RWMutex to be held for
+// writing. Previously, state lock was used as the noticeCond locker, which is
+// unlocked when noticeCond.Wait is called, but now the notices RWMutex RLocker
+// is used instead, so a locked state would remain locked and noticeCond.Wait
+// would block until the context is cancelled.
 func (s *State) WaitNotices(ctx context.Context, filter *NoticeFilter) ([]*Notice, error) {
-	s.reading()
+	s.noticesMu.RLock()
+	defer s.noticesMu.RUnlock()
+
+	// It's important that we do not attempt to lock noticesMu for reading again
+	// during the rest of the function call, since any attempt to lock it for
+	// writing (either within the context timeout callback or externally) will
+	// block any attempted call to RLock in this function, and thus prevent us
+	// from releasing the RLock we already hold, and lead to deadlock.
 
 	// If there are existing notices, return them right away.
 	//
-	// State is already locked here by the caller, so notices won't be added
-	// concurrently.
-	notices := s.Notices(filter)
+	// noticesMu is already locked here, so notices won't be added concurrently.
+	notices := s.doNotices(filter)
 	if len(notices) > 0 {
 		return notices, nil
 	}
@@ -572,28 +672,34 @@ func (s *State) WaitNotices(ctx context.Context, filter *NoticeFilter) ([]*Notic
 	//
 	// TODO: replace this with context.AfterFunc once we're on Go 1.21.
 	stop := contextAfterFunc(ctx, func() {
-		// We need to acquire the cond lock here to be sure that the Broadcast
-		// below won't occur before the call to Wait, which would result in a
-		// missed signal (and deadlock).
-		s.noticeCond.L.Lock()
-		defer s.noticeCond.L.Unlock()
+		// We need to acquire a lock mutually exclusive with the cond lock here
+		// to be sure that the Broadcast below won't occur before the call to
+		// Wait, which would result in a missed signal (and deadlock). Since
+		// the cond lock is noticesMu.RLocker(), we need to acquire the lock
+		// for writing.
+		s.noticesMu.Lock()
+		defer s.noticesMu.Unlock()
 
 		s.noticeCond.Broadcast()
 	})
 	defer stop()
 
 	for {
-		// Since the state lock is held for the duration of AddNotice, there
-		// can be no notices currently in-flight which have timestamps before
-		// now but have not yet been added to the notices map. Therefore, if
-		// the current time is after the BeforeOrAt filter, we know there can
-		// be no new notices which match the filter.
+		// Since the noticesMu is held for writing for the duration of
+		// AddNotice, there can be no notices destined for the state notices
+		// map currently in-flight which have timestamps before now but have
+		// not yet been added to the notices map. Therefore, if the current
+		// time is after the BeforeOrAt filter, we know there can be no new
+		// notices which match the filter.
 		now := time.Now()
 		if !filter.futureNoticesPossible(now) {
 			return nil, nil
 		}
 
 		// Wait till a new notice occurs or a context is cancelled.
+		// This unlocks noticeCond.L, so for this reason, it is essential that
+		// noticeCond.L is noticesMu.RLocker, since that is what we hold during
+		// this function call.
 		s.noticeCond.Wait()
 
 		// If this context is cancelled, return the error.
@@ -603,7 +709,7 @@ func (s *State) WaitNotices(ctx context.Context, filter *NoticeFilter) ([]*Notic
 		}
 
 		// Otherwise check if there are now matching notices.
-		notices = s.Notices(filter)
+		notices = s.doNotices(filter)
 		if len(notices) > 0 {
 			return notices, nil
 		}
