@@ -759,6 +759,8 @@ func (ss *stateSuite) TestMethodEntrance(c *C) {
 		func() { st.Warnf("hello") },
 		func() { st.OkayWarnings(time.Time{}) },
 		func() { st.UnshowAllWarnings() },
+		func() { st.AddNotice(nil, state.WarningNotice, "foo", nil) },
+		func() { st.DrainNotices(nil) },
 	}
 
 	reads := []func(){
@@ -772,9 +774,6 @@ func (ss *stateSuite) TestMethodEntrance(c *C) {
 		func() { st.MarshalJSON() },
 		func() { st.Prune(time.Now(), time.Hour, time.Hour, 100) },
 		func() { st.TaskCount() },
-		func() { st.AllWarnings() },
-		func() { st.PendingWarnings() },
-		func() { st.WarningsSummary() },
 	}
 
 	for i, f := range reads {
@@ -1132,12 +1131,13 @@ func (ss *stateSuite) TestTaskChangedHandler(c *C) {
 	defer st.Unlock()
 
 	var taskObservedChanges []taskAndStatus
-	oId := st.AddTaskStatusChangedHandler(func(t *state.Task, old, new state.Status) {
+	oId := st.AddTaskStatusChangedHandler(func(t *state.Task, old, new state.Status) bool {
 		taskObservedChanges = append(taskObservedChanges, taskAndStatus{
 			t:   t,
 			old: old,
 			new: new,
 		})
+		return false
 	})
 
 	t1 := st.NewTask("foo", "...")
@@ -1171,6 +1171,37 @@ func (ss *stateSuite) TestTaskChangedHandler(c *C) {
 	})
 }
 
+func (ss *stateSuite) TestTaskChangedHandlerCanRemoveItself(c *C) {
+	st := state.New(nil)
+	st.Lock()
+	defer st.Unlock()
+
+	var transitions []taskAndStatus
+	st.AddTaskStatusChangedHandler(func(t *state.Task, old, new state.Status) bool {
+		transitions = append(transitions, taskAndStatus{t: t, old: old, new: new})
+		return new == state.ErrorStatus
+	})
+
+	t1 := st.NewTask("foo", "...")
+	t1.SetStatus(state.DoingStatus)
+	// should remove handler after this transition
+	t1.SetStatus(state.ErrorStatus)
+	t1.SetStatus(state.UndoneStatus)
+
+	c.Check(transitions, DeepEquals, []taskAndStatus{
+		{
+			t:   t1,
+			old: state.DefaultStatus,
+			new: state.DoingStatus,
+		},
+		{
+			t:   t1,
+			old: state.DoingStatus,
+			new: state.ErrorStatus,
+		},
+	})
+}
+
 type changeAndStatus struct {
 	chg      *state.Change
 	old, new state.Status
