@@ -37,15 +37,32 @@ import (
 	"github.com/snapcore/snapd/testutil"
 )
 
+type (
+	ResealKeysWithTPMParams = resealKeysWithTPMParams
+	PreinstallCheckResult   = preinstallCheckResult
+)
+
 var (
 	UnwrapPreinstallCheckError         = unwrapPreinstallCheckError
 	ConvertPreinstallCheckErrorType    = convertPreinstallCheckErrorType
 	ConvertPreinstallCheckErrorActions = convertPreinstallCheckErrorActions
+	Save                               = (*preinstallCheckResult).save
 
 	EFIImageFromBootFile = efiImageFromBootFile
 	LockTPMSealedKeys    = lockTPMSealedKeys
+
+	ResealKeysWithTPM          = resealKeysWithTPM
+	ResealKeysWithFDESetupHook = resealKeysWithFDESetupHook
 )
 
+func ExtractSbRunChecksContext(checkContext *PreinstallCheckContext) *sb_preinstall.RunChecksContext {
+	return checkContext.sbRunChecksContext
+}
+
+func NewPreinstallChecksContext(sbRunChecksContext *sb_preinstall.RunChecksContext) *PreinstallCheckContext {
+	return &PreinstallCheckContext{sbRunChecksContext}
+}
+
 func MockSbPreinstallNewRunChecksContext(f func(initialFlags sb_preinstall.CheckFlags, loadedImages []sb_efi.Image, profileOpts sb_preinstall.PCRProfileOptionsFlags) *sb_preinstall.RunChecksContext) (restore func()) {
 	old := sbPreinstallNewRunChecksContext
 	sbPreinstallNewRunChecksContext = f
@@ -346,6 +363,7 @@ func MockDeleteLUKS2ContainerKey(f func(devicePath string, keyslotName string) e
 }
 
 type KeyRevealerV3 = keyRevealerV3
+type TaggedHandle = taggedHandle
 
 func MockAddLUKS2ContainerUnlockKey(f func(devicePath string, keyslotName string, existingKey sb.DiskUnlockKey, newKey sb.DiskUnlockKey) error) (restore func()) {
 	old := sbAddLUKS2ContainerUnlockKey
@@ -538,3 +556,15 @@ func MockSbKeyDataChangePassphrase(f func(d *sb.KeyData, oldPassphrase string, n
 func NewKeyData(kd *sb.KeyData) KeyData {
 	return &keyData{kd: kd}
 }
+
+func MockResealKeysWithFDESetupHook(f func(keys []KeyDataLocation, primaryKeyDevices []string, fallbackPrimaryKeyFiles []string, models []ModelForSealing, bootModes []string) error) (restore func()) {
+	return testutil.Mock(&resealKeysWithFDESetupHook, f)
+}
+
+func MockResealKeysWithTPM(f func(params *resealKeysWithTPMParams, newPCRPolicyVersion bool) (UpdatedKeys, error)) (restore func()) {
+	return testutil.Mock(&resealKeysWithTPM, f)
+}
+
+func MockSbKeyDataPlatformName(f func(d *sb.KeyData) string) (restore func()) {
+	return testutil.Mock(&sbKeyDataPlatformName, f)
+}
