@@ -177,6 +177,14 @@ type KeyDataLocation struct {
 	SlotName string
 }
 
+const (
+	PlatformTpm2       = "tpm2"
+	PlatformTpm2Legacy = "tpm2-legacy"
+	PlatformPlainkey   = "plainkey"
+	PlatformFdeHookV2  = "fde-hook-v2"
+	PlatformFdeHooksV3 = "fde-hooks-v3"
+)
+
 // KeyData represents a disk unlock key protected by a platform's secure device.
 type KeyData interface {
 	PlatformName() string
@@ -193,15 +201,6 @@ type KeyData interface {
 // opaque binary blob outside of secboot package.
 type SerializedPCRProfile []byte
 
-type ResealKeysParams struct {
-	// The snap model parameters
-	PCRProfile SerializedPCRProfile
-	// The locations to the key data
-	Keys []KeyDataLocation
-	// The primary key
-	PrimaryKey []byte
-}
-
 // UnlockVolumeUsingSealedKeyOptions contains options for unlocking encrypted
 // volumes using keys sealed to the TPM.
 type UnlockVolumeUsingSealedKeyOptions struct {
@@ -255,6 +254,17 @@ type UnlockResult struct {
 	UnlockMethod UnlockMethod
 }
 
+type ProtectKeyParams struct {
+	// The serialized PCR profile
+	PCRProfile SerializedPCRProfile
+	// The handle at which to create a NV index for dynamic authorization policy revocation support
+	PCRPolicyCounterHandle uint32
+	// The key role (run, run+recover, recover)
+	KeyRole string
+	// Optional volume authentication options
+	VolumesAuth *device.VolumesAuthOptions
+}
+
 // EncryptedPartitionName returns the name/label used by an encrypted partition
 // corresponding to a given name.
 func EncryptedPartitionName(name string) string {
@@ -290,3 +300,23 @@ func MarkSuccessful() error {
 const (
 	defaultKeyringPrefix = "ubuntu-fde"
 )
+
+type ResealKeyParams struct {
+	// PrimaryKeyDevices is the list of all devices that might
+	// have been unlocked and provides a primary key.
+	PrimaryKeyDevices []string
+	// FallbackPrimaryKeyFiles is the list of files that might contain
+	// the primary key.
+	FallbackPrimaryKeyFiles []string
+	// The allowed boot modes (run, recover, factory-reset)
+	BootModes []string
+	// The allowed models
+	Models []ModelForSealing
+	// The TPM policy profile. May be be nil when using hooks.
+	TpmPCRProfile []byte
+	// Whether a incremented value of the counter is allowed
+	// (before a revocation)
+	NewPCRPolicyVersion bool
+	// Whether old ambiguous key formats interpreted as FDE hook keys.
+	HintExpectFDEHook bool
+}
