@@ -659,12 +659,21 @@ func sbNewSealedKeyDataImpl(k *sb.KeyData) (MaybeSealedKeyData, error) {
 
 var sbNewSealedKeyData = sbNewSealedKeyDataImpl
 
-// ResealKeys updates the PCR protection policy for the sealed encryption keys
+type resealKeysWithTPMParams struct {
+	// The snap model parameters
+	PCRProfile SerializedPCRProfile
+	// The locations to the key data
+	Keys []KeyDataLocation
+	// The primary key
+	PrimaryKey []byte
+}
+
+// resealKeysWithTPMImpl updates the PCR protection policy for the sealed encryption keys
 // according to the specified parameters.
 // If newPCRPolicyVersion is true, the keys will use a new policy version
 // so that the policy counter can be incremented. The function will
 // then also return the updated keys in order to increase the counter.
-func ResealKeys(params *ResealKeysParams, newPCRPolicyVersion bool) (UpdatedKeys, error) {
+func resealKeysWithTPMImpl(params *resealKeysWithTPMParams, newPCRPolicyVersion bool) (UpdatedKeys, error) {
 	numSealedKeyObjects := len(params.Keys)
 	if numSealedKeyObjects < 1 {
 		return nil, fmt.Errorf("at least one key file is required")
@@ -761,6 +770,8 @@ func ResealKeys(params *ResealKeysParams, newPCRPolicyVersion bool) (UpdatedKeys
 	return updatedKeys, nil
 }
 
+var resealKeysWithTPM = resealKeysWithTPMImpl
+
 func buildPCRProtectionProfile(modelParams []*SealKeyModelParams, allowInsufficientDmaProtection bool) (*sb_tpm2.PCRProtectionProfile, error) {
 	numModels := len(modelParams)
 	modelPCRProfiles := make([]*sb_tpm2.PCRProtectionProfile, 0, numModels)
@@ -1089,7 +1100,7 @@ func GetPCRHandle(node, keySlot, keyFile string, hintExpectFDEHook bool) (uint32
 			if err != nil {
 				return 0, fmt.Errorf("cannot read key data for slot '%s': %w", keySlot, err)
 			}
-			if keyData.PlatformName() != "tpm2" {
+			if keyData.PlatformName() != platformTpm2 {
 				return 0, nil
 			}
 			sealedKeyData, err := keyData.GetTPMSealedKeyData()
@@ -1122,7 +1133,7 @@ func GetPCRHandle(node, keySlot, keyFile string, hintExpectFDEHook bool) (uint32
 	}
 
 	if loadedKey.KeyData != nil {
-		if loadedKey.KeyData.PlatformName() != "tpm2" {
+		if loadedKey.KeyData.PlatformName() != platformTpm2 {
 			return 0, nil
 		}
 		sealedKeyData, err := loadedKey.KeyData.GetTPMSealedKeyData()
@@ -1166,7 +1177,7 @@ func RemoveOldCounterHandles(device string, possibleOldKeys map[string]bool, pos
 			if err != nil {
 				return fmt.Errorf("cannot read key data for slot '%s': %w", slot, err)
 			}
-			if keyData.PlatformName() != "tpm2" {
+			if keyData.PlatformName() != platformTpm2 {
 				continue
 			}
 			sealedKeyData, err := keyData.GetTPMSealedKeyData()
@@ -1201,7 +1212,7 @@ func RemoveOldCounterHandles(device string, possibleOldKeys map[string]bool, pos
 				logger.Noticef("WARNING: we are deleting an unexpected handle. That should never have happened.")
 			}
 		} else if loadedKey.KeyData != nil {
-			if loadedKey.KeyData.PlatformName() != "tpm2" {
+			if loadedKey.KeyData.PlatformName() != platformTpm2 {
 				continue
 			}
 			sealedKeyData, err := loadedKey.KeyData.GetTPMSealedKeyData()
@@ -1265,3 +1276,62 @@ func FindFreeHandle() (uint32, error) {
 
 	return 0, fmt.Errorf("no free handle on TPM")
 }
+
+// AddContainerRecoveryKey adds a new TPM protected key to specified device.
+//
+// Note: The unlock and primary keys are implicitly obtained from the kernel
+// keyring so this will not work if the disk was unlocked with a recovery key
+// during boot.
+func AddContainerTPMProtectedKey(devicePath, slotName string, params *ProtectKeyParams) (err error) {
+	var pcrProfile sb_tpm2.PCRProtectionProfile
+	if _, err := mu.UnmarshalFromBytes(params.PCRProfile, &pcrProfile); err != nil {
+		return fmt.Errorf("cannot unmarshal PCR profile: %v", err)
+	}
+
+	// this will fail if the disk was unlocked with a recovery key during boot.
+	primaryKey, err := sbGetPrimaryKeyFromKernel(defaultKeyringPrefix, devicePath, false)
+	if err != nil {
+		return fmt.Errorf("cannot get primary key from kernel keyring for unlocked disk %s: %v", devicePath, err)
+	}
+
+	unlockKey, err := sbGetDiskUnlockKeyFromKernel(defaultKeyringPrefix, devicePath, false)
+	if err != nil {
+		return fmt.Errorf("cannot get unlock key from kernel keyring for unlocked disk %s: %v", devicePath, err)
+	}
+
+	tpm, err := sbConnectToDefaultTPM()
+	if err != nil {
+		return fmt.Errorf("cannot connect to TPM: %v", err)
+	}
+	defer tpm.Close()
+	if !isTPMEnabled(tpm) {
+		return errors.New("TPM device is not enabled")
+	}
+
+	creationParams := &sb_tpm2.ProtectKeyParams{
+		PCRProfile:             &pcrProfile,
+		Role:                   params.KeyRole,
+		PCRPolicyCounterHandle: tpm2.Handle(params.PCRPolicyCounterHandle),
+		PrimaryKey:             primaryKey,
+	}
+
+	protectedKey, _, newKey, err := newTPMProtectedKey(tpm, creationParams, params.VolumesAuth)
+	if err != nil {
+		return fmt.Errorf("cannot seal key: %v", err)
+	}
+
+	if err := sbAddLUKS2ContainerUnlockKey(devicePath, slotName, unlockKey, newKey); err != nil {
+		return fmt.Errorf("cannot add key: %v", err)
+	}
+
+	defer func() {
+		if err != nil {
+			if deleteErr := sbDeleteLUKS2ContainerKey(devicePath, slotName); deleteErr != nil {
+				logger.Noticef("cannot remove added key %s:%s during cleanup", devicePath, slotName)
+			}
+		}
+	}()
+
+	keyData := keyData{kd: protectedKey}
+	return keyData.WriteTokenAtomic(devicePath, slotName)
+}
