@@ -201,7 +201,7 @@ plugs:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"ipv6-aware": true},
+		Attrs:     map[string]any{"ipv6-aware": true},
 		Unscoped:  true,
 	})
 }
@@ -227,10 +227,10 @@ plugs:
 		Snap:      info,
 		Name:      "iface",
 		Interface: "complex",
-		Attrs: map[string]interface{}{
+		Attrs: map[string]any{
 			"i": int64(3),
-			"l": []interface{}{int64(1), int64(2), int64(3)},
-			"m": map[string]interface{}{"a": "A", "b": "B"},
+			"l": []any{int64(1), int64(2), int64(3)},
+			"m": map[string]any{"a": "A", "b": "B"},
 		},
 		Unscoped: true,
 	})
@@ -256,7 +256,7 @@ plugs:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"attr": int64(2)},
+		Attrs:     map[string]any{"attr": int64(2)},
 		Unscoped:  true,
 	})
 }
@@ -405,7 +405,7 @@ plugs:
 		Snap:      info,
 		Name:      "network-client",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"ipv6-aware": true},
+		Attrs:     map[string]any{"ipv6-aware": true},
 		Unscoped:  true,
 	})
 }
@@ -602,7 +602,7 @@ slots:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"ipv6-aware": true},
+		Attrs:     map[string]any{"ipv6-aware": true},
 		Unscoped:  true,
 	})
 }
@@ -627,10 +627,10 @@ slots:
 		Snap:      info,
 		Name:      "iface",
 		Interface: "complex",
-		Attrs: map[string]interface{}{
+		Attrs: map[string]any{
 			"i": int64(3),
-			"l": []interface{}{int64(1), int64(2)},
-			"m": map[string]interface{}{"a": "A"},
+			"l": []any{int64(1), int64(2)},
+			"m": map[string]any{"a": "A"},
 		},
 		Unscoped: true,
 	})
@@ -656,7 +656,7 @@ slots:
 		Snap:      info,
 		Name:      "net",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"attr": int64(2)},
+		Attrs:     map[string]any{"attr": int64(2)},
 		Unscoped:  true,
 	})
 }
@@ -766,7 +766,7 @@ slots:
 		Snap:      info,
 		Name:      "network-client",
 		Interface: "network-client",
-		Attrs:     map[string]interface{}{"ipv6-aware": true},
+		Attrs:     map[string]any{"ipv6-aware": true},
 		Unscoped:  true,
 	})
 }
@@ -1359,7 +1359,7 @@ slots:
 	c.Check(slot1.Snap, Equals, info)
 	c.Check(slot1.Name, Equals, "foo-socket-slot")
 	c.Check(slot1.Interface, Equals, "socket")
-	c.Check(slot1.Attrs, DeepEquals, map[string]interface{}{
+	c.Check(slot1.Attrs, DeepEquals, map[string]any{
 		"protocol": "foo", "path": "$SNAP_DATA/socket"})
 	c.Check(slot1.Label, Equals, "")
 	c.Check(slot1.Apps, DeepEquals, map[string]*snap.AppInfo{app1.Name: app1})
@@ -1463,6 +1463,37 @@ version: 1.0
 	c.Assert(info.Confinement, Equals, snap.StrictConfinement)
 }
 
+func (s *YamlSuite) TestSnapYamlGradeComplete(c *C) {
+	const (
+		str = `
+name: snap
+version: '0.1.0'
+grade: %q
+`
+		invalidGrade = "invalid"
+	)
+
+	yStable := []byte(fmt.Sprintf(str, "stable"))
+	infoStable, err := snap.InfoFromSnapYaml(yStable)
+	c.Assert(infoStable.Grade, Equals, snap.StableGrade)
+	c.Assert(err, IsNil)
+
+	yDevel := []byte(fmt.Sprintf(str, "devel"))
+	infoDevel, err := snap.InfoFromSnapYaml(yDevel)
+	c.Assert(infoDevel.Grade, Equals, snap.DevelGrade)
+	c.Assert(err, IsNil)
+
+	yEmpty := []byte(fmt.Sprintf(str, ""))
+	infoEmpty, err := snap.InfoFromSnapYaml(yEmpty)
+	c.Assert(infoEmpty.Grade, Equals, snap.EmptyGrade)
+	c.Assert(err, IsNil)
+
+	yInvalid := []byte(fmt.Sprintf(str, invalidGrade))
+	infoInvalid, err := snap.InfoFromSnapYaml(yInvalid)
+	c.Assert(infoInvalid, IsNil)
+	c.Assert(err, ErrorMatches, fmt.Sprintf("^.*unknown grade type: %q", invalidGrade))
+}
+
 func (s *YamlSuite) TestSnapYamlMultipleArchitecturesParsing(c *C) {
 	y := []byte(`name: binary
 version: 1.0
@@ -1996,7 +2027,7 @@ system-usernames:
 	c.Assert(info.SystemUsernames["baz"], DeepEquals, &snap.SystemUsernameInfo{
 		Name:  "baz",
 		Scope: "private",
-		Attrs: map[string]interface{}{
+		Attrs: map[string]any{
 			"attr1": "norf",
 			"attr2": "corge",
 			"attr3": "",
@@ -2180,11 +2211,11 @@ func (s *YamlSuite) TestUnmarshalComponents(c *C) {
 name: snap
 components:
   test1:
-    type: test
+    type: standard
     summary: test component
     description: long component description
   test2:
-    type: test
+    type: standard
     summary: test component 2
     description: long component description 2
 `))
@@ -2192,13 +2223,13 @@ components:
 	c.Check(info.InstanceName(), Equals, "snap")
 	c.Check(info.Components, DeepEquals, map[string]*snap.Component{
 		"test1": {
-			Type:        "test",
+			Type:        "standard",
 			Summary:     "test component",
 			Description: "long component description",
 			Name:        "test1",
 		},
 		"test2": {
-			Type:        "test",
+			Type:        "standard",
 			Summary:     "test component 2",
 			Description: "long component description 2",
 			Name:        "test2",
@@ -2211,7 +2242,7 @@ func (s *YamlSuite) TestUnmarshalComponentsHook(c *C) {
 name: snap
 components:
   test1:
-    type: test
+    type: standard
     summary: test component
     description: long component description
     hooks:
@@ -2239,7 +2270,7 @@ func (s *YamlSuite) TestUnmarshalComponentsHooksWithPlugs(c *C) {
 name: snap
 components:
   test1:
-    type: test
+    type: standard
     summary: test component
     description: long component description
     hooks:
@@ -2324,7 +2355,7 @@ func (s *YamlSuite) TestUnmarshalComponentsHooksUnsupported(c *C) {
 name: snap
 components:
   test1:
-    type: test
+    type: standard
     summary: test component
     description: long component description
     hooks:
@@ -2352,7 +2383,7 @@ func (s *YamlSuite) TestUnmarshalComponentsHookSlotsError(c *C) {
 name: snap
 components:
   test1:
-    type: test
+    type: standard
     hooks:
       install:
         slots: [slot-for-install]
