@@ -21,14 +21,11 @@ package pack_test
 
 import (
 	"bytes"
-	"encoding/json"
 	"fmt"
-	"io"
 	"os"
 	"os/exec"
 	"path/filepath"
 	"regexp"
-	"strconv"
 	"strings"
 	"testing"
 
@@ -39,7 +36,6 @@ import (
 	// for SanitizePlugsSlots
 	_ "github.com/snapcore/snapd/interfaces/builtin"
 	"github.com/snapcore/snapd/snap"
-	"github.com/snapcore/snapd/snap/integrity"
 	"github.com/snapcore/snapd/snap/pack"
 	"github.com/snapcore/snapd/snap/squashfs"
 	"github.com/snapcore/snapd/testutil"
@@ -167,7 +163,65 @@ apps:
 `)
 	c.Assert(os.Remove(filepath.Join(sourceDir, "bin", "hello-world")), IsNil)
 	_, err := pack.Pack(sourceDir, pack.Defaults)
-	c.Assert(err, Equals, snap.ErrMissingPaths)
+	c.Check(err, testutil.ErrorIs, snap.ErrMissingPaths)
+	c.Assert(err, ErrorMatches, `snap is unusable due to missing files: path "bin/hello-world" does not exist`)
+}
+
+func (s *packSuite) TestPackKernelGadgetOSAppWithConfigureHookHappy(c *C) {
+	for _, snapType := range []string{"kernel", "gadget", "os", "app"} {
+		snapYaml := fmt.Sprintf(`name: %[1]s
+version: 0
+type: %[1]s`, snapType)
+		sourceDir := makeExampleSnapSourceDir(c, snapYaml)
+		c.Assert(os.Mkdir(filepath.Join(sourceDir, "meta", "hooks"), 0755), IsNil)
+		c.Assert(os.WriteFile(filepath.Join(sourceDir, "meta", "hooks", "configure"), []byte("#!/bin/sh"), 0755), IsNil)
+		_, err := pack.Pack(sourceDir, pack.Defaults)
+		c.Assert(err, IsNil)
+	}
+}
+
+func (s *packSuite) TestPackKernelGadgetAppWithDefaultConfigureAndConfigureHookHappy(c *C) {
+	for _, snapType := range []string{"kernel", "gadget", "app"} {
+		snapYaml := fmt.Sprintf(`name: %[1]s
+version: 0
+type: %[1]s`, snapType)
+		sourceDir := makeExampleSnapSourceDir(c, snapYaml)
+		configureHooks := []string{"default-configure", "configure"}
+		c.Assert(os.Mkdir(filepath.Join(sourceDir, "meta", "hooks"), 0755), IsNil)
+		for _, hook := range configureHooks {
+			c.Assert(os.WriteFile(filepath.Join(sourceDir, "meta", "hooks", hook), []byte("#!/bin/sh"), 0755), IsNil)
+		}
+		_, err := pack.Pack(sourceDir, pack.Defaults)
+		c.Assert(err, IsNil)
+	}
+}
+
+func (s *packSuite) TestPackSnapdBaseWithConfigureHookError(c *C) {
+	for _, snapType := range []string{"snapd", "base"} {
+		snapYaml := fmt.Sprintf(`name: %[1]s
+version: 0
+type: %[1]s`, snapType)
+		sourceDir := makeExampleSnapSourceDir(c, snapYaml)
+		c.Assert(os.Mkdir(filepath.Join(sourceDir, "meta", "hooks"), 0755), IsNil)
+		c.Assert(os.WriteFile(filepath.Join(sourceDir, "meta", "hooks", "configure"), []byte("#!/bin/sh"), 0755), IsNil)
+		_, err := pack.Pack(sourceDir, pack.Defaults)
+		c.Check(err, ErrorMatches, fmt.Sprintf(`cannot validate snap %[1]q: cannot specify "configure" hook for %[1]q snap %[1]q`, snapType))
+	}
+}
+
+func (s *packSuite) TestPackSnapdBaseOSWithDefaultConfigureHookError(c *C) {
+	for _, snapType := range []string{"snapd", "base", "os"} {
+		snapYaml := fmt.Sprintf(`name: %[1]s
+version: 0
+type: %[1]s`, snapType)
+		sourceDir := makeExampleSnapSourceDir(c, snapYaml)
+		c.Assert(os.Mkdir(filepath.Join(sourceDir, "meta", "hooks"), 0755), IsNil)
+		c.Assert(os.WriteFile(filepath.Join(sourceDir, "meta", "hooks", "default-configure"), []byte("#!/bin/sh"), 0755), IsNil)
+		_, err := pack.Pack(sourceDir, pack.Defaults)
+		// an error due to a prohibited hook for the snap type takes precedence over the
+		// error for missing a configure hook when default-configure is present
+		c.Check(err, ErrorMatches, fmt.Sprintf(`cannot validate snap %[1]q: cannot specify "default-configure" hook for %[1]q snap %[1]q`, snapType))
+	}
 }
 
 func (s *packSuite) TestPackDefaultConfigureWithoutConfigureError(c *C) {
@@ -180,7 +234,7 @@ apps:
 	c.Assert(os.Mkdir(filepath.Join(sourceDir, "meta", "hooks"), 0755), IsNil)
 	c.Assert(os.WriteFile(filepath.Join(sourceDir, "meta", "hooks", "default-configure"), []byte("#!/bin/sh"), 0755), IsNil)
 	_, err := pack.Pack(sourceDir, pack.Defaults)
-	c.Check(err, ErrorMatches, "cannot validate snap \"hello\": cannot specify \"default-configure\" hook without \"configure\" hook")
+	c.Check(err, ErrorMatches, `cannot validate snap "hello": cannot specify "default-configure" hook without "configure" hook`)
 }
 
 func (s *packSuite) TestPackConfigureHooksPermissionsError(c *C) {
@@ -193,25 +247,12 @@ apps:
 	c.Assert(os.Mkdir(filepath.Join(sourceDir, "meta", "hooks"), 0755), IsNil)
 	configureHooks := []string{"configure", "default-configure"}
 	for _, hook := range configureHooks {
-		c.Assert(os.WriteFile(filepath.Join(sourceDir, "meta", "hooks", hook), []byte("#!/bin/sh"), 0666), IsNil)
+		c.Assert(os.WriteFile(filepath.Join(sourceDir, "meta", "hooks", hook), []byte("#!/bin/sh"), 0644), IsNil)
 		_, err := pack.Pack(sourceDir, pack.Defaults)
-		c.Check(err, ErrorMatches, "snap is unusable due to bad permissions")
-	}
-}
-
-func (s *packSuite) TestPackConfigureHooksHappy(c *C) {
-	sourceDir := makeExampleSnapSourceDir(c, `name: hello
-version: 0
-apps:
- foo:
-  command: bin/hello-world
-`)
-	c.Assert(os.Mkdir(filepath.Join(sourceDir, "meta", "hooks"), 0755), IsNil)
-	configureHooks := []string{"configure", "default-configure"}
-	for _, hook := range configureHooks {
-		c.Assert(os.WriteFile(filepath.Join(sourceDir, "meta", "hooks", hook), []byte("#!/bin/sh"), 0755), IsNil)
-		_, err := pack.Pack(sourceDir, pack.Defaults)
-		c.Assert(err, IsNil)
+		c.Check(err, testutil.ErrorIs, snap.ErrBadModes)
+		c.Check(err, ErrorMatches, fmt.Sprintf(`snap is unusable due to bad permissions: "meta/hooks/%s" should be executable, and isn't: -rw-r--r--`, hook))
+		// Fix hook error to catch next hook's error
+		c.Assert(os.Chmod(filepath.Join(sourceDir, "meta", "hooks", hook), 0755), IsNil)
 	}
 }
 
@@ -246,7 +287,8 @@ apps:
 `
 	c.Assert(os.WriteFile(filepath.Join(sourceDir, "meta", "snapshots.yaml"), []byte(invalidSnapshotYaml), 0411), IsNil)
 	_, err := pack.Pack(sourceDir, pack.Defaults)
-	c.Assert(err, ErrorMatches, "snap is unusable due to bad permissions")
+	c.Check(err, testutil.ErrorIs, snap.ErrBadModes)
+	c.Assert(err, ErrorMatches, `snap is unusable due to bad permissions: "meta/snapshots.yaml" should be world-readable, and isn't: -r----x--x`)
 }
 
 func (s *packSuite) TestPackSnapshotYamlHappy(c *C) {
@@ -283,7 +325,8 @@ apps:
 	c.Assert(os.Remove(filepath.Join(sourceDir, "bin", "hello-world")), IsNil)
 
 	err = pack.CheckSkeleton(&buf, sourceDir)
-	c.Assert(err, Equals, snap.ErrMissingPaths)
+	c.Check(err, testutil.ErrorIs, snap.ErrMissingPaths)
+	c.Assert(err, ErrorMatches, `snap is unusable due to missing files: path "bin/hello-world" does not exist`)
 	c.Check(buf.String(), Equals, "")
 }
 
@@ -346,9 +389,9 @@ func (s *packSuite) TestDebArchitecture(c *C) {
 	c.Check(pack.DebArchitecture(&snap.Info{Architectures: nil}), Equals, "all")
 }
 
-func (s *packSuite) TestPackSimple(c *C) {
+func (s *packSuite) TestPackComponentSimple(c *C) {
 	sourceDir := makeExampleComponentSourceDir(c, `component: hello+test
-type: test
+type: standard
 version: 1.0.1
 `)
 
@@ -395,7 +438,49 @@ version: 1.0.1
 	}
 }
 
-func (s *packSuite) TestPackComponentSimple(c *C) {
+func (s *packSuite) TestPackComponentProvenance(c *C) {
+	sourceDir := makeExampleComponentSourceDir(c, `component: hello+test
+type: standard
+version: 1.0.1
+provenance: prov
+`)
+
+	result, err := pack.Pack(sourceDir, nil)
+	c.Assert(err, IsNil)
+
+	// check that there is result
+	_, err = os.Stat(result)
+	c.Assert(err, IsNil)
+	c.Assert(result, Equals, "hello+test_1.0.1.comp")
+
+	// check that the content looks sane
+	output, err := exec.Command("unsquashfs", "-ll", result).CombinedOutput()
+	c.Assert(err, IsNil)
+	expr := fmt.Sprintf(`(?ms).*%s.*`, regexp.QuoteMeta("meta/component.yaml"))
+	c.Assert(string(output), Matches, expr)
+}
+
+func (s *packSuite) TestPackComponentNoVersion(c *C) {
+	sourceDir := makeExampleComponentSourceDir(c, `component: hello+test
+type: standard
+`)
+
+	result, err := pack.Pack(sourceDir, nil)
+	c.Assert(err, IsNil)
+
+	// check that there is result
+	_, err = os.Stat(result)
+	c.Assert(err, IsNil)
+	c.Assert(result, Equals, "hello+test.comp")
+
+	// check that the content looks sane
+	output, err := exec.Command("unsquashfs", "-ll", result).CombinedOutput()
+	c.Assert(err, IsNil)
+	expr := fmt.Sprintf(`(?ms).*%s.*`, regexp.QuoteMeta("meta/component.yaml"))
+	c.Assert(string(output), Matches, expr)
+}
+
+func (s *packSuite) TestPackSimple(c *C) {
 	sourceDir := makeExampleSnapSourceDir(c, `name: hello
 version: 1.0.1
 architectures: ["i386", "amd64"]
@@ -537,90 +622,3 @@ func (s *packSuite) TestPackWithCompressionUnhappy(c *C) {
 		c.Assert(snapfile, Equals, "")
 	}
 }
-
-func (s *packSuite) TestPackWithIntegrity(c *C) {
-	sourceDir := makeExampleSnapSourceDir(c, "{name: hello, version: 0}")
-	targetDir := c.MkDir()
-
-	// 8192 is the hash size that is created when running 'veritysetup format'
-	// on a minimally sized snap. there is not an easy way to calculate this
-	// value dynamically.
-	const verityHashSize = 8192
-
-	// mock the verity-setup command, what it does is make a copy of the snap
-	// and then returns pre-calculated output
-	vscmd := testutil.MockCommand(c, "veritysetup", fmt.Sprintf(`
-case "$1" in
-	--version)
-		echo "veritysetup 2.2.6"
-		exit 0
-		;;
-	format)
-		truncate -s %[1]d %[2]s/hello_0_all.snap.verity
-		echo "VERITY header information for %[2]s/hello_0_all.snap.verity"
-		echo "UUID:            	606d10a2-24d8-4c6b-90cf-68207aa7c850"
-		echo "Hash type:       	1"
-		echo "Data blocks:     	4"
-		echo "Data block size: 	4096"
-		echo "Hash block size: 	4096"
-		echo "Hash algorithm:  	sha256"
-		echo "Salt:            	eba61f2091bb6122226aef83b0d6c1623f095fc1fda5712d652a8b34a02024ea"
-		echo "Root hash:      	3fbfef5f1f0214d727d03eebc4723b8ef5a34740fd8f1359783cff1ef9c3f334"
-		;;
-esac
-`, verityHashSize, targetDir))
-	defer vscmd.Restore()
-
-	snapPath, err := pack.Pack(sourceDir, &pack.Options{
-		TargetDir: targetDir,
-		Integrity: true,
-	})
-	c.Assert(err, IsNil)
-	c.Check(snapPath, testutil.FilePresent)
-	c.Assert(vscmd.Calls(), HasLen, 2)
-	c.Check(vscmd.Calls()[0], DeepEquals, []string{"veritysetup", "--version"})
-	c.Check(vscmd.Calls()[1], DeepEquals, []string{"veritysetup", "format", snapPath, snapPath + ".verity"})
-
-	magic := []byte{'s', 'n', 'a', 'p', 'e', 'x', 't'}
-
-	snapFile, err := os.Open(snapPath)
-	c.Assert(err, IsNil)
-	defer snapFile.Close()
-
-	fi, err := snapFile.Stat()
-	c.Assert(err, IsNil)
-
-	integrityStartOffset := squashfs.MinimumSnapSize
-	if fi.Size() > int64(65536) {
-		// on openSUSE, the squashfs image is padded up to 64k,
-		// including the integrator data, the overall size is > 64k
-		integrityStartOffset = 65536
-	}
-
-	// example snap has a size of 16384 (4 blocks)
-	_, err = snapFile.Seek(integrityStartOffset, io.SeekStart)
-	c.Assert(err, IsNil)
-
-	integrityHdr := make([]byte, integrity.HeaderSize)
-	_, err = snapFile.Read(integrityHdr)
-	c.Assert(err, IsNil)
-
-	c.Assert(bytes.HasPrefix(integrityHdr, magic), Equals, true)
-
-	var hdr interface{}
-	integrityHdr = bytes.Trim(integrityHdr, "\x00")
-	err = json.Unmarshal(integrityHdr[len(magic):], &hdr)
-	c.Check(err, IsNil)
-
-	integrityDataHeader, ok := hdr.(map[string]interface{})
-	c.Assert(ok, Equals, true)
-	hdrSizeStr, ok := integrityDataHeader["size"].(string)
-	c.Assert(ok, Equals, true)
-	hdrSize, err := strconv.ParseUint(hdrSizeStr, 10, 64)
-	c.Assert(err, IsNil)
-	c.Check(hdrSize, Equals, uint64(integrity.HeaderSize+verityHashSize))
-
-	fi, err = snapFile.Stat()
-	c.Assert(err, IsNil)
-	c.Check(fi.Size(), Equals, int64(integrityStartOffset+(integrity.HeaderSize+verityHashSize)))
-}
