@@ -323,7 +323,7 @@ func init() {
 	defaultConfig.DetailFields = jsonutil.StructFields((*snapDetails)(nil), "snap_yaml_raw")
 	defaultConfig.InfoFields = jsonutil.StructFields((*storeSnap)(nil), "snap-yaml")
 	defaultConfig.FindFields = append(jsonutil.StructFields((*storeSnap)(nil),
-		"architectures", "created-at", "epoch", "name", "snap-id", "snap-yaml"),
+		"architectures", "created-at", "epoch", "name", "snap-id", "snap-yaml", "resources"),
 		"channel")
 }
 
@@ -614,6 +614,13 @@ func (r *requestOptions) addHeader(k, v string) {
 	r.ExtraHeaders[k] = v
 }
 
+// iconRequestOptions specifies parameters for icon requests, which do not
+// require headers related to the store or snap. Just an ordinary GET request.
+type iconRequestOptions struct {
+	url  *url.URL
+	etag string
+}
+
 func cancelled(ctx context.Context) bool {
 	select {
 	case <-ctx.Done():
@@ -623,7 +630,7 @@ func cancelled(ctx context.Context) bool {
 	}
 }
 
-var expectedCatalogPreamble = []interface{}{
+var expectedCatalogPreamble = []any{
 	json.Delim('{'),
 	"_embedded",
 	json.Delim('{'),
@@ -693,7 +700,7 @@ func decodeCatalog(resp *http.Response, names io.Writer, db SnapAdder) error {
 	return nil
 }
 
-func decodeJSONBody(resp *http.Response, success interface{}, failure interface{}) error {
+func decodeJSONBody(resp *http.Response, success any, failure any) error {
 	ok := (resp.StatusCode == 200 || resp.StatusCode == 201)
 	// always decode on success; decode failures only if body is not empty
 	if !ok && resp.ContentLength == 0 {
@@ -710,7 +717,7 @@ func decodeJSONBody(resp *http.Response, success interface{}, failure interface{
 }
 
 // retryRequestDecodeJSON calls retryRequest and decodes the response into either success or failure.
-func (s *Store) retryRequestDecodeJSON(ctx context.Context, reqOptions *requestOptions, user *auth.UserState, success interface{}, failure interface{}) (resp *http.Response, err error) {
+func (s *Store) retryRequestDecodeJSON(ctx context.Context, reqOptions *requestOptions, user *auth.UserState, success any, failure any) (resp *http.Response, err error) {
 	return httputil.RetryRequest(reqOptions.URL.String(), func() (*http.Response, error) {
 		return s.doRequest(ctx, s.client, reqOptions, user)
 	}, func(resp *http.Response) error {
@@ -766,6 +773,26 @@ func (s *Store) doRequest(ctx context.Context, client *http.Client, reqOptions *
 	}
 }
 
+// doIconRequest does an unauthenticated GET request to the given URL.
+func doIconRequest(ctx context.Context, client *http.Client, reqOptions iconRequestOptions) (*http.Response, error) {
+	var body io.Reader // empty body
+	req, err := http.NewRequestWithContext(ctx, "GET", reqOptions.url.String(), body)
+	if err != nil {
+		return nil, err
+	}
+
+	if reqOptions.etag != "" {
+		req.Header.Set("If-None-Match", reqOptions.etag)
+	}
+
+	resp, err := client.Do(req)
+	if err != nil {
+		return nil, err
+	}
+
+	return resp, err
+}
+
 func (s *Store) buildLocationString() (string, error) {
 	if s.dauthCtx == nil {
 		return "", nil
@@ -1560,7 +1587,7 @@ func (s *Store) ReadyToBuy(user *auth.UserState) error {
 
 // abbreviated info structs just for the download info
 type storeInfoChannelAbbrev struct {
-	Download storeSnapDownload `json:"download"`
+	Download storeDownload `json:"download"`
 }
 
 type storeInfoAbbrev struct {
@@ -1572,9 +1599,7 @@ var errUnexpectedConnCheckResponse = errors.New("unexpected response during conn
 
 func (s *Store) snapConnCheck() ([]string, error) {
 	var hosts []string
-	// NOTE: "core" is possibly the only snap that's sure to be in all stores
-	//       when we drop "core" in the move to snapd/core18/etc, change this
-	infoURL, err := s.endpointURL(path.Join(snapInfoEndpPath, "core"), url.Values{
+	infoURL, err := s.endpointURL(path.Join(snapInfoEndpPath, "snapd"), url.Values{
 		// we only want the download URL
 		"fields": {"download"},
 		// we only need *one* (but can't filter by channel ... yet)
