@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2020 Canonical Ltd
+ * Copyright (C) 2020, 2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -35,6 +35,7 @@ import (
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/osutil/kcmdline"
 	"github.com/snapcore/snapd/strutil"
 )
 
@@ -70,7 +71,7 @@ func DisableCloudInit(rootDir string) error {
 // cloud-init configuration file.
 type supportedFilteredCloudConfig struct {
 	Datasource map[string]supportedFilteredDatasource `yaml:"datasource,omitempty"`
-	Network    map[string]interface{}                 `yaml:"network,omitempty"`
+	Network    map[string]any                         `yaml:"network,omitempty"`
 	// DatasourceList is a pointer so we can distinguish between:
 	// datasource_list: []
 	// and not setting the datasource at all
@@ -742,6 +743,15 @@ func CloudInitStatus() (CloudInitState, error) {
 		return CloudInitDisabledPermanently, nil
 	}
 
+	// if it was explicitly disabled via the kernel commandline, then
+	// return special status for that
+	cmdline, err := kcmdline.KeyValues("cloud-init")
+	if err != nil {
+		logger.Noticef("WARNING: cannot obtain cloud-init from kernel command line: %v", err)
+	} else if cmdline["cloud-init"] == "disabled" {
+		return CloudInitDisabledPermanently, nil
+	}
+
 	ciBinary, err := exec.LookPath("cloud-init")
 	if err != nil {
 		logger.Noticef("cannot locate cloud-init executable: %v", err)
@@ -767,9 +777,19 @@ func CloudInitStatus() (CloudInitState, error) {
 		return CloudInitErrored, fmt.Errorf("invalid cloud-init output: %v", osutil.OutputErrCombine(out, stderr, err))
 	}
 
+	hasError := false
+	if err != nil {
+		exitError, isExitError := err.(*exec.ExitError)
+		if isExitError && exitError.ExitCode() == 2 {
+			logger.Noticef("cloud-init status returned 'recoverable error' status: cloud-init completed but experienced errors")
+		} else {
+			hasError = true
+		}
+	}
+
 	// otherwise we had a successful match, but we need to check if the status
 	// command errored itself
-	if err != nil {
+	if hasError {
 		if string(match[1]) == "error" {
 			// then the status was indeed error and we should treat this as the
 			// "positively identified" error case
