@@ -21,7 +21,7 @@
 : "${NESTED_UBUNTU_IMAGE_PRESEED_KEY:=}"
 : "${NESTED_UBUNTU_SEED_SIZE:=}"
 : "${NESTED_KEEP_FIRMWARE_STATE:=}"
-: "${NESTED_BIOS_FILE:=}"
+: "${NESTED_CUSTOM_FIRMWARE:=}"
 : "${NESTED_ENABLE_ARM_TRUSTZONE:=false}"
 
 : "${NESTED_DISK_PHYSICAL_BLOCK_SIZE:=512}"
@@ -1226,6 +1226,9 @@ nested_start_core_vm_unit() {
     elif [ "$SPREAD_BACKEND" = "qemu-nested" ] || [ "$SPREAD_BACKEND" = "garden" ]; then
         PARAM_MEM="-m ${NESTED_MEM:-2048}"
         PARAM_SMP="-smp ${NESTED_CPUS:-1}"
+    elif [ "$SPREAD_BACKEND" = "openstack-ext-ps7" ]; then
+        PARAM_MEM="-m ${NESTED_MEM:-4096}"
+        PARAM_SMP="-smp ${NESTED_CPUS:-2}"
     else
         echo "unknown spread backend $SPREAD_BACKEND"
         exit 1
@@ -1292,7 +1295,7 @@ nested_start_core_vm_unit() {
     PARAM_REEXEC_ON_FAILURE=""
 
     if nested_is_core_lt 20; then
-        if [[ "$SPREAD_BACKEND" = google-nested* ]]; then
+        if [[ "$SPREAD_BACKEND" = google-nested* ]] || [[ "$SPREAD_BACKEND" = openstack* ]]; then
             PARAM_MACHINE="-machine ubuntu${ATTR_KVM}"
         elif [ "$SPREAD_BACKEND" = "qemu-nested" ] || [ "$SPREAD_BACKEND" = "garden" ]; then
             # check if we have nested kvm
@@ -1311,31 +1314,30 @@ nested_start_core_vm_unit() {
     fi
 
     if nested_is_core_ge 20; then
-        nested_ensure_ovmf
-        local OVMF_CODE OVMF_VARS OVMF_VARS_SECBOOT OVMF_VARS_CURRENT OVMF 
-        if os.query is-arm; then
-            OVMF=AAVMF
-            OVMF_VARS_SECBOOT="${NESTED_ASSETS_DIR}/ovmf/fw/${OVMF}_VARS.ms.fd"
-        else
-            OVMF=OVMF
-            OVMF_VARS_SECBOOT="${NESTED_ASSETS_DIR}/ovmf/fw/${OVMF}_VARS.enrolled.fd"
-        fi
-        OVMF_CODE="${NESTED_ASSETS_DIR}/ovmf/fw/${OVMF}_CODE.fd"
-        OVMF_VARS="${NESTED_ASSETS_DIR}/ovmf/fw/${OVMF}_VARS.fd"
-        OVMF_VARS_CURRENT="${NESTED_ASSETS_DIR}/ovmf/fw/${OVMF}_VARS.current.fd"
-
-        if [ -z "$NESTED_KEEP_FIRMWARE_STATE" ] || ! [ -e "${OVMF_VARS_CURRENT}" ]; then
-            if nested_is_secure_boot_enabled; then
-                cp -fv "${OVMF_VARS_SECBOOT}" "${OVMF_VARS_CURRENT}"
+        if [ -z "$NESTED_CUSTOM_FIRMWARE" ]; then
+            nested_ensure_ovmf
+            local OVMF_CODE OVMF_VARS OVMF_VARS_SECBOOT OVMF_VARS_CURRENT OVMF
+            if os.query is-arm; then
+                OVMF=AAVMF
+                OVMF_VARS_SECBOOT="${NESTED_ASSETS_DIR}/ovmf/fw/${OVMF}_VARS.ms.fd"
             else
-                cp -fv "${OVMF_VARS}" "${OVMF_VARS_CURRENT}"
+                OVMF=OVMF
+                OVMF_VARS_SECBOOT="${NESTED_ASSETS_DIR}/ovmf/fw/${OVMF}_VARS.enrolled.fd"
             fi
-        fi
+            OVMF_CODE="${NESTED_ASSETS_DIR}/ovmf/fw/${OVMF}_CODE.fd"
+            OVMF_VARS="${NESTED_ASSETS_DIR}/ovmf/fw/${OVMF}_VARS.fd"
+            OVMF_VARS_CURRENT="${NESTED_ASSETS_DIR}/ovmf/fw/${OVMF}_VARS.current.fd"
 
-        if [ -z "$NESTED_BIOS_FILE" ]; then
+            if [ -z "$NESTED_KEEP_FIRMWARE_STATE" ] || ! [ -e "${OVMF_VARS_CURRENT}" ]; then
+                if nested_is_secure_boot_enabled; then
+                    cp -fv "${OVMF_VARS_SECBOOT}" "${OVMF_VARS_CURRENT}"
+                else
+                    cp -fv "${OVMF_VARS}" "${OVMF_VARS_CURRENT}"
+                fi
+            fi
             PARAM_BIOS="-drive file=${OVMF_CODE},if=pflash,format=raw,readonly=on -drive file=${OVMF_VARS_CURRENT},if=pflash,format=raw"
         else
-            PARAM_BIOS="-drive file=${NESTED_BIOS_FILE},if=pflash,format=raw,readonly=on  -drive file=${OVMF_VARS_CURRENT},if=pflash,format=raw"
+            PARAM_BIOS="-drive file=${NESTED_CUSTOM_FIRMWARE},if=pflash,format=raw,readonly=on"
         fi
 
         local ENABLE_ARM_TRUSTZONE
@@ -1458,6 +1460,55 @@ nested_start_core_vm_unit() {
                 fi
             fi
         fi
+        nested_setup_vm
+    fi
+}
+
+nested_setup_vm(){
+    if nested_is_core_ge 20; then
+        remote.exec "sudo snap set system journal.persistent=true"
+    fi
+    if [ "${SNAPD_USE_PROXY:-}" = true ]; then
+        nested_no_proxy="${NO_PROXY},10.0.2.2"
+
+        # Ensure the nameservers used are the same than the host vm
+        net_interface="$(ip route show default | awk '{print $5}')"
+        nameservers="$(resolvectl status "$net_interface" | grep "DNS Servers:" | cut -d: -f2)"
+        if [ -n "$nameservers" ]; then
+            remote.exec "grep -v '^nameserver' /etc/resolv.conf > /tmp/resolv.conf"
+            remote.exec "sudo cp /tmp/resolv.conf /etc/resolv.conf"
+            for nameserver in $nameservers; do
+                remote.exec "echo nameserver $nameserver | sudo tee -a /etc/resolv.conf"
+            done
+        fi
+
+        # Add proxy configuration in /etc/environment
+        remote.exec "echo HTTPS_PROXY=$HTTPS_PROXY | sudo tee -a /etc/environment"
+        remote.exec "echo https_proxy=$HTTPS_PROXY | sudo tee -a /etc/environment"
+        remote.exec "echo HTTP_PROXY=$HTTP_PROXY | sudo tee -a /etc/environment"
+        remote.exec "echo http_proxy=$HTTP_PROXY | sudo tee -a /etc/environment"
+        remote.exec "echo NO_PROXY=$nested_no_proxy | sudo tee -a /etc/environment"
+        remote.exec "echo no_proxy=$nested_no_proxy | sudo tee -a /etc/environment"
+        remote.exec "echo SNAPD_USE_PROXY=$SNAPD_USE_PROXY | sudo tee -a /etc/environment"
+
+        # Configure snapd to use the proxy
+        remote.retry -n 10 --wait 3 "systemctl is-enabled snapd"
+        remote.exec "sudo systemctl stop snapd.service snapd.socket"
+        remote.exec "sudo mkdir -p /etc/systemd/system/snapd.service.d"
+        remote.exec "echo [Service] | sudo tee /etc/systemd/system/snapd.service.d/proxy.conf"
+        remote.exec "echo Environment=HTTPS_PROXY=$HTTPS_PROXY HTTP_PROXY=$HTTP_PROXY https_proxy=$HTTPS_PROXY http_proxy=$HTTP_PROXY NO_PROXY=$nested_no_proxy no_proxy=$nested_no_proxy SNAPD_USE_PROXY=$SNAPD_USE_PROXY | sudo tee -a /etc/systemd/system/snapd.service.d/proxy.conf"
+        remote.exec "sudo systemctl daemon-reload"
+        remote.exec "sudo systemctl start snapd.service snapd.socket"
+        remote.exec "sudo sync"
+    fi
+    if [ -n "${NTP_SERVER:-}" ]; then
+        # Configure systemd-timesyncd to use the predefined ntp server
+        CONF_FILE="/etc/systemd/timesyncd.conf"
+        remote.exec "sudo sed -i -e '/^NTP=/d' -e '/^FallbackNTP=/d' \"$CONF_FILE\""
+        remote.exec "sudo sed -i '/^\[Time\]/a NTP='\"$NTP_SERVER\" \"$CONF_FILE\""
+        remote.exec "sudo sed -i '/^\[Time\]/a FallbackNTP=' \"$CONF_FILE\""
+        remote.exec "sudo systemctl restart systemd-timesyncd"
+        remote.exec "sudo sync"
     fi
 }
 
@@ -1620,6 +1671,9 @@ nested_start_classic_vm() {
     elif [ "$SPREAD_BACKEND" = "qemu-nested" ] || [ "$SPREAD_BACKEND" = "garden" ]; then
         PARAM_MEM="-m ${NESTED_MEM:-2048}"
         PARAM_SMP="-smp ${NESTED_CPUS:-1}"
+    elif [ "$SPREAD_BACKEND" = "openstack-ext-ps7" ]; then
+        PARAM_MEM="-m ${NESTED_MEM:-4096}"
+        PARAM_SMP="-smp ${NESTED_CPUS:-2}"        
     else
         echo "unknown spread backend $SPREAD_BACKEND"
         exit 1
@@ -1648,7 +1702,7 @@ nested_start_classic_vm() {
     fi
 
     local PARAM_MACHINE PARAM_IMAGE PARAM_SEED PARAM_SERIAL PARAM_BIOS PARAM_TPM
-    if [[ "$SPREAD_BACKEND" = google-nested* ]]; then
+    if [[ "$SPREAD_BACKEND" = google-nested* ]] || [[ "$SPREAD_BACKEND" = openstack* ]]; then
         PARAM_MACHINE="-machine ubuntu,accel=kvm"
         PARAM_CPU="-cpu host"
     elif [ "$SPREAD_BACKEND" = "qemu-nested" ] || [ "$SPREAD_BACKEND" = "garden" ]; then
@@ -1724,7 +1778,9 @@ nested_start_classic_vm() {
     fi
 
     # Copy tools to be used on tests
+    nested_wait_for_ssh
     nested_prepare_tools
+    nested_setup_vm
 }
 
 nested_destroy_vm() {
@@ -1860,7 +1916,13 @@ nested_get_core_revision_installed() {
 nested_fetch_spread() {
     if [ ! -f "$NESTED_WORK_DIR/spread" ]; then
         mkdir -p "$NESTED_WORK_DIR"
-        curl -s https://storage.googleapis.com/snapd-spread-tests/spread/spread-amd64.tar.gz | tar -xz -C "$NESTED_WORK_DIR"
+        if os.query is-arm; then
+            curl -s https://storage.googleapis.com/snapd-spread-tests/spread/spread-plus-arm64.tar.gz | tar -xz -C "$NESTED_WORK_DIR"
+        elif nested_is_core_ge 22; then
+            curl -s https://storage.googleapis.com/snapd-spread-tests/spread/spread-plus-amd64.tar.gz | tar -xz -C "$NESTED_WORK_DIR"
+        else
+            curl -s https://storage.googleapis.com/snapd-spread-tests/spread/spread-amd64.tar.gz | tar -xz -C "$NESTED_WORK_DIR"
+        fi
         # make sure spread really exists
         test -x "$NESTED_WORK_DIR/spread"
     fi
