@@ -436,7 +436,7 @@ distro_install_build_snapd(){
         distro_install_local_package $packages
 
         case "$SPREAD_SYSTEM" in
-            fedora-*|centos-*)
+            fedora-*|centos-*|opensuse-*-selinux-*)
                 # We need to wait until the man db cache is updated before do daemon-reexec
                 # Otherwise the service fails and the system will be degraded during tests executions
                 for i in $(seq 20); do
@@ -489,7 +489,7 @@ distro_install_build_snapd(){
         if ! systemctl is-enabled snapd.socket ; then
             # Can't use --now here as not all distributions we run on support it
             systemctl enable snapd.socket
-            systemctl start snapd.socket
+            systemctl restart snapd.socket
         fi
     fi
 }
@@ -545,6 +545,23 @@ pkg_dependencies_ubuntu_generic(){
         "
 }
 
+pkg_dependencies_ubuntu_nested(){
+    echo "
+        ca-certificates
+        cloud-image-utils
+        genisoimage
+        kpartx
+        mtools
+        ovmf
+        qemu-kvm
+        qemu-utils
+        snapd
+        sshpass
+        xdelta3
+        xz-utils
+        "
+}
+
 pkg_dependencies_ubuntu_classic(){
     echo "
         avahi-daemon
@@ -609,9 +626,10 @@ pkg_dependencies_ubuntu_classic(){
                 shellcheck
                 "
             ;;
-        ubuntu-22.*|ubuntu-23.*|ubuntu-24.04*)
+        ubuntu-22.04*|ubuntu-24.04*)
             # bpftool is part of linux-tools package
             echo "
+                cifs-utils
                 dbus-user-session
                 fwupd
                 golang
@@ -623,9 +641,9 @@ pkg_dependencies_ubuntu_classic(){
                 qemu-utils
                 "
             ;;
-	ubuntu-24.10*|ubuntu-25.*)
+        ubuntu-25.*)
             # bpftool is part of linux-tools package
-            # ubuntu-24.10+ systemd-dev is optional
+            # ubuntu-25.04+ systemd-dev is optional
             echo "
                 dbus-user-session
                 fwupd
@@ -721,23 +739,21 @@ pkg_dependencies_fedora_centos_common(){
         xdg-utils
         strace
         zsh
+        glibc-static
+        libcap-devel
         "
-    if ! os.query is-centos 9; then
-        echo "
-            fish
-            redhat-lsb-core
-        "
-    fi
 }
 
 pkg_dependencies_fedora(){
     echo "
-         libcap-static
-        "
+        fish
+        libcap-static
+        script
+    "
 }
 
 pkg_dependencies_amazon(){
-    if os.query is-amazon-linux 2 || os.query is-centos 7; then
+    if os.query is-amazon-linux 2; then
         echo "
             fish
             fwupd
@@ -775,6 +791,7 @@ pkg_dependencies_amazon(){
         xdg-utils
         udisks2
         zsh
+        glibc-static
         "
 }
 
@@ -805,8 +822,10 @@ pkg_dependencies_opensuse(){
         nss-mdns
         osc
         PackageKit
+        procps
         python3-yaml
         strace
+        sysvinit-tools
         netcat-openbsd
         rpm-build
         udisks2
@@ -815,6 +834,8 @@ pkg_dependencies_opensuse(){
         xdg-user-dirs
         xdg-utils
         zsh
+        libcap-progs
+        glibc-static
         "
     if os.query is-opensuse tumbleweed; then
         echo "
@@ -874,8 +895,13 @@ pkg_dependencies(){
             pkg_dependencies_ubuntu_core
             ;;
         ubuntu-*|debian-*)
-            pkg_dependencies_ubuntu_generic
-            pkg_dependencies_ubuntu_classic
+            if tests.nested is-nested &>/dev/null; then
+                pkg_dependencies_ubuntu_generic
+                pkg_dependencies_ubuntu_nested
+            else
+                pkg_dependencies_ubuntu_generic
+                pkg_dependencies_ubuntu_classic
+            fi
             ;;
         amazon-*)
             pkg_dependencies_amazon
