@@ -87,7 +87,47 @@ disable_refreshes() {
     snap refresh --time --abs-time | MATCH "last: 2[0-9]{3}"
 }
 
+setup_snapd_proxy() {
+    if [ "${SNAPD_USE_PROXY:-}" != true ]; then
+        return
+    fi
+
+    mkdir -p /etc/systemd/system/snapd.service.d
+    cat <<EOF > /etc/systemd/system/snapd.service.d/proxy.conf
+[Service]
+Environment=HTTPS_PROXY=$HTTPS_PROXY HTTP_PROXY=$HTTP_PROXY https_proxy=$HTTPS_PROXY http_proxy=$HTTP_PROXY NO_PROXY=$NO_PROXY no_proxy=$NO_PROXY
+EOF
+
+    # We change the service configuration so reload and restart
+    # the units to get them applied
+    systemctl daemon-reload
+    # restart the service (it pulls up the socket)
+    systemctl restart snapd.service
+}
+
+setup_system_proxy() {
+    mkdir -p "$SNAPD_WORK_DIR"
+    if [ "${SNAPD_USE_PROXY:-}" = true ]; then    
+        cp -f /etc/environment "$SNAPD_WORK_DIR"/environment.bak
+        {
+            echo "HTTPS_PROXY=$HTTPS_PROXY"
+            echo "HTTP_PROXY=$HTTP_PROXY"
+            echo "https_proxy=$HTTPS_PROXY"
+            echo "http_proxy=$HTTP_PROXY"
+            echo "NO_PROXY=$NO_PROXY"
+            echo "no_proxy=$NO_PROXY"
+        } >> /etc/environment
+    fi
+}
+
 setup_systemd_snapd_overrides() {
+    local burst
+    burst=10
+    if [ "$SPREAD_BACKEND" = "garden" ]; then
+        # the tests execute much faster and the repeated stop/start of snapd in
+        # prepare may eventually go over the start limit
+        burst=30
+    fi
     mkdir -p /etc/systemd/system/snapd.service.d
     cat <<EOF > /etc/systemd/system/snapd.service.d/local.conf
 [Service]
@@ -97,7 +137,16 @@ ExecStartPre=/bin/touch /dev/iio:device0
 [Unit]
 # The default limit is usually 5, which can be easily hit in 
 # a fast system with few systemd units
-StartLimitBurst=10
+StartLimitBurst=${burst}
+StartLimitIntervalSec=10s
+EOF
+
+    mkdir -p /etc/systemd/system/snapd.socket.d
+    cat <<EOF > /etc/systemd/system/snapd.socket.d/local.conf
+[Unit]
+# The default limit is usually 5, which can be easily hit in
+# a fast system with few systemd units
+StartLimitBurst=${burst}
 StartLimitIntervalSec=10s
 EOF
 
@@ -200,7 +249,7 @@ update_core_snap_for_classic_reexec() {
     esac
 
     case "$SPREAD_SYSTEM" in
-        fedora-*|centos-*|amazon-*)
+        fedora-*|centos-*|amazon-*|opensuse-*-selinux-*)
             if selinuxenabled ; then
                 # On these systems just unpacking core snap to $HOME will
                 # automatically apply user_home_t label on all the contents of the
@@ -231,7 +280,7 @@ update_core_snap_for_classic_reexec() {
 
     # Now mount the new core snap, first discarding the old mount namespace
     snapd.tool exec snap-discard-ns core
-    mount "$snap" "$core"
+    mount -t squashfs "$snap" "$core"
 
     check_file() {
         if ! cmp "$1" "$2" ; then
@@ -354,11 +403,33 @@ prepare_each_core() {
 }
 
 prepare_classic() {
+    # Configure the proxy in the system when it is required
+    setup_system_proxy   
+
     # Skip building snapd when REUSE_SNAPD is set to 1
     if [ "$REUSE_SNAPD" != 1 ]; then
         distro_install_build_snapd
     fi
 
+    case "$SPREAD_SYSTEM" in
+        opensuse-*-selinux-*)
+            # openSUSE SELinux variant may have restorecond installed, which
+            # apparently is unable to deal with changes to the policy, such as a
+            # new module, done at runtime
+            if systemctl is-active restorecond.service; then
+                systemctl restart restorecond.service
+            fi
+            ;;
+        amazon-linux-2*)
+            # Cloud init service fails in openstack depending on the environment
+            # being used when the metadata retrieved does not contain the proper
+            # networking information
+            if [[ "$SPREAD_BACKEND" =~ openstack ]]; then
+                systemctl restart cloud-init.service
+            fi
+            ;;
+    esac
+
     if snap --version |MATCH unknown; then
         echo "Package build incorrect, 'snap --version' mentions 'unknown'"
         snap --version
@@ -412,6 +483,16 @@ prepare_classic() {
     fi
     snap list snapd
 
+    setup_snapd_proxy
+
+    mount_dir="$(os.paths snap-mount-dir)"
+    if ! getcap "$mount_dir"/snapd/current/usr/lib/snapd/snap-confine | grep "cap_sys_admin"; then
+        echo "snapd snap is missing file capabilities on snap-confine"
+        echo "and is not usable"
+        echo "ensure it has been correctly built (wipe snapcraft containers and rebuild)"
+        exit 1
+    fi
+
     setup_systemd_snapd_overrides
 
     if [ "$REMOTE_STORE" = staging ]; then
@@ -463,6 +544,7 @@ prepare_classic() {
 
         prepare_reexec_override
         prepare_state_lock "SNAPD PROJECT"
+        prepare_tag_features
         prepare_memory_limit_override
         disable_refreshes
 
@@ -478,6 +560,28 @@ prepare_classic() {
             fi
         fi
 
+        # lxd-installer is in cloud images starting from 24.04. This package
+        # installs lxd when any lxc command is run. This caused problems
+        # because if we install snapcraft & lxd, in the restore step lxd is
+        # removed, and after that snapcraft is removed. However, snapcraft's
+        # remove hook calls lxd and triggers a new installation of lxd, and in
+        # turn when we try to remove core22, it fails as lxd has been
+        # re-installed and depends on that base. Therefore, we remove it to
+        # prevent these issues, and we do that before we get the list of
+        # installed packages to make sure we do not re-install it again.
+        if ( os.query is-ubuntu || os.query is-debian ) && tests.pkgs is-installed lxd-installer; then
+            extra=
+            if os.query is-ubuntu-ge 25.10; then
+                # the following dependency is in place in 25.10:
+                # ubuntu-server:amd64 Depends lxd-installer
+                #
+                # NOTE: this will leave some packages without explicit
+                # dependency pulling them in
+                extra=ubuntu-server
+            fi
+            apt remove -y --purge lxd-installer $extra
+        fi
+
         setup_experimental_features
 
         systemctl stop snapd.{service,socket}
@@ -530,6 +634,8 @@ build_snapd_snap() {
     local snapd_snap_cache
     TARGET="${1}"
 
+    mkdir -p "${TARGET}"
+
     snapd_snap_cache="$SNAPD_WORK_DIR/snapd_snap"
     mkdir -p "${snapd_snap_cache}"
     for snap in "${snapd_snap_cache}"/snapd_*.snap; do
@@ -553,7 +659,6 @@ build_snapd_snap() {
                         exit 1
                         ;;
                 esac
-                [ -d "${TARGET}" ] || mkdir -p "${TARGET}"
                 touch "${PROJECT_PATH}"/test-build
                 chmod -R go+r "${PROJECT_PATH}/tests"
                 # TODO: run_snapcraft does not currently guarantee or check the required version for building snapd
@@ -563,44 +668,51 @@ build_snapd_snap() {
         fi
         break
     done
-
     cp "${snapd_snap_cache}"/snapd_*.snap "${TARGET}/"
 }
 
-build_snapd_snap_with_run_mode_firstboot_tweaks() {
-    local snapd_snap_cache
+_get_snapd() {
     local TARGET
-
     TARGET="${1}"
 
-    snapd_snap_cache="$SNAPD_WORK_DIR/snapd_snap_with_tweaks"
-    mkdir -p "${snapd_snap_cache}"
-    for snap in "${snapd_snap_cache}"/snapd_*.snap; do
-        if [ -f "${snap}" ]; then
-            cp "${snap}" "${TARGET}/"
-            return
-        fi
-    done
+    mkdir -p "${TARGET}"
 
     if [ "${USE_PREBUILT_SNAPD_SNAP}" = true ]; then
         if [ -n "${USE_SNAPD_SNAP_URL}" ]; then
-            wget -q "$USE_SNAPD_SNAP_URL" -O /tmp/snapd_from_snapcraft.snap
+            wget -q "$USE_SNAPD_SNAP_URL" -O "${TARGET}"/snapd_from_snapcraft.snap
         else
-            cp "${PROJECT_PATH}/built-snap"/snapd_1337.*.snap.keep "/tmp/snapd_from_snapcraft.snap"
+            cp "${PROJECT_PATH}/built-snap"/snapd_1337.*.snap.keep "${TARGET}/snapd_from_snapcraft.snap"
         fi
     else
+        touch "${PROJECT_PATH}"/test-build
         chmod -R go+r "${PROJECT_PATH}/tests"
         run_snapcraft --use-lxd --verbosity quiet --output="snapd_from_snapcraft.snap"
-        mv "${PROJECT_PATH}/snapd_from_snapcraft.snap" "/tmp/snapd_from_snapcraft.snap"
+        mv "${PROJECT_PATH}/snapd_from_snapcraft.snap" "${TARGET}/snapd_from_snapcraft.snap"
     fi
+}
 
-    # TODO set up a trap to clean this up properly?
+_add_gpio_iio_slots() {
     local UNPACK_DIR
-    UNPACK_DIR="$(mktemp -d /tmp/snapd-unpack.XXXXXXXX)"
-    unsquashfs -no-progress -f -d "$UNPACK_DIR" /tmp/snapd_from_snapcraft.snap
+    UNPACK_DIR="${1}"
+
+    cat >> "${UNPACK_DIR}/meta/snap.yaml" <<-EOF
+slots:
+    gpio-pin:
+        interface: gpio
+        number: 100
+        direction: out
+    iio0:
+        interface: iio
+        path: /dev/iio:device0
+EOF
+}
+
+_add_run_mode_tweaks() {
+    local UNPACK_DIR
+    UNPACK_DIR="${1}"
 
     # now install a unit that sets up enough so that we can connect
-    cat > "$UNPACK_DIR"/lib/systemd/system/snapd.spread-tests-run-mode-tweaks.service <<'EOF'
+    cat > "${UNPACK_DIR}"/lib/systemd/system/snapd.spread-tests-run-mode-tweaks.service <<'EOF'
 [Unit]
 Description=Tweaks to run mode for spread tests
 Before=snapd.service
@@ -615,7 +727,7 @@ RemainAfterExit=true
 WantedBy=multi-user.target
 EOF
     # XXX: this duplicates a lot of setup_test_user_by_modify_writable()
-    cat > "$UNPACK_DIR"/usr/lib/snapd/snapd.spread-tests-run-mode-tweaks.sh <<'EOF'
+    cat > "${UNPACK_DIR}"/usr/lib/snapd/snapd.spread-tests-run-mode-tweaks.sh <<'EOF'
 #!/bin/sh
 set -ex
 # ensure we don't enable ssh in install mode or spread will get confused
@@ -667,12 +779,66 @@ fi
 
 touch /root/spread-setup-done
 EOF
-    chmod 0755 "$UNPACK_DIR"/usr/lib/snapd/snapd.spread-tests-run-mode-tweaks.sh
+    chmod 0755 "${UNPACK_DIR}"/usr/lib/snapd/snapd.spread-tests-run-mode-tweaks.sh
+}
 
-    mkdir -p "${snapd_snap_cache}"
-    snap pack "$UNPACK_DIR" "${snapd_snap_cache}/"
-    rm -rf "$UNPACK_DIR"
-    cp "${snapd_snap_cache}"/snapd_*.snap "${TARGET}/"
+build_snapd_snap_for_core18() {
+    local TARGET
+    local SNAP_CACHE
+    
+    TARGET="${1}"
+    SNAP_CACHE="$SNAPD_WORK_DIR/snapd_snap_core18"
+
+    mkdir -p "${SNAP_CACHE}"
+    for snap in "${SNAP_CACHE}"/snapd_*.snap; do
+        if [ -f "${snap}" ]; then
+            cp "${snap}" "${TARGET}/"
+            return
+        fi
+    done
+
+    _get_snapd "${SNAP_CACHE}/downloads"
+
+    mkdir -p "${SNAP_CACHE}/unpack"
+    unsquashfs -no-progress -f -d "${SNAP_CACHE}/unpack" "${SNAP_CACHE}"/downloads/snapd_from_snapcraft.snap
+
+    # add gpio and iio slots required for the tests
+    _add_gpio_iio_slots "${SNAP_CACHE}/unpack"
+
+    snap pack "${SNAP_CACHE}/unpack" "${SNAP_CACHE}/"
+    rm -rf "${SNAP_CACHE}/unpack"
+    cp "${SNAP_CACHE}"/snapd_*.snap "${TARGET}/"
+}
+
+build_snapd_snap_with_run_mode_firstboot_tweaks() {
+    local SNAP_CACHE
+    local TARGET
+
+    TARGET="${1}"
+    SNAP_CACHE="$SNAPD_WORK_DIR/snapd_snap_with_tweaks"
+
+    mkdir -p "${SNAP_CACHE}"
+    for snap in "${SNAP_CACHE}"/snapd_*.snap; do
+        if [ -f "${snap}" ]; then
+            cp "${snap}" "${TARGET}/"
+            return
+        fi
+    done
+
+    _get_snapd "${SNAP_CACHE}/downloads"
+
+    mkdir -p "${SNAP_CACHE}/unpack"
+    unsquashfs -no-progress -f -d "${SNAP_CACHE}/unpack" "${SNAP_CACHE}"/downloads/snapd_from_snapcraft.snap
+
+    # add tweaks to run mode for spread tests
+    _add_run_mode_tweaks "${SNAP_CACHE}/unpack"
+
+    # add gpio and iio slots required for the tests
+    _add_gpio_iio_slots "${SNAP_CACHE}/unpack"
+
+    snap pack "${SNAP_CACHE}/unpack" "${SNAP_CACHE}/"
+    rm -rf "${SNAP_CACHE}/unpack"
+    cp "${SNAP_CACHE}"/snapd_*.snap "${TARGET}/"
 }
 
 repack_core_snap_with_tweaks() {
@@ -708,6 +874,10 @@ EOF
 
     cp "${SPREAD_PATH}"/data/completion/bash/complete.sh "${UNPACK_DIR}"/usr/lib/snapd/complete.sh
 
+    if [ "${SNAPD_USE_PROXY:-}" = true ]; then
+        cp /etc/environment "${UNPACK_DIR}"/etc/environment
+    fi
+
     snap pack --filename="$TARGET" "$UNPACK_DIR"
 
     rm -rf "$UNPACK_DIR"
@@ -809,7 +979,7 @@ EOF
 uc20_build_initramfs_kernel_snap() {
     quiet apt install software-properties-common -y
     # carries ubuntu-core-initframfs
-    quiet add-apt-repository ppa:snappy-dev/image -y
+    retry -n 10 quiet add-apt-repository ppa:snappy-dev/image -y
     # On focal, lvm2 does not reinstall properly after being removed.
     # So we need to clean up in case the VM has been re-used.
     if os.query is-focal; then
@@ -818,7 +988,7 @@ uc20_build_initramfs_kernel_snap() {
     # TODO: install the linux-firmware as the current version of
     # ubuntu-core-initramfs does not depend on it, but nonetheless requires it
     # to build the initrd
-    quiet apt install ubuntu-core-initramfs linux-firmware -y
+    retry -n 10 quiet apt install ubuntu-core-initramfs linux-firmware -y
 
     local ORIG_SNAP="$1"
     local TARGET="$2"
@@ -994,6 +1164,8 @@ uc24_build_initramfs_kernel_snap() {
     ubuntu-core-initramfs create-initrd --kernelver="$kernelver" --kerneldir pc-kernel/modules/"$kernelver" \
                           --firmwaredir pc-kernel/firmware --output initrd.img
 
+    # Check that manifest is generated
+    stat manifest-initramfs.yaml-"$kernelver"
     initrd_f=initrd.img-"$kernelver"
     unmkinitramfs "$initrd_f" initrd
 
@@ -1231,7 +1403,7 @@ setup_reflash_magic() {
     export UBUNTU_IMAGE_SNAP_CMD="$IMAGE_HOME/snap"
 
     if is_test_target_core 18; then
-        build_snapd_snap "${IMAGE_HOME}"
+        build_snapd_snap_for_core18 "${IMAGE_HOME}"
         # FIXME: fetch directly once its in the assertion service
         cp "$TESTSLIB/assertions/ubuntu-core-18-amd64.model" "$IMAGE_HOME/pc.model"
     elif is_test_target_core 20; then
@@ -1246,7 +1418,12 @@ setup_reflash_magic() {
         fi
     elif is_test_target_core 24; then
         build_snapd_snap_with_run_mode_firstboot_tweaks "$IMAGE_HOME"
-        cp "$TESTSLIB/assertions/ubuntu-core-24-amd64.model" "$IMAGE_HOME/pc.model"
+        if os.query is-arm; then
+            cp "$TESTSLIB/assertions/ubuntu-core-24-arm64.model" "$IMAGE_HOME/pc.model"
+        else
+            cp "$TESTSLIB/assertions/ubuntu-core-24-amd64.model" "$IMAGE_HOME/pc.model"
+        fi
+        
     else
         # FIXME: install would be better but we don't have dpkg on
         #        the image
@@ -1347,7 +1524,7 @@ EOF
         # so for now, don't include snapd.debug=1, but eventually it would be
         # nice to have this on
 
-        if [[ "$SPREAD_BACKEND" =~ google ]]; then
+        if [[ "$SPREAD_BACKEND" =~ google ]] || [[ "$SPREAD_BACKEND" =~ openstack ]]; then
             # the default console settings for snapd aren't super useful in GCE,
             # instead it's more useful to have all console go to ttyS0 which we 
             # can read more easily than tty1 for example
@@ -1423,6 +1600,18 @@ EOF
         # * core20 (to avoid the automatic refresh issue)
         if [ "$IMAGE_CHANNEL" != "$BASE_CHANNEL" ]; then
             unsquashfs -d "${BASE}-snap" "${BASE}.snap"
+
+            # We setup the ntp server in case it is defined in the current env
+            # This is not needed in classic systems because the images already have ntp configured
+            if [ -n "${NTP_SERVER:-}" ]; then
+                TARGET_TIME_CONF="$(find "${BASE}-snap" -name timesyncd.conf)"
+                if [ -z "$TARGET_TIME_CONF" ]; then
+                    echo "File timesyncd.conf not found in core image"
+                    exit 1
+                fi
+                cp /etc/systemd/timesyncd.conf "$TARGET_TIME_CONF"
+            fi
+
             snap pack --filename="${BASE}-repacked.snap" "${BASE}-snap"
             rm -r "${BASE}-snap"
             mv "${BASE}-repacked.snap" "${IMAGE_HOME}/${BASE}.snap"
@@ -1445,7 +1634,7 @@ EOF
                     $EXTRA_FUNDAMENTAL \
                     --snap "${extra_snap[0]}" \
                     --output-dir "$IMAGE_HOME"
-    rm -f ./pc-kernel_*.{snap,assert} ./pc-kernel.{snap,assert} ./pc_*.{snap,assert} ./snapd_*.{snap,assert} ./core{20,22}.{snap,assert}
+    rm -f ./pc-kernel_*.{snap,assert} ./pc-kernel.{snap,assert} ./pc_*.{snap,assert} ./snapd_*.{snap,assert} ./core{20,22,24}.{snap,assert}
 
     if os.query is-arm; then
         LOOP_PARTITION=1
@@ -1606,13 +1795,45 @@ EOF
     fi
 }
 
+prepare_tag_features(){
+    CONF_FILE="/etc/systemd/system/snapd.service.d/99-feature-tags.conf"
+    RESTART=false
+
+    if [ -n "$TAG_FEATURES" ]; then
+        # Generate the config file when it does not exist and when the threshold has changed different
+        if ! [ -f "$CONF_FILE" ]; then
+            cat <<EOF > "$CONF_FILE"
+[Service]
+Environment=SNAPPY_TESTING=1
+Environment=SNAPD_TRACE=1
+Environment=SNAPD_JSON_LOGGING=1
+EOF
+            RESTART=true
+        fi
+    elif [ -f "$CONF_FILE" ]; then
+        rm -f "$CONF_FILE"
+        RESTART=true
+    fi
+
+    if [ "$RESTART" = "true" ]; then
+        # the service setting may have changed in the service so we need
+        # to ensure snapd is reloaded
+        systemctl daemon-reload
+        systemctl restart snapd
+    fi
+}
+
 # prepare_ubuntu_core will prepare ubuntu-core 16+
 prepare_ubuntu_core() {
+    # Configure the proxy in the system when it is required
+    setup_system_proxy
+
     # we are still a "classic" image, prepare the surgery
     if [ -e /var/lib/dpkg/status ]; then
         setup_reflash_magic
         REBOOT
     fi
+    setup_snapd_proxy
 
     disable_journald_rate_limiting
     disable_journald_start_limiting
@@ -1665,6 +1886,8 @@ prepare_ubuntu_core() {
             rsync_snap="test-snapd-rsync-core22"
         elif os.query is-core24; then
             rsync_snap="test-snapd-rsync-core24"
+        elif os.query is-core26; then
+            rsync_snap="test-snapd-rsync-core26"
         fi
         snap install --devmode --edge "$rsync_snap"
         snap alias "$rsync_snap".rsync rsync
@@ -1695,6 +1918,9 @@ prepare_ubuntu_core() {
         if os.query is-core24; then
             cache_snaps test-snapd-sh-core24
         fi
+        if os.query is-core26; then
+            cache_snaps test-snapd-sh-core26
+        fi
     fi
 
     disable_refreshes
@@ -1720,6 +1946,7 @@ prepare_ubuntu_core() {
         remove_disabled_snaps
         prepare_memory_limit_override
         prepare_state_lock "SNAPD PROJECT"
+        prepare_tag_features
         setup_experimental_features
         systemctl stop snapd.service snapd.socket
         save_snapd_state
