@@ -77,6 +77,10 @@ class Retriever(ABC):
     '''
     Retrieves features tags from a data source.
     '''
+
+    def __init__(self):
+        self.cache = {}
+
     @abstractmethod
     def get_sorted_timestamps_and_systems(self) -> list[dict[str, Any]]:
         '''
@@ -84,29 +88,66 @@ class Retriever(ABC):
         Format: [{"timestamp":<timestamp>,"systems":[<system1>,...,<systemN>]}]
         '''
 
-    @abstractmethod
     def get_single_json(self, timestamp: str, system: str) -> SystemFeatures:
         '''
         Given the timestamp and system, gets a single dictionary entry.
 
         :raises RuntimeError: when there is not exactly one entry for the system at the timestamp
         '''
+        if timestamp in self.cache and 'systems' in self.cache[timestamp]:
+            for s in self.cache[timestamp]['systems']:
+                if s['system'] == system:
+                    return s
+        return self._get_single_json(timestamp, system)
 
     @abstractmethod
+    def _get_single_json(self, timestamp: str, system: str) -> SystemFeatures:
+        pass
+
     def get_systems(self, timestamp: str, systems: list[str] = None) -> Iterable[SystemFeatures]:
         '''
         Retrieves dictionary entries for all indicated systems at 
         the given timestamp. If systems is None, then it retrieves 
         all dictionary entries for all systems at the given timestamp.
         '''
+        if timestamp in self.cache and 'systems' in self.cache[timestamp]:
+            if not systems:
+                return self.cache[timestamp]['systems']
+            else:
+                return [sys for sys in self.cache[timestamp]['systems'] if sys['system'] in systems]
+            
+        if systems:
+            # Do not cache if only a subset of systems is specified
+            # otherwise the cached list won't be complete
+            return self._get_systems(timestamp, systems)
+        else:
+            if timestamp not in self.cache:
+                self.cache[timestamp] = {}
+            self.cache[timestamp]['systems'] = list(self._get_systems(timestamp, systems))
+            return self.cache[timestamp]['systems']
+        
 
     @abstractmethod
+    def _get_systems(self, timestamp: str, systems: list[str] = None) -> Iterable[SystemFeatures]:
+        pass
+
     def get_all_features(self, timestamp: str) -> dict[str, list[Any]]:
         '''
         Retrives a dictionary of all feature data at the given timestmap.
         It contains only feature keys (e.g. cmds, endpoints) and the list
         of all possible feature data at the indicated moment in time.
         '''
+        if timestamp in self.cache and 'all-features' in self.cache[timestamp]:
+            return self.cache[timestamp]['all-features']
+        if timestamp not in self.cache:
+            self.cache[timestamp] = {}
+        all_feat = self._get_all_features(timestamp)
+        self.cache[timestamp]['all-features'] = all_feat
+        return all_feat
+
+    @abstractmethod
+    def _get_all_features(self, timestamp: str) -> dict[str, list[Any]]:
+        pass
 
     @abstractmethod
     def close(self) -> None:
@@ -122,6 +163,7 @@ class MongoRetriever(Retriever):
     '''
 
     def __init__(self, creds_file):
+        super().__init__()
         config = json.load(creds_file)
         self.client = pymongo.MongoClient(
             host=config['host'], port=config['port'], username=config['user'], password=config['password'])
@@ -140,7 +182,7 @@ class MongoRetriever(Retriever):
                 result['system'])
         return [{"timestamp": entry[0], "systems": entry[1]} for entry in sorted(dictionary.items(), reverse=True)]
 
-    def get_systems(self, timestamp: str, systems: list[str] = None) -> Iterable[SystemFeatures]:
+    def _get_systems(self, timestamp: str, systems: list[str] = None) -> Iterable[SystemFeatures]:
         if systems:
             for system in systems:
                 system_jsons = self.collection.find(
@@ -152,14 +194,14 @@ class MongoRetriever(Retriever):
                 if 'all_features' not in result or not result['all_features']:
                     yield result
 
-    def get_single_json(self, timestamp: str, system: str) -> SystemFeatures:
+    def _get_single_json(self, timestamp: str, system: str) -> SystemFeatures:
         json_result = self.collection.find(
             {'timestamp': datetime.datetime.fromisoformat(timestamp), 'system': system}).to_list()
         if len(json_result) != 1:
             raise RuntimeError(f'{len(json_result)} entries of system {system} found in collection {timestamp}')
         return json_result[0]
     
-    def get_all_features(self, timestamp) -> dict[str, list[Any]]:
+    def _get_all_features(self, timestamp) -> dict[str, list[Any]]:
         json_result = self.collection.find(
             {'timestamp': datetime.datetime.fromisoformat(timestamp), 'all_features': True}).to_list()
         if len(json_result) != 1:
@@ -183,6 +225,7 @@ class DirRetriever(Retriever):
     '''
 
     def __init__(self, dir: str):
+        super().__init__()
         if not os.path.exists(dir):
             raise RuntimeError(f'directory {dir} does not exist')
         self.dir = dir
@@ -206,7 +249,7 @@ class DirRetriever(Retriever):
                     dictionary[timestamp].append(system)
         return [{"timestamp": entry[0], "systems": entry[1]} for entry in sorted(dictionary.items(), reverse=True)]
 
-    def get_systems(self, timestamp: str, systems: list[str] = None) -> Iterable[SystemFeatures]:
+    def _get_systems(self, timestamp: str, systems: list[str] = None) -> Iterable[SystemFeatures]:
         timestamp_dir = os.path.join(self.dir, timestamp)
         if not os.path.isdir(timestamp_dir):
             raise RuntimeError(
@@ -218,14 +261,14 @@ class DirRetriever(Retriever):
                 with open(os.path.join(timestamp_dir, filename), 'r', encoding='utf-8') as f:
                     yield json.load(f)
 
-    def get_single_json(self, timestamp: str, system: str) -> SystemFeatures:
+    def _get_single_json(self, timestamp: str, system: str) -> SystemFeatures:
         sys_path = os.path.join(self.dir, timestamp, system+'.json')
         if not os.path.exists(sys_path):
             raise RuntimeError(f'system file not found {sys_path}')
         with open(sys_path, 'r', encoding='utf-8') as f:
             return json.load(f)
         
-    def get_all_features(self, timestamp) -> dict[str, list[Any]]:
+    def _get_all_features(self, timestamp) -> dict[str, list[Any]]:
         sys_path = os.path.join(self.dir, timestamp, ALL_FEATURES_FILE)
         if not os.path.exists(sys_path):
             raise RuntimeError(f'{ALL_FEATURES_FILE} not found')
@@ -285,6 +328,26 @@ def minus(first: dict[str, list], second: dict[str, list]) -> dict:
     return minus
 
 
+def subtract_features(first: dict[str, list], second: dict[str, list], match_snap_types: bool) -> dict:
+    if match_snap_types:
+        return minus(first, second)
+    
+    diff = minus({'cmds': first['cmds'], 'ensures': first['ensures'], 'endpoints': first['endpoints']}, second)
+    tasks = [af for af in first['tasks'] 
+            if not any(af['kind'] == sf['kind'] and af['last_status'] == sf['last_status'] for sf in second['tasks'])]
+    changes = [af for af in first['changes'] 
+            if not any(af['kind'] == sf['kind'] for sf in second['changes'])]
+    interfaces = [af for af in first['interfaces'] 
+                if not any(af['name'] == sf['name'] for sf in second['interfaces'])]
+    if tasks:
+        diff['tasks'] = tasks
+    if changes:
+        diff['changes'] = changes
+    if interfaces:
+        diff['interfaces'] = interfaces
+    return diff
+
+
 def list_tasks(system_json: SystemFeatures, remove_failed: bool) -> set[TaskIdVariant]:
     '''
     Lists all tasks present in the SystemFeatures dictionary
@@ -299,7 +362,7 @@ def list_tasks(system_json: SystemFeatures, remove_failed: bool) -> set[TaskIdVa
     return tasks
 
 
-def diff(retriever: Retriever, timestamp1: str, system1: str, timestamp2: str, system2: str, remove_failed: bool, only_same: bool) -> dict:
+def diff(retriever: Retriever, timestamp1: str, system1: str, timestamp2: str, system2: str, remove_failed: bool, only_same: bool, match_snap_types: bool = True) -> dict:
     '''
     Calculates set(system1_features) - set(system2_features), each at their
     respective timestamps. 
@@ -323,8 +386,8 @@ def diff(retriever: Retriever, timestamp1: str, system1: str, timestamp2: str, s
         system_json1, include_tasks=include_tasks1)
     features2 = consolidate_system_features(
         system_json2, include_tasks=include_tasks2)
-    mns = minus(features1, features2)
-    return mns
+    
+    return subtract_features(features1, features2, match_snap_types)
 
 
 def diff_all_features(retriever: Retriever, timestamp: str, system: str, remove_failed: bool) -> dict:
@@ -338,20 +401,7 @@ def diff_all_features(retriever: Retriever, timestamp: str, system: str, remove_
     '''
     sys_features = feat_sys(retriever, timestamp, system, remove_failed)
     all_features = retriever.get_all_features(timestamp)
-    diff = minus({'cmds': all_features['cmds'], 'ensures': all_features['ensures'], 'endpoints': all_features['endpoints']}, sys_features)
-    tasks = [af for af in all_features['tasks'] 
-             if not any(af['kind'] == sf['kind'] and af['last_status'] == sf['last_status'] for sf in sys_features['tasks'])]
-    changes = [af for af in all_features['changes'] 
-               if not any(af['kind'] == sf['kind'] for sf in sys_features['changes'])]
-    interfaces = [af for af in all_features['interfaces'] 
-                  if not any(af['name'] == sf['name'] for sf in sys_features['interfaces'])]
-    if tasks:
-        diff['tasks'] = tasks
-    if changes:
-        diff['changes'] = changes
-    if interfaces:
-        diff['interfaces'] = interfaces
-    return diff
+    return subtract_features(all_features, sys_features, False)
 
 
 def feat_sys(retriever: Retriever, timestamp: str, system: str, remove_failed: bool, suite: str = None, task: str = None, variant: str = None) -> dict:
@@ -363,7 +413,12 @@ def feat_sys(retriever: Retriever, timestamp: str, system: str, remove_failed: b
     system_json = retriever.get_single_json(timestamp, system)
 
     if suite or task or variant:
-        system_json['tests'] = [test for test in system_json['tests'] 
+        tests = system_json['tests']
+        system_json = SystemFeatures(schema_version=system_json['schema_version'] if 'schema_version' in system_json else '', 
+                                     system=system_json['system'] if 'system' in system_json else '', 
+                                     scenarios=system_json['scenarios'] if 'scenarios' in system_json else '',
+                                     env_variables=system_json['env_variables'] if 'env_variables' in system_json else '')
+        system_json['tests'] = [test for test in tests 
                                 if (not suite or test['suite'] == suite) and 
                                 (not task or test['task_name'] == task) and 
                                 (not variant or test['variant'] == variant)]
@@ -392,7 +447,7 @@ def get_feature_name_from_feature(feat: dict) -> str:
     return ''
 
 
-def find_feat(retriever: Retriever, timestamp: str, feat: dict, remove_failed: bool, system: str = None) -> dict[str, TaskIdVariant]:
+def find_feat(retriever: Retriever, timestamp: str, feat: dict, remove_failed: bool, system: str = None, match_snap_types: bool = False) -> dict[str, TaskIdVariant]:
     '''
     Given a timestamp, a feature, and optionally a system, finds
     all tests that contain the indicated feature. If no system
@@ -410,6 +465,7 @@ def find_feat(retriever: Retriever, timestamp: str, feat: dict, remove_failed: b
     A feature match happens for interfaces when the names match.
 
     :param remove_failed: if true, will remove all instances of tests where success == False
+    :param force_exact: if true, will match the entire feature when searching for tests
     :returns: dictionary where each key is a system and each value is a list of tests that contain the feature
     '''
 
@@ -417,15 +473,14 @@ def find_feat(retriever: Retriever, timestamp: str, feat: dict, remove_failed: b
     if not feat_name:
         raise RuntimeError(f'feature {feat} not a recognized feature')
 
-    feat_in_test=lambda _: False
-    if feat_name == 'cmds' or feat_name == 'ensures' or feat_name == 'endpoints':
-        feat_in_test = lambda test: feat_name in test and feat in test[feat_name]
-    elif feat_name == 'tasks':
-        feat_in_test = lambda test: feat_name in test and any(t['kind'] == feat['kind'] and t['last_status'] == feat['last_status'] for t in test['tasks'])
-    elif feat_name == 'changes':
-        feat_in_test = lambda test: feat_name in test and any(c['kind'] == feat['kind'] for c in test['changes'])
-    elif feat_name == 'interfaces':
-        feat_in_test = lambda test: feat_name in test and any(i['name'] == feat['name'] for i in test['interfaces'])
+    feat_in_test=lambda test: feat_name in test and feat in test[feat_name]
+    if not match_snap_types:
+        if feat_name == 'tasks':
+            feat_in_test = lambda test: feat_name in test and any(t['kind'] == feat['kind'] and t['last_status'] == feat['last_status'] for t in test['tasks'])
+        elif feat_name == 'changes':
+            feat_in_test = lambda test: feat_name in test and any(c['kind'] == feat['kind'] for c in test['changes'])
+        elif feat_name == 'interfaces':
+            feat_in_test = lambda test: feat_name in test and any(i['name'] == feat['name'] for i in test['interfaces'])
 
     system_list = None
     if system:
@@ -466,8 +521,13 @@ def dup(retriever: Retriever, timestamp: str, system: str, remove_failed: bool)
     system_json = retriever.get_single_json(timestamp, system)
 
     if remove_failed:
+        tests = system_json['tests']
+        system_json = SystemFeatures(schema_version=system_json['schema_version'] if 'schema_version' in system_json else '', 
+                                     system=system_json['system'] if 'system' in system_json else '', 
+                                     scenarios=system_json['scenarios'] if 'scenarios' in system_json else '',
+                                     env_variables=system_json['env_variables'] if 'env_variables' in system_json else '')
         system_json['tests'] = [
-            task for task in system_json['tests'] if task['success']]
+            task for task in tests if task['success']]
 
     duplicates = []
     with concurrent.futures.ProcessPoolExecutor() as executor:
@@ -586,6 +646,7 @@ def add_diff_parsers(subparsers: argparse._SubParsersAction) -> tuple[str, str,
     sys.add_argument('-s2', '--system2', help='system of second execution', type=str, required=True)
     sys.add_argument('--remove-failed', help='remove all tasks that failed', action='store_true')
     sys.add_argument('--only-same', help='only compare tasks that were executed on both systems', action='store_true')
+    sys.add_argument('--match-snap-types', help='match the entire feature, including snap types', action='store_true')
 
     all = diff_subparsers.add_parser(cmd_all, help='calculate diff between system features and all features',
                                      description=all_description, formatter_class=argparse.RawDescriptionHelpFormatter)
@@ -667,6 +728,7 @@ def add_all_features_parser(subparsers: argparse._SubParsersAction) -> tuple[str
     find.add_argument('-s', '--system', help='(optional) system to search for feature in', default=None, type=str)
     find.add_argument('--feat', help='feature to search for (json format)', required=True, type=str)
     find.add_argument('--remove-failed', help='remove all tasks that failed', action='store_true')
+    find.add_argument('--match-snap-types', help='match the entire feature, including snap types', action='store_true')
     return cmd, cmd_all, cmd_sys, cmd_find
 
 
@@ -696,7 +758,7 @@ def main():
         if args.command == diff_cmd:
             if args.diff_cmd == diff_sys_cmd:
                 result = diff(retriever, args.timestamp1, args.system1,
-                            args.timestamp2, args.system2, args.remove_failed, args.only_same)
+                            args.timestamp2, args.system2, args.remove_failed, args.only_same, args.match_snap_types)
             elif args.diff_cmd == diff_all_cmd:
                 result = diff_all_features(retriever, args.timestamp, args.system, args.remove_failed)
             else:
@@ -725,7 +787,7 @@ def main():
             elif args.features_cmd == feat_find_cmd:
                 try:
                     feat = json.loads(args.feat)
-                    result = find_feat(retriever, args.timestamp, feat, args.remove_failed, args.system)
+                    result = find_feat(retriever, args.timestamp, feat, args.remove_failed, args.system, args.match_snap_types)
                     json.dump(result, sys.stdout, default=lambda x: str(x))
                 except Exception as e:
                     raise RuntimeError(f'Error parsing feature {args.feat}: {e}')
